#ifdef WIN32
#pragma warning(disable : 4786 )
#endif
#include "CSAnimation.h"
#include "CSXMLHelper.h"

CSAnimationLoader CSAnimationLoader::INSTANCE;
const char *CSAnimation::CLASS = "CSAnimation";

CSAnimation::CSAnimation(const std::string &filename)
{
	static char *functionName="CSAnimation";
	LOG_ENTER 
	CSAnimationData data;
	loadAnimationData(filename, data);
	initialize(data);
	LOG_EXIT
}

CSAnimation::CSAnimation(const CSAnimation &animation)
{
	static char *functionName="CSAnimation";
	LOG_ENTER 
	mSecsPerFrame = animation.mSecsPerFrame;
	mSizePictures = animation.mSizePictures;
	mSizePhases = animation.mSizePhases;
	mMoveAllowed = animation.mMoveAllowed;
	mRepeatable = animation.mRepeatable;
	mIsBreakable = animation.mIsBreakable;
	mIsSolid = animation.mIsSolid;

	mPhases = new unsigned int[mSizePhases];
	int i;
	for (i=0; i < mSizePhases; i++)
	{
		mPhases[i] = animation.mPhases[i];
	}

	mPictures = new CSPictures();
	mAnimationRates = new float[mSizePictures];
	mMoveX = new signed int[mSizePictures];
	mMoveY = new signed int[mSizePictures];
	i = 0;
	CSPictures::iterator iter = animation.mPictures->begin();
	while (iter != animation.mPictures->end())
	{
		CSPicture *pic = *iter;
		// picture need not be loaded again->
		// we can use the same pointer since it is a singleton anyway!
		mPictures->push_back(pic);

		mAnimationRates[i] = animation.mAnimationRates[i];
		mMoveX[i] = animation.mMoveX[i];
		mMoveY[i] = animation.mMoveY[i];
		i++;
		iter++;
	}
	LOG_EXIT
}

void CSAnimation::initialize(void)
{
	static char *functionName="initialize";
	LOG_ENTER 
	mMoveX = 0;
	mMoveY = 0;
	mPictures = 0;
	mSizePictures = 0;
	mSizePhases = 0;
	mPhases = 0;
	mAnimationRates = 0;
	mMoveAllowed = true;
	mRepeatable = 0;
	mIsBreakable = 0;
	mIsSolid = false;;
	mSecsPerFrame = 0.01f;
	LOG_EXIT
}

void CSAnimation::initialize(const CSAnimationData &data)
{
	static char *functionName="initialize";
	LOG_ENTER 
	int i = 0;
	initialize();
	mIsBreakable = data.isBreakable;
	mRepeatable = data.repeatable;
	mSizePhases = data.sizePhases;
	mPhases = new unsigned int[mSizePhases];
	for (i; i < mSizePhases; i++)
	{
		mPhases[i] = data.phases[i];
	}

	// how many pictures are there?
	// is there a better way, to get the size of a vector?
	// data.pictures->size() :-)
	CSAnimationPictures::const_iterator iter = data.pictures->begin();
	while (iter != data.pictures->end())
	{
		iter++;
		mSizePictures++;
	}

	mPictures = new CSPictures();
	mAnimationRates = new float[mSizePictures];
	mMoveX = new signed int[mSizePictures];
	mMoveY = new signed int[mSizePictures];
	i = 0;
	iter = data.pictures->begin();
	while (iter != data.pictures->end())
	{
		// picture must be loaded
		CSPicture *pic = CSPictureLoader::INSTANCE.load((*iter)->filename);

		mPictures->push_back(pic);
		mAnimationRates[i] = (*iter)->animationRate;
		// negative animation rates are not in animtions per second
		// but aniamtion each # of seconds!
		if (mAnimationRates[i]<0)
		{
			mAnimationRates[i] *= -1;
			mAnimationRates[i] = (float) (1.0f/mAnimationRates[i]);
		}
		mMoveX[i] = (*iter)->moveX;
		mMoveY[i] = (*iter)->moveY;
		i++;
		iter++;
	}
	LOG_EXIT
}

CSAnimation::~CSAnimation()
{
	static char *functionName="~CSAnimation";
	LOG_ENTER 
	if (mPictures != 0)
	{
		// CSPictures are deleted by CSPictureLoader!
		delete mPictures;
		mPictures = 0;
	}	
	if (mPhases != 0)
	{
		delete []mPhases;
		mPhases = 0;
	}
	if (mAnimationRates != 0)
	{
		delete []mAnimationRates;
		mAnimationRates = 0;
	}
	if (mMoveX !=0)
	{
		delete []mMoveX;
		mMoveX = 0;
	}
	if (mMoveY !=0)
	{
		delete []mMoveY;
		mMoveY = 0;
	}
	LOG_EXIT
}

void CSAnimation::setSecPerFrame(float secPerFrame)
{
	static char *functionName="setSecPerFrame";
	LOG_ENTER 
	mSecsPerFrame = secPerFrame;
	LOG_EXIT
}

void CSAnimation::setSolid(bool b)
{
	static char *functionName="setSolid";
	LOG_ENTER 
	mIsSolid = b;
	for (CSPictures::iterator iter = mPictures->begin(); iter != mPictures->end(); iter++)
	{
		CSPicture *pic = *iter;
		pic->setSolid(b);
	}
	LOG_EXIT
}

void CSAnimation::setAlpha(int alpha)
{
	static char *functionName="setAlpha";
	LOG_ENTER 
	for (CSPictures::iterator iter = mPictures->begin(); iter != mPictures->end(); iter++)
	{
		CSPicture *pic = *iter;
		pic->setAlpha(alpha);
	}
	LOG_EXIT
}

//! build an initial state for this aniamtion, sets variables to defaults
AnimationState CSAnimation::buildState()
{
	static char *functionName="buildState";
	AnimationState state;
	state.mXPos = 0;
	state.mYPos = 0;
	state.mCurrentPhase = 0;
	state.mCurrentPicture = mPhases[state.mCurrentPhase];
	state.mPicture = mPictures->at(state.mCurrentPicture);
	state.mAnimationRateCounter = 0;
	state.mForward = true;
	state.mFinished = false;
	return state;
}

//! resets a state to default values
//! doesn't  update mDisplayParams! 
//! (this position and mDisplayPicture do not change!)
void CSAnimation::resetState(AnimationState& state)
{
	static char *functionName="resetState";
	state.mCurrentPhase = 0;
	state.mCurrentPicture = mPhases[state.mCurrentPhase];
	state.mPicture = mPictures->at(state.mCurrentPicture);
	state.mAnimationRateCounter = 0;
	state.mSpeedX = ANIMATION_SPEED;
	state.mSpeedY = ANIMATION_SPEED;
	state.mForward = true;
	state.mFinished = false;
	state.mXPos = 0;
	state.mYPos = 0;
}

//! resets a state to start values of this animations mDisplayParams!  -> immediately
//! THIS will be displayed during next display() call
//! happens when an event for another action occurs or so
void CSAnimation::startAnimation(AnimationState& state)
{
	static char *functionName="startAnimation";
	state.mCurrentPhase = 0;
	state.mCurrentPicture = mPhases[state.mCurrentPhase];
	state.mPicture = mPictures->at(state.mCurrentPicture);
	state.mAnimationRateCounter = 0;
	state.mForward = true;
	state.mFinished = false;
	state.mDisplayParams.mXPos = state.mXPos;
	state.mDisplayParams.mYPos = state.mYPos;
	state.mDisplayPicture = state.mPicture;
}

//! resets a state to start values of this animation
//! this will not be displayed during next display() call
//! happens when one animation ends during update, and a next
//! animation is initiated - this means the next animation is
//! active from the start of next update, doesn't interfere with
//! current mDisplayParams!
void CSAnimation::startNextAnimation(AnimationState& state)
{
	static char *functionName="startNextAnimation";
	state.mCurrentPhase = 0;
	state.mCurrentPicture = mPhases[state.mCurrentPhase];
	state.mPicture = mPictures->at(state.mCurrentPicture);
	state.mAnimationRateCounter = 0;
	state.mForward = true;
	state.mFinished = false;
}

//! of all pictures belonging to this animation 
//! return the largest width
unsigned int CSAnimation::getMaxX()
{
	static char *functionName="getMaxX";
	unsigned int maxX = 0;
	unsigned int x;
	for (CSPictures::iterator iter = mPictures->begin(); iter != mPictures->end(); iter++)
	{
		CSPicture *pic = *iter;
		x = pic->getMaxX(); if (maxX < x) maxX = x;
	}
	return maxX;
}

//! of all pictures belonging to this animation 
//! return the largest height
unsigned int CSAnimation::getMaxY()
{
	static char *functionName="getMaxY";
	unsigned int maxY = 0;
	unsigned int y;
	for (CSPictures::iterator iter = mPictures->begin(); iter != mPictures->end(); iter++)
	{
		CSPicture *pic = *iter;
		y = pic->getMaxY(); if (maxY < y) maxY = y;
	}
	return maxY;
}

//! if finished and not repeat -> false, true otherwise
//! does not display anything, but prepares (update) for next displaying
bool CSAnimation::next(AnimationState& state)
{
	static char *functionName="next";
	if (state.mFinished)
	{
		return !state.mFinished;
	}

	// if an animation has only one picture, it is not really
	// much of an aniamtion -> display it, regardless
	// of any counters,
	// thus an aniamtion with only one picture never ends!
	if (mSizePictures <= 1)
	{
		if (mMoveAllowed)
		{
			// speed calculated vi frame per second
			if (state.mSpeedX == ANIMATION_SPEED)
			{
				state.mXPos += (float) (mMoveX[state.mCurrentPicture]*mSecsPerFrame);
			}
			else if (state.mSpeedX == ADJUST_SPEED)
			{
				int speed = 0;
				if (mMoveX[state.mCurrentPicture] < 0) 
					speed = -state.mAdjustSpeed;
				if (mMoveX[state.mCurrentPicture] > 0) 
					speed = state.mAdjustSpeed;
				state.mXPos += (float) (speed*mSecsPerFrame);
			}
			else 
			{
				state.mXPos += (float) (state.mSpeedX*mSecsPerFrame);
			}

			if (state.mSpeedY == ANIMATION_SPEED)
			{
				state.mYPos += (float) (mMoveY[state.mCurrentPicture]*mSecsPerFrame);
			}
			else if (state.mSpeedY == ADJUST_SPEED)
			{
				int speed = 0;
				if (mMoveY[state.mCurrentPicture] < 0) 
					speed = -state.mAdjustSpeed;
				if (mMoveY[state.mCurrentPicture] > 0) 
					speed = state.mAdjustSpeed;
				state.mYPos += (float) (speed*mSecsPerFrame);
			}
			else
			{
				state.mYPos += (float) (state.mSpeedY*mSecsPerFrame);
			}
		}

		state.mDisplayParams.mXPos = state.mXPos;
		state.mDisplayParams.mYPos = state.mYPos;
		state.mDisplayPicture = state.mPicture;
		return true;
	}

	// animation rate calculated vi frame per second
	state.mAnimationRateCounter += (float) (mAnimationRates[state.mCurrentPicture] * mSecsPerFrame);
	// if animation frame should change
	if (state.mAnimationRateCounter >= 1)
	{
		state.mAnimationRateCounter = 0;
		if (mRepeatable == CS_RANDOM)
		{
			// TODO
		}
		else
		{
			if (state.mForward)
			{
				state.mCurrentPhase++;
				if (state.mCurrentPhase == mSizePhases)
				{
					if (mRepeatable == CS_INVERSE_REPEATABLE)
					{
						state.mForward = !state.mForward;
						if (mSizePhases > 1)
						{
							state.mCurrentPhase-=2;
						}
						else
						{
							state.mCurrentPhase--;
						}
					}
					else
					if (mRepeatable == CS_INVERSE_ONCE)
					{
						state.mForward = !state.mForward;
						if (mSizePhases > 1)
						{
							state.mCurrentPhase-=2;
						}
						else
						{
							state.mCurrentPhase--;
						}
					}
					else
					{
						if (mRepeatable == CS_NOT_REPEATABLE)
						{
							state.mFinished = true;
							return !state.mFinished;
						}
						state.mCurrentPhase = 0;
					}
				}
			}
			else // mForward
			{
				state.mCurrentPhase--;
				if (state.mCurrentPhase == -1)
				{
					if (mRepeatable == CS_INVERSE_REPEATABLE)
					{
						state.mForward = !state.mForward;
						if (mSizePhases > 1)
						{
							state.mCurrentPhase+=2;
						}
						else
						{
							state.mCurrentPhase++;
						}
					}
					else
					{
						if (mRepeatable == CS_INVERSE_ONCE)
						{
							state.mFinished = true;
							return !state.mFinished;
						}
						state.mCurrentPhase = 0;
					}
				}
			}
		} // else (mRepeatable == CS_RANDOM)
		state.mCurrentPicture = mPhases[state.mCurrentPhase];
		state.mPicture = mPictures->at(state.mCurrentPicture);
	} // (state.mDelayCounter == mDelays[mPhases[state.mCurrentPhase]])

	if (mMoveAllowed)
	{
		// speed calculated vi frame per second
		if (state.mSpeedX == ANIMATION_SPEED)
		{
			state.mXPos += (float) (mMoveX[state.mCurrentPicture]*mSecsPerFrame);
		}
		else if (state.mSpeedX == ADJUST_SPEED)
		{
			int speed = 0;
			if (mMoveX[state.mCurrentPicture] < 0) 
				speed = -(state.mAdjustSpeed);
			if (mMoveX[state.mCurrentPicture] > 0) 
				speed = state.mAdjustSpeed;
			state.mXPos += (float) (speed*mSecsPerFrame);
		}
		else 
		{
			state.mXPos += (float) (state.mSpeedX*mSecsPerFrame);
		}

		if (state.mSpeedY == ANIMATION_SPEED)
		{
			state.mYPos += (float) (mMoveY[state.mCurrentPicture]*mSecsPerFrame);
		}
		else if (state.mSpeedY == ADJUST_SPEED)
		{
			int speed = 0;
			if (mMoveY[state.mCurrentPicture] < 0) 
				speed = -(state.mAdjustSpeed);
			if (mMoveY[state.mCurrentPicture] > 0) 
				speed = state.mAdjustSpeed;
			state.mYPos += (float) (speed*mSecsPerFrame);
		}
		else
		{
			state.mYPos += (float) (state.mSpeedY*mSecsPerFrame);
		}

		state.mDisplayParams.mXPos = state.mXPos;
		state.mDisplayParams.mYPos = state.mYPos;
	}
	state.mDisplayPicture = state.mPicture;
	return true;
}

//! not much to do here, to display a picture
void CSAnimation::display(AnimationState& state)
{
	static char *functionName="display";
	state.mDisplayPicture->display(&(state.mDisplayParams));
}

void CSAnimation::loadAnimationData(const std::string &filename, CSAnimationData &data)
{
	static char *functionName="loadAnimationData";
	LOG_ENTER 
	char tmp[10];
	
	CSXMLHelper xmlSupport(filename, "ANIMATION");
	try
	{
		if (xmlSupport.getError())
		{
			LOG_EXIT
			throw "error";
		}

		data.id = strdup(xmlSupport.getString("ID").c_str());
		data.isBreakable = xmlSupport.getInt("IS_BREAKABLE") == 1;
		data.repeatable = xmlSupport.getInt("REPEATABLE");

		data.sizePhases = xmlSupport.getInt("count(/ANIMATION/PHASE)");
		data.phases = new unsigned int[data.sizePhases];
		int i;
		for (i=0; i<data.sizePhases; i++)
		{
			// i+1, predicates in xpath start with 1
			data.phases[i] = xmlSupport.getInt((std::string)"/ANIMATION/PHASE["+itoa(i+1,tmp,10)+"]");
		}
		CSAnimationPictures *pictures = new CSAnimationPictures();			// above data for animation pictures
		data.pictures = pictures;
		int imageCount = xmlSupport.getInt("count(/ANIMATION/IMAGE_DATA)");
		for (i = 0; i<imageCount; i++)
		{
			CSAnimationPicture *picStruct = new CSAnimationPicture();
			if (picStruct==NULL)
			{
				LOG_EXIT
				SDLMain::shutdown((std::string)"Out of memory in XML-Loading: ", 1);
			}
			// i+1, predicates in xpath start with 1
			picStruct->filename = strdup(xmlSupport.getString((std::string)"/ANIMATION/IMAGE_DATA["+itoa(i+1,tmp,10)+"]/PICTURE").c_str());
			picStruct->animationRate = xmlSupport.getInt((std::string)"/ANIMATION/IMAGE_DATA["+itoa(i+1,tmp,10)+"]/ANIMATION_RATE");
			picStruct->moveX = xmlSupport.getInt((std::string)"/ANIMATION/IMAGE_DATA["+itoa(i+1,tmp,10)+"]/MOVE_X");
			picStruct->moveY = xmlSupport.getInt((std::string)"/ANIMATION/IMAGE_DATA["+itoa(i+1,tmp,10)+"]/MOVE_Y");
			pictures->push_back(picStruct);
		}
	}
	catch(...)
	{
		LOG_EXIT
		SDLMain::shutdown((std::string)"XML error \"" + filename + "\": " + xmlSupport.getErrorMessage().c_str(), 1);
	}
	LOG_EXIT
}

/////////////////////////////////////////////////////////
void CSAnimation::scale(int width, int tilesX, int height, int tilesY)
{
	static char *functionName="scale";
	LOG_ENTER 
	for (CSPictures::iterator iter = mPictures->begin(); iter != mPictures->end(); iter++)
	{
		CSPicture *pic = *iter;
		pic->scale(width, tilesX, height, tilesY);;
	}
	LOG_EXIT
}
