#ifdef WIN32
#pragma warning(disable : 4786 )
#endif
#include "CSBitmap.h"
#include "SDLMain.h"
#include "CSXMLHelper.h"

using namespace std;

CSBitmapLoader CSBitmapLoader::INSTANCE;
CSBitmapDirectLoader CSBitmapDirectLoader::INSTANCE;

CSBitmap *CSBitmapLoader::create(const std::string &filename)
{
	CSXMLHelper xmlSupport(filename, "BITMAP");
	char *id;
	char *bitmapFilename;
	signed int r,g,b;
	try
	{
		if (xmlSupport.getError())
		{
			throw "error";
		}

		id = strdup(xmlSupport.getString("ID").c_str());
		bitmapFilename = strdup(xmlSupport.getString("NAME").c_str());
		r = xmlSupport.getInt("TRANSPARENT_COLOR/RED");
		g = xmlSupport.getInt("TRANSPARENT_COLOR/GREEN");
		b = xmlSupport.getInt("TRANSPARENT_COLOR/BLUE");
	}
	catch(...)
	{
		LOG_EXIT
		SDLMain::shutdown((std::string)"XML error \"" + filename + "\": " + xmlSupport.getErrorMessage().c_str(), 1);
	}

	SDL_Surface *image = 0;

    /* Load the BMP file into a surface */
    image = SDL_LoadBMP(bitmapFilename);

    if (image == 0) 
	{
		LOG_EXIT
		SDLMain::shutdown((std::string)"Couldn't load \"" + bitmapFilename + "\": " + SDL_GetError(), 1);
    }

	// free loaded parse entities!
	free (bitmapFilename);
	bitmapFilename = 0;
	free (id);
	id = 0;

	CSBitmap *bitmap = new CSBitmap();
	bitmap->r = r;
	bitmap->g = g;
	bitmap->b = b;
	bitmap->picture = image;
	return bitmap;
}

void CSBitmapLoader::destroy(CSBitmap *image)
{
	SDLMain::freeSurface(image->picture);
	delete image;
}


typedef std::map<std::string, CSBitmap *> BitmapMap;

void CSBitmapLoader::scale(char *name, int width, int divideX, int height, int divideY)
{
	BitmapMap *map = CSBitmapLoader::INSTANCE.getMap();
	BitmapMap::iterator iter = map->find(name);
	if (iter == map->end())
	{
		return;
	}
	CSBitmap *bitmap = iter->second;
	SDL_Surface *surface = bitmap->picture;
	bitmap->picture = SDLMain::getInstance()->scale(surface, width, divideX, height, divideY);
	SDLMain::freeSurface(surface);
}

CSBitmap *CSBitmapDirectLoader::create(const std::string &filename)
{
	SDL_Surface *image = 0;

    /* Load the BMP file into a surface */
    image = SDL_LoadBMP(filename.c_str());

    if (image == 0) 
	{
		LOG_EXIT
		SDLMain::shutdown((std::string)"Couldn't load \"" + filename + "\": " + SDL_GetError(), 1);
    }

	// free loaded parse entities!
	CSBitmap *bitmap = new CSBitmap();
	bitmap->r = 255;
	bitmap->g = 0;
	bitmap->b = 255;
	bitmap->picture = image;
	return bitmap;
}
