#ifndef CSBitmapLoader_h
#define CSBitmapLoader_h

#include "CSLoadable.h"

// USES SDL_Surface
// USES SDL_Rect 

struct SDL_Surface;


//! this class is little more than a wrapper - not really c++ like
//! but ...
class CSBitmap
{
	public:
		SDL_Surface *picture;
		int r;
		int g;
		int b;
};


/**
	The Bitmap class is the innermost class for graphical entities,
	Here we use SDL images loaded thru the
	    image = SDL_LoadBMP(bitmapFilename);
	function.

	The Class provides:
		\arg The use of a CSLoadable Interface
		\arg loads a "ini" - file beforehand and sets (if appropriate 
	 	   (all RGB values >0)) a transparancy color
	
	But (as of now) now, values are NOT stored in any "data" section (like with 
	the other graphical entities).
	The Loader returns a (CSBitmap *).<BR>

	This class (or the loader... that is) holds the images as they are loaded,
	Upon switching between different resolutions, the surfaces are "recreated" using 
	the images that are hold within the loader class.
	Thus the used images are allways recreated from the original bitmap.
	Also this one holds the transparent-rgb values for (re-)setting the 
	color-key values.<BR>
	
	The Loader IS responsible for freeing the surface.(parent class!)
	(thru destroy)<BR>
	*/
class CSBitmapLoader : public Loadable<CSBitmap>
{
	protected:
		virtual void destroy(CSBitmap *image);
		virtual CSBitmap *create(const std::string &filename);

	public:
		static CSBitmapLoader INSTANCE;
		static void scale(char *name, int width, int divideX, int height, int divideY);
};

class CSBitmapDirectLoader : public CSBitmapLoader
{
	protected:
		virtual CSBitmap *create(const std::string &filename);

	public:
		static CSBitmapDirectLoader INSTANCE;
};

#endif //CSBitmapLoader_h