#ifdef WIN32
#pragma warning(disable : 4786 )
#endif
#include "CSFont.h"
#include "CSXMLHelper.h"

using namespace std;

CSFontLoader CSFontLoader::INSTANCE;
const char *CSFont::CLASS = "CSFont";
CSFonts CSFont::mAllFonts;

//! constructor using "*.pic" file
CSFont::CSFont(const std::string &filename)
{
	static char *functionName="CSFont";
	LOG_ENTER 
	isInit = false;
	mKey = -1;
	mPicture = 0;
	mFilename = 0;

	mSpaceWidth = 0;
	mH = -1;
	mId = 0;
	CSFontData data;
	loadFontData(filename, data);
	initialize(data);
	addFont(this);
	isInit = true;
	LOG_EXIT
}

void CSFont::initialize(const CSFontData &data)
{
	static char *functionName="initialize";
	LOG_ENTER 
	// picture muss geladen werden
    // Load the BMP file into a surface 
	mId = strdup(data.id);
	mPicture = CSPictureLoader::INSTANCE.load(data.filename);
	mPicture->getRGB(mR,mG,mB);
	mFilename = strdup(data.filename);

    int x = 0, i = 0;
    if (SDL_MUSTLOCK(mPicture->getSurface())) 
	{
		SDL_LockSurface(mPicture->getSurface());
	}
    while ( x < mPicture->getSurface()->w ) 
	{
		if (SDLMain::getPixel(mPicture->getSurface(),x,0) == SDL_MapRGB(mPicture->getSurface()->format,255,0,255)) 
		{ 
    	    mCharPos[i++]=x;

    	    while (	( x < mPicture->getSurface()->w-1) && (SDLMain::getPixel(mPicture->getSurface(),x,0) ==SDL_MapRGB(mPicture->getSurface()->format,255,0,255)) )
			{
				x++;
			}
		    mCharPos[i++]=x;
		}
		x++;
    }
    if (SDL_MUSTLOCK(mPicture->getSurface())) 
	{
		SDL_UnlockSurface(mPicture->getSurface());
	}
    mH = mPicture->getSurface()->h;

	// width of a space is like - 'n'
	mSpaceWidth = mCharPos[('n'-33)*2+1+1] - mCharPos[('n'-33)*2+1];
	mMonoSpaced = data.monospaced;
	LOG_EXIT
}

CSFont::~CSFont()
{
	static char *functionName="~CSFont";
	LOG_ENTER 
	removeFont(this);
	if (mFilename)
	{
		free(mFilename);
		mFilename = 0;
	}
	if (mId)
	{
		free(mId);
		mId = 0;
	}

	// picture is singleton - not freed
	deleteColorFonts();
	isInit = false;
	LOG_EXIT
}

void CSFont::setSolid(bool b)
{
	static char *functionName="setSolid";
	if (mPicture == 0)
	{
		return;
	}
	mPicture->setSolid(b);
}


void CSFont::loadFontData(const std::string &filename, CSFontData &data)
{
	static char *functionName="loadFontData";
	LOG_ENTER 
	CSXMLHelper xmlSupport(filename, "FONT");
	try
	{
		if (xmlSupport.getError())
		{
			LOG_EXIT
			throw "error";
		}

		data.id = strdup(xmlSupport.getString("ID").c_str());
		data.filename = strdup(xmlSupport.getString("PICTURE").c_str());
		data.monospaced = xmlSupport.getInt("MONO_SPACED");
	}
	catch(...)
	{
		LOG_EXIT
		SDLMain::shutdown((std::string)"XML error \"" + filename + "\": " + xmlSupport.getErrorMessage().c_str(), 1);
	}
	LOG_EXIT
}

void CSFont::putString(SDL_Surface *surface, int x, int y, const char *text)
{
	static char *functionName="putString";
    int ofs;
    int i=0;
    SDL_Rect srcrect,dstrect; 
	int sx = x;
	int sy = y;
	srcrect.y = 1;
	dstrect.y = y;
	srcrect.h = dstrect.h = mPicture->getSurface()->h-1;
    while (text[i]!='\0') 
	{
		dstrect.y = y;	// with resetting of y, the y pos of the string is not altered if 
						// partial printing is done out of bounds (clipped)
		if (text[i] <32)
		{
			i++;
			continue;
		}
        if (text[i]==' ') 
		{
    	    dstrect.x = x;
			dstrect.w =  mSpaceWidth;
            x+=dstrect.w;
			if (mPicture->isSolid())
			{
				SDL_FillRect(surface, &dstrect, mPicture->getColorKey());
			}
			i++;
		}
		else 
		{
			ofs= (text[i]-33)*2+1;

            srcrect.x = mCharPos[ofs];
    	    dstrect.x = x;
			srcrect.w = dstrect.w =  mCharPos[ofs+1] - mCharPos[ofs];
            x+=dstrect.w;

			SDL_BlitSurface(mPicture->getSurface(), &srcrect, surface, &dstrect); 

            i++;
        }
    }

	SDL_Rect mUpdateRegion;		// has position and size
	mUpdateRegion.x = sx;
	mUpdateRegion.y = sy;
	mUpdateRegion.w = dstrect.x + srcrect.w;
	mUpdateRegion.h = dstrect.h;
	SDLMain::addUpdateRect(mUpdateRegion);
}

int CSFont::getWidth(const char *text)
{
	static char *functionName="getWidth";
    int ofs=0;
    int i=0,x=0;

    while (text[i]!='\0') 
	{
        if (text[i]==' ') 
		{
            x+=mSpaceWidth;
            i++;
		}
		else 
		{
			ofs=(text[i]-33)*2+1;
			if (ofs < 0) break;
            x+=mCharPos[ofs+1]-mCharPos[ofs];
            i++;
        }
    }
    return x;
}

void CSFont::xCenteredString(SDL_Surface *surface, int y, const char *text)
{
	static char *functionName="xCenteredString";
    putString(surface, surface->w/2-getWidth(text)/2, y, text);
}

// untested
void CSFont::inputText(SDL_Surface *dest, int x, int y, int w, char *text)
{
	static char *functionName="inputText";
	LOG_ENTER 
    SDL_Event event;
    int ch=-1,blink=0;
    long blinktimer=0;
    SDL_Surface *back;
    SDL_Rect rect;
    int previous;
    
    back = SDL_AllocSurface(dest->flags,
    			    dest->w,
    			    mH,
    			    dest->format->BitsPerPixel,
    			    dest->format->Rmask,
    			    dest->format->Gmask,
					dest->format->Bmask, 0);
    rect.x=0;
    rect.y=y;
    rect.w=dest->w;
    rect.h=mPicture->getSurface()->h;
    SDL_BlitSurface(dest, &rect, back, NULL);
    putString(dest,x,y,text);
    SDL_UpdateRects(dest, 1, &rect);
        
    // start input
    previous=SDL_EnableUNICODE(1);
    blinktimer=SDL_GetTicks();
	SDL_PollEvent(&event);
    while (ch!=SDLK_RETURN) 
	{
		if (event.type==SDL_KEYDOWN) 
		{
			ch=event.key.keysym.unicode;
			if (((ch>31)||(ch=='\b')) && (ch<128)) 
			{
				if ((ch=='\b')&&(strlen(text)>0))
				{
					text[strlen(text)-1]='\0';
				}
				else if (ch!='\b')
				{
					sprintf(text,"%s%c",text,ch);
				}
				if (getWidth(text)>w) 
				{
					text[strlen(text)-1]='\0';
				}
				SDL_BlitSurface( back, NULL, dest, &rect);
				putString(dest, x, y, text);
				SDL_UpdateRects(dest, 1, &rect);
				SDL_WaitEvent(&event);
			}
		}
		if (SDL_GetTicks()>blinktimer) 
		{
			blink=1-blink;
			blinktimer=SDL_GetTicks()+500;
			if (blink) 
			{
				putString(dest, x+getWidth(text), y, "|");
				SDL_UpdateRects(dest, 1, &rect);
		    } 
			else 
			{
				SDL_BlitSurface( back, NULL, dest, &rect);
				putString(dest, x, y, text);
				SDL_UpdateRects(dest, 1, &rect);
		    }
		}
		SDL_Delay(1);
		SDL_PollEvent(&event);
    }
    text[strlen(text)]='\0';
    SDLMain::freeSurface(back);
    SDL_EnableUNICODE(previous);  //restore the previous state
	LOG_EXIT
}

void CSFont::putString(SDL_Surface *surface, int x, int y, int color, const char *text, int style)
{
	static char *functionName="putString";
	bool isNewColor = true;
	SDL_Surface *fontPicture = 0;
	if (color == mKey)	// cheating here - but I cant' be bothered to make it
						// exact - transaparent color should allways be different
						// from the font color!!!
	{
		color = color + 0x010101;
	}

	ColorFontMap::iterator iter = mColorFontMap.find(color);
	if (iter != mColorFontMap.end())
	{
		fontPicture = iter->second;
		isNewColor = false;
	}
	else
	{
		SDL_Rect rect;
		rect.x = 0;
		rect.y = 0;
		rect.w = mPicture->getSurface()->w;
		rect.h = mPicture->getSurface()->h;

		int flags = SDLMain::getScreen()->flags;
		if ((flags & SDL_HWSURFACE) == SDL_HWSURFACE)
		{
			flags -= SDL_HWSURFACE;
		}
		if ((flags & SDL_DOUBLEBUF) == SDL_DOUBLEBUF)
		{
			flags -= SDL_DOUBLEBUF;
		}
		if ((flags & SDL_FULLSCREEN) == SDL_FULLSCREEN)
		{
			flags -= SDL_FULLSCREEN;
		}
		
		fontPicture = SDL_CreateRGBSurface(flags, 
								   rect.w, 
								   rect.h, 
								   SDLMain::getScreen()->format->BitsPerPixel, 
								   SDLMain::getScreen()->format->Rmask, 
								   SDLMain::getScreen()->format->Gmask, 
								   SDLMain::getScreen()->format->Bmask, 
								   SDLMain::getScreen()->format->Amask);
		if (SDL_BlitSurface(mPicture->getSurface(), &rect, fontPicture, 0) < 0)
		{
			SDLMain::shutdown((std::string)"Display Format error: " + SDL_GetError(), 1);
		}
		mKey = SDL_MapRGB(fontPicture->format, mR, mG, mB);
		SDL_SetColorKey(fontPicture, SDL_SRCCOLORKEY, mKey); 
		mColorFontMap.insert(ColorFontMap::value_type(color, fontPicture));
	}

	if (isNewColor)
	{
		int newColor = COLOR(fontPicture, color);
		int x,y;

		if (SDL_MUSTLOCK(fontPicture)) 
		{
			SDL_LockSurface(fontPicture);
		}
		for (y=0;y<fontPicture->h; y++)
		{
			for (x=0;x<fontPicture->w; x++)
			{
				int color = SDLMain::getPixel(fontPicture,x,y);

				if (color != mKey)		
				{
					SDLMain::setPixel(fontPicture,x,y,newColor);
				}
			}
		}
		if (SDL_MUSTLOCK(fontPicture)) 
		{
			SDL_UnlockSurface(fontPicture);
		}
	}

	int sx = x;
	int sy = y;
	SDL_Rect srcrect,dstrect; 
	srcrect.h=0;
	srcrect.w=0;
	srcrect.x=0;
	srcrect.y=0;
	dstrect.h=0;
	dstrect.w=0;
	dstrect.x=0;
	dstrect.y=0;
	int ofs;
	int i=0;
	if (!((style & FONT_STYLE_ITALIC) == FONT_STYLE_ITALIC))
	{
		srcrect.y = 1;
		srcrect.h = dstrect.h = fontPicture->h-1;
		while (text[i]!='\0') 
		{
			dstrect.y = y;	// with resetting of y, the y pos of the string is not altered if 
							// partial printing is done out of bounds (clipped)
			if (text[i] <32)
			{
				i++;
				continue;
			}
			if (text[i]==' ') 
			{
    			dstrect.x = x;
				dstrect.w =  mSpaceWidth;
				if (mPicture->isSolid())
				{
					SDL_FillRect(surface, &dstrect, mKey);
				}
				if ((style & FONT_STYLE_UNDERLINE)  == FONT_STYLE_UNDERLINE)
				{
					SDLMain::line(surface, x, y + srcrect.h -1 , x + mSpaceWidth, y + srcrect.h-1, (color<<8)+255);
				}
				x += mSpaceWidth;
				i++;
			}
			else 
			{
				ofs= (text[i]-33)*2+1;
				
				srcrect.x = mCharPos[ofs];
    			dstrect.x = x;
				srcrect.w = dstrect.w =  mCharPos[ofs+1] - mCharPos[ofs];
				SDL_Rect copyDest = dstrect;

				SDL_BlitSurface(fontPicture, &srcrect, surface, &dstrect); 
				if ((style & FONT_STYLE_BOLD)  == FONT_STYLE_BOLD)
				{
					copyDest.x += 1;
					SDL_BlitSurface(fontPicture, &srcrect, surface, &copyDest); 
				}
				if ((style & FONT_STYLE_UNDERLINE)  == FONT_STYLE_UNDERLINE)
				{
					SDLMain::line(surface, x, y + srcrect.h -1 , x + srcrect.w, y + srcrect.h-1, (color<<8)+255);
				}
				x+=srcrect.w;
				i++;
			}

		}
	}
	else // FONT_STYLE_ITALIC
	{
		srcrect.y = 1;
		srcrect.h = dstrect.h = fontPicture->h-1;
		while (text[i]!='\0') 
		{
			dstrect.y = y;	// with resetting of y, the y pos of the string is not altered if 
							// partial printing is done out of bounds (clipped)
			if (text[i] <32)
			{
				i++;
				continue;
			}
			if (text[i]==' ') 
			{
    			dstrect.x = x;
				dstrect.w =  mSpaceWidth;
				if (mPicture->isSolid())
				{
					SDL_FillRect(surface, &dstrect, mKey);
				}
				if ((style & FONT_STYLE_UNDERLINE)  == FONT_STYLE_UNDERLINE)
				{
					SDLMain::line(surface, x, y + (fontPicture->h-1) -1 , x + mSpaceWidth, y + (fontPicture->h-1) -1, (color<<8)+255);
				}
				x += mSpaceWidth;
				i++;
			}
			else 
			{
				ofs= (text[i]-33)*2+1;
				int many = 6;
				int addy = 	(fontPicture->h-1+(many-1))/many;
				srcrect.h = dstrect.h = addy;

				for (int ii = 0; ii< many; ii++)
				{
					srcrect.y = 1 + (ii*addy);
					dstrect.y = y + (ii*addy);

					srcrect.h = dstrect.h = addy;
					srcrect.x = mCharPos[ofs];
    				dstrect.x = x+((many-1)-ii);
					srcrect.w = dstrect.w =  mCharPos[ofs+1] - mCharPos[ofs];
					SDL_Rect copyDest = dstrect;

					SDL_BlitSurface(fontPicture, &srcrect, surface, &dstrect); 
					if ((style & FONT_STYLE_BOLD)  == FONT_STYLE_BOLD)
					{
						copyDest.x += 1;
						SDL_BlitSurface(fontPicture, &srcrect, surface, &copyDest); 
					}
				}
				if ((style & FONT_STYLE_UNDERLINE)  == FONT_STYLE_UNDERLINE)
				{
					SDLMain::line(surface, x, y + (fontPicture->h-1) -1 , x + srcrect.w, y + (fontPicture->h-1)-1, (color<<8)+255);
				}
				i++;
				x+=srcrect.w;
			}
		}
	}
	SDL_Rect mUpdateRegion;		// has position and size
	mUpdateRegion.x = sx;
	mUpdateRegion.y = sy;
	mUpdateRegion.w = dstrect.x + srcrect.w;
	mUpdateRegion.h = dstrect.h;
	SDLMain::addUpdateRect(mUpdateRegion);
}

void CSFont::resetSurface()
{
	static char *functionName="resetSurface";
	LOG_ENTER 
	deleteColorFonts();
}

// static stuff to enable converting fonts to and from fullscreen
// all fonts ever loaded from disc are remembered in the Vector "mAllFonts"
// (only neccessary for "color" putString())
void CSFont::addFont(CSFont *font)
{
	if (font == 0)
	{
		return;
	}
	removeFont(font);
	mAllFonts.push_back(font);
}

void CSFont::removeFont(CSFont *font)
{
	if (font == 0)
	{
		return;
	}

	if (!font->isInit) return;

	CSFonts::iterator iter = mAllFonts.begin();

	// for some reason or another mAllFonts is on deinit
	// empty, and strangly enough
	// a empty start cannot be compared to an empty end, even
	// in the same collection, it throws an exception uncomparable...
	// (at least in MS)
// csa 2016	if (iter._Ptr == 0) return;

	while ( iter != mAllFonts.end())
	{
		if ((*iter) == font)
		{
			mAllFonts.erase(iter);
			iter = mAllFonts.begin();	// restart to be absolutly save!
		}
		else
		{
			iter++;
		}
	}
}

// resets all surfaces
void CSFont::resetSurfaces()
{
	static char *functionName="resetSurfaces";
	LOG_ENTER 
	CSFonts::iterator iter = mAllFonts.begin();
	while ( iter != mAllFonts.end())
	{
		(*iter)->resetSurface();
		iter++;
	}
	LOG_EXIT
}
