#ifndef CSFONT_H
#define CSFONT_H

// USES SDL_Surface

#include "SDL.h"
#include "CSPicture.h"
#include "CSLoadable.h"
#include "CSLog.h"
#include <map>

#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

class CSFont;
class CSFontLoader;
typedef std::vector<CSFont *> CSFonts;

typedef std::map<int, SDL_Surface *> ColorFontMap;

const int FONT_STYLE_NONE = 0;
const int FONT_STYLE_UNDERLINE = 1;
const int FONT_STYLE_BOLD = 2;
const int FONT_STYLE_ITALIC = 4;

class CSFontData
{
	public:
		char *id;
		char *filename;		//!< of bitmap file
		int monospaced;
		CSFontData()
		{
			id = 0;
			filename = 0;
			monospaced = 0;
		}

		~CSFontData()
		{
			if (id != 0) 
			{
				free(id);
				id = 0;
			}
			if (filename != 0) 
			{
				free(filename);
				filename = 0;
			}
		}
};

class CSFont
{
	friend CSFontLoader;
	private:
		static CSFonts mAllFonts;
		static void addFont(CSFont *font);
		static void removeFont(CSFont *font);
		bool isInit;

		ColorFontMap mColorFontMap;
		char *mId;
		char *mFilename;
		CSPicture *mPicture;
		int mKey;
		int mSpaceWidth;
		int mCharPos[512];
		int mMonoSpaced;
		int mH;
		int mR;
		int mG;
		int mB;
		CSFont(const std::string &filename);
		virtual void initialize(const CSFontData &data);
		static void loadFontData(const std::string &filename, CSFontData &data);
		void buildSigns(void);
		void deleteColorFonts()
		{
			ColorFontMap::iterator iter = mColorFontMap.begin();
			while (iter != mColorFontMap.end())
			{
				SDLMain::freeSurface(iter->second);
				iter++;
			}
			mColorFontMap.clear();
		}
	
	public:
		CSFont(const CSFont &font) {}; // not available!
		virtual ~CSFont();
		static const char *CLASS;
		virtual std::string getType() {return (std::string) CLASS;}
		static void resetSurfaces();							//!< resets all surfaces
		void resetSurface();		//!< converts current surface to display surface
									//!< used (e.g.) upon fullscreen-toggle

		void setSolid(bool b);
		int getHeight() {return mH;}
		
		//! Returns the width of "text" in pixels
		int getWidth(const char *text);
		int getWidth(const std::string &text) {return getWidth(text.c_str());}

		//! Blits a string to a surface
		//! Destination: the suface you want to blit to
		//! text: a string containing the text you want to blit.
		void putString(SDL_Surface *surface, int x, int y, const char *text);
		void putString(SDL_Surface *surface, int x, int y, const std::string &text)
			{putString(surface, x, y, text.c_str());}
		
		void putString(int x, int y, const char *text) 
			{putString(SDLMain::getScreen(), x, y, text);}
		void putString(int x, int y, const std::string &text) 
			{putString(SDLMain::getScreen(), x, y, text.c_str());}

		void putString(SDL_Surface *surface, int x, int y, int color, const char *text, int style=0);
		void putString(SDL_Surface *surface, int x, int y, int color, const std::string &text, int style = 0)
			{putString(surface, x, y, color, text.c_str(), style);}
		
		void putString(int x, int y, int color, const char *text) 
			{putString(SDLMain::getScreen(), x, y, color, text);}
		void putString(int x, int y, int color, const std::string &text) 
			{putString(SDLMain::getScreen(), x, y, color, text.c_str());}

		//! Blits a string to with centered x position
		void xCenteredString(SDL_Surface *surface, int y, const char *text);
		void xCenteredString(SDL_Surface *surface, int y, const std::string &text)
			{xCenteredString(surface, y, text.c_str());}
		void xCenteredString(int y, const char *text) 
			{xCenteredString(SDLMain::getScreen(), y, text);}
		void xCenteredString(int y, const std::string &text) 
			{xCenteredString(SDLMain::getScreen(), y, text.c_str());}

		//! Allows the user to enter text
		//! Width: What is the maximum width of the text (in pixels)
		//! text: This string contains the text which was entered by the user
		void inputText(SDL_Surface *Destination, int x, int y, int w, char *text);
		void inputText(int x, int y, int w, char *text) 
			{inputText(SDLMain::getScreen(), x, y, w, text);}
};

class CSFontLoader : public Loadable<CSFont>
{
	
	protected:
		virtual CSFont *create(const std::string &filename)
		{
			return new CSFont(filename);
		}

	public:
		static CSFontLoader INSTANCE;
};


#endif CSFONT_H
