#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

#include "SDLMain.h"
#include "CSFont.h"
#include <conio.h>
#include <string>

#include "CSPanel.h"
#include "CSWindow.h"
#include "CSLabel.h"
#include "CSButton.h"
#include "CSCheckBox.h"
#include "CSTextfield.h"
#include "CSScrollbar.h"
#include "CSTextarea.h"
#include "CSListbox.h"
#include "CSCombobox.h"
#include "CSHTMLArea.h"
#include "CSMenu.h"

#include "CSIcon.h"
#include "CSMessageBox.h"
#include "CSLAF.h"

#include "CSGame.h"
#include "CSMenu.h"
#include "CSMenuItem.h"
#include "CSSample.h"

const char *CSGame::CLASS = "CSGame";

const int QUIT_ACTION_ID = 1;
const int ABOUT_ACTION_ID = 2;

/** \mainpage Index done by Chrissalo


PAralaxx Info:
Muss background haben, sonst wird die unterstee eben der welt als background genommen
mit der groesse der untersteen welt,, -> nur teilweise erase des BG


  \section gui something to about GUI

  \subsection widgets something to say about widgets
*/



/*! \page page1 A documentation page
  Leading text.
  \section sec An example section
  This page contains the subsections \ref subsection1 and \ref subsection2.
  For more info see page \ref page2.
  \subsection subsection1 The first subsection
  Text.
  \subsection subsection2 The second subsection
  More text.
*/

/*! \page page2 Another page
  Even more info.
*/


CSGame::CSGame(std::string gameFileName)
{
	static char *functionName="CSGame";
	LOG_ENTER 
	initialize();
	loadGameData(gameFileName);
	LOG_EXIT 
}

CSGame::~CSGame()
{
	static char *functionName="~CSGame";
	LOG_ENTER 
	freeLevel();
	
	TileCollisionCheckDatas::iterator iter = mTileCollisionCheckDatas.begin();
	while (iter != mTileCollisionCheckDatas.end())
	{
		TileCollisionCheckData *data = *iter;
		mTileCollisionCheckDatas.erase(iter);
		delete data;
		iter = mTileCollisionCheckDatas.begin();
	}

	SpriteCollisionCheckDatas::iterator titer = mSpriteCollisionCheckDatas.begin();
	while (titer != mSpriteCollisionCheckDatas.end())
	{
		SpriteCollisionCheckData *data = *titer;
		mSpriteCollisionCheckDatas.erase(titer);
		delete data;
		titer = mSpriteCollisionCheckDatas.begin();
	}
	for (StringVector::iterator siter = mLevels.begin(); siter != mLevels.end(); siter++)
	{
		delete *siter;
	}
	mLevels.clear();
	if (mId)
	{
		free(mId);
		mId = 0;
	}
	if (mEventMap != 0)
	{
		CSEventMap::iterator iter;
		for (iter = mEventMap->begin(); iter!=mEventMap->end(); iter++)
		{
			delete (iter->second);
		}
		
		mEventMap->clear();
		delete mEventMap;
		mEventMap = 0;
	}
	if (mLevelData != 0)
	{
		delete mLevelData;
		mLevelData = 0;
	}
	if (mMenuBar)
	{
		delete (mMenuBar);
		mMenuBar = 0;
	}
	if (mDesktop)
	{
		delete (mDesktop);
		mDesktop = 0;
	}
	LOG_EXIT 
}

void CSGame::initialize()
{
	static char *functionName="initialize";
	LOG_ENTER 
	dummyGame = false;
	mDesktop = new CSDesktop();

	mMenuBar = new CSMenuBar();
	mMenuBar->setVisible(true);
	mFileMenu = new CSMenu((std::string)"File");
	mFileMenu->setTooltipText("File menu, exit game with \"Exit\"...");
	mHelpMenu = new CSMenu((std::string)"Help");
	mHelpMenu->setTooltipText("Help and about...");

	mMenuItemExit = new CSMenuItem("Exit");
	mMenuItemAbout = new CSMenuItem("About");
	mMenuItemExit->setActionId(QUIT_ACTION_ID);
	mMenuItemExit->addMessageListener(this, GUI_MESSAGE);
	mMenuItemAbout->setActionId(ABOUT_ACTION_ID);
	mMenuItemAbout->addMessageListener(this, GUI_MESSAGE);
	mFileMenu->addMenuItem(mMenuItemExit);
	mHelpMenu->addMenuItem(mMenuItemAbout);
	mMenuBar->addElement(mFileMenu);
	mMenuBar->addElement(mHelpMenu);

    mDesktop->addElementNorth(mMenuBar);

	mIsGui = false;
	mLastIsGui = false;
	mEnableCheck = true;
	mDisplayScaled = false;
	mEventMap = new CSEventMap();
	mCurrentTime = 0;
	mGameTime = 0;
	mAverageFrameRate = 0;
	mDiffTime = 0;
	mLevelData = 0;
	mSecPerFrame = 0;
	mScaleCenterSprite = 0;
	mQuit = false;
	mPause = false;
	mLevelFinished = false;
	mStartLevel = 0;
	mMainSprite = 0;
	mWorld = 0;
	mWorldCount = 0;
	mThreshold = 150;
	mIsWorldOutOfBounds = false;
	mCurrentWorld = 0;
	mCurrentLevel = 0;
	mCurrentFrameRate;
	mDiffTimeGUI = 0;
	mDiffTimeGame = 0;
	mDiffTimeAll = 0;
	mId = 0;
	initSpeed();
	addMessageListener(this, GAME_MESSAGE);
	
	SDLMain::getInstance()->addMessageListener(this, SDL_MESSAGE);
	mFont = CSFontLoader::INSTANCE.load("font\\default_font.xml");
	mFont->setSolid(true);

	MESSAGE_END.setSubtype(END_MESSAGE);
	MESSAGE_PAUSE.setSubtype(PAUSE_MESSAGE);
	MESSAGE_TOGGLE_FULLSCREEN.setSubtype(TOGGLE_FULLSCREEN_MESSAGE);
	MESSAGE_SPRITE_TILE_COLLISION.setSubtype(SPRITE_TILE_COLLISION_MESSAGE);
	MESSAGE_SPRITE_SPRITE_COLLISION.setSubtype(SPRITE_SPRITE_COLLISION_MESSAGE);
	MESSAGE_SPRITE_OUT_OF_BOUNDS.setSubtype(SPRITE_OUT_OF_BOUNDS_MESSAGE);

	MESSAGE_DEFAULT.setSubtype(DEFAULT_MESSAGE);
	MESSAGE_UP.setSubtype(UP_MESSAGE);
	MESSAGE_DOWN.setSubtype(DOWN_MESSAGE);
	MESSAGE_LEFT.setSubtype(LEFT_MESSAGE);
	MESSAGE_RIGHT.setSubtype(RIGHT_MESSAGE);
	MESSAGE_FIRE.setSubtype(FIRE_MESSAGE);
	MESSAGE_DEATH.setSubtype(DEATH_MESSAGE);
	MESSAGE_KEY_UP.setSubtype(KEY_UP_MESSAGE);
	MESSAGE_KEY_DOWN.setSubtype(KEY_DOWN_MESSAGE);
	MESSAGE_KEY_LEFT.setSubtype(KEY_LEFT_MESSAGE);
	MESSAGE_KEY_RIGHT.setSubtype(KEY_RIGHT_MESSAGE);
	MESSAGE_KEY_FIRE.setSubtype(KEY_FIRE_MESSAGE);
	MESSAGE_TOGGLE_GUI.setSubtype(TOGGLE_GUI_MESSAGE);

	MESSAGE_SPRITE_STATE_CHANGE.setSubtype(SPRITE_STATE_CHANGE_MESSAGE);
	LOG_EXIT 
}

SDL_Rect CSGame::getDisplayArea()
{
	static char *functionName="getDisplayArea";
	LOG_ENTER 
	SDL_Rect area;
	area.x = 0;
	area.y = 0;
	area.w = SDLMain::getInstance()->getScreenWidth();
	area.h = SDLMain::getInstance()->getScreenHeight();
	LOG_EXIT 
	return area;
}

SDL_Rect CSGame::getScaledDisplayArea()
{
	static char *functionName="getScaledDisplayArea";
	LOG_ENTER 
	SDL_Rect area;
	area.x = 0;
	area.y = 0;
	area.w = 100;
	area.h = 100;
	LOG_EXIT 
	return area;
}

void CSGame::enableOutOfBoundsCheck(CSSprite *sprite, CSMessageListener *listener, int threshold)
{
	static char *functionName="enableOutOfBoundsCheck";
	LOG_ENTER 
 	OutOfBoundsData *data = new OutOfBoundsData();
	data->sprite = sprite;
	data->listener = listener;
	data->threshold = threshold;
	mOutOfBoundsDatas.push_back(data);
	LOG_EXIT 
}

void CSGame::disableOutOfBoundsCheck(CSSprite *sprite, CSMessageListener *listener)
{
	static char *functionName="disableOutOfBoundsCheck";
	LOG_ENTER 
	OutOfBoundsDatas::iterator iter = mOutOfBoundsDatas.begin();
	while (iter != mOutOfBoundsDatas.end())
	{
		OutOfBoundsData *data = *iter;
		if ((data->sprite == sprite) &&
			(data->listener == listener) 
			)
		{
			mOutOfBoundsDatas.erase(iter);
			delete data;
			break;
		}
		iter++;
	}
	LOG_EXIT 
}

void CSGame::enableTileCollisionCheck(CSSprite *sprite, int layer, CSMessageListener *listener)
{
	static char *functionName="enableTileCollisionCheck";
	LOG_ENTER 
	TileCollisionCheckData *data = new TileCollisionCheckData();
	data->type = CHECK_HIT;
	data->sprite = sprite;
	data->layer = layer;
	data->listener = listener;
	mTileCollisionCheckDatas.push_back(data);
	LOG_EXIT 
}

void CSGame::enableTileCollisionCheckJustHit(CSSprite *sprite, int layer, CSMessageListener *listener)
{
	static char *functionName="enableTileCollisionCheckJustHit";
	LOG_ENTER 
	TileCollisionCheckData *data = new TileCollisionCheckData();
	data->type = CHECK_JUST_HIT;
	data->sprite = sprite;
	data->layer = layer;
	data->listener = listener;
	mTileCollisionCheckDatas.push_back(data);
	LOG_EXIT 
}

void CSGame::enableTileCollisionCheckPixel(CSSprite *sprite, int layer, CSMessageListener *listener, int pixelThreshold)
{
	static char *functionName="enableTileCollisionCheckPixel";
	LOG_ENTER 
	TileCollisionCheckData *data = new TileCollisionCheckData();
	data->type = CHECK_PIXEL;
	data->sprite = sprite;
	data->layer = layer;
	data->listener = listener;
	data->threshold = pixelThreshold;
	mTileCollisionCheckDatas.push_back(data);
	LOG_EXIT 
}

void CSGame::enableSpriteCollisionCheck(CSSprite *sprite, CSSprite *other, CSMessageListener *listener)
{
	static char *functionName="enableSpriteCollisionCheck";
	LOG_ENTER 
	SpriteCollisionCheckData *data = new SpriteCollisionCheckData();
	data->type = CHECK_HIT;
	data->sprite = sprite;
	data->other = other;
	data->listener = listener;
	mSpriteCollisionCheckDatas.push_back(data);
	LOG_EXIT 
}

void CSGame::enableSpriteCollisionCheckPixel(CSSprite *sprite, CSSprite *other, CSMessageListener *listener, int pixelThreshold)
{
	static char *functionName="enableSpriteCollisionCheckPixel";
	LOG_ENTER 
	SpriteCollisionCheckData *data = new SpriteCollisionCheckData();
	data->type = CHECK_PIXEL;
	data->sprite = sprite;
	data->other = other;
	data->listener = listener;
	data->threshold = pixelThreshold;
	mSpriteCollisionCheckDatas.push_back(data);
	LOG_EXIT 
}

void CSGame::disableTileCollisionCheck(CSSprite *sprite, int layer, CSMessageListener *listener)
{
	static char *functionName="disableTileCollisionCheck";
	LOG_ENTER 
	TileCollisionCheckDatas::iterator iter = mTileCollisionCheckDatas.begin();
	while (iter != mTileCollisionCheckDatas.end())
	{
		TileCollisionCheckData *data = *iter;
		if ((data->sprite == sprite) &&
			(data->layer == layer) &&
			(data->listener == listener) 
			)
		{
			mTileCollisionCheckDatas.erase(iter);
			delete data;
			break;
		}
		iter++;
	}
	LOG_EXIT 
}

void CSGame::disableSpriteCollisionCheck(CSSprite *sprite, CSSprite *other, CSMessageListener *listener)
{
	static char *functionName="disableSpriteCollisionCheck";
	LOG_ENTER 
	SpriteCollisionCheckDatas::iterator iter = mSpriteCollisionCheckDatas.begin();
	while (iter != mSpriteCollisionCheckDatas.end())
	{
		SpriteCollisionCheckData *data = *iter;
		if ((data->sprite == sprite) &&
			(data->other == other) &&
			(data->listener == listener) 
			)
		{
			mSpriteCollisionCheckDatas.erase(iter);
			delete data;
			break;
		}
		iter++;
	}
	LOG_EXIT 
}

void CSGame::checkCollisions()
{
	static char *functionName="checkCollisions";
	// sprite
	SpriteCollisionCheckDatas::iterator iter = mSpriteCollisionCheckDatas.begin();
	while (iter != mSpriteCollisionCheckDatas.end())
	{
		SpriteCollisionCheckData *data = *iter;
		int count = 0;
		if (data->type == CHECK_HIT)
		{
			if (data->sprite->checkCollision(data->other))
			{
				count = 10000000; // something like max int
			}
		}
		else //CHECK_PIXEL
		{
			count = data->sprite->checkCollisionPixels(data->other);
		}
		if (count>data->threshold)
		{
			MESSAGE_SPRITE_SPRITE_COLLISION.tiles.clear();
			MESSAGE_SPRITE_SPRITE_COLLISION.tile = 0;
			MESSAGE_SPRITE_SPRITE_COLLISION.layer = -1;

			MESSAGE_SPRITE_SPRITE_COLLISION.sprite = data->sprite;
			MESSAGE_SPRITE_SPRITE_COLLISION.other = data->other;
			MESSAGE_SPRITE_SPRITE_COLLISION.pixels = count;
			sendMessage(MESSAGE_SPRITE_SPRITE_COLLISION);
		}
		iter++;
	}

	// tile
	TileCollisionCheckDatas::iterator titer = mTileCollisionCheckDatas.begin();
	while (titer != mTileCollisionCheckDatas.end())
	{
		TileCollisionCheckData *data = *titer;
		int count = 0;
		if (data->type == CHECK_HIT)
		{
			CSTiles tiles = data->sprite->getCollidingTiles(mCurrentWorld->getLayer(data->layer));
			if (tiles.size() >0)
			{
				CSTiles::iterator ttiter = tiles.begin();
				while (ttiter != tiles.end())
				{
					MESSAGE_SPRITE_TILE_COLLISION.tiles.clear();
					MESSAGE_SPRITE_TILE_COLLISION.tile = *ttiter;
					MESSAGE_SPRITE_TILE_COLLISION.other = 0;
					MESSAGE_SPRITE_TILE_COLLISION.sprite = data->sprite;
					MESSAGE_SPRITE_TILE_COLLISION.layer = data->layer;
					MESSAGE_SPRITE_TILE_COLLISION.pixels = 1;
					sendMessage(MESSAGE_SPRITE_TILE_COLLISION);
					ttiter++;
				}
			}
		}
		else if (data->type == CHECK_PIXEL)
		{
			CSTiles tiles = data->sprite->getCollidingTiles(mCurrentWorld->getLayer(data->layer));
			if (tiles.size() >0)
			{
				CSTiles::iterator ttiter = tiles.begin();
				while (ttiter != tiles.end())
				{
					MESSAGE_SPRITE_TILE_COLLISION.tiles.clear();
					MESSAGE_SPRITE_TILE_COLLISION.tile = *ttiter;
					MESSAGE_SPRITE_TILE_COLLISION.other = 0;
					MESSAGE_SPRITE_TILE_COLLISION.sprite = data->sprite;
					MESSAGE_SPRITE_TILE_COLLISION.layer = data->layer;
					MESSAGE_SPRITE_TILE_COLLISION.pixels = data->sprite->checkCollisionPixels(*ttiter);
					sendMessage(MESSAGE_SPRITE_TILE_COLLISION);
					ttiter++;
				}
			}
		}
		else // CHECK_JUST_HIT
		{
			CSTiles tiles = data->sprite->getCollidingTiles(mCurrentWorld->getLayer(data->layer));
			if (tiles.size() >0)
			{
				MESSAGE_SPRITE_TILE_COLLISION.tiles = tiles;
				MESSAGE_SPRITE_TILE_COLLISION.tile = 0;
				MESSAGE_SPRITE_TILE_COLLISION.other = 0;
				MESSAGE_SPRITE_TILE_COLLISION.sprite = data->sprite;
				MESSAGE_SPRITE_TILE_COLLISION.layer = data->layer;
				MESSAGE_SPRITE_TILE_COLLISION.pixels = 1;
				sendMessage(MESSAGE_SPRITE_TILE_COLLISION);
			}
		}
		titer++;
	}
}
	
void CSGame::addSprite(CSSprite *sprite)
{
	static char *functionName="addSprite";
	LOG_ENTER 
	// savety meassure, not to add twice!
	removeSprite(sprite);
	for (int i=0; i < mWorldCount; i++)
	{
		mWorld[i]->addSprite(sprite);
	}
	mSprites.push_back(sprite);
	LOG_EXIT 
}

void CSGame::removeSprite(CSSprite *sprite)
{
	static char *functionName="removeSprite";
	LOG_ENTER 
	for (int i=0; i < mWorldCount; i++)
	{
		mWorld[i]->removeSprite(sprite);
	}
	CSSprites::iterator iter = mSprites.begin();
	while (iter != mSprites.end())
	{
		if ((*iter) == sprite)
		{
			mSprites.erase(iter);
			iter = mSprites.begin();	// restart!
		}
		else
		{
			iter++;
		}
	}

	// TODO: ensure, that the listener is not currently enqueued with
	//       a waiting message!
	LOG_EXIT 
}

void CSGame::toggleGui()
{
	static char *functionName="toggleGui";
	LOG_ENTER 
	mIsGui = !mIsGui;
	initSpeed();
	LOG_EXIT 
}

void CSGame::toggleFullscreen()
{
	static char *functionName="toggleFullscreen";
	LOG_ENTER 
	SDLMain *main = SDLMain::getInstance();
	main->toggleFullscreen();
	LOG_EXIT 
}

void CSGame::initSpeed()
{
	static char *functionName="initSpeed";
	LOG_ENTER 
	mInitCount = INIT_COUNT;
	mSecPerFrame = 0.0f;
	LOG_EXIT 
}

void CSGame::reactOnMessage(CSMessage *message)
{
	reactOnMessageGame(message);
}

void CSGame::reactOnMessageGame(CSMessage *message)
{
	static char *functionName="reactOnMessage";
	switch (message->getType())
	{
		case GAME_MESSAGE:
		{
			switch (message->getSubtype())
			{
				case END_MESSAGE:
				{
					mQuit = true;
					return;
				}
				
				case PAUSE_MESSAGE:
				{
					doPause();
					return;
				}
				
				case TOGGLE_FULLSCREEN_MESSAGE:
				{
					toggleFullscreen();
					return;
				}
				case TOGGLE_GUI_MESSAGE:
				{
					toggleGui();
					return;
				}
			}
			break;
		}
		case SDL_MESSAGE:
		{
			switch (message->getSubtype())
			{
				case SCREEN_CHANGED_MESSAGE:
				{
					screenChanged();
					return;
				}
			}
			break;
		}
		case GUI_MESSAGE:
		{
			GuiMessage *gm = (GuiMessage *) message;
			switch (gm->actionId)
			{
				case QUIT_ACTION_ID:
				{
					mQuit = true;
					return;
				}
			}
			break;
		}
	}
}
void CSGame::previousLevel()
{
	static char *functionName="previousLevel";
	if (mCurrentLevel == 0) return;
	LOG_ENTER 
	mCurrentLevel = (mCurrentLevel-1);
	freeLevel();
	initLevel(mLevels.at(mCurrentLevel));
	initSpeed();
	checkWorld();
	LOG_EXIT 
}

void CSGame::nextLevel()
{
	static char *functionName="nextLevel";
	LOG_ENTER 
	mCurrentLevel = (mCurrentLevel+1) % mLevels.size();
	freeLevel();
	initLevel(mLevels.at(mCurrentLevel));
	initSpeed();
	checkWorld();
	LOG_EXIT 
}

static int p = 0;
void CSGame::run(void)
{
	static char *functionName="run";
	LOG_ENTER 
	mCurrentTime = SDL_GetTicks();
	mDiffTime = 0;
	mSecPerFrame = 0;
	mGameTime = 0;
	mCurrentLevel = mStartLevel;
	
	initSpeed();
	if (!dummyGame)
	{
		freeLevel();
		initLevel(mLevels.at(mCurrentLevel));

		checkWorld();

		for (int i=0; i < mWorldCount; i++)
		{
			mWorld[i]->update();
		}
		if (mIsWorldOutOfBounds)
		{
			adjustDisplay();
		}
	}
	
	while (!mQuit)
	{
		static int count = 0;
		static int frameRateTotal = 0;
		long allDrawStart = SDL_GetTicks();
		
		mCurrentTime = SDL_GetTicks();
		do
		{
			handleEvents();
			if (mPause)
			{
				SDL_Delay(50);
			}
		}
		while ((mPause) && (!mQuit));
/*
{
char str[100];
sprintf(str, "GUI Update time in millseconds: %i    ", mDiffTimeGUI);
mFont->putString(SDLMain::getScreen(), 0, 40+50, str);

sprintf(str, "Game Update time in millseconds: %i   ", mDiffTimeGame);
mFont->putString(SDLMain::getScreen(), 0, 55+50, str);

sprintf(str, "All Update time in millseconds: %i   ", mDiffTimeAll);
mFont->putString(SDLMain::getScreen(), 0, 70+50, str);

sprintf(str, "Framerate (average, current): %i, %i   ", mAverageFrameRate, mCurrentFrameRate);
mFont->putString(SDLMain::getScreen(), 0, 85+50, str);
}
 */
		if (mIsGui)
		{
			long desktopDrawStart = SDL_GetTicks();
			
			SDL_Rect area;
			area.x = 0;
			area.y = 0;
			area.w = SDLMain::getScreenWidth();
			area.h = SDLMain::getScreenHeight();
			mDesktop->paint(SDLMain::getScreen(), 0);
			SDLMain::updateScreen();
			SDL_FillRect(SDLMain::getScreen(), &area, COLOR(SDLMain::getScreen(), 0));
			SDLMain::addUpdateRect(area);
			mDiffTimeGUI = SDL_GetTicks() - desktopDrawStart;
		}
		else
		{
			if (mLastIsGui) // make sure double buffers are deleted
			{
				SDL_Rect area;
				area.x = 0;
				area.y = 0;
				area.w = SDLMain::getScreenWidth();
				area.h = SDLMain::getScreenHeight();
				SDLMain::updateScreen();
				SDL_FillRect(SDLMain::getScreen(), &area, COLOR(SDLMain::getScreen(), 0));
				SDLMain::addUpdateRect(area);
			}
			else
			{
				SDLMain::updateScreen();
			}
		}
		long gameStart = SDL_GetTicks();
		mLastIsGui = mIsGui;
		startRun();
		for (int i=0; i < mWorldCount; i++)
		{
			mWorld[i]->update();
		}
		if (mIsWorldOutOfBounds)
		{
			adjustDisplay();
		}
		checkOutOfBounds();
		// checkCollisions(); // before // - not done yet
		if (!dummyGame)
		{
			mCurrentWorld->display();
			if ((mDisplayScaled) && (mScaleCenterSprite != 0))
			{
				mCurrentWorld->displayScaled(mScaleCenterSprite);
			}
		}

		dispatchQueuedMessages();
		if (mEnableCheck)
		{
			checkCollisions(); // after
		}
		checkTimedEvent();
		endRun();
		displayGameData();
		mDiffTime = SDL_GetTicks() - mCurrentTime;
		mGameTime += mDiffTime;
		mCurrentFrameRate = (int)(1/(float (mDiffTime/1000.0f)));
		frameRateTotal += mCurrentFrameRate;
		count++;
		mAverageFrameRate = frameRateTotal/count;
		mDiffTimeGame = SDL_GetTicks() - gameStart;
		if (mInitCount)
		{
			count = 0;
			frameRateTotal = 0;
			mInitCount--;
			mSecPerFrame += (float (mDiffTime/1000.0f));

			if (mInitCount == 0)
			{
				mSecPerFrame = (float) (mSecPerFrame / ((float)INIT_COUNT));
				for (int i=0; i < mWorldCount; i++)
				{
					mWorld[i]->setSecPerFrame(mSecPerFrame);
				}
			}
		}

		if ((p==1) || (mLevelFinished))
		{
			nextLevel();
			p=0;
		}
		if (p==-1) 
		{
			previousLevel();
			p=0;
		}
		mDiffTimeAll = SDL_GetTicks() - allDrawStart;
	}
	LOG_EXIT 
}

// variable "mIsWorldOutOfBounds" is set to true or false, depending on the
// current available world (bigger than screen resolution or not)
void CSGame::checkWorld()
{
	static char *functionName="checkWorld";
	LOG_ENTER 
	mIsWorldOutOfBounds = false;

	unsigned int ww = mCurrentWorld->getWidth();
	unsigned int wh = mCurrentWorld->getHeight();

	unsigned int dx = mCurrentWorld->getDisplayXLowerBound();
	unsigned int dX = mCurrentWorld->getDisplayXUpperBound();
	unsigned int dy = mCurrentWorld->getDisplayYLowerBound();
	unsigned int dY = mCurrentWorld->getDisplayYUpperBound();

	unsigned int dw = dX - dx;
	unsigned int dh = dY - dy;

	if ((ww > dw) || (wh > dh)) mIsWorldOutOfBounds = true;
	LOG_EXIT 
}

// TODO optimize the variables to class variables!
// they don't change!
void CSGame::adjustDisplay()
{
	static char *functionName="adjustDisplay";
	unsigned int dx = mCurrentWorld->getDisplayXLowerBound();
	unsigned int dX = mCurrentWorld->getDisplayXUpperBound();
	unsigned int dy = mCurrentWorld->getDisplayYLowerBound();
	unsigned int dY = mCurrentWorld->getDisplayYUpperBound();

	unsigned int dw = dX - dx;
	unsigned int dh = dY - dy;

	unsigned int sx = mMainSprite->getDisplayXLower();
	unsigned int sX = mMainSprite->getDisplayXUpper();
	unsigned int sy = mMainSprite->getDisplayYLower();
	unsigned int sY = mMainSprite->getDisplayYUpper();

	// world coordinates of main sprite
	unsigned int swx = mMainSprite->getXPos();
	unsigned int swy = mMainSprite->getYPos();
	

	int x = mCurrentWorld->getWorldPositionX();
	int y = mCurrentWorld->getWorldPositionY();

	if (sx < dx + mThreshold)
	{
		x = x - (dx + mThreshold - sx);
	}

	if (sX > dX - mThreshold)
	{
		x = x + mThreshold - (dX-sX);
	}

	if (sy < dy + mThreshold)
	{
		y = y - (dy + mThreshold - sy);
	}

	if (sY > dY - mThreshold)
	{
		y = y + mThreshold - (dY-sY);
	}

	if (x < 0) x = 0;
	if (y < 0) y = 0;
	if (x+dw > mCurrentWorld->getWidth() ) x = mCurrentWorld->getWidth() - dw;
	if (y+dh > mCurrentWorld->getHeight()) y = mCurrentWorld->getHeight() - dh;

	mCurrentWorld->setWorldPosition(x, y);
}

void CSGame::loadLevelData(const std::string &filename, CSLevelData *data)
{
	static char *functionName="loadLevelData";
	LOG_ENTER 
	char tmp[10];
	int i;
	CSXMLHelper xmlSupport(filename, "LEVEL");
	try
	{
		if (xmlSupport.getError())
		{
			throw "error";
		}

		data->id = strdup(xmlSupport.getString("ID").c_str());
		int dCount = xmlSupport.getInt("count(DESCRIPTION)");
		if (dCount > 0)
		{
			for (i=0; i<dCount; i++)
			{
				char *line;
				// i+1, predicates in xpath start with 1
				std::string a = xmlSupport.getString((std::string)"DESCRIPTION["+itoa(i+1,tmp,10)+"]");
				line =  strdup(a.c_str());

				data->descriptions.push_back(line);
			}
		}
		int wCount = xmlSupport.getInt("count(WORLD)");
		for (i=0; i<wCount; i++)
		{
			char *line;
			// i+1, predicates in xpath start with 1
			std::string a = xmlSupport.getString((std::string)"WORLD["+itoa(i+1,tmp,10)+"]");
			line =  strdup(a.c_str());

			data->worlds.push_back(line);
		}
		data->mainSprite.name = strdup(xmlSupport.getString((std::string)"MAIN_SPRITE/NAME").c_str());
		data->mainSprite.speed = xmlSupport.getInt("MAIN_SPRITE/SPEED");
		data->mainSprite.x = xmlSupport.getInt("MAIN_SPRITE/START_POSITION/X");
		data->mainSprite.y = xmlSupport.getInt("MAIN_SPRITE/START_POSITION/Y");
		data->mainSprite.z = xmlSupport.getInt("MAIN_SPRITE/START_POSITION/Z");

		int sCount = xmlSupport.getInt("count(SPRITE)");
		for (i=0; i<sCount; i++)
		{
			SpriteData *sData = new SpriteData();
			sData->name = strdup(xmlSupport.getString((std::string)"SPRITE["+itoa(i+1,tmp,10)+"]/NAME").c_str());
			sData->speed = xmlSupport.getInt((std::string)"SPRITE["+itoa(i+1,tmp,10)+"]/SPEED");
			sData->x = xmlSupport.getInt((std::string)"SPRITE["+itoa(i+1,tmp,10)+"]/START_POSITION/X");
			sData->y = xmlSupport.getInt((std::string)"SPRITE["+itoa(i+1,tmp,10)+"]/START_POSITION/Y");
			sData->z = xmlSupport.getInt((std::string)"SPRITE["+itoa(i+1,tmp,10)+"]/START_POSITION/Z");
			sData->behaviour = xmlSupport.getInt((std::string)"SPRITE["+itoa(i+1,tmp,10)+"]/BEHAVIOUR");

			data->sprites.push_back(sData);
		}

		sCount = xmlSupport.getInt("count(SAMPLE)");
		for (i=0; i<sCount; i++)
		{
			SampleData *sData = new SampleData();
			sData->name = strdup(xmlSupport.getString((std::string)"SAMPLE["+itoa(i+1,tmp,10)+"]/NAME").c_str());
			sData->sid = strdup(xmlSupport.getString((std::string)"SAMPLE["+itoa(i+1,tmp,10)+"]/SID").c_str());
			data->samples.push_back(sData);
		}
		loadLevelDataSpecific(xmlSupport);
	}
	catch(...)
	{
		LOG_EXIT
		SDLMain::shutdown((std::string)"XML error \"" + filename + "\": " + xmlSupport.getErrorMessage().c_str(), 1);
	}
	LOG_EXIT 
}

void CSGame::freeLevel()
{
	static char *functionName="freeLevel";
	LOG_ENTER 
	freeLevelSpecific();
	clearTimedEvents();
	clearSamples();
	if (mMainSprite != 0)
	{
		removeSprite(mMainSprite);
		delete mMainSprite;
		mMainSprite = 0;
	}
	mScaleCenterSprite = 0;

	CSSprites::iterator iter = mSprites.begin();
	while (iter != mSprites.end())
	{
		CSSprite *sprite = *iter;
		removeSprite(sprite);
		delete sprite;
		iter = mSprites.begin();	
	}
	mSprites.clear();

	if (mWorld != 0)
	{
		for (int i=0; i < mWorldCount; i++)
		{
			delete mWorld[i];
		}

		delete[] mWorld;
		mWorld = 0;
	}
	if (mLevelData != 0)
	{
		delete mLevelData;
		mLevelData = 0;
	}
	LOG_EXIT 
}

void CSGame::initLevel(std::string levelName)
{
	static char *functionName="initLevel";
	LOG_ENTER 
	int i;
	mLevelData = new CSLevelData();
	loadLevelData(levelName, mLevelData);

	mWorld = (CSWorld **) new void *[mLevelData->worlds.size()];
	mWorldCount = 0;
	for (i = 0; i < mLevelData->worlds.size(); i++)
	{
		mWorld[i] = CSWorldLoader::INSTANCE.load(mLevelData->worlds.at(i));
		mWorld[i]->setWorldNo(i);
		mWorld[i]->scaleBackgroundTiles(getDisplayArea().w, getDisplayArea().h);
		mWorldCount++;
	}

	SpriteDatas::iterator iter = mLevelData->sprites.begin();
	for (i = 0; i < mLevelData->sprites.size(); i++)
	{
//LOG_AMESSAGE("Loading Sprite: " +std::string(mLevelData->sprites.at(i)->name))
		CSSprite* sprite;
		sprite = CSSpriteLoader::INSTANCE.load(mLevelData->sprites.at(i)->name);
		sprite->setPosition(mLevelData->sprites.at(i)->x, 
							mLevelData->sprites.at(i)->y, 
							mLevelData->sprites.at(i)->z);
		sprite->adjustSpeed(mLevelData->sprites.at(i)->speed);
		addSprite(sprite);
	}
//LOG_AMESSAGE("Done")

	if (mLevelData->mainSprite.name != 0)
	{
		mMainSprite = CSSpriteLoader::INSTANCE.load(mLevelData->mainSprite.name);
		mMainSprite->setPosition(mLevelData->mainSprite.x, mLevelData->mainSprite.y, mLevelData->mainSprite.z);
		mMainSprite->adjustSpeed(mLevelData->mainSprite.speed);
		addMainSprite(mMainSprite);
	}
	
	for (i = 0; i < mLevelData->samples.size(); i++)
	{
		CSSample* sample;
		sample = CSSampleLoader::INSTANCE.load(mLevelData->samples.at(i)->name);
		addSample(sample, mLevelData->samples.at(i)->sid);
	}

	setCurrentWorld(mMainSprite->getWorldPos());
	mScaleCenterSprite = mMainSprite;
	initLevelSpecific(mLevelData);

	if (mDisplayScaled)
	{
		mCurrentWorld->setScaleDisplayArea(getScaledDisplayArea());
		mCurrentWorld->setScaleFactor(getScaleFactor());
	}
	mLevelFinished = false;

	LOG_EXIT 
}

void CSGame::setCurrentWorld(unsigned int w)
{
	static char *functionName="setCurrentWorld";
	LOG_ENTER 
	mCurrentWorld = mWorld[w];
	mCurrentWorld->setDisplayArea(getDisplayArea());
	for (int i=0; i < mWorldCount; i++)
	{
		mWorld[i]->setActive(i==w);
	}
	checkWorld();
	LOG_EXIT 
}

void CSGame::loadGameData(const std::string &filename)
{
	static char *functionName="loadGameData";
	LOG_ENTER 
	char tmp[10];
	int i;
	if (filename.size() == 0)
	{
		// a dummy game?
		// just the gui?
		dummyGame = true;
		return;
	}

	CSXMLHelper xmlSupport(filename, "GAME");
	try
	{
		if (xmlSupport.getError())
		{
			throw "error";
		}

		mId = strdup(xmlSupport.getString("ID").c_str());
		int lCount = xmlSupport.getInt("count(LEVEL)");
		if (lCount > 0)
		{
			for (i=0; i<lCount; i++)
			{
				char *line;
				// i+1, predicates in xpath start with 1
				std::string a = xmlSupport.getString((std::string)"LEVEL["+itoa(i+1,tmp,10)+"]");
				line =  strdup(a.c_str());
				mLevels.push_back(line);
			}
		}
	}
	catch(...)
	{
		LOG_EXIT
		SDLMain::shutdown((std::string)"XML error \"" + filename + "\": " + xmlSupport.getErrorMessage().c_str(), 1);
	}
	LOG_EXIT 
}

void CSGame::addSample(CSSample *sample, const char *sid)
{
	static char *functionName="addSample";
	mSampleMap.insert(std::map<std::string, CSSample *>::value_type(sid, sample));
}

void CSGame::removeSample(const char *sid)
{
	static char *functionName="removeSample";
	// delete single object
	std::map<std::string, CSSample *>::iterator iter;
	for (iter = mSampleMap.begin(); iter!=mSampleMap.end(); iter++)
	{
		if (strcmp(sid, iter->first.c_str()) == 0)
		{
			mSampleMap.erase(iter);
		}
	}
}

void CSGame::clearSamples()
{
	static char *functionName="clearSamples";
	LOG_ENTER 
	mSampleMap.clear();
	LOG_EXIT 
}

void CSGame::playSample(const char *sid)
{
	static char *functionName="playSample";
	std::map<std::string, CSSample *>::iterator iter = mSampleMap.find(sid);
	if (iter != mSampleMap.end())
	{
		iter->second->play();
	}
}

void CSGame::checkOutOfBounds()
{
	static char *functionName="checkOutOfBounds";
	// sprite
	OutOfBoundsDatas::iterator iter = mOutOfBoundsDatas.begin();
	while (iter != mOutOfBoundsDatas.end())
	{
		OutOfBoundsData *data = *iter;
		int count = 0;
		int yl = data->sprite->getYPos() - data->threshold;
		int xl = data->sprite->getXPos() - data->threshold;
		int yh = yl+data->sprite->getMaxHeight() + data->threshold;
		int xh = xl+data->sprite->getMaxWidth() + data->threshold;

		if (yh > getWorld()->getHeight())
		{
			MESSAGE_SPRITE_OUT_OF_BOUNDS.setSubsubtype(Y_HIGHER);
			MESSAGE_SPRITE_OUT_OF_BOUNDS.sprite = data->sprite;
			sendMessage(MESSAGE_SPRITE_OUT_OF_BOUNDS);
		}
		else if (yl < 0)
		{
			MESSAGE_SPRITE_OUT_OF_BOUNDS.setSubsubtype(Y_LOWER);
			MESSAGE_SPRITE_OUT_OF_BOUNDS.sprite = data->sprite;
			sendMessage(MESSAGE_SPRITE_OUT_OF_BOUNDS);
		}
		if (xh > getWorld()->getWidth())
		{
			MESSAGE_SPRITE_OUT_OF_BOUNDS.setSubsubtype(X_HIGHER);
			MESSAGE_SPRITE_OUT_OF_BOUNDS.sprite = data->sprite;
			sendMessage(MESSAGE_SPRITE_OUT_OF_BOUNDS);
		}
		else if (xl < 0)
		{
			MESSAGE_SPRITE_OUT_OF_BOUNDS.setSubsubtype(X_LOWER);
			MESSAGE_SPRITE_OUT_OF_BOUNDS.sprite = data->sprite;
			sendMessage(MESSAGE_SPRITE_OUT_OF_BOUNDS);
		}
		iter++;
	}
}

void CSGame::doPause(void)
{
	static char *functionName="doPause";
	LOG_ENTER 
	mPause = !mPause;
	if (mPause)
	{
		CSSample::pauseAll();
	}
	else
	{
		CSSample::resumeAll();
		mCurrentTime = SDL_GetTicks();
	}
	LOG_EXIT 
}

void CSGame::addTimedEvent(unsigned int timeSpace, int id, void *data, void (*answer)(TimedEvent *), int repeat, bool repeatUse)
{
	static char *functionName="addTimedEvent";
	// don't add two events at exactly the same time!
	CSEventMap::iterator iter = mEventMap->find(mGameTime + timeSpace);
	while (iter != mEventMap->end())
	{
		timeSpace++;
		iter = mEventMap->find(mGameTime + timeSpace);
	}
	
	TimedEvent *event = new TimedEvent();
	event->id = id;
	event->data = data;
	event->repeat = repeat;
	event->finishTime = mGameTime + timeSpace;
	event->intervall = timeSpace;
	event->useRepeat = repeatUse;
	event->answer = answer;

	mEventMap->insert(CSEventMap::value_type(event->finishTime, event));
}

void CSGame::addTimedEvent(unsigned int timeSpace, int id, void *data, int repeat,  bool repeatUse)
{
	static char *functionName="addTimedEvent";
	// don't add two events at exactly the same time!
	CSEventMap::iterator iter = mEventMap->find(mGameTime + timeSpace);
	while (iter != mEventMap->end())
	{
		timeSpace++;
		iter = mEventMap->find(mGameTime + timeSpace);
	}
	
	TimedEvent *event = new TimedEvent();
	event->id = id;
	event->data = data;
	event->repeat = repeat;
	event->finishTime = mGameTime + timeSpace;
	event->intervall = timeSpace;
	event->useRepeat = repeatUse;

	mEventMap->insert(CSEventMap::value_type(event->finishTime, event));
}

void CSGame::removeTimedEvent(int id, void *data)
{
	static char *functionName="removeTimedEvent";
	CSEventMap::iterator iter = mEventMap->begin();
	while (iter!=mEventMap->end())
	{
		TimedEvent *event = iter->second;
		if ((event->id == id) && (data == event->data))
		{
			mEventMap->erase(iter);
			delete (event);
			iter = mEventMap->begin();
			continue;
		}
		iter++;
	}
}

void CSGame::removeTimedEvent(int id)
{
	static char *functionName="removeTimedEvent";
	CSEventMap::iterator iter = mEventMap->begin();
	while (iter!=mEventMap->end())
	{
		TimedEvent *event = iter->second;
		if (event->id == id)
		{
			mEventMap->erase(iter);
			delete (event);
			iter = mEventMap->begin();
			continue;
		}
		iter++;
	}
}

void CSGame::clearTimedEvents()
{
	static char *functionName="clearTimedEvents";
	LOG_ENTER 
	CSEventMap::iterator iter = mEventMap->begin();
	while (iter!=mEventMap->end())
	{
		TimedEvent *event = iter->second;
		mEventMap->erase(iter);
		delete (event);
		iter = mEventMap->begin();
	}
	LOG_EXIT 
}

// would be "better" if the map was sorted
void CSGame::checkTimedEvent()
{
	static char *functionName="checkTimedEvent";
	CSEventMap::iterator iter = mEventMap->begin();
	while (iter!=mEventMap->end())
	{
		if (iter->first <= mGameTime)
		{
			TimedEvent *event = iter->second;
			mEventMap->erase(iter); // erase befor the event,
									// since within the event, mEventMap might be changed (BAD!)
			if (event->answer == 0)
			{
				reactOnTimedEvent(event);
			}
			else
			{
				event->answer(event);
			}
			if (event->useRepeat)
			{
				event->repeat--;
				if (event->repeat > 0)
				{
					event->finishTime = mGameTime + event->intervall;
					mEventMap->insert(CSEventMap::value_type(event->finishTime, event));
				}
				else
				{
					delete (event);
				}
			}
			else
			{
				delete (event);
			}
			iter = mEventMap->begin(); // restart, since mEventMap was surely changed!
			continue;
		}
		iter++;
	}
}

// resets sprite to initial Level values!
void CSGame::resetSprites()
{
	static char *functionName="resetSprites";
	LOG_ENTER 
	int i;
	CSSprites::iterator iter = mSprites.begin();
	for (i = 0; i < mLevelData->sprites.size(); i++)
	{
		CSSprite* sprite = *iter;
		sprite->setPosition(mLevelData->sprites.at(i)->x, 
							mLevelData->sprites.at(i)->y, 
							mLevelData->sprites.at(i)->z);
		iter++;
	}

	if (mMainSprite != 0)
	{
		mMainSprite->setPosition(mLevelData->mainSprite.x, mLevelData->mainSprite.y, mLevelData->mainSprite.z);
		setCurrentWorld(mLevelData->mainSprite.z);
	}
	resetSpritesSpecific();
	LOG_EXIT 
}

void CSGame::disableSpriteMovement()
{
	static char *functionName="disableSpriteMovement";
	LOG_ENTER 
	CSSprites::iterator iter = mSprites.begin();
	while (iter != mSprites.end())
	{
		CSSprite *sprite = *iter;
		sprite->adjustSpeed(0);
		iter++;
	}
	LOG_EXIT 
}

bool CSGame::isSamplePlaying(void)
{
	static char *functionName="isSamplePlaying";
	return CSSample::isPlaying();
}

void CSGame::screenChanged()
{
	static char *functionName="screenChanged";
	LOG_ENTER 
	// following classes include surfaces that are screen dependent
	// the classes must take care of themselfs
	// but we must at least tell them that the video mode has changed!
	CSPicture::resetSurfaces();
	CSFont::resetSurfaces();
	initSpeed();
	if (mDisplayScaled)
	{
		mCurrentWorld->resetScaledWorld();
	}
	LOG_EXIT 
}
					
void CSGame::setScaleDisplayLayer(bool b, int layer)
{
	static char *functionName="setScaleDisplayLayer";
	LOG_ENTER 
	mCurrentWorld->setScaleDisplayLayer(b,layer);
	LOG_EXIT 
}

void CSGame::enableDesktop(bool b)
{
	static char *functionName="enableDesktop";
	LOG_ENTER 
	if (b == mIsGui)
	{
		LOG_EXIT 
		return;
	}
	mIsGui = b;
	LOG_EXIT 
}

void CSGame::onMouseMotion(SDL_MouseMotionEvent event)
{
	static char *functionName="onMouseMotion";
	if (mIsGui)
	{
		if (mDesktop->isIn(event.x,event.y, 0))
		{
			mDesktop->onMouseMotion(event.x,event.y);
		}
	}
}

void CSGame::onMouseButtonDown(SDL_MouseButtonEvent event)
{
	static char *functionName="onMouseButtonDown";
	LOG_ENTER 
	if (mIsGui)
	{
		if (mDesktop->isIn(event.x,event.y, 0))
		{
			mDesktop->onMouseDown(event.x,event.y);
		}
	}
	LOG_EXIT 
}

void CSGame::onMouseButtonUp(SDL_MouseButtonEvent event)
{
	static char *functionName="onMouseButtonUp";
	LOG_ENTER 
	if (mIsGui)
	{
		if (mDesktop->isIn(event.x,event.y, 0))
		{
			mDesktop->onMouseUp(event.x,event.y);
		}
	}
	LOG_EXIT 
}

void CSGame::handleEvents()
{
	static char *functionName="handleEvents";
	SDL_Event event;

	if (SDL_PeepEvents(&event, 1, SDL_PEEKEVENT,SDL_MOUSEMOTIONMASK)==1)
	{
		do 
		{
			SDL_PeepEvents(&event,1, SDL_GETEVENT,SDL_MOUSEMOTIONMASK);
		} while (SDL_PeepEvents(&event,1 , SDL_PEEKEVENT,SDL_MOUSEMOTIONMASK)==1);
		
		onMouseMotion(event.motion);
	}
	
	if (SDL_PollEvent(&event))
	{
		switch(event.type)
		{
			case SDL_MOUSEMOTION:
			{
		onMouseMotion(event.motion);
				break;
			}
			case SDL_MOUSEBUTTONDOWN:
			{
				onMouseButtonDown(event.button); 
				break;
			}
			case SDL_MOUSEBUTTONUP:
			{
				onMouseButtonUp(event.button); 
				break;
			}
			case SDL_KEYUP:
			{
				if (mIsGui)
				{
					mDesktop->dispatchKeyEvent(event, SDL_KEYUP);
				}
				break;
			}
			case SDL_KEYDOWN:
			{
				bool used = false;
				if (mIsGui)
				{
					used = mDesktop->dispatchKeyEvent(event, SDL_KEYDOWN);
				}
				if (!used)
				{
					switch(event.key.keysym.sym) 
					{
						case SDLK_ESCAPE:
							sendMessage(MESSAGE_END);
							break;

						case SDLK_RIGHT:
						case SDLK_r:
							sendMessage(MESSAGE_KEY_RIGHT);
							break;

						case SDLK_LEFT:
						case SDLK_l:
							sendMessage(MESSAGE_KEY_LEFT);
							break;

						case SDLK_UP:
						case SDLK_u:
							sendMessage(MESSAGE_KEY_UP);
							break;

						case SDLK_DOWN:
						case SDLK_d:
							sendMessage(MESSAGE_KEY_DOWN);
							break;
						case SDLK_p:
							sendMessage(MESSAGE_PAUSE);
							break;
						case SDLK_g:
							sendMessage(MESSAGE_TOGGLE_GUI);
							break;

						case SDLK_SPACE:
	//						sendMessage(MESSAGE_KEY_FIRE);
	p = 1;
							break;
						case SDLK_BACKSPACE:
	p = -1;
							break;

						case SDLK_F4:
							sendMessage(MESSAGE_TOGGLE_FULLSCREEN);
							break;
						default:
							break;
					}
				}
			}
			break;
		}
	}
}

void CSGame::intro(void)
{
}

int CSGame::getLevelDataNum(CSSprite *sprite)
{
	SpriteDatas::iterator iter = mLevelData->sprites.begin();

	for (int i = 0; i < mLevelData->sprites.size(); i++)
	{
		SpriteData* data = (SpriteData *) mLevelData->sprites.at(i);
		if (strcmp(data->name, sprite->getFilename().c_str()) == 0)
		{
			return i;
		}
	}
	return -1;
}



