#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

#include "CSLAF.h"
#include "SDLMain.h"
#include "CSGrafikElement.h"
#include "CSMouseCursor.h"
#include "CSFont.h"
#include "CSIcon.h"
#include "CSBorder.h"
#include "CSArea.h"
#include "CSHelper.h"
#include "CSButton.h"
#include "CSWindow.h"
#include "CSCheckBox.h"
#include "CSCombobox.h"
#include "CSListbox.h"
#include "CSLabel.h"
#include "CSMenu.h"
#include "CSMenuBar.h"
#include "CSMenuItem.h"
#include "CSScrollbar.h"
#include "CSPanel.h"
#include "CSTextfield.h"
#include "CSTextarea.h"
#include "CSHTMLPanel.h"
#include "CSHTMLarea.h"


#pragma warning(disable : 4786 )

	//! \todo rectFill berall beseitigen! 


CSLAF *CSLAF::mLAF = 0;
CSGrafikElements CSLAF::mGUIElements;
int CSLAF::mCurrentLAFId = 0;
const char *CSLAF::CLASS = "CSLAF";

void CSLAF::setCurrentLAF(CSLAF &laf)
{
	(*mLAF) = laf;
	// pointers are copied also!
	// beware, that we don't delete pointers of the class we are copiing from
	// therefor we set the actual CSMouseCursor instances to 0!
	mLAF->mSizingMouseCursorInstance = 0;
	mLAF->mDefaultMouseCursorInstance = 0;
	lafChanged();
}

CSLAF *CSLAF::getCurrentLAF()
{
	if (mLAF == 0)
	{
		mLAF = new CSLAF();
	}
	return mLAF;
}

CSLAF::CSLAF() 
{
	mSizingMouseCursorInstance = 0;
	mDefaultMouseCursorInstance = 0;
	reset();
}

CSLAF::~CSLAF()
{
	destroyMouseCursors();
}


void CSLAF::addElement(CSGrafikElement *element)
{
	static char *functionName="addElement";
	LOG_ENTER 
	element->setCurrentLAFId(mCurrentLAFId);
	mGUIElements.push_back(element);
	LOG_EXIT
}

void CSLAF::removeElement(CSGrafikElement *element)
{
	static char *functionName="removeElement";
	LOG_ENTER 
	CSGrafikElements::iterator iter = mGUIElements.begin();
	while (iter != mGUIElements.end())
	{
		if ((*iter) == element)
		{
			mGUIElements.erase(iter);
			iter = mGUIElements.begin();	// restart!
		}
		else
		{
			iter++;
		}
	}
	LOG_EXIT
}

void CSLAF::lafChanged()
{
	static char *functionName="lafChanged";
	LOG_ENTER 
	if (mLAF != 0)
	{
		mLAF->destroyMouseCursors();
	}
	mCurrentLAFId++;
	
	CSGrafikElements::iterator iter = mGUIElements.begin();
	while (iter != mGUIElements.end())
	{
		CSGrafikElement *element = *iter;
		int id = element->getCurrentLAFId();
		if (id != mCurrentLAFId)
		{
			element->lafChanged(mCurrentLAFId);
			element->setCurrentLAFId(mCurrentLAFId);
			iter = mGUIElements.begin();
		}
		else
		{
			iter++;
		}
	}

	LOG_EXIT
}

int CSLAF::getHorizontalElementSpacingInternal(const std::string &classType)
{
	if (classType.compare(CSButton::CLASS) == 0)
	{
		return buttonHorizontalSpacing;
	}
	if (classType.compare(CSDragButton::CLASS) == 0)
	{
		return buttonHorizontalSpacing;
	}
	if (classType.compare("Title") == 0)
	{
		return windowTitleInterIconOffset;
	}
	if (classType.compare(CSMenuBar::CLASS) == 0)
	{
		return menuBarHorizontalSpacing;
	}
	if (classType.compare(CSMenu::CLASS) == 0)
	{
		return menuItemHorzintalSpacing;
	}
//	if (classType.compare("Menu") == 0)
//	{
//		return menuItemHorzintalSpacing;
//	}
	if (classType.compare("MenuWindow") == 0)
	{
		return menuItemHorzintalSpacing;
	}
	if (classType.compare(CSMenuItem::CLASS) == 0)
	{
		return menuItemHorzintalSpacing;
	}
	if ( (classType.compare("CSIcon") == 0) || (classType.compare("CSPictureIcon") == 0)  || (classType.compare("CSTextIcon") == 0) )
	{
		return 0;
	}
	if (classType.compare(CSCheckBox::CLASS) == 0)
	{
		return buttonHorizontalSpacing;
	}
	if (classType.compare(CSListbox::CLASS) == 0)
	{
		return listboxHorizontalLineSpacing;
	}
	if (classType.compare(CSTextfield::CLASS) == 0)
	{
		return textfieldHorizontalSpacing;
	}
	if (classType.compare(CSTextarea::CLASS) == 0)
	{
		return textAreaHorizontalLineSpacing;
	}
	return elementHorizontalSpacing;
}

int CSLAF::getVerticalElementSpacingInternal(const std::string &classType)
{
	if (classType.compare(CSButton::CLASS) == 0)
	{
		return buttonVerticalSpacing;
	}
	if (classType.compare(CSDragButton::CLASS) == 0)
	{
		return buttonVerticalSpacing;
	}
	if (classType.compare("Title") == 0)
	{
		return windowTitleInterIconOffset;
	}
	if (classType.compare(CSMenuBar::CLASS) == 0)
	{
		return menuBarVerticalSpacing;
	}
	if (classType.compare(CSMenu::CLASS) == 0)
	{
		return menuItemVerticalSpacing;
	}
	if (classType.compare("MenuWindow") == 0)
	{
		return menuItemVerticalSpacing;
	}
//	if (classType.compare("Menu") == 0)
//	{
//		return menuItemVerticalSpacing;
//	}
	if (classType.compare(CSMenuItem::CLASS) == 0)
	{
		return menuItemVerticalSpacing;
	}
	if ( (classType.compare("CSIcon") == 0) || (classType.compare("CSPictureIcon") == 0)  || (classType.compare("CSTextIcon") == 0) )
	{
		return 0;
	}
	if (classType.compare(CSCheckBox::CLASS) == 0)
	{
		return buttonVerticalSpacing;
	}
	if (classType.compare(CSListbox::CLASS) == 0)
	{
		return listboxVerticalLineSpacing;
	}
	if (classType.compare(CSTextfield::CLASS) == 0)
	{
		return textfieldVerticalSpacing;
	}
	if (classType.compare(CSTextarea::CLASS) == 0)
	{
		return textAreaVerticalLineSpacing;
	}
	return elementVerticalSpacing;
}
		
CSArea *CSLAF::getAreaInternal(const std::string &classType, int state)
{
	static CSArea *area = new CSArea();
	if (classType.compare(CSLabel::CLASS) == 0)
	{
		static CSArea *area = new CSArea();
		return 0;
		//return area;
	}
	if (classType.compare(CSPanel::CLASS) == 0)
	{
		static CSArea *area = new CSArea();
		return 0;
		//return area;
	}
	if (classType.compare(CSButton::CLASS) == 0)
	{
		static CSRoundedArea *area = new CSRoundedArea();
		area->setCorner(3);
		return area;
	}
	if (classType.compare(CSDragButton::CLASS) == 0)
	{
		static CSRoundedArea *area = new CSRoundedArea();
		area->setCorner(4);
		return area;
	}
	if (classType.compare(CSWindow::CLASS) == 0)
	{
		static CSRoundedArea *area = new CSRoundedArea();
		area->setCorner(3);
//		static CSPictureArea *area = new CSPictureArea();
//		area->setPicture(CSPictureDirectLoader::INSTANCE.load("PAC_MAN.bmp"));
		return area;
	}

	if (classType.compare(CSMenuBar::CLASS) == 0)
	{
		/*
		static CSGradientArea *area = new CSGradientArea();
		area->setColor(CSHelper::colorPercent(getBackgroundColorEnabledInternal(classType), 50));
		area->setColor(CSHelper::colorPercent(0xffffff, 50));
		area->setSecondColor(CSHelper::colorPercent(getBackgroundColorEnabledInternal(classType), 50));
		area->setSecondColor(CSHelper::colorPercent(00000000, 50));
		area->setType(TYPE_VERTICAL);
		*/
		static CSArea *area = new CSArea();
		area->setColor(CSHelper::colorPercent(getBackgroundColorEnabledInternal(classType), 50));
		return area;
	}
	if (classType.compare(CSTextfield::CLASS) == 0)
	{
		static CSRoundedArea *area = new CSRoundedArea();
		area->setCorner(6);
		return area;
	}
	if (classType.compare(CSBar::CLASS) == 0)
	{
		static CSGradientArea *area = new CSGradientArea();
		area->setColor(CSHelper::colorPercent(getBackgroundColorEnabledInternal(classType), 50));
		area->setSecondColor(CSHelper::colorPercent(getBackgroundColorEnabledInternal(classType), 50));
		return area;
	}
	if (classType.compare(CSTextarea::CLASS) == 0)
	{
		static CSRoundedArea *area = new CSRoundedArea();
		area->setCorner(6);
		return area;
	}
	return area;
}

CSFont *CSLAF::getFontInternal(const std::string &classType)
{
	static char *functionName="getProportionalFont";
	if (classType.compare("MonoSpaced"))
	{
		static CSFont *font = CSFontLoader::INSTANCE.load(monoSpacedFontName);
		font->setSolid(false);
		return font;
	}
	else if (classType.compare(CSTextarea::CLASS))
	{
		static CSFont *font = CSFontLoader::INSTANCE.load(monoSpacedFontName);
		font->setSolid(false);
		return font;
	}
	static CSFont *font = CSFontLoader::INSTANCE.load(proportionalFontName);
	font->setSolid(false);
	return font;
}

CSMouseCursor *CSLAF::getMouseCursorInternal(int type)
{
	switch (type)
	{
		case LAF_MOUSE_CURSOR_DEFAULT:
		{
			if (mDefaultMouseCursorInstance == 0)
			{
				mDefaultMouseCursorInstance = new CSMouseCursor(mDefaultMouseCursor);
			}
			return mDefaultMouseCursorInstance;
		}
		case LAF_MOUSE_CURSOR_SIZING_DOWN_RIGHT:
		case LAF_MOUSE_CURSOR_SIZING_DOWN_LEFT:
		case LAF_MOUSE_CURSOR_SIZING_UP_RIGHT:
		case LAF_MOUSE_CURSOR_SIZING_UP_LEFT:
		case LAF_MOUSE_CURSOR_SIZING_DOWN:
		case LAF_MOUSE_CURSOR_SIZING_UP:
		case LAF_MOUSE_CURSOR_SIZING_LEFT:
		case LAF_MOUSE_CURSOR_SIZING_RIGHT:
		{
			if (mSizingMouseCursorInstance == 0)
			{
				mSizingMouseCursorInstance = new CSMouseCursor(mSizingMouseCursor);
			}
			return mSizingMouseCursorInstance;
		}
	}
	return 0;
}

int CSLAF::getBackgroundColorEnabledInternal(const std::string &classType) 
{
	if (classType.compare(CSPanel::CLASS) == 0)
	{
		return windowBackColorEnabled;
	}
	if (classType.compare(CSDragButton::CLASS) == 0)
	{
		return buttonBackColorEnabled;
	}
	if (classType.compare(CSButton::CLASS) == 0)
	{
		return buttonBackColorEnabled;
	}
	if (classType.compare(CSWindow::CLASS) == 0)
	{
		return windowBackColorEnabled;
	}
	if (classType.compare("Title") == 0)
	{
		return titleColorEnabled;
	}
	if (classType.compare(CSMenuBar::CLASS) == 0)
	{
		return menuBarBackColorEnabled;
	}
	if (classType.compare(CSMenu::CLASS) == 0)
	{
		return menuBarBackColorEnabled;
	}
	if (classType.compare("MenuWindow") == 0)
	{
		return menuBackColorEnabled;
	}
//	if (classType.compare("Menu") == 0)
//	{
//		return menuBackColorEnabled;
//	}
	if (classType.compare(CSMenuItem::CLASS) == 0)
	{
		return menuItemBackColorEnabled;
	}
	if (classType.compare("CSMenuItem") == 0)
	{
		return menuItemBackColorEnabled;
	}
	if (classType.compare(CSCheckBox::CLASS) == 0)
	{
		return checkBoxColorEnabled;
	}
	if (classType.compare(CSScrollbar::CLASS) == 0)
	{
		return scrollbarBackColor;
	}
	if (classType.compare(CSListbox::CLASS) == 0)
	{
		return listboxColorEnabled;
	}
	if (classType.compare(CSCombobox::CLASS) == 0)
	{
		return textfieldColorEnabled;
	}
	if (classType.compare(CSTextfield::CLASS) == 0)
	{
		return textfieldColorEnabled;
	}
	if (classType.compare(CSTextarea::CLASS) == 0)
	{
		return textAreaColorEnabled;
	}
	return -1;
}

int CSLAF::getBackgroundColorDisabledInternal(const std::string &classType) 
{
	if (classType.compare(CSPanel::CLASS) == 0)
	{
		return windowBackColorDisabled;
	}
	if (classType.compare(CSMenuItem::CLASS) == 0)
	{
		return menuItemBackColorDisabled;
	}
	if (classType.compare("CSMenuItem") == 0)
	{
		return menuItemBackColorDisabled;
	}
	if (classType.compare(CSDragButton::CLASS) == 0)
	{
		return buttonBackColorDisabled;
	}
	if (classType.compare(CSButton::CLASS) == 0)
	{
		return buttonBackColorDisabled;
	}
	if (classType.compare(CSScrollbar::CLASS) == 0)
	{
		return scrollbarBackColor;
	}
	if (classType.compare(CSWindow::CLASS) == 0)
	{
		return windowBackColorDisabled;
	}
	if (classType.compare("Title") == 0)
	{
		return titleColorDisabled;
	}
	if (classType.compare(CSCheckBox::CLASS) == 0)
	{
		return checkBoxColorDisabled;
	}
	if (classType.compare(CSListbox::CLASS) == 0)
	{
		return listboxColorDisabled;
	}
	if (classType.compare(CSCombobox::CLASS) == 0)
	{
		return textfieldColorDisabled;
	}
	if (classType.compare(CSTextfield::CLASS) == 0)
	{
		return textfieldColorDisabled;
	}
	if (classType.compare(CSTextarea::CLASS) == 0)
	{
		return textAreaColorDisabled;
	}
	return -1;
}

int CSLAF::getTextColorEnabledInternal(const std::string &classType) 
{
	if (classType.compare(CSPanel::CLASS) == 0)
	{
		return windowTextColorEnabled;
	}
	if (classType.compare(CSWindow::CLASS) == 0)
	{
		return windowTextColorEnabled;
	}
	if (classType.compare("Title") == 0)
	{
		return titleTextColorEnabled;
	}
	if (classType.compare(CSMenu::CLASS) == 0)
	{
		return menuTextColorEnabled;
	}
	if (classType.compare("MenuWindow") == 0)
	{
		return menuTextColorEnabled;
	}
//	if (classType.compare("Menu") == 0)
//	{
//		return menuTextColorEnabled;
//	}
	if (classType.compare(CSMenuItem::CLASS) == 0)
	{
		return menuItemTextColorEnabled;
	}
	if (classType.compare("CSMenuItem") == 0)
	{
		return menuItemTextColorEnabled;
	}
	if (classType.compare(CSListbox::CLASS) == 0)
	{
		return listboxTextColorEnabled;
	}
	if (classType.compare(CSTextarea::CLASS) == 0)
	{
		return textAreaTextColorEnabled;
	}
	if (classType.compare(CSTextfield::CLASS) == 0)
	{
		return textfieldTextColorEnabled;
	}
	if (classType.compare(CSCombobox::CLASS) == 0)
	{
		return textfieldTextColorEnabled;
	}
	return -1;
}

int CSLAF::getTextColorDisabledInternal(const std::string &classType) 
{
	if (classType.compare(CSPanel::CLASS) == 0)
	{
		return windowTextColorDisabled;
	}
	if (classType.compare(CSWindow::CLASS) == 0)
	{
		return windowTextColorDisabled;
	}
	if (classType.compare("Title") == 0)
	{
		return titleTextColorDisabled;
	}
	if (classType.compare(CSMenu::CLASS) == 0)
	{
		return menuTextColorDisabled;
	}
	if (classType.compare("MenuWindow") == 0)
	{
		return menuTextColorDisabled;
	}
//	if (classType.compare("Menu") == 0)
//	{
//		return menuTextColorDisabled;
//	}
	if (classType.compare(CSMenuItem::CLASS) == 0)
	{
		return menuItemTextColorDisabled;
	}
	if (classType.compare("CSMenuItem") == 0)
	{
		return menuItemTextColorDisabled;
	}
	if (classType.compare(CSListbox::CLASS) == 0)
	{
		return listboxTextColorDisabled;
	}
	if (classType.compare(CSCombobox::CLASS) == 0)
	{
		return textfieldTextColorDisabled;
	}
	if (classType.compare(CSTextfield::CLASS) == 0)
	{
		return textfieldTextColorDisabled;
	}
	if (classType.compare(CSTextarea::CLASS) == 0)
	{
		return textAreaTextColorDisabled;
	}
	return -1;
}

CSBorder *CSLAF::getBorderInternal(const std::string &classType, int state)
{
	static CSBorder *theBorder = CSBorder::getBorder(BORDER_TYPE_NONE);

	// default
	int type = -1;
	if (classType.compare(CSPanel::CLASS) == 0)
	{
		static CSBorder *border = CSBorder::getBorder(BORDER_TYPE_NONE);
//		static CSRoundedBorder *border = CSRoundedBorder::getBorder(BORDER_TYPE_WINDOW);
//		border->setState(state);
//		border->setCorner(3);
		return border;
	}
	if (classType.compare(CSButton::CLASS) == 0)
	{
//		static CSBorder *border = CSBorder::getBorder(BORDER_TYPE_BUTTON);
//		border->setState(state);
		static CSRoundedBorder *border = CSRoundedBorder::getBorder(BORDER_TYPE_BUTTON);
		border->setState(state);
		border->setCorner(4);
		return border;
	}
	if (classType.compare(CSDragButton::CLASS) == 0)
	{
//		static CSBorder *border = CSBorder::getBorder(BORDER_TYPE_BUTTON);
//		border->setState(state);
		static CSRoundedBorder *border = CSRoundedBorder::getBorder(BORDER_TYPE_BUTTON);
		border->setState(state);
		border->setCorner(6);
		return border;
	}
	if (classType.compare(CSWindow::CLASS) == 0)
	{
//		static CSBorder *border = CSBorder::getBorder(BORDER_TYPE_WINDOW);
//		border->setState(state);
		static CSRoundedBorder *border = CSRoundedBorder::getBorder(BORDER_TYPE_WINDOW);
		border->setState(state);
		border->setCorner(3);
		return border;
	}
	if (classType.compare(CSBar::CLASS) == 0)
	{
		static CSBorder *border = CSBorder::getBorder(BORDER_TYPE_UP);
		border->setState(state);
		return border;
	}
	if (classType.compare("Title") == 0)
	{
		static CSBorder *border = CSBorder::getBorder(BORDER_TYPE_UP);
		border->setState(state);
		return border;
	}
	if (classType.compare(CSMenu::CLASS) == 0)
	{
		static CSBorder *border = CSBorder::getBorder(BORDER_TYPE_MENU);
		border->setState(state);
		return border;
	}
//	if (classType.compare("Menu") == 0)
//	{
//		static CSBorder *border = CSBorder::getBorder(BORDER_TYPE_MENU);
//		border->setState(state);
//		return border;
//	}
	if (classType.compare(CSMenuBar::CLASS) == 0)
	{
		static CSBorder *border = CSBorder::getBorder(BORDER_TYPE_UP);
		border->setState(state);
		return border;
	}
	if (classType.compare("MenuWindow") == 0)
	{
		static CSBorder *border = CSBorder::getBorder(BORDER_TYPE_WINDOW);
		border->setState(state);
		return border;
	}

	if (classType.compare(CSMenuItem::CLASS) == 0)
	{
		static CSBorder *border = CSBorder::getBorder(BORDER_TYPE_MENU);
		border->setState(state);
		return border;
	}
	if (classType.compare(CSCheckBox::CLASS) == 0)
	{
		static CSBorder *border = CSBorder::getBorder(BORDER_TYPE_CHECK_BUTTON);
		border->setState(state);
		return border;
	}
	if (classType.compare(CSListbox::CLASS) == 0)
	{
		static CSBorder *border = CSBorder::getBorder(BORDER_TYPE_TEXTFIELD);
		border->setState(state);
		return border;
	}
	if (classType.compare(CSCombobox::CLASS) == 0)
	{
		static CSBorder *border = CSBorder::getBorder(BORDER_TYPE_TEXTFIELD);
		border->setState(state);
		return border;
	}
	if (classType.compare(CSTextfield::CLASS) == 0)
	{
//		static CSBorder *border = CSBorder::getBorder(BORDER_TYPE_TEXTFIELD);
//		border->setState(state);
		static CSRoundedBorder *border = CSRoundedBorder::getBorder(BORDER_TYPE_TEXTFIELD);
		border->setState(state);
		border->setCorner(2);
		return border;
	}
	if (classType.compare(CSTextarea::CLASS) == 0)
	{
		static CSRoundedBorder *border = CSRoundedBorder::getBorder(BORDER_TYPE_TEXTFIELD);
		border->setState(state);
		border->setCorner(2);
//		static CSBorder *border = CSBorder::getBorder(BORDER_TYPE_TEXTFIELD);
//		border->setState(state);
		return border;
	}
	if (classType.compare(CSHTMLPanel::CLASS) == 0)
	{
		static CSBorder *border = CSBorder::getBorder(BORDER_TYPE_NONE);
		border->setState(state);
		return border;
	}
	if (classType.compare(CSLabel::CLASS) == 0)
	{
		static CSBorder *border = CSBorder::getBorder(BORDER_TYPE_NONE);
		border->setState(state);
		return border;
	}
/*	
	if (classType.compare(CSHTMLArea::CLASS) == 0)
	{
		static CSBorder *border = CSBorder::getBorder(BORDER_TYPE_TEXTFIELD);
		border->setState(state);
		return border;
	}
*/
	return theBorder;
}
		
CSIcon *CSLAF::getIconInternal(int iconType)
{
	switch (iconType)
	{
		case LAF_ICON_TYPE_CLOSE:
			return CSPictureIcon::buildIcon(closeIconFileEnabled, closeIconFileDisabled);		
		case LAF_ICON_TYPE_WARN:
			return CSPictureIcon::buildIcon(warnIconFileEnabled, warnIconFileDisabled);
		case LAF_ICON_TYPE_ERROR:
			return CSPictureIcon::buildIcon(errorIconFileEnabled, errorIconFileDisabled);
		case LAF_ICON_TYPE_INFO:
			return CSPictureIcon::buildIcon(infoIconFileEnabled, infoIconFileDisabled);
		case LAF_ICON_TYPE_QUESTION:
			return CSPictureIcon::buildIcon(questionIconFileEnabled, questionIconFileDisabled);
		case LAF_ICON_TYPE_UP:
			return CSPictureIcon::buildIcon(upIconFileEnabled, upIconFileDisabled);
		case LAF_ICON_TYPE_LEFT:
			return CSPictureIcon::buildIcon(leftIconFileEnabled, leftIconFileDisabled);
		case LAF_ICON_TYPE_DOWN:
			return CSPictureIcon::buildIcon(downIconFileEnabled, downIconFileDisabled);
		case LAF_ICON_TYPE_RIGHT:
			return CSPictureIcon::buildIcon(rightIconFileEnabled, rightIconFileDisabled);
	}
	return 0;
}

void CSLAF::reset(void)
{
	static char *functionName="reset";
	LOG_ENTER 

	borderDefaultBrightestColor = 0xe0e0e0;
	windowBackColorEnabled = 0xc0c0c0;
	windowBackColorDisabled = 0xc0c0c0;
	windowTextColorEnabled = 0xffffff;
	windowTextColorDisabled = 0x101010;

	buttonBackColorEnabled = 0xc0c0c0;
	buttonBackColorDisabled = 0xc0c0c0;
	scrollbarBackColor = 0xd0d0d0;

	menuBarBackColorEnabled = 0xc0c0c0;
	menuBarBackColorDisabled = 0xc0c0c0;
	menuBarHorizontalSpacing = 2;
	menuBarVerticalSpacing = 0;

	menuItemHorzintalSpacing = 2;
	menuItemVerticalSpacing = 2;
	
	menuBackColorEnabled = 0xb0b0b0;
	menuBackColorDisabled = 0xb0b0b0;
	menuTextColorEnabled = 0x101010;
	menuTextColorDisabled = 0x101010;

	menuItemBackColorEnabled = 0xd0d0d0;
	menuItemBackColorDisabled = 0xd0d0d0;
	menuItemTextColorEnabled = 0x101010;
	menuItemTextColorDisabled = 0x101010;

	titleTextColorEnabled = 0xe0e0e0;
	titleTextColorDisabled = 0xe0e0e0;
	titleColorEnabled = 0x000080;
	titleColorDisabled = 0xc0c0c0;

	checkBoxColorEnabled = 0xffffff;
	checkBoxColorDisabled = 0xa0a0a0;
	
	listboxColorEnabled = 0xffffff;
	listboxColorDisabled = 0xa0a0a0;
	listboxTextColorEnabled = 0x101010;
	listboxTextColorDisabled = 0x101010;

	textfieldTextColorEnabled = 0x101010;
	textfieldTextColorDisabled = 0xffffff;
	textfieldColorDisabled = 0xa0a0a0;
	textfieldColorEnabled = 0xffffff;
	
	textAreaTextColorEnabled = 0x101010;
	textAreaTextColorDisabled = 0xffffff;
	textAreaColorEnabled = 0xffffff;
	textAreaColorDisabled = 0xa0a0a0;

	textAreaCursorColor = 0x101010;

	proportionalFontName = "font\\win_font.xml";
	monoSpacedFontName = "font\\defaultMonoFont.xml";

	comboboxItemsShown = 4;
	menuOpenXOffset = 0;
	menuOpenYOffset = 2;
	tooltipThreshold = 2000;

	windowTitleInterIconOffset = 0;
	windowTitleIconBorderVerticalOffset = 1;
	windowTitleIconBorderHorizontalOffset = 3;
	windowSizingOffset = 20;

	elementHorizontalSpacing = 2;
	elementVerticalSpacing = 2;
	textfieldHorizontalSpacing = 2;
	
	textfieldVerticalSpacing = 2;		
	labelHorizontalSpacing = 2;		
	labelVerticalSpacing = 2;
	buttonHorizontalSpacing = 2;	
	buttonVerticalSpacing = 2;
	textAreaVerticalLineSpacing = 2;
	textAreaHorizontalLineSpacing = 2;

	listboxVerticalLineSpacing = 4;	
	listboxHorizontalLineSpacing = 2;

	cursorRate = 500; // 500 milliseconds
	checkBoxSize = 12;

	closeIconFileEnabled="picture\\WindowClose.xml";
	closeIconFileDisabled="picture\\WindowClose.xml";

	warnIconFileEnabled="picture\\IconWarning.xml";
	warnIconFileDisabled="picture\\IconWarning.xml";

	errorIconFileEnabled="picture\\IconError.xml";
	errorIconFileDisabled="picture\\IconError.xml";

	infoIconFileEnabled="picture\\IconInfo.xml";
	infoIconFileDisabled="picture\\IconInfo.xml";

	questionIconFileEnabled="picture\\IconQuestion.xml";
	questionIconFileDisabled="picture\\IconQuestion.xml";

	upIconFileEnabled="picture\\IconArrowUpSmall.xml";
	upIconFileDisabled="picture\\IconArrowUpSmall.xml";

	downIconFileEnabled="picture\\IconArrowDownSmall.xml";
	downIconFileDisabled="picture\\IconArrowDownSmall.xml";

	leftIconFileEnabled="picture\\IconArrowLeftSmall.xml";
	leftIconFileDisabled="picture\\IconArrowLeftSmall.xml";

	rightIconFileEnabled="picture\\IconArrowRightSmall.xml";
	rightIconFileDisabled="picture\\IconArrowRightSmall.xml";
	
	mDefaultMouseCursor = "picture\\ArrowCursor.xml";
	mSizingMouseCursor = "picture\\SizingCursor.xml";

	LOG_EXIT
}

void CSLAF::destroyMouseCursors()
{
	if (mSizingMouseCursorInstance)
	{
		delete (mSizingMouseCursorInstance);
		mSizingMouseCursorInstance = 0;
	}
	if (mDefaultMouseCursorInstance)
	{
		delete (mDefaultMouseCursorInstance);
		mDefaultMouseCursorInstance = 0;
	}
}

