#ifndef CSLAF_h
#define CSLAF_h

// USES SDL_Surface
// USES SDL_Rect 


// TODO: Insets in LAF!

#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

#include <vector>
#include <string>
#include "CSLog.h"
#include "CSMouseCursor.h"

class CSGrafikElement;
typedef std::vector<CSGrafikElement *> CSGrafikElements;

#include <SDL.h>

const int LAF_ICON_TYPE_CLOSE = 1;
const int LAF_ICON_TYPE_WARN =	2;
const int LAF_ICON_TYPE_ERROR = 3;
const int LAF_ICON_TYPE_INFO =	4;
const int LAF_ICON_TYPE_QUESTION = 5;
const int LAF_ICON_TYPE_UP =	6;
const int LAF_ICON_TYPE_LEFT =	7;
const int LAF_ICON_TYPE_DOWN =	8;
const int LAF_ICON_TYPE_RIGHT = 9;

const int LAF_MOUSE_CURSOR_DEFAULT = 1;
const int LAF_MOUSE_CURSOR_SIZING_DOWN_RIGHT = 2;
const int LAF_MOUSE_CURSOR_SIZING_DOWN_LEFT = 3;
const int LAF_MOUSE_CURSOR_SIZING_UP_RIGHT = 4;
const int LAF_MOUSE_CURSOR_SIZING_UP_LEFT = 5;
const int LAF_MOUSE_CURSOR_SIZING_DOWN = 6;
const int LAF_MOUSE_CURSOR_SIZING_UP = 7;
const int LAF_MOUSE_CURSOR_SIZING_LEFT = 8;
const int LAF_MOUSE_CURSOR_SIZING_RIGHT = 9;

class CSFont;
class CSBorder;
class CSArea;
class CSIcon;
class CSLAF
{
	private:
		static CSGrafikElements mGUIElements;
		static int mCurrentLAFId;
		static CSLAF *mLAF;
		CSMouseCursor *mSizingMouseCursorInstance;
		CSMouseCursor *mDefaultMouseCursorInstance;

	protected:
		int cursorRate;
		int windowTitleInterIconOffset;
		int windowTitleIconBorderVerticalOffset;
		int windowTitleIconBorderHorizontalOffset;
		int windowSizingOffset;				//!< \todo to sizing allow sizing not just in lower right corner!

		int borderDefaultBrightestColor;

		int windowBackColorEnabled;
		int windowBackColorDisabled;
		int windowTextColorEnabled;
		int windowTextColorDisabled;
		int	buttonBackColorEnabled;
		int	buttonBackColorDisabled;
		int scrollbarBackColor;
		int menuBarBackColorEnabled;
		int menuBarBackColorDisabled;
		int menuBackColorEnabled;
		int menuBackColorDisabled;

		int menuItemBackColorEnabled;
		int menuItemBackColorDisabled;
		int menuItemTextColorEnabled;
		int menuItemTextColorDisabled;

		int titleTextColorEnabled;			//!< \todo Window (title) with disabled color
		int titleTextColorDisabled;
		int titleColorEnabled;
		int titleColorDisabled;

		int checkBoxColorEnabled;
		int checkBoxColorDisabled;
		int listboxColorEnabled;
		int listboxColorDisabled;
		int listboxTextColorEnabled;
		int listboxTextColorDisabled;
		int menuTextColorEnabled;
		int menuTextColorDisabled;

		int textfieldColorDisabled;
		int textfieldColorEnabled;
		int textfieldTextColorEnabled;		//!< \todo textField with disabled color
		int textfieldTextColorDisabled;

		int textAreaTextColorEnabled;
		int textAreaTextColorDisabled;		//!< \todo textArea with disabled color
		int textAreaColorEnabled;
		int textAreaColorDisabled;
		int textAreaCursorColor;
		
		int listboxVerticalLineSpacing;
		int listboxHorizontalLineSpacing;
		int textAreaVerticalLineSpacing;
		int textAreaHorizontalLineSpacing; //!<space between border
		int textfieldHorizontalSpacing;
		int textfieldVerticalSpacing;
		int labelHorizontalSpacing;
		int labelVerticalSpacing;
		int buttonHorizontalSpacing;
		int buttonVerticalSpacing;
		int menuBarHorizontalSpacing;
		int menuBarVerticalSpacing;
		int menuItemHorzintalSpacing;
		int menuItemVerticalSpacing;
		int elementHorizontalSpacing;	//!< in Window (e.g.) if elements are arranged automatically
										//!< these is the space between two elements
		int elementVerticalSpacing;		//!< in Window (e.g.) if elements are arranged automatically
										//!< these is the space between two elements

		int comboboxItemsShown;
		int checkBoxSize;
		
		int menuOpenXOffset;
		int menuOpenYOffset;

		void destroyMouseCursors();

		int getBackgroundColorEnabledInternal(const std::string &classType);
		int getBackgroundColorDisabledInternal(const std::string &classType);

		int getTextColorEnabledInternal(const std::string &classType);
		int getTextColorDisabledInternal(const std::string &classType);

		int getHorizontalElementSpacingInternal(const std::string &classType);
		int getVerticalElementSpacingInternal(const std::string &classType);
		
		CSArea *getAreaInternal(const std::string &classType, int state = 0);
		CSFont *getFontInternal(const std::string &classType);
		CSBorder *getBorderInternal(const std::string &classType, int state);
		CSIcon *getIconInternal(int iconType);
		CSMouseCursor *getMouseCursorInternal(int type);

	public:
		static const char *CLASS;

		int tooltipThreshold;

		//! Mouse Cursor changes are only recognized with
		//! a lafChanged() call!
		std::string mDefaultMouseCursor;
		std::string mSizingMouseCursor;

		std::string proportionalFontName;
		std::string monoSpacedFontName;

		std::string closeIconFileEnabled;
		std::string closeIconFileDisabled;
		std::string warnIconFileEnabled;
		std::string warnIconFileDisabled;
		std::string errorIconFileEnabled;
		std::string errorIconFileDisabled;
		std::string infoIconFileEnabled;
		std::string infoIconFileDisabled;
		std::string questionIconFileEnabled;
		std::string questionIconFileDisabled;
		std::string upIconFileEnabled;
		std::string upIconFileDisabled;
		std::string downIconFileEnabled;
		std::string downIconFileDisabled;
		std::string leftIconFileEnabled;
		std::string leftIconFileDisabled;
		std::string rightIconFileEnabled;
		std::string rightIconFileDisabled;


		static void addElement(CSGrafikElement *element);
		static void removeElement(CSGrafikElement *element);
		static void lafChanged();
		static void setCurrentLAF(CSLAF &laf);
		static CSLAF *getCurrentLAF();

		CSLAF() ;
		virtual ~CSLAF();


		virtual int getBackgroundColorEnabled(const std::string &classType) {return getBackgroundColorEnabledInternal(classType);}
		virtual int getBackgroundColorDisabled(const std::string &classType) {return getBackgroundColorDisabledInternal(classType);}

		virtual int getTextColorEnabled(const std::string &classType){return getTextColorEnabledInternal(classType);}
		virtual int getTextColorDisabled(const std::string &classType){return getTextColorDisabledInternal(classType);}

		virtual int getHorizontalElementSpacing(const std::string &classType) {return getHorizontalElementSpacingInternal(classType);}
		virtual int getVerticalElementSpacing(const std::string &classType) {return getVerticalElementSpacingInternal(classType);}
		
		virtual int getBorderDefaultBrightestColor() {return borderDefaultBrightestColor;}
		virtual int getComboboxItemsShown() {return comboboxItemsShown;}
		virtual int getMenuOpenXOffset() {return menuOpenXOffset;}
		virtual int getMenuOpenYOffset() {return menuOpenYOffset;}
		virtual int getWindowSizingOffset() {return windowSizingOffset;}				//!< \todo to sizing allow sizing not just in lower right corner!
		
		//! state for buttons!
		//! otherwise ignored
		virtual CSBorder *getBorder(const std::string &classType, int state = 0) {return getBorderInternal(classType, state);}
		virtual CSArea *getArea(const std::string &classType, int state = 0) {return getAreaInternal(classType, state);}
		virtual CSIcon *getIcon(int iconType) {return getIconInternal(iconType);}
		
		CSMouseCursor *getMouseCursor(int type) {return getMouseCursorInternal(type);}
		CSFont *getFont(std::string classType){return getFontInternal(classType);}
		virtual int getTextAreaCursorColor() {return textAreaCursorColor;}

		virtual void reset(void);
		virtual std::string getType() {return (std::string) CLASS;}
		virtual int getCursorRate() {return cursorRate;}; // 500 milliseconds
		virtual int getCheckBoxSize() {return checkBoxSize;}

};

#endif CSLAF_h