/*! \page cslib1 Concept of CSLib 
  \section cslibutilis1 Concepts & Style

  \subsection cslibutilisdebug1 Logging
  <UL>
	  <LI> 
		Log
	  </LI> 
  </UL>

  \subsection cslibutilisloading1 Loading globally!
  <UL>
	  <LI> 
		Loadable
	  </LI> 
  </UL>

  \subsection cslibutilisxml1 XML & XPath
  <UL>
	  <LI> 
		XML
	  </LI> 
	  <LI> 
		XPATH
	  </LI> 
  </UL>

  \subsection cslibutilismessage1 Messaging
  <UL>
	  <LI> 
		Log
	  </LI> 
  </UL>

  \section cslibgame1 CSGame
  \subsection cslibgamegraphic1 Graphics and Animations
  <UL>
	  <LI> 
		Sprites
	  </LI> 
	  <LI> 
		Actions
	  </LI> 
	  <LI> 
		Animations
	  </LI> 
	  <LI> 
		Pictures
	  </LI> 
  </UL>

  \subsection cslibgamesound1 Music & Samples
  <UL>
	  <LI> 
		Music
	  </LI> 
	  <LI> 
		Samples
	  </LI> 
  </UL>

  \subsection cslibgameworld1 World & Tile
  <UL>
	  <LI> 
		World
	  </LI> 
	  <LI> 
		Tilemap
	  </LI> 
	  <LI> 
		Tileset
	  </LI> 
	  <LI> 
		Tile
	  </LI> 
  </UL>

  \subsection cslibgametext1 Fonts and printing
  <UL>
	  <LI> 
		Font
	  </LI> 
  </UL>

  \section cslibgui1 GUI
  \subsection cslibguielement3 GraphicElement
  <UL>
	  <LI> 
		GraphicElement
	  </LI> 
	  <LI> 
		Desktop
	  </LI> 
	  <LI> 
		other elements
	  </LI> 
	  <LI> 
		Layout (adding, removing, autosizing)
	  </LI> 
  </UL>
  \subsection cslibguielement2 LAF
  \subsection cslibguielement1 Messages

*/


/*! \page gui1 GUI and screen updates
	The GUI wihtin this library assumes, that befor each "update round" the complete
	area ist blanked.

    For an example how the gui should be updated see the folloing code (from \a CSGame):
	<PRE>
	while (!mQuit)
	{
		do
		{
			handleEvents();
			if (mPause)
			{
				SDL_Delay(50);
			}
		}
		while ((mPause) && (!mQuit));

		if (mIsGui)
		{
			SDL_Rect area;
			area.x = 0;
			area.y = 0;
			area.w = SDLMain::getScreenWidth();
			area.h = SDLMain::getScreenHeight();
			mDesktop->paint(SDLMain::getScreen(), 0);
			SDLMain::updateScreen();
			SDL_FillRect(SDLMain::getScreen(), &area, COLOR(SDLMain::getScreen(), 0));
			SDLMain::addUpdateRect(area);
			mDiffTimeGUI = SDL_GetTicks() - desktopDrawStart;
		}
		// .. draw non gui stuff at the background of the gui e.g. the game
	}
	</PRE>
	That also means that each and every gui component is drawn at each update!
	There are some (good ones, I think) reasons for this:
	\arg Computers are now easliy able to do it, they are fast enough!
	\arg It saves all the trouble one has in keeping in mind which things to update,
	     which backgrounds to save, redrawing stuff under the mouse-cursor etc...
	\arg <i>This library was intended this way.</i> It was started of as a means to
	     create a user frontend for "in game" communication. That also implies the
		 possiblity for the game to continue while changing stuff with the gui.
		 The gui itself does not <b>know</B> anything about any game! So it wouldn't
		 know which areas to update! <b> And </b> the game doesn't know about the gui
		 either!
	\arg Again - it makes the whole screen updating very simple!
		 (Believe me! I once wrote a gui library, that only updated areas of the screen
		  where changes occured, including keeping track of game information, it was 
		  an updating horror, for people interested, look at the gui of the DOS Vectrex
		  emulator, that's the gui I'm talking about. But at the time I wrote that gui,
		  it was neccessary, computers were not fast enough (or I was a bad programmer :-)))
*/

/*! \page gui2 Concept of the CSLib GUI
	The GUI of CSLib is based one single class <B>CSGrafikElement</B>.
	This class is the base class of all other elements, widgets, windows, desktop and other entities.
	The base class keeps track of position, sizing and the like.
	Also it provides mechanisms for painting and string output and LAF support and children (other CSGrafikElement elements).
	For a complete list of features and supporting methods see \a CSGrafikElement.

	\section structuregraphicelement  GraphicElement Structure
	Each CSGraphicElement constists of:
	\arg an area (CSArea)
	\arg and its children (or direct paint directives)
	\arg a border (CSBorder)

	During each painting of a GraphicElement these are drawn on screen in the above order.
	Usually the area and the border are determined by the currently set LAF. But one
	can set elements their own special version, the own area's and border's must fullfill
	the interface defined in the classes \a CSBorder and \a CSArea.<BR>

	\section structuregraphicelementborder Borders
	As of now their are two different types of borders.
	\arg CSBorder (a normal rectangular border with shades)
	\arg CSRoundedBorder (as above but with rounded corners)

	One can define own borders, which (as a thought) include animations, changing colors
	or the like.

	\section structuregraphicelementarea Areas
	As of now their are several different types of areas.
	\arg CSArea (a normal rectangular area with)
	\arg CSRoundedArea (as above but with rounded corners)
	\arg CSGradientArea (a rectangular area, with two differnt colors, 
	     which gradually change from one to the other), horizontally or vertically
	\arg CSPictureArea, which takes a picture as the "background" of an element, stretched, or centered
	
	One can define own areas, which (as a thought) include animations, changing colors
	or the like.

	\section structuregraphicelementchildren Children
	Each \a GraphicElement can take a(ny) number of GraphicElements as children. Children
	can be arranged via a CSLayoutManager. 
	\arg CSLayoutManagerXY
	\arg CSLayoutManagerBorder (north, south, east, west, center)
	\arg CSLayoutManagerStack (horizontally, and vertically)

	elements added to another element can have different options:
	\arg autosizing (horizontally, vertically, both)
	\arg viewported 
	\arg centered
	\arg space (between itself and other elements)

    \section structuregraphicelementmessage Messages
	If something happens -> a message is passed to everyone who is interested.
	A listener can be added to each element. The message that is sent will be of
	the type \a CSMessage in special \a GuiMessage.<BR>
	
    \section structuregraphicelementmessageinternal Internal Messages
	The receiver of Gui-driven events (mouse movement, clicks, etc) will allways be
	the element that is furthest down the element hierachie.<BR>
	e.g.<BR>
	The desktop has a window, the window has a panel, which has a button. 
	If the mouse is hovering above the button and the user clicks the (mouse) button.
	The CSButton, which is part of the panel, which is part of the window... receives 
	a \a MOUSE_BUTTON_PRESSED_MESSAGE - message. The innermost element is responsible for
	the message. If it not interested in it, it should pass the message on to
	it's parent - element (one up in the hierachy)!




*/

/*! \page gui3 Viewported versus not Viewported
	Every grafical element has a size, internally they are called <TT>mWidth</TT> and <TT>mHeight</TT>.
	
	E.g. a window constructed with <TT>new CSWindow(200, 400)</TT> has a height of
	200 and a width of 400. This is space the window occupies on the screen
	(if not out of bounds).
	<P>
	(The following text refers to windows only, since windows are easy to understand,
	 but the viewport concept holds true for <b>ALL</B> CSGraphikElements.)</P>

	The maximum size of the viewport of the window is at this moment also 200 x 400
	(that doesn't imply, that all elements added to the window are size restricted to 
	 be smaller than 200 x 400 -> but that later).
	
	The viewport of the window is the hole, thru that a spectator can watch the action
	that takes place behind the window.
	A real life example, take a window: a <b> REAL </B> window - out of glass.
	What makes up the window? Say you see it from two meters distant.
	\arg the glass, thru that you can see the kids playing in the garden
	\arg the window frame (plastic, or wood?)
	\arg a handle to open/close the window

	This example has everything in it a computer window needs. Here the translation:
	\arg glass <-> viewport 
	\arg frame <-> borders and titlebar
	\arg handle <-> buttons and icons for sizing, menu

	The viewport is the part of the window "thru" which you can see elements of the 
	window.
	With this example you see that there are defenitly elements of a window, that
	don't reside in the viewport.
	Elements that usually <B>are not</B> viewported:
	\arg titlebar 
	\arg menubar
	\arg borders
	\arg closebuttons 
	\arg scrollbars 
	\arg etc.

	Elements that usually <B>are</B> viewported:
	\arg textfields 
	\arg buttons (ordinary)
	\arg textarea
	\arg html-output 
	\arg combobox
	\arg etc.

	The viewport is (nearly all the time) smaller than the window size.
	Internally the size is kept in <TT>mViewportHeight</TT> and <TT>mViewportHeight</TT>.
	The viewport has an offset to the position of the window start 
	(kept in <TT>mViewportX</TT> and <TT>mViewportY</TT>).

	Non viewported elements are usually autosizing (menubars stretch all the way, even
	if the size of the window changes). If they are not autosizing, it is assumed they
	don't influence the viewport size! -> \sa CSGrafikElement::buildArea()
	
	  The size of the viewport is changed, when adding non viewported elements (since the size
	of the window does not change).

    When adding a viewported element, the position of the element is in relation to the
	viewport start. That means you cannot position a viewported element on the window
	on a position that is not taken up by the viewport. The window thinks it
	belongs to the viewport and is actually displaying only parts of it if positioned out of bounds
	(like -> you don't see the whole sky if you look out of the real window, only part of it).

    When adding elements to a window that are not viewported, they are in relation 
	to the actual start position of the window.

	All elements can either be viewported or non viewported. <B> Befor</B> they are
	added to another element, they must be set to either viewported or non viewported, using:
	<PRE>
			void setViewported(bool viewported) 
	</PRE>
	All elements have sensible default values!
*/


/*! \page gui4 Structure of Grafikal Elements
	
*/





