#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

#include <time.h>
#include "CSLog.h"
using namespace std;

LogMap CSLog::mMap;
int CSLog::mInit = 0;

std::string CSLog::mDebugLevelString[]={
			((std::string) "Message"),
			((std::string) "Debug"),
			((std::string) "Info"),
			((std::string) "Warning"),
			((std::string) "Error"),
			((std::string) "None")
		};

CSLog::CSLog(const std::string &identifier)
 : mIdentifier(identifier)
{
	mLogFilename = mIdentifier + ".log";
	mTrackClassName = "";
	mTrackMethodName = "";
	for (int i=0; i < MAX_METHOD_DEPTH; i++)
	{
		mCurrentMethod[i] = "";
		mCurrentClass[i] = "";
	}
	mLogLevel = LOG_LEVEL_ERROR;
	mDebugType = LOG_TYPE_HIGHER_LEVEL;
	mFileState = false;
	mDebugFile = 0;
	mFunctionDepth = 0;
	mFunctionTracking = false;
	mShortMessage = false;
	printInternal(-1, "--- LOG STARTING ---");
}

CSLog::~CSLog()
{
	closeFile();
}

bool CSLog::openFile()
{
	closeFile();
	mDebugFile = fopen(mLogFilename.c_str(), "a");
	if (mDebugFile == 0)
	{
		return false;
	}
	
	if (setvbuf(mDebugFile, NULL, _IONBF, 0 ) != 0 )
	{
		// error
        // printf( "Incorrect type or size of buffer for mDebugFile\n" );
	}
 
	mFileState = true;
	return true;
}

bool CSLog::closeFile()
{
	if (mDebugFile == 0)
	{
		mFileState = false;
		return true;
	}
	fflush(mDebugFile);
	fclose(mDebugFile);
	
	mFileState = false;
	mDebugFile = 0;
	return true;
}

bool CSLog::printInternal(int level, const std::string &text)
{
	if ((level < mLogLevel) && (level != -1))
	{
		return true;
	}

	if ((mTrackClassName.length() != 0) && (level != -1))
	{
		int result = mTrackClassName.compare(0, mTrackClassName.length(), mCurrentClass[mFunctionDepth], 0, mTrackClassName.length());
		if ( result != 0)
		{
			return true;
		}
	}

	if ((mTrackMethodName.length() != 0) && (level != -1))
	{
		int result = mTrackMethodName.compare(0, mTrackMethodName.length(), mCurrentMethod[mFunctionDepth], 0, mTrackMethodName.length());
		if (result != 0)
		{
			return true;
		}
	}
	if (level > LOG_LEVEL_NONE)
	{
		level =  LOG_LEVEL_NONE;
	}
	
	std::string out;
	if (!mShortMessage)
	{
		static char depth[MAX_METHOD_DEPTH];
		int i;
		for (i=0; ((i<mFunctionDepth) && (i < MAX_METHOD_DEPTH-1)); i++)
		{
			depth[i] = ' ';
		}
		depth[i]=0;

		struct tm *newtime;
		static char ctime[80];
		time_t aclock;
		time( &aclock );                 // Get time in seconds 
		newtime = localtime( &aclock );  // Convert time to struct 
										 // tm form 

		// Print local time as a string 
		sprintf(ctime, "%s", asctime(newtime));
		//	free(newtime); // must not be freed!
		ctime[strlen(ctime) - 1] = 0;
		out = depth + ((std::string)"---") + "\n";
		out = out + depth + "|" + ((std::string)ctime);
		
		if (mCurrentClass[mFunctionDepth].length() != 0)
		{
			out = out + " -> " + mCurrentClass[mFunctionDepth] + "::";
		}
		else
		{
			;//out = out + depth + "UnkownClass::";
		}
		if (mCurrentMethod[mFunctionDepth].length() != 0)
		{
			if (mCurrentClass[mFunctionDepth].length() == 0)
			{
				out = out + " -> UnkownClass::";
			}
			out = out + mCurrentMethod[mFunctionDepth] + "()\n";
		}
		else
		{
			;//out = out + "UnkownMethod()\n";
			out = out + "\n";
		}


		out = out + depth  + "|" + mDebugLevelString[level+1]+ ": "+text + "\n";
		out = out + depth + "---" + "\n";
	}
	else
	{
////////////////////////////
		if (mCurrentClass[mFunctionDepth].length() != 0)
		{
			out = mCurrentClass[mFunctionDepth] + "::";
		}
		if (mCurrentMethod[mFunctionDepth].length() != 0)
		{
			out = out + mCurrentMethod[mFunctionDepth] + "()";
		}
		out = out + ": "+text + "\n";
////////////////////////////
	}

	bool fileError = openFile();
	if (fileError)
	{
		fprintf(mDebugFile, out.c_str());
		closeFile();
	}
	fprintf(stderr, out.c_str());

	CSLogCallbacks::iterator iter = mCallbacks.begin();
	while (iter != mCallbacks.end())
	{
		CSLogCallback *callback = *iter;
		callback->printLog(text);
		iter++;
	}
	return fileError;
}

CSLog *CSLog::getInstance(const std::string &identifier)
{
	CSLog *instance = 0;
	try
	{
		// MS STL and statics...
		if ((mInit == 1) && (mMap.size() == 0)) return 0;
		LogMap::iterator iter = mMap.find(identifier);
		if (iter != mMap.end())
		{
			instance = iter->second;
		}
		else
		{
			mInit = 1;
			instance = new CSLog(identifier);
			mMap.insert(LogMap::value_type(identifier, instance));
		}
	}
	catch (...)
	{
	}
	return instance;
}

void CSLog::deleteInstance(const std::string &identifier)
{
	CSLog *instance = 0;
	LogMap::iterator iter = mMap.find(identifier);
	if (iter != mMap.end())
	{
		instance = iter->second;
		delete(instance);
		mMap.erase(iter);
	}
}

void CSLog::print(int level, const std::string &text)
{
	printInternal(level, text);
}

void CSLog::print(int level, const std::string &text, long value)
{
	std::string newText;
	static char valueText[80];
	sprintf(valueText, "%i", value);
	newText = text + valueText;
	print(level, newText);
}

void CSLog::print(int level, const std::string &text, int value)
{
	std::string newText;
	static char valueText[80];
	sprintf(valueText, "%i", value);
	newText = text + valueText;
	print(level, newText);
}

void CSLog::print(int level, const std::string &text, float value)
{
	std::string newText;
	static char valueText[80];
	sprintf(valueText, "%f", value);
	newText = text + valueText;
	print(level, newText);
}

void CSLog::startFunction(const std::string &methodName, const std::string &className)
{
	if (mFunctionDepth<MAX_METHOD_DEPTH-1) 
		mFunctionDepth++;
	mCurrentMethod[mFunctionDepth] = methodName;
	mCurrentClass[mFunctionDepth] = className;
	if (mFunctionTracking)
	{
#ifdef SDL_TIMING
		mCurrentTime[mFunctionDepth] = SDL_GetTicks();
#endif
		if (!mShortMessage)
		{
			printInternal(LOG_LEVEL_DEBUG, ((std::string) "Entering..."));
		}
	}
}

void CSLog::endFunction()
{
	if (!mShortMessage)
	{
		if (mFunctionTracking)
		{
#ifdef SDL_TIMING
			long timeBetween = SDL_GetTicks() - mCurrentTime[mFunctionDepth];
			mCurrentTime[mFunctionDepth] = 0;
			std::string newText;
			static char valueText[80];
			sprintf(valueText, "Time remaining in Method (ms): %i", timeBetween);
			printInternal(LOG_LEVEL_DEBUG, ((std::string) "Exiting... " + valueText));
#else
			printInternal(LOG_LEVEL_DEBUG, ((std::string) "Exiting..."));
#endif
		}
	}
	mCurrentMethod[mFunctionDepth] = "";
	mCurrentClass[mFunctionDepth] = "";
	mFunctionDepth--;
	if (mFunctionDepth < 0) 
	{
		mFunctionDepth = 0;
	}
}

void CSLog::setDebugType(int type)
{
	mDebugType = type;
}

int CSLog::getDebugType()
{
	return mDebugType;
}

void CSLog::setDebugLevel(int level)
{
	mLogLevel = level;
}

int CSLog::getDebugLevel()
{
	return mLogLevel;
}

void CSLog::addCallback(CSLogCallback *callback)
{
	removeCallback(callback);
	mCallbacks.push_back(callback);
}

void CSLog::removeCallback(CSLogCallback *callback)
{
	CSLogCallbacks::iterator iter = mCallbacks.begin();
	while ( iter != mCallbacks.end())
	{
		if ((*iter) == callback)
		{
			mCallbacks.erase(iter);
			iter = mCallbacks.begin();	// restart!
		}
		else
		{
			iter++;
		}
	}
}
