#ifndef CS_LOG_H
#define CS_LOG_H



// Debugging and logging enabled at all?
// #define LOG_ENABLE 1


// USES SDL_Timing ?
//#define SDL_TIMING 1
#define LOG_GLOBAL_NAME "CSLIB"

#include <stdio.h>
#include <vector>
#include <string>
#include <map>

#ifdef SDL_TIMING
 #include <SDL.h>
#endif

class CSLog;
class CSLogCallback;

typedef std::map<std::string, CSLog *> LogMap;
typedef std::vector<CSLogCallback *> CSLogCallbacks;


const int MAX_METHOD_DEPTH = 80;

const int LOG_TYPE_HIGHER_LEVEL = 0;
const int LOG_TYPE_UNIQUE_LEVEL = 1;

const int LOG_LEVEL_DEBUG = 0;
const int LOG_LEVEL_INFO = 1;
const int LOG_LEVEL_WARNING = 2;
const int LOG_LEVEL_ERROR = 3;
const int LOG_LEVEL_NONE = 4;

#ifdef LOG_ENABLE

#ifdef SDL_TIMING
#define NLOG_EXACT_TIMING(logname,b) \
{ \
	CSLog *localLog = CSLog::getInstance(logname);\
	if (localLog) localLog->setExactTiming((b)); \
} 
#endif

#define NLOG_TRACK_METHOD(logname,methodname) \
{ \
	CSLog *localLog = CSLog::getInstance(logname);\
	if (localLog)localLog->setOnlyTrackMethod((methodname)); \
} 

#define NLOG_TRACK_CLASS(logname,classname) \
{ \
	CSLog *localLog = CSLog::getInstance(logname);\
	if (localLog) localLog->setOnlyTrackClass((classname)); \
} 

#define NLOG_SHORT_MESSAGE(logname,b) \
{ \
	CSLog *localLog = CSLog::getInstance(logname);\
	if (localLog) localLog->setShortMessage((b)); \
} 

#define NLOG_FUNCTION_TRACKING(logname,b) \
{ \
	CSLog *localLog = CSLog::getInstance(logname);\
	if (localLog) localLog->setFunctionTracking((b)); \
} 

#define NLOG_AMESSAGE(logname,text) \
{ \
	CSLog *localLog = CSLog::getInstance(logname);\
	if (localLog) localLog->print(-1, (text)); \
} 

#define NLOG_APRINT(logname,text,value) \
{ \
	CSLog *localLog = CSLog::getInstance(logname);\
	if (localLog) localLog->print(-1, (text), (value)); \
} 

#define NLOG_ENTER(logname) \
{ \
	CSLog *localLog = CSLog::getInstance(logname); \
	if (localLog) localLog->startFunction(functionName, CLASS); \
} 

#define NLOG_EXIT(logname) \
{ \
	CSLog *localLog = CSLog::getInstance(logname); \
	if (localLog) localLog->endFunction(); \
} 

#define NLOG_MESSAGE(logname,level,text) \
{ \
	CSLog *localLog = CSLog::getInstance(logname);\
	if (localLog) localLog->print((level), (text)); \
} 

#define NLOG_PRINT(logname,level,text,value) \
{ \
	CSLog *localLog = CSLog::getInstance(logname);\
	if (localLog) localLog->print((level), (text), (value)); \
} 

#define NLOG_SET_LEVEL(logname,level) \
{ \
	CSLog *localLog = CSLog::getInstance(logname);\
	if (localLog) localLog->setDebugLevel((level)); \
} 

#define NLOG_SET_TYPE(logname,type) \
{ \
	CSLog *localLog = CSLog::getInstance(logname);\
	if (localLog) localLog->setDebugType((level)); \
} 

#else
#ifdef SDL_TIMING
#define NLOG_EXACT_TIMING(logname,b)
#endif
#define NLOG_TRACK_METHOD(logname,methodname) 
#define NLOG_TRACK_CLASS(logname,classname) 
#define NLOG_FUNCTION_TRACKING(logname,b) 
#define NLOG_SHORT_MESSAGE(logname,b) 
#define NLOG_AMESSAGE(logname,text) 
#define NLOG_APRINT(logname,text,value) 
#define NLOG_ENTER(logname) 
#define NLOG_EXIT(logname) 
#define NLOG_MESSAGE(logname,level,text) 
#define NLOG_PRINT(logname,level,text,value) 
#define NLOG_SET_LEVEL(logname,level) 
#define NLOG_SET_TYPE(logname,type) 
#endif

#define NLOG_DEBUG_MESSAGE(logname,text) NLOG_MESSAGE((logname),(LOG_LEVEL_DEBUG),(text))
#define NLOG_DEBUG_PRINT(logname,text,value) NLOG_PRINT((logname),(LOG_LEVEL_DEBUG),(text),(value))
#define NLOG_INFO_MESSAGE(logname,text) NLOG_MESSAGE((logname),(LOG_LEVEL_INFO),(text))
#define NLOG_INFO_PRINT(logname),text,value) NLOG_PRINT((logname),(LOG_LEVEL_INFO),(text),(value))
#define NLOG_WARN_MESSAGE(logname,text) NLOG_MESSAGE((logname),(LOG_LEVEL_WARNING),(text))
#define NLOG_WARN_PRINT(logname,text,value) NLOG_PRINT((logname),(LOG_LEVEL_WARNING),(text),(value))
#define NLOG_ERROR_MESSAGE(logname,text) NLOG_MESSAGE((logname),(LOG_LEVEL_ERROR),(text))
#define NLOG_ERROR_PRINT(logname),text,value) NLOG_PRINT((logname),(LOG_LEVEL_ERROR),(text),(value))

#ifdef SDL_TIMING
#define LOG_EXACT_TIMING(b) NLOG_EXACT_TIMING(LOG_GLOBAL_NAME,b)
#else
#define LOG_EXACT_TIMING(b)
#endif
#define LOG_ENTER NLOG_ENTER(LOG_GLOBAL_NAME)
#define LOG_EXIT NLOG_EXIT(LOG_GLOBAL_NAME)
#define LOG_MESSAGE(level,text) NLOG_MESSAGE((LOG_GLOBAL_NAME),(level),(text))
#define LOG_PRINT(level,text,value) NLOG_PRINT((LOG_GLOBAL_NAME),(level),(text),(value))
#define LOG_AMESSAGE(text) NLOG_AMESSAGE((LOG_GLOBAL_NAME),(text))
#define LOG_APRINT(text,value) NLOG_APRINT((LOG_GLOBAL_NAME),(text),(value))
#define LOG_DEBUG_MESSAGE(text) NLOG_MESSAGE((LOG_GLOBAL_NAME),(LOG_LEVEL_DEBUG),(text))
#define LOG_DEBUG_PRINT(text,value) NLOG_PRINT((LOG_GLOBAL_NAME),(LOG_LEVEL_DEBUG),(text),(value))
#define LOG_INFO_MESSAGE(text) NLOG_MESSAGE((LOG_GLOBAL_NAME),(LOG_LEVEL_INFO),(text))
#define LOG_INFO_PRINT(text,value) NLOG_PRINT((LOG_GLOBAL_NAME),(LOG_LEVEL_INFO),(text),(value))
#define LOG_WARN_MESSAGE(text) NLOG_MESSAGE((LOG_GLOBAL_NAME),(LOG_LEVEL_WARNING),(text))
#define LOG_WARN_PRINT(text,value) NLOG_PRINT((LOG_GLOBAL_NAME),(LOG_LEVEL_WARNING),(text),(value))
#define LOG_ERROR_MESSAGE(text) NLOG_MESSAGE((LOG_GLOBAL_NAME),(LOG_LEVEL_ERROR),(text))
#define LOG_ERROR_PRINT(text,value) NLOG_PRINT((LOG_GLOBAL_NAME),(LOG_LEVEL_ERROR),(text),(value))
#define LOG_SET_LEVEL(level) NLOG_SET_LEVEL((LOG_GLOBAL_NAME),level)
#define LOG_SET_TYPE(type) NLOG_SET_TYPE((LOG_GLOBAL_NAME),type)
#define LOG_TRACK_METHOD(methodname) NLOG_TRACK_METHOD((LOG_GLOBAL_NAME),methodname) 
#define LOG_TRACK_CLASS(classname) NLOG_TRACK_CLASS((LOG_GLOBAL_NAME),classname) 
#define LOG_FUNCTION_TRACKING(b) NLOG_FUNCTION_TRACKING((LOG_GLOBAL_NAME),b) 
#define LOG_SHORT_MESSAGE(b) NLOG_SHORT_MESSAGE((LOG_GLOBAL_NAME),b) 

class CSLogCallback
{
	public:
		virtual void printLog(const std::string &text) = 0;
};

class CSLog
{
	private:
		static LogMap mMap;
		static int mInit;

		static std::string mDebugLevelString[];

		CSLogCallbacks mCallbacks;
		const std::string mIdentifier;
		std::string mLogFilename;
		std::string mCurrentMethod[MAX_METHOD_DEPTH];
		std::string mCurrentClass[MAX_METHOD_DEPTH];
#ifdef SDL_TIMING
		long mCurrentTime[MAX_METHOD_DEPTH];
		bool mExactTiming;
#endif
		bool mFileState; //!< false = closed, true = open 
		bool mFunctionTracking;
		bool mShortMessage;
		int mLogLevel;
		int mDebugType;
		int mFunctionDepth;
		FILE *mDebugFile;
		std::string mTrackClassName;
		std::string mTrackMethodName;

		CSLog(const std::string &identifier);
		bool openFile();
		bool closeFile();
		bool printInternal(int level, const std::string &text);
		virtual ~CSLog();
	protected:
	public:
		static CSLog *getInstance(const std::string &identifier);
		static void deleteInstance(const std::string &identifier);
		void print(int level, const std::string &text);
		void print(int level, const std::string &text, int value);
		void print(int level, const std::string &text, long value);
		void print(int level, const std::string &text, float value);
		void startFunction(const std::string &methodName, const std::string &className);
		void endFunction();
		void setDebugType(int type);
		int getDebugType();
		void setDebugLevel(int level);
		int getDebugLevel();
		void addCallback(CSLogCallback *callback);
		void removeCallback(CSLogCallback *callback);
		void setFunctionTracking(bool b) {mFunctionTracking = b;} //!< Debug Level
		void setShortMessage(bool b) {mShortMessage = b;} 

		/** Only up to characters of name givven are compared, that means name - "r" 
		 * enables to track all classes beginning with "r"!
		 */
		void setOnlyTrackClass(const std::string &className) {mTrackClassName = className;}
		void setOnlyTrackMethod(const std::string &methodName) {mTrackMethodName = methodName;}
#ifdef SDL_TIMING
		void setExactTiming(bool b) {mExactTiming = b;}
#endif
};
#endif // CS_LOG_H

