#ifndef CS_MESSAGE_DISPATCHABLE_H
#define CS_MESSAGE_DISPATCHABLE_H
 
#include <vector>
#include <string>
#include "CSLog.h"
#include "CSObject.h"

class CSMessageListener;
class CSMessage;
class CSMessageDispatchable;
class ListenerInterested;

typedef std::vector<CSMessageListener*> CSMessageListeners;
typedef std::vector<ListenerInterested *> ListenerInteresteds;

class ListenerInterested
{
	friend CSMessageDispatchable;
	private:
		CSMessageListener *mListener;
		int mInterestedIn;
		int mLastSentMessageSequence;
	public:
		ListenerInterested(CSMessageListener *l, int i = -1)
		{
			mListener = l;
			mInterestedIn = i;
			mLastSentMessageSequence = 0;
		}
		std::string getType();
};

typedef std::vector<CSMessageListener*> CSMessageListeners;

/**	
	This is the base class for all classes that wish to dispatch messages
	to other classes.<BR>

	Just inherit from this class and everything should be fine.
	There are only three self explanatory methods:<BR>
	<PRE>
		// send a message according to its type and priority - now, or later 
		void sendMessage(CSMessage &message);

		// can't add same listener twice! 
		void addMessageListener(CSMessageListener* listener);
		
		// return # of listeners removed - should allways be 0 or 1 
		int removeMessageListener(CSMessageListener* listener);

	</PRE>
	A message is allways sent to ALL listeners!

	*/
class CSMessageDispatchable : public CSObject
{
	private:
		ListenerInteresteds mListeners;
		bool mActive;
	public:
		CSMessageDispatchable() 
		{
			mActive = false;
		}
		virtual ~CSMessageDispatchable() 
		{
			if (mActive)
			{
				printf("\n!!!! DELETING CSMessageDispatchable while processing an event!\n");
				exit(1);
			}
		}
		static const char *CLASS;
		virtual std::string getType() {return (std::string) CLASS;}
		
		/** send a message according to its type and priority - now, or later */
		void sendMessage(CSMessage &message);

		/** can't add same listener twice! */
		void addMessageListener(CSMessageListener* listener, int typeOfMessage = -1);

		/** return # of listeners removed - should allways be 0 or 1 */
		int removeMessageListener(CSMessageListener* listener);

		/** return # of listeners removed - should allways be 0 or 1 */
		int removeMessageListener(CSMessageListener* listener, int typeOfMessage);
};

#endif // CS_MESSAGE_DISPATCHABLE_H
