#ifndef CSMESSAGE_LISTENER_H
#define CSMESSAGE_LISTENER_H

#include <vector>
#include "CSObject.h"
#include "CSTypes.h"
#include "CSLog.h"

class CSMessageListener;
class CSMessageDispatchable;
class CSMessage;

struct CSInternalMessage
{
	CSMessageListener *listener;	// never to be freed
	CSMessage *message;				// never to be freed
};

typedef std::vector<CSInternalMessage*> CSInternalMessages;

/**
	This class is the base class for all message listeners.
	It also provides Singleton features for dispatching messages 
	at a later time.<BR>

	The method to dispatch a message to a (n allways given) listerner
	is:<PRE>
		void CSMessageListener::dispatchMessage(CSMessage *message)
	</PRE>
	The message itself "knows" if it is a message that must be dispatched
	at once - or at a later point in time.<BR>

	If the message is to be dispatched at one, the method:
	<PRE>
		virtual void reactOnMessage(CSMessage *message) = 0;
	</PRE>
	will be called.<BR>
	(But this should not be called by a client, this method is called by
	 CSMEessageDispatchable. The client, that wishes to "send a message"
	 should allways use the method:
	 <PRE>
	 		void sendMessage(CSMessage &message);
	</PRE>
	 of the CSMEessageDispatchable class!)<BR>


	All classes implementing "CSMessageListener", must implement that method.<BR>

    Otherwise (if the message is supposed to be dispatched later), the 
	message AND the listener is added to a static Vector.
	At the apropriate time some "HANDLER" (main look...) must call:
	<PRE>
		void CSMessageListener::dispatchQueuedMessages()
	</PRE>
	At that point all pending messaged will be send to the corresponing 
	listeners.




das ganze event handling NEU machen

  alle Listener bekommen eine ID
  die ID wird zentral verwaltet, dort gibt es ein mapping zwischen ID und listener
  im destrujtor wird die ID aus dem Mapping genommen ->
  es kann nie ein mapping zwischen einem gelschten und einem element und eine ID geben!

  */
class CSMessageListener: public CSObject
{
	friend CSMessageDispatchable;
	private:
		static CSInternalMessages mCSInternalMessages;

	protected:
		static void addMessageToQueue(CSMessageListener *listener, CSMessage *message);

	public:
		CSMessageListener();
		virtual ~CSMessageListener();
		static const char *CLASS;
		virtual std::string getType() {return (std::string) CLASS;}
		
		static void dispatchQueuedMessages();
		virtual void dispatchMessage(CSMessage *message);
		virtual void reactOnMessage(CSMessage *message) = 0;
};


#endif // CSMESSAGE_LISTENER_H

