/**
	CSMusic
	  CSMusic can only be created thru a loader (CSMusicLoader).

*/

#ifdef WIN32
#pragma warning(disable : 4786 )
#endif
#include "CSMusic.h"
#include "CSXMLHelper.h"

using namespace std;
const char *CSMusic::CLASS = "CSMusic";

CSMusicLoader CSMusicLoader::INSTANCE;

// constructor using "*.pic" file
CSMusic::CSMusic(const std::string &filename)
{
	static char *functionName="CSMusic";
	LOG_ENTER 
	CSMusicData data;
	mMusic = 0;
	mMPEG = 0;
	loadMusicData(filename, data);
	initialize(data);
	LOG_EXIT
}

void CSMusic::initialize(const CSMusicData &data)
{
	static char *functionName="initialize";
	LOG_ENTER 
	// if Musics are to large - they get garbled during
	// conversion if in FULLSCREEN (Windows)
	if (isMP3(data.filename))
	{
		/* Note the last parameter is zero! */
		SMPEG_Info info;
		mMPEG = SMPEG_new(data.filename, &info, 0);
		if (!SMPEG_error( mMPEG ))
		{
			/* Play the movie, using SDL_mixer for audio */
			SMPEG_enablevideo(mMPEG, 0);
			SMPEG_enableaudio(mMPEG, 0);
		}
		else
		{
			SMPEG_delete(mMPEG);
			mMPEG = 0;
			mMusic = Mix_LoadMUS(data.filename);
		}
	}
	else
	{
		mMusic = Mix_LoadMUS(data.filename);
	}
	LOG_EXIT
}

CSMusic::CSMusic(const CSMusic &music)
{
	static char *functionName="CSMusic";
	LOG_ENTER 
	mMusic = music.mMusic;
	LOG_EXIT
}

CSMusic::~CSMusic()
{
	static char *functionName="~CSMusic";
	LOG_ENTER 
	stop();
	
	if (mMPEG)
	{
		SMPEG_delete(mMPEG);
		mMPEG = 0;
	}
	
	if (mMusic)
	{
		Mix_FreeMusic(mMusic);
		mMusic = 0;
	}
	LOG_EXIT
}

void CSMusic::play()
{
	static char *functionName="play";
	LOG_ENTER 
	// CSA TODO: Uncomment to play music!

	stop();
	if (mMPEG)
	{
		SDL_AudioSpec audiofmt;
		Uint16 format;
		int freq, channels;

		// Tell SMPEG what the audio format is 
		Mix_QuerySpec(&freq, &format, &channels);
		audiofmt.format = format;
		audiofmt.freq = freq;
		audiofmt.channels = channels;
		SMPEG_actualSpec(mMPEG, &audiofmt);

		// Hook in the MPEG music mixer 
		Mix_HookMusic(SMPEG_playAudioSDL, mMPEG);
		SMPEG_enableaudio(mMPEG, 1);
		SMPEG_play(mMPEG);
	}
	else if (mMusic)
	{
		Mix_PlayMusic(mMusic, 0);
	}

	LOG_EXIT
}

void CSMusic::stop()
{
	static char *functionName="stop";
	LOG_ENTER 
	int frequency;
	Uint16 format;
	int channels;
	int state = Mix_QuerySpec(&frequency, &format, &channels);
	if (state)
	{
		if (mMPEG)
		{
			/* Stop the movie and unhook SMPEG from the mixer */
			SMPEG_stop(mMPEG);
			Mix_HookMusic(NULL, NULL);
			SMPEG_enableaudio(mMPEG, 0);
		}
		else if (mMusic)
		{
			Mix_HaltMusic();
		}
	}
	LOG_EXIT
}

void CSMusic::pause()
{
	static char *functionName="pause";
	LOG_ENTER 
	int frequency;
	Uint16 format;
	int channels;
	int state = Mix_QuerySpec(&frequency, &format, &channels);
	if (state)
	{
		if (mMPEG)
		{
			/* Pause/Resume playback of an SMPEG object */
			SMPEG_pause(mMPEG);
		}
		else if (mMusic)
		{
			Mix_PauseMusic();
		}
	}
	LOG_EXIT
}

void CSMusic::resume()
{
	static char *functionName="resume";
	LOG_ENTER 
	int frequency;
	Uint16 format;
	int channels;
	int state = Mix_QuerySpec(&frequency, &format, &channels);
	if (state)
	{
		if (mMPEG)
		{
			/* Pause/Resume playback of an SMPEG object */
			SMPEG_pause(mMPEG);
		}
		else if (mMusic)
		{
			Mix_ResumeMusic();
		}
	}
	LOG_EXIT
}

void CSMusic::loadMusicData(const std::string &filename, CSMusicData &data)
{
	static char *functionName="loadMusicData";
	LOG_ENTER 
	data.filename = strdup(filename.c_str());
	LOG_EXIT
}

bool CSMusic::isMP3(const char *name)
{
	static char *functionName="isMP3";
	LOG_ENTER 
	// for now only simple checkking!
	// filename!
	if (strstr(name,".mp3") ||
		strstr(name,".Mp3") ||
		strstr(name,".mP3") ||
		strstr(name,".MP3"))
	{
		LOG_EXIT
		return true;
	}
	LOG_EXIT
	return false;
}
