#ifndef CSMusic_h
#define CSMusic_h

#include <string>
#include <vector>
#include "SDLMain.h"
#include "CSLoadable.h"
#include "CSLog.h"

#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

class CSMusic;
class CSMusicLoader;

typedef std::vector<CSMusic *> CSMusics;

class CSMusicData
{
	public:
		char *id;
		char *filename;		// of bitmap file

		CSMusicData()
		{
			id = 0;
			filename = 0;
		}

		~CSMusicData()
		{
			if (id != 0) 
			{
				free(id);
				id = 0;
			}
			if (filename != 0) 
			{
				free(filename);
				filename = 0;
			}
		}
};

class CSMusic
{
	friend CSMusicLoader;
	private:
		Mix_Music *mMusic;
		SMPEG *mMPEG;
		// constructor using name
		CSMusic(const std::string &filename);
		void initialize(const CSMusicData &data);
		static void loadMusicData(const std::string &filename, CSMusicData &data);
		static bool isMP3(const char *name);

	public:
		// copy constructor
		CSMusic(const CSMusic &Music);

		// destructor
		virtual ~CSMusic();	
		static const char *CLASS;
		virtual std::string getType() {return (std::string) CLASS;}

		void play();	
		void pause();
		void resume();
		void stop();
};

// factory - class
// that is used to create CSMusic
class CSMusicLoader : public Loadable<CSMusic>
{
	protected:
		virtual CSMusic *create(const std::string &filename)
		{
			return new CSMusic(filename);
		}

	public:
		static CSMusicLoader INSTANCE;
};


#endif // CSMusic_h
