#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

#include "CSPicture.h"
#include "CSBitmap.h"
#include "CSXMLHelper.h"

using namespace std;
const char *CSPicture::CLASS = "CSPicture";
CSPictures CSPicture::mAllPictures;

CSPictureLoader CSPictureLoader::INSTANCE;
CSPictureDirectLoader CSPictureDirectLoader::INSTANCE;

// constructor using "*.xml" file
CSPicture::CSPicture(const std::string &filename, bool isDirectLoad)
{
	static char *functionName="CSPicture";
	mLoadName = filename;
	mFactor = 0;
	mScaledPicture = 0; 
	mRLEUse = RLE_PERHAPS;
	mAlpha = -1;
	mId = 0;
	mR = 0;
	mG = 0;
	mB = 0;
    mPicture = 0;
	mScaledWidth = 0;
	mScaledHeight = 0;
	mStartPixelsX = 0;
	mStartPixelsY = 0;
	mProjection = 0;
	mPixelProjectionCount = 0;
	mRLEAccelleration = 0;
	mIsCreatedAsScaled = false;
	mColorKey = -1;
	mIsSolid = false;
	CSPictureData data;
	mIsDirectLoad = isDirectLoad;
	if (!isDirectLoad)
	{
		loadPictureData(filename, data);
	}
	initialize(data, isDirectLoad);
	addPicture(this);
}

/** if pictures are to large - they get garbled during
  * conversion if in SDL_HWSURFACE (Windows) (FONT)
  * so I guessed 1000 pixel - and do them if encountered as NOT! Hardware
  * this is guess only - and I don't know WHY
  * it happens - bad work around!!!
  */
SDL_Surface *CSPicture::convertToCurrentScreen(CSBitmap *image, SDL_Rect rect)
{
	static char *functionName="convertToCurrentScreen";
	SDL_Surface *picture = 0;
	int flags = SDLMain::getScreen()->flags;
	if (image->picture->w > 1000)
	{
		if ((flags & SDL_HWSURFACE) == SDL_HWSURFACE)
		{
			flags -= SDL_HWSURFACE;
		}
	}
	
	// at least under windows this is needed!
	// direct Draw gives a surface locked error otherwise!
	if (mAlpha >=0)
	{
		if ((flags & SDL_HWSURFACE) == SDL_HWSURFACE)
		{
			flags -= SDL_HWSURFACE;
		}
	}
	picture = SDL_CreateRGBSurface(flags, 
							   rect.w, 
							   rect.h, 
							   SDLMain::getScreen()->format->BitsPerPixel, 
							   SDLMain::getScreen()->format->Rmask, 
							   SDLMain::getScreen()->format->Gmask, 
							   SDLMain::getScreen()->format->Bmask, 
							   SDLMain::getScreen()->format->Amask);

    if (picture == 0)
	{
		SDLMain::shutdown((std::string)"Create Surface Error error: " + SDL_GetError(), 1);
	}
	LOG_DEBUG_PRINT("image->picture->w: ", image->picture->w)
	LOG_DEBUG_PRINT("image->picture->h: ", (long) image->picture->h)
	LOG_DEBUG_PRINT("rect.x: ", (long) rect.x)
	LOG_DEBUG_PRINT("rect.y: ", (long) rect.y)
	LOG_DEBUG_PRINT("rect.w: ", (long) rect.w)
	LOG_DEBUG_PRINT("rect.h: ", (long) rect.h)
	LOG_DEBUG_PRINT("picture->w: ", (long) picture->w)
	LOG_DEBUG_PRINT("picture->h: ", (long) picture->h) 
	LOG_DEBUG_PRINT("image->picture->format->BytesPerPixel: ", (long) image->picture->format->BytesPerPixel) 
	LOG_DEBUG_PRINT("picture->format->BytesPerPixel: ", (long) picture->format->BytesPerPixel) 

	LOG_DEBUG_PRINT("picture: ", (long) picture) 
	LOG_DEBUG_PRINT("image->picture: ", (long) image->picture) 
	LOG_DEBUG_PRINT("image->picture->format_version: ", (long) image->picture->format_version) 
	LOG_DEBUG_PRINT("picture->format_version: ", (long) picture->format_version) 

	// oversight in SDL, a "picture" that was just deleted, can be reused with
	// exactly the same address and different BytesPerPixel (FullscreenToggle)
	// -> than there is a segmentation fault!
	// we circumvent that by forcing a mapping!
	// with this error SDL_MapSurface() is not called and thus we have a wrong 
	// Blit1to... function set internally
	/* this doesn't work
	if (SDL_MapSurface(image->picture, picture) == -1)
	{
		SDLMain::shutdown((std::string)"Mapping error: " + SDL_GetError(), 1);
	}
	*/

	// here the "working" "work arround"...
	static long helperSingleton = 0;
	picture->format_version+=helperSingleton++;// = rand()*1292;

 	if (SDL_BlitSurface(image->picture, &rect, picture, 0) < 0)
	{
		SDLMain::shutdown((std::string)"Display Format error: " + SDL_GetError(), 1);
	}
	
	mR = image->r;
	mG = image->g;
	mB = image->b;
	if ((image->r>=0) && (image->g>=0) && (image->b>=0))
	{
		if (!(SDLMain::getScreen()->flags & SDL_HWSURFACE))
		{
			int key;
			key = SDL_MapRGB(picture->format, image->r, image->g, image->b);
			SDL_SetColorKey(picture, SDL_SRCCOLORKEY | mRLEAccelleration, key); // new
		}
	}

	// hardware Surface do later
	if ((image->r>=0) && (image->g>=0) && (image->b>=0))
	{
		if (SDLMain::getScreen()->flags & SDL_HWSURFACE)
		{
			int key;
			key = SDL_MapRGB(picture->format, image->r, image->g, image->b);
			SDL_SetColorKey(picture, SDL_SRCCOLORKEY | mRLEAccelleration , key); // new
		}
	}
	mColorKey = picture->format->colorkey;
	if (mAlpha >=0) SDL_SetAlpha(picture, mRLEAccelleration | SDL_SRCALPHA, mAlpha);

	bool mIsDoublebuffer = (picture->flags & SDL_DOUBLEBUF) == SDL_DOUBLEBUF;
	bool mIsHardware = (picture->flags & SDL_HWSURFACE) == SDL_HWSURFACE;
	bool mIsFullscreen = (picture->flags & SDL_FULLSCREEN) == SDL_FULLSCREEN;
	return picture;
}

void CSPicture::initialize(const CSPictureData &data, bool isDirectLoad)
{
	static char *functionName="initialize";
	// picture muss geladen werden
    // Load the BMP file into a surface 
	if (isDirectLoad)
	{
		mId = 0;
		CSBitmap *image = CSBitmapDirectLoader::INSTANCE.load(mLoadName);
		mUpdateRegion.x = mPictureRegion.x = 0;
		mUpdateRegion.y = mPictureRegion.y = 0;
		mUpdateRegion.h = mPictureRegion.h = image->picture->h;
		mUpdateRegion.w = mPictureRegion.w = image->picture->w;

		buildPixelProjection(image, mUpdateRegion);
		mAlpha = -1;
		mPicture = convertToCurrentScreen(image, mPictureRegion);
		mRLEUse = false;

		mPixelPictureRegion = mPictureRegion;
		mPixelPictureRegion.x = 0;
		mPixelPictureRegion.y = 0;
		buildOptimizeData();
		checkRLEAccelleration();
	}
	else
	{
		mId = strdup(data.id);
		mFileName = strdup(data.filename);

		// if pictures are to large - they get garbled during
		// conversion if in FULLSCREEN (Windows)
		CSBitmap *image = CSBitmapLoader::INSTANCE.load(data.filename);
		if (data.position.x + data.position.y + data.position.w + data.position.h == 0)
		{
			mUpdateRegion.x = mPictureRegion.x = data.position.x;
			mUpdateRegion.y = mPictureRegion.y = data.position.y;
			mUpdateRegion.h = mPictureRegion.h = image->picture->h;
			mUpdateRegion.w = mPictureRegion.w = image->picture->w;
		}
		else
		{
			mUpdateRegion.x = mPictureRegion.x = data.position.x;
			mUpdateRegion.y = mPictureRegion.y = data.position.y;
			mUpdateRegion.h = mPictureRegion.h = data.position.h;
			mUpdateRegion.w = mPictureRegion.w = data.position.w;
		}

		buildPixelProjection(image, mUpdateRegion);
		mAlpha = data.alpha;
		mPicture = convertToCurrentScreen(image, mPictureRegion);
		mRLEUse = data.rleUse;

		mPixelPictureRegion = mPictureRegion;
		mPixelPictureRegion.x = 0;
		mPixelPictureRegion.y = 0;
		buildOptimizeData();
		checkRLEAccelleration();
	}
}

CSPicture::CSPicture(const CSPicture &picture)
{
	static char *functionName="CSPicture";
	LOG_ENTER 
	mFactor = 0;
	mScaledWidth = 0;
	mScaledHeight = 0;
	mScaledPicture = 0;
	mPicture = 0;
	mProjection = 0;
	mUpdateRegion = picture.mUpdateRegion;
	mPictureRegion = picture.mPictureRegion;
	mRLEUse = picture.mRLEUse;
	mAlpha = picture.mAlpha;
	mIsCreatedAsScaled = picture.mIsCreatedAsScaled;
	mLoadName = picture.mLoadName;
	mIsDirectLoad = picture.mIsDirectLoad;

	mPixelPictureRegion = picture.mPixelPictureRegion;
	mStartPixelsX = picture.mStartPixelsX;
	mStartPixelsY = picture.mStartPixelsY;
	
	mPixelProjectionCount = picture.mPixelProjectionCount;
	mRLEAccelleration = picture.mRLEAccelleration;
	if (picture.mId)
	{
		mId = strdup(picture.mId);
	}
	else
	{
		mId = 0;
	}
	mFileName = strdup(picture.mFileName);
	mIsSolid = picture.mIsSolid;
	mColorKey = picture.mColorKey;
	mR = picture.mR;
	mG = picture.mG;
	mB = picture.mB;
	mFactor = picture.mFactor;
	
	if (mIsDirectLoad)
	{
		mPicture = convertToCurrentScreen(CSBitmapDirectLoader::INSTANCE.load(mLoadName), mPictureRegion);
	}
	else
	{
		mPicture = convertToCurrentScreen(CSBitmapLoader::INSTANCE.load(mFileName), mPictureRegion);
	}
	if (mIsSolid)
	{
		SDL_SetColorKey(mPicture, mRLEAccelleration, mColorKey+11);
		SDL_Surface *h = mPicture;
		mPicture = SDL_DisplayFormat(h);
		SDLMain::freeSurface(h);
	}
	else
	{
		SDL_Surface *h = mPicture;
		// software surface needs befor
		if (!(SDLMain::getScreen()->flags & SDL_HWSURFACE))
		{
			SDL_SetColorKey(mPicture, SDL_SRCCOLORKEY| mRLEAccelleration, mColorKey);
		}
		mPicture = SDL_DisplayFormat(h);

		// hardware surface needs after
		if (SDLMain::getScreen()->flags & SDL_HWSURFACE)
		{
			SDL_SetColorKey(mPicture, SDL_SRCCOLORKEY| mRLEAccelleration, mColorKey);
		}
		buildOptimizeData();
		SDLMain::freeSurface(h);
	}

	int x,y;
	if (picture.mProjection != 0)
	{
		mProjection = (unsigned char **) new void *[mUpdateRegion.h];
		for (y = 0; y<mUpdateRegion.h; y++)
		{
			mProjection[y] = (unsigned char *) new unsigned char[mUpdateRegion.w];
		}	
		for (y = 0; y<mUpdateRegion.h; y++)
		{
			for (x = 0; x<mUpdateRegion.w; x++)
			{
				mProjection[y][x] = picture.mProjection[y][x];
			}
		}
	}
	addPicture(this);
	LOG_EXIT
}

CSPicture::~CSPicture()
{
	static char *functionName="~CSPicture";
	LOG_ENTER 
	removePicture(this);
	if (mId)
	{
		free(mId);
		mId = 0;
	}
	if (mFileName)
	{
		free(mFileName);
		mFileName = 0;
	}
	if (mProjection != 0)
	{
		for (int y = 0; y<mPictureRegion.h; y++)
		{
			delete[] mProjection[y];
		}	
		delete[] mProjection;
		mProjection = 0;
	}
	if (mScaledPicture != 0)
	{
		if (SDLMain::isInitialized())
		{		
			SDLMain::freeSurface(mScaledPicture);
		}
		mScaledPicture = 0;
	}
	LOG_EXIT
}

void CSPicture::setSolid(bool b)
{
	static char *functionName="setSolid";
	if (b == mIsSolid)
	{
		return;
	}

	SDL_Surface *h = mPicture;
	mIsSolid = b;
	if (mIsSolid)
	{
		SDL_SetColorKey(mPicture, mRLEAccelleration, mColorKey+11);
		mPicture = SDL_DisplayFormat(h);
		mPixelPictureRegion = mPictureRegion;
		mPixelPictureRegion.x = 0;
		mPixelPictureRegion.y = 0;
		mStartPixelsX = 0;
		mStartPixelsY = 0;
		mUpdateRegion.h = mPictureRegion.h;
		mUpdateRegion.w = mPictureRegion.w;
	}
	else
	{
		// software surface needs befor
		if (!(SDLMain::getScreen()->flags & SDL_HWSURFACE))
		{
			SDL_SetColorKey(mPicture, SDL_SRCCOLORKEY| mRLEAccelleration, mColorKey);
		}
		mPicture = SDL_DisplayFormat(h);

		// hardware surface needs after
		if (SDLMain::getScreen()->flags & SDL_HWSURFACE)
		{
			SDL_SetColorKey(mPicture, SDL_SRCCOLORKEY| mRLEAccelleration, mColorKey);
		}
		buildOptimizeData();
	}
	SDLMain::freeSurface(h);
}

void CSPicture::setAlpha(int alpha)
{
	static char *functionName="setAlpha";
	LOG_ENTER 
	if (alpha < 0)
	{
		mAlpha = -1;
		SDL_SetAlpha(mPicture, mRLEAccelleration, 0);
		LOG_EXIT
		return;
	}
	mAlpha = alpha;
	SDL_SetAlpha(mPicture, mRLEAccelleration | SDL_SRCALPHA, alpha);
	LOG_EXIT
}

void CSPicture::display(SDL_Surface *destination, int x, int y)
{
	static char *functionName="display";
	if ((!mIsSolid) && (mPixelProjectionCount == 0))
	{
		return;
	}

	mUpdateRegion.x = mStartPixelsX + x;
	mUpdateRegion.y = mStartPixelsY + y;

    if (SDL_BlitSurface(mPicture, &mPixelPictureRegion, destination, &mUpdateRegion)< 0)
	{
		SDLMain::shutdown((std::string)"BlitSurface error: " + SDL_GetError(), 1);
	}
}

void CSPicture::display(int x, int y)
{
	static char *functionName="display";
	if ((!mIsSolid) && (mPixelProjectionCount == 0))
	{
		return;
	}

	mUpdateRegion.x = mStartPixelsX + x;
	mUpdateRegion.y = mStartPixelsY + y;
    /* Blit onto the screen surface */
    if (SDL_BlitSurface(mPicture, &mPixelPictureRegion, SDLMain::getScreen(), &mUpdateRegion)< 0)
	{
		SDLMain::shutdown((std::string)"BlitSurface error: " + SDL_GetError(), 1);
	}
	
	SDLMain::addUpdateRect(mUpdateRegion);
}

void CSPicture::display(CSDisplayParams *displayParams)
{
	static char *functionName="CSPicture::display(CSDisplayParams *displayParams)";
	if ((!mIsSolid) && (mPixelProjectionCount == 0))
	{
		return;
	}

	// optimize - perhaps have some part of below as
	// constants - uses less indirections...
	mUpdateRegion.x = mStartPixelsX + displayParams->mXDisplayStart + displayParams->mXPos - displayParams->mXWorldStart;
	mUpdateRegion.y = mStartPixelsY + displayParams->mYDisplayStart + displayParams->mYPos - displayParams->mYWorldStart;

    /* Blit onto the screen surface */
    if (SDL_BlitSurface(mPicture, &mPixelPictureRegion, SDLMain::getScreen(), &mUpdateRegion)< 0)
	{
		SDLMain::shutdown((std::string)"BlitSurface error: " + SDL_GetError(), 1);
	}
	
	SDLMain::addUpdateRect(mUpdateRegion);
}

void CSPicture::resetSurface()
{
	static char *functionName="resetSurface";
	LOG_ENTER 
	SDLMain::freeSurface(mPicture);

	if (mIsCreatedAsScaled)
	{
		SDL_Surface *picture = 0;
		if (mIsDirectLoad)
		{
			picture = convertToCurrentScreen(CSBitmapDirectLoader::INSTANCE.load(mLoadName), mPictureRegion);
		}
		else
		{
			picture = convertToCurrentScreen(CSBitmapLoader::INSTANCE.load(mFileName), mPictureRegion);
		}
		if (picture == 0)
		{
			SDLMain::shutdown((std::string)"ConvertError: " + SDL_GetError(), 13);
		}
		mPicture = SDLMain::getInstance()->scale(picture, mFactor);
		if (mPicture == 0)
		{
			SDLMain::shutdown((std::string)"ScaleError: " + SDL_GetError(), 12);
		}
		SDLMain::freeSurface(picture);

		if (mIsSolid)
		{
			SDL_SetColorKey(mPicture, mRLEAccelleration, mColorKey+11);
			SDL_Surface *h = mPicture;
			mPicture = SDL_DisplayFormat(h);
			if (mPicture == 0)
			{
				SDLMain::shutdown((std::string)"Format Error: " + SDL_GetError(), 14);
			}
			SDLMain::freeSurface(h);
		}
		else
		{
			SDL_Surface *h = mPicture;
			// software surface needs befor
			if (!(SDLMain::getScreen()->flags & SDL_HWSURFACE))
			{
				SDL_SetColorKey(mPicture, SDL_SRCCOLORKEY| mRLEAccelleration, mColorKey);
			}
			mPicture = SDL_DisplayFormat(h);

			// hardware surface needs after
			if (SDLMain::getScreen()->flags & SDL_HWSURFACE)
			{
				SDL_SetColorKey(mPicture, SDL_SRCCOLORKEY| mRLEAccelleration, mColorKey);
			}
			SDLMain::freeSurface(h);
		}
	}
	else
	{
		// the image is not loaded from disk!
		// the loader "buffers" the image - therefor "loading"
		// here is ok!
		if (mIsDirectLoad)
		{
			mPicture = convertToCurrentScreen(CSBitmapDirectLoader::INSTANCE.load(mLoadName), mPictureRegion);
		}
		else
		{
			mPicture = convertToCurrentScreen(CSBitmapLoader::INSTANCE.load(mFileName), mPictureRegion);
		}
		if (mPicture == 0)
		{
			SDLMain::shutdown((std::string)"ConvertError: " + SDL_GetError(), 16);
		}
		if (mIsSolid)
		{
			SDL_SetColorKey(mPicture, mRLEAccelleration, mColorKey+11);
			SDL_Surface *h = mPicture;
			mPicture = SDL_DisplayFormat(h);
			if (mPicture == 0)
			{
				SDLMain::shutdown((std::string)"Format Error: " + SDL_GetError(), 15);
			}
			SDLMain::freeSurface(h);
		}
		else
		{
			SDL_Surface *h = mPicture;
			// software surface needs befor
			if (!(SDLMain::getScreen()->flags & SDL_HWSURFACE))
			{
				SDL_SetColorKey(mPicture, SDL_SRCCOLORKEY| mRLEAccelleration, mColorKey);
			}
			mPicture = SDL_DisplayFormat(h);

			// hardware surface needs after
			if (SDLMain::getScreen()->flags & SDL_HWSURFACE)
			{
				SDL_SetColorKey(mPicture, SDL_SRCCOLORKEY| mRLEAccelleration, mColorKey);
			}
			SDLMain::freeSurface(h);
		}
		mFactor = 0;
	}
	if (mScaledPicture != 0)
	{
		SDLMain::freeSurface(mScaledPicture);
		mScaledPicture = 0;
	}
	setAlpha(mAlpha);
	LOG_EXIT
}

// static stuff to enable converting picture to and from fullscreen
// all pictures ever loaded from disc are remembered in the Vector "mAllPictures"
void CSPicture::addPicture(CSPicture *picture)
{
	if (picture == 0)
	{
		return;
	}
	removePicture(picture);
	mAllPictures.push_back(picture);
}

void CSPicture::removePicture(CSPicture *picture)
{
	if (picture == 0)
	{
		return;
	}

	CSPictures::iterator iter = mAllPictures.begin();
	while ( iter != mAllPictures.end())
	{
		if ((*iter) == picture)
		{
			mAllPictures.erase(iter);
			iter = mAllPictures.begin();	// restart to be absolutly save!
		}
		else
		{
			iter++;
		}
	}
}

// resets all surfaces
// loaded via the CSPictureLoader (all!)
void CSPicture::resetSurfaces()
{
	static char *functionName="resetSurfaces";
	LOG_ENTER 
	CSPictures::iterator iter = mAllPictures.begin();
	while ( iter != mAllPictures.end())
	{
		(*iter)->resetSurface();
		iter++;
	}
	LOG_EXIT
}

// a pixel projection
// has a one for every non transparent pixel
// and a zero for every transparent pixel
// one "pixel" in the projection i an 8-bit value
// one could use a real bitmap - one bit for each pixel
// than one could AND the pixels and be still a bit faster
// but for now - it seems to be sufficient
void CSPicture::buildPixelProjection(CSBitmap *image, const SDL_Rect &position)
{
	static char *functionName="buildPixelProjection";
	// position data is not available for ALL pictures - must test this
	if ( (position.x == 0) &&
		 (position.y == 0) &&
		 (position.h == 0) &&
		 (position.w == 0) 
	   )
	{
		// fonts...
		// but perhaps we should than use the size of the picture?
		// well do that some other time - as this is
		// not needed yet
		return;
	}
	
	// count all non zero pixels
	mPixelProjectionCount = 0;
	
	// get transparent pixels-value
	int key = SDL_MapRGB(image->picture->format, image->r, image->g, image->b);
	int x,y;

	// if an old projection should already exist?
	// delete it
	if (mProjection != 0)
	{
		for (y = 0; y<mPictureRegion.h; y++)
		{
			delete[] mProjection[y];
		}	
		delete[] mProjection;
		mProjection = 0;
	}

	// build a two dimensional array of chars - (zero or one)
	mProjection = (unsigned char **) new void *[position.h];
	for (y = 0; y<position.h; y++)
	{
		mProjection[y] = 0;
	}	
	for (y = 0; y<position.h; y++)
	{
		mProjection[y] = (unsigned char *) new unsigned char[position.w];
	}	
	
	if (SDL_MUSTLOCK(image->picture)) 
	{
		SDL_LockSurface(image->picture);
	}
	for (y = 0; y<position.h; y++)
	{
		for (x = 0; x<position.w; x++)
		{
			Uint32 p = SDLMain::getPixel(image->picture, x+position.x, y+position.y);

			if (key == p)
			{
				mProjection[y][x] = 0;
			}
			else
			{
				mProjection[y][x] = 1;
				mPixelProjectionCount++;
			}

		}
	}

    if (SDL_MUSTLOCK(image->picture)) 
	{
		SDL_UnlockSurface(image->picture);
	}
}
    
void CSPicture::checkRLEAccelleration()
{
	static char *functionName="checkRLEAccelleration";
	if 	(mRLEUse == RLE_YES)
	{
    	mRLEAccelleration = SDL_RLEACCEL;
		return;
	}

	if 	(mRLEUse == RLE_NO)
	{
		return;
	}

	// this should be a little bit more sophisticated 
	// - can't be bothered right now!
	if (mPixelProjectionCount <((mUpdateRegion.h*mUpdateRegion.w)/4))
	{

		// is faster completely without RLE?
		mRLEAccelleration = SDL_RLEACCEL;
	}
}

void CSPicture::loadPictureData(const std::string &filename, CSPictureData &data)
{
	static char *functionName="loadPictureData";
	LOG_ENTER 
	CSXMLHelper xmlSupport(filename, "PICTURE");
	try
	{
		if (xmlSupport.getError())
		{
			LOG_EXIT
			throw "error";
		}

		data.id = strdup(xmlSupport.getString("ID").c_str());
		int hasPositionData = xmlSupport.getInt("count(/PICTURE/X)");
		if (hasPositionData == 1)
		{
			data.position.x = xmlSupport.getInt("X");
			data.position.y = xmlSupport.getInt("Y");
			data.position.w = xmlSupport.getInt("WIDTH");
			data.position.h = xmlSupport.getInt("HEIGHT");
		}
		else
		{
			data.position.x = 0;
			data.position.y = 0;
			data.position.w = 0;
			data.position.h = 0;
		}

		int hasRLEData = xmlSupport.getInt("count(/PICTURE/RLE_USE)");
		if (hasRLEData == 1)
		{
			data.rleUse = xmlSupport.getInt("RLE_USE");
		}
		int hasAlpha = xmlSupport.getInt("count(/PICTURE/ALPHA)");
		if (hasAlpha == 1)
		{
			data.alpha = xmlSupport.getInt("ALPHA");
		}
		data.filename = strdup(xmlSupport.getString("BITMAP").c_str());
	}
	catch(...)
	{
		LOG_EXIT
		SDLMain::shutdown((std::string)"XML error \"" + filename + "\": " + xmlSupport.getErrorMessage().c_str(), 1);
	}
	LOG_EXIT
}

// e.g. if a picture is surrounded by transparent pixels
// the surrounding is "removed" and the start x,y is
// set to the "inner" of the surrounding...
void CSPicture::buildOptimizeData()
{
	static char *functionName="buildOptimizeData";
	if (mProjection == 0)
	{
		return;
	}

	int x,y;
	unsigned int startPixelsX = 0;
	unsigned int startPixelsY = 0;
	unsigned int endPixelsX = mPictureRegion.w;
	unsigned int endPixelsY = mPictureRegion.h;

	for (y = 0; y<mPictureRegion.h; y++)
	{
		bool hasY = false;
		for (x = 0; x<mPictureRegion.w; x++)
		{
			if (mProjection[y][x] == 1)
			{
				hasY = true;
				break;
			}
		}
		if (hasY)
		{
			break;
		}
		startPixelsY++;
	}

	for (x = 0; x<mPictureRegion.w; x++)
	{
		bool hasX = false;
		for (y = 0; y<mPictureRegion.h; y++)
		{
			if (mProjection[y][x] == 1)
			{
				hasX = true;
				break;
			}
		}
		if (hasX)
		{
			break;
		}
		startPixelsX++;
	}

	for (y = mPictureRegion.h-1; y>=startPixelsY; y--)
	{
		bool hasY = false;
		for (x = 0; x<mPictureRegion.w; x++)
		{
			if (mProjection[y][x] == 1)
			{
				hasY = true;
				break;
			}
		}
		if (hasY)
		{
			break;
		}
		endPixelsY = y;
	}

	for (x = mPictureRegion.w-1; x>=startPixelsX; x--)
	{
		bool hasX = false;
		for (y = 0; y<mPictureRegion.h; y++)
		{
			if (mProjection[y][x] == 1)
			{
				hasX = true;
				break;
			}
		}
		if (hasX)
		{
			break;
		}
		endPixelsX = x;
	}

	mStartPixelsX = startPixelsX;
	mStartPixelsY = startPixelsY;
	mPixelPictureRegion.x += startPixelsX;
	mPixelPictureRegion.y += startPixelsY;
	mPixelPictureRegion.w = endPixelsX - startPixelsX;
	mPixelPictureRegion.h = endPixelsY - startPixelsY;
	mUpdateRegion.w = endPixelsX - startPixelsX;
	mUpdateRegion.h = endPixelsY - startPixelsY;
}

/** This function scales the current picture "deeply", each following reference, even to
  * the buffered Bitmap will produce a scaled version of the picture.
  * Only way to get rid of scaling is unload from the Loader!
  *
  */
void CSPicture::scale(int width, int divideX, int height, int divideY)
{
	static char *functionName="scale";
	if (mIsDirectLoad)
	{
		CSBitmapDirectLoader::INSTANCE.scale((char *) mLoadName.c_str(), width, divideX*mUpdateRegion.w, height, divideY*mUpdateRegion.h);
	}
	else
	{
		CSBitmapLoader::INSTANCE.scale(mFileName, width, divideX*mUpdateRegion.w, height, divideY*mUpdateRegion.h);
	}

	if (mProjection != 0)
	{
		for (int y = 0; y<mPictureRegion.h; y++)
		{
			delete[] mProjection[y];
		}	
		delete[] mProjection;
		mProjection = 0;
	}

	mUpdateRegion.x = mPictureRegion.x = SDLMain::getInstance()->scaleX(width, divideX*mUpdateRegion.w, mUpdateRegion.x);
	mUpdateRegion.y = mPictureRegion.y = SDLMain::getInstance()->scaleY(height, divideY*mUpdateRegion.h, mUpdateRegion.y);
	mUpdateRegion.w = mPictureRegion.w = SDLMain::getInstance()->scaleX(width, divideX*mUpdateRegion.w, mUpdateRegion.w);
	mUpdateRegion.h = mPictureRegion.h = SDLMain::getInstance()->scaleY(height, divideY*mUpdateRegion.h, mUpdateRegion.h);

	resetSurface();

	CSBitmap *image = 0;
	if (mIsDirectLoad)
	{
		image = CSBitmapDirectLoader::INSTANCE.load(mLoadName);
	}
	else
	{
		image = CSBitmapLoader::INSTANCE.load(mFileName);
	}
	buildPixelProjection(image, mUpdateRegion);
	mPixelPictureRegion = mPictureRegion;
	mPixelPictureRegion.x = 0;
	mPixelPictureRegion.y = 0;
	buildOptimizeData();
	checkRLEAccelleration();
}

void CSPicture::getRGB(int &r, int &b, int &g)
{
	static char *functionName="getRGB";
	r = mR;
	g = mG;
	b = mB;
}

/** Get a scaled version of this CSPicture.
  *
  */
/** This function scales the current picture with a factor. The scaled result is
  * a part of this picture! Both the scaled and unscaled picture can be got with functions.
  * The scaled picture is "buffered" by CSPicture and can be got "fast!" again and again
  */
SDL_Surface *CSPicture::getScaledPicture(double factor)
{
	static char *functionName="getScaledPicture";
	if ((mFactor == factor) && (mScaledPicture != 0))
	{
		return mScaledPicture;
	}
	mFactor = factor;
	if (mScaledPicture != 0)
	{
		SDLMain::freeSurface(mScaledPicture);
		mScaledPicture = 0;
	}
	
	mScaledPicture = SDLMain::getInstance()->scale(mPicture, factor);
	if (!isSolid())
	{
		int key;
		key = SDL_MapRGB(mScaledPicture->format, mR, mG, mB);
		SDL_SetColorKey(mScaledPicture, SDL_SRCCOLORKEY, key);
	}
	return mScaledPicture;
}
//! Get a scaled version of this CSPicture.
/** This function scales the current picture with a factor. The scaled result is
  * a part of this picture! Both the scaled and unscaled picture can be got with functions.
  * The scaled picture is "buffered" by CSPicture and can be got "fast!" again and again
  * \return CSPicture -> should not be freed
  */
// scale to a given height and width

SDL_Surface *CSPicture::getScaledPicture(int w, int h)
{
	static char *functionName="getScaledPicture";
	
	if ((mScaledWidth == h) && (mScaledHeight == h) && (mScaledPicture != 0))
	{
		return mScaledPicture;
	}
	mScaledWidth = w;
	mScaledHeight = h;
	if (mScaledPicture != 0)
	{
		SDLMain::freeSurface(mScaledPicture);
		mScaledPicture = 0;
	}
	
	mScaledPicture = SDLMain::getInstance()->scale(mPicture, w,h);
	if (!isSolid())
	{
		int key;
		key = SDL_MapRGB(mScaledPicture->format, mR, mG, mB);
		SDL_SetColorKey(mScaledPicture, SDL_SRCCOLORKEY, key);
	}
	return mScaledPicture;
}

bool CSPicture::isTransparent(int x, int y)
{
	static char *functionName="isTransparent";
	LOG_ENTER 
    if (SDL_MUSTLOCK(mPicture)) 
	{
		SDL_LockSurface(mPicture);
	}
	bool val = mColorKey == SDLMain::getPixel(mPicture, x, y);
    if (SDL_MUSTLOCK(mPicture)) 
	{
		SDL_UnlockSurface(mPicture);
	}
	LOG_EXIT
	return val;
}

/** This function duplicates the current picture, that is builds a new pciture,
  * which is scaled to the factor given!
  *
  */
CSPicture* CSPicture::createScaledPicture(CSPicture* picture, double factor)
{
	CSPicture *newPicture = new CSPicture(*picture);
	newPicture->mIsCreatedAsScaled = true;
	newPicture->mFactor = factor;
 	newPicture->resetSurface();
	
	newPicture->mUpdateRegion.x = newPicture->mUpdateRegion.x * newPicture->mFactor;
	newPicture->mUpdateRegion.y = newPicture->mUpdateRegion.y * newPicture->mFactor;
	newPicture->mUpdateRegion.w = newPicture->mUpdateRegion.w * newPicture->mFactor;
	newPicture->mUpdateRegion.h = newPicture->mUpdateRegion.h * newPicture->mFactor;
	newPicture->mPixelPictureRegion = newPicture->mUpdateRegion;
	newPicture->mPixelPictureRegion.x = 0;
	newPicture->mPixelPictureRegion.y = 0;


	// following is not done yet for copied scaled pictures
//	CSBitmap *image = CSBitmapLoader::INSTANCE.load(mFileName);
//	buildPixelProjection(image, mUpdateRegion);
	newPicture->buildOptimizeData();
	newPicture->checkRLEAccelleration();
	
	return newPicture;
}
