/**
	CSSample
	  CSSample can only be created thru a loader (CSSampleLoader).

*/

#ifdef WIN32
#pragma warning(disable : 4786 )
#endif
#include "CSSample.h"
#include "CSXMLHelper.h"

using namespace std;

CSSampleLoader CSSampleLoader::INSTANCE;
CSSampleMap *CSSample::mAllPlayingSamples = 0;
bool CSSample::mLock = false;
int CSSample::mIsInit = 0;
const char *CSSample::CLASS = "CSSample";

// constructor using "*.pic" file
CSSample::CSSample(const std::string &filename)
{
	static char *functionName="CSSample";
	LOG_ENTER 
	CSSampleData data;
	mSample = 0;
	mChannel = -1;
	loadSampleData(filename, data);
	initialize(data);
	init();
	LOG_EXIT
}

CSSample::~CSSample()
{
	static char *functionName="~CSSample";
	LOG_ENTER 
	stop();
	if (mSample)
	{
		Mix_FreeChunk(mSample);
	}
	mSample = 0;
	deinit();
	LOG_EXIT
}

void CSSample::initialize(const CSSampleData &data)
{
	static char *functionName="initialize";
	LOG_ENTER 
	// if Samples are to large - they get garbled during
	// conversion if in FULLSCREEN (Windows)
	mSample = Mix_LoadWAV(data.filename);
	LOG_EXIT
}

CSSample::CSSample(const CSSample &sample)
{
	static char *functionName="~CSSample";
	LOG_ENTER 
	mSample = sample.mSample;
	LOG_EXIT
}

void CSSample::play()
{
	static char *functionName="play";
	LOG_ENTER 
	stop();
	// CSA TODO: Uncomment to play samples!
 mChannel = Mix_PlayChannel(-1,mSample, 0);
 addCallback(mChannel, this);
	LOG_EXIT
}

void CSSample::stop()
{
	static char *functionName="stop";
	LOG_ENTER 
	int frequency;
	Uint16 format;
	int channels;
	removeCallback(mChannel);
	int state = Mix_QuerySpec(&frequency, &format, &channels);
	if (state)
	{
		if (mChannel != -1)
		{
			Mix_HaltChannel(mChannel);
			mChannel = -1;
		}
	}
	LOG_EXIT
}

void CSSample::loadSampleData(const std::string &filename, CSSampleData &data)
{
	static char *functionName="loadSampleData";
	LOG_ENTER 
	data.filename = strdup(filename.c_str());
	LOG_EXIT
}

void CSSample::pause()	
{
	static char *functionName="pause";
	LOG_ENTER 
	Mix_Pause(mChannel);
	LOG_EXIT
}

void CSSample::resume()	
{
	static char *functionName="resume";
	LOG_ENTER 
	Mix_Resume(mChannel);
	LOG_EXIT
}

void CSSample::pauseAll()
{
	static char *functionName="pauseAll";
	LOG_ENTER 
	while (mLock)
	{
		SDL_Delay(50);
	}

	mLock = true;
	for (CSSampleMap::iterator iter = mAllPlayingSamples->begin(); iter != mAllPlayingSamples->end(); iter++)
	{
		iter->second->pause();
	}
	mLock = false;
	LOG_EXIT
}

void CSSample::resumeAll()
{
	static char *functionName="resumeAll";
	LOG_ENTER 
	while (mLock)
	{
		SDL_Delay(50);
	}

	mLock = true;
	for (CSSampleMap::iterator iter = mAllPlayingSamples->begin(); iter != mAllPlayingSamples->end(); iter++)
	{
		iter->second->resume();
	}
	mLock = false;
	LOG_EXIT
}

void CSSample::channel_finished(int channel)
{
	static char *functionName="channel_finished";
	LOG_ENTER 
	removeCallback(channel);
	LOG_EXIT
}
	
void CSSample::addCallback(int channel, CSSample *sample)
{
	static char *functionName="addCallback";
	LOG_ENTER 
	if (channel < 0)
	{
		LOG_EXIT
		return;
	}
	while (mLock)
	{
		SDL_Delay(50);
	}

	mLock = true;
	mAllPlayingSamples->insert(CSSampleMap::value_type(channel, sample));
	mLock = false;
	LOG_EXIT
}

void CSSample::removeCallback(int channel)
{
	static char *functionName="removeCallback";
	LOG_ENTER 
	if (channel < 0)
	{
		LOG_EXIT
		return;
	}

	while (mLock)
	{
		SDL_Delay(50);
	}

	mLock = true;
	CSSampleMap::iterator iter;
	for (iter = mAllPlayingSamples->begin(); iter!=mAllPlayingSamples->end(); iter++)
	{
		if (iter->first == channel)
		{
			mAllPlayingSamples->erase(iter);
			break;
		}
	}
	mLock = false;
	LOG_EXIT
}

void CSSample::init()
{
	static char *functionName="init";
	LOG_ENTER 
	if (mIsInit == 0)
	{
		mAllPlayingSamples = new CSSampleMap();
		Mix_ChannelFinished(CSSample::channel_finished);
	}
	mIsInit++;
	LOG_EXIT
}

void CSSample::deinit()
{
	static char *functionName="deinit";
	LOG_ENTER 
	mIsInit--;
	if (mIsInit == 0)
	{
		mAllPlayingSamples->clear();
		delete (mAllPlayingSamples);
		mAllPlayingSamples = 0;
		Mix_ChannelFinished(0);
	}
	LOG_EXIT
}

bool CSSample::isPlaying()
{
	static char *functionName="isPlaying";
	LOG_ENTER 
	bool b= mAllPlayingSamples->size() > 0;
	LOG_EXIT
	return b;
}
