#ifndef CSSample_h
#define CSSample_h

#include <string>
#include <vector>
#include <map>
#include "SDLMain.h"
#include "CSLoadable.h"
#include "CSLog.h"

#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

class CSSample;
class CSSampleLoader;

typedef std::vector<CSSample *> CSSamples;
typedef std::map<int, CSSample *> CSSampleMap;

class CSSampleData
{
	public:
		char *id;
		char *filename;		// of bitmap file

		CSSampleData()
		{
			id = 0;
			filename = 0;
		}

		~CSSampleData()
		{
			if (id != 0) 
			{
				free(id);
				id = 0;
			}
			if (filename != 0) 
			{
				free(filename);
				filename = 0;
			}
		}
};

class CSSample
{
	friend CSSampleLoader;
	private:
		static CSSampleMap *mAllPlayingSamples;
		static bool mLock;
		static int mIsInit;
		Mix_Chunk *mSample;
		int mChannel;

		// constructor using name
		CSSample(const std::string &filename);
		void initialize(const CSSampleData &data);
		static void loadSampleData(const std::string &filename, CSSampleData &data);

	public:
		// copy constructor
		CSSample(const CSSample &Sample);

		// destructor
		virtual ~CSSample();	

		static const char *CLASS;
		virtual std::string getType() {return (std::string) CLASS;}

		void play();	
		void stop();	
		void pause();	
		void resume();	

		static void pauseAll();
		static void resumeAll();
		static void channel_finished(int channel);
		static void addCallback(int channel, CSSample *sample);
		static void removeCallback(int channel);
		static void init();
		static void deinit();
		static bool isPlaying();
};

// factory - class
// that is used to create CSSample
class CSSampleLoader : public Loadable<CSSample>
{
	protected:
		virtual CSSample *create(const std::string &filename)
		{
			return new CSSample(filename);
		}

	public:
		static CSSampleLoader INSTANCE;
};


#endif // CSSample_h
