#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

#include "CSSprite.h"

#include "SDLMain.h"
#include "CSTileMap.h"
#include "CSXMLHelper.h"
#include "CSNavigator.h"

CSSpriteLoader CSSpriteLoader::INSTANCE;
const char *CSSprite::CLASS = "CSSprite";

CSSprite::CSSprite(const std::string &filename)
{
	static char *functionName="CSSprite";
	LOG_ENTER 
	CSSpriteData data;
	loadSpriteData(filename, data);
	mFilename = filename;
	initialize(data);
	LOG_EXIT
}

// not done! TODO
// not listener added as original!
CSSprite::CSSprite(const CSSprite &sprite)
{
	static char *functionName="CSSprite";
	LOG_ENTER 
	initialize();
	mId = strdup(sprite.mId);
	mFilename = sprite.mFilename;

	LOG_EXIT
}

void CSSprite::initialize(void)
{
	static char *functionName="initialize";
	LOG_ENTER 
	mActive = false;
	mDefaultBundle = 0;
	mNavigator = 0;
	layer_position = 0;
	y_position = 0;
	mWidth = 0;
	mHeight = 0;
	mId = 0;
	mType = 0;
	mSubtype = 0;
	MESSAGE_SPRITE_ACTION_CHANGE.setSubtype(SPRITE_ACTION_CHANGE_MESSAGE);
	LOG_EXIT
}

// data used to initialize sprite
// is not used as itself, everything needed
// is instantiated fresh or copied...
// that meens data can be reused to build another sprite!
void CSSprite::initialize(const CSSpriteData &data)
{
	static char *functionName="initialize";
	LOG_ENTER 
	unsigned int maxX = 0, maxY = 0;
	unsigned int x, y;
	initialize();

	layer_position = data.layer_position;
	y_position = data.y_position;
	mDefaultBundle = data.defaultBundle;
	for (int i=0; i < data.stateDatas.size(); i++)
	{
		StateData *stateData = data.stateDatas.at(i);
		ActionBundle *bundle = new 	ActionBundle();
		bundle->stateId = stateData->stateId;
		// in this vector all of "my" actions will be stored!
		bundle->mActions = new CSActions();

		// create all actions (with ini - filename)
		for (StringVector::iterator iter = stateData->actionNames.begin(); iter != stateData->actionNames.end(); iter++)
		{
			char *help = (char *) *iter; 
			if (help == NULL)
			{
				LOG_EXIT
				SDLMain::shutdown((std::string)"Char Pointer == 0 Sprite Loading: " + SDL_GetError(), 1);
			}
			CSAction *action = CSActionLoader::INSTANCE.load((char *)(help));
			x = action->getMaxX(); if (maxX < x) maxX =x;
			y = action->getMaxY(); if (maxY < y) maxY =y;

			bundle->mActions->push_back(action);
		} 
		if ((stateData->defaultActionName == NULL) || (strlen(stateData->defaultActionName) == 0))
		{
			LOG_EXIT
			SDLMain::shutdown((std::string)"No default action given! Sprite Loading: " + SDL_GetError(), 1);
		}

		// create default action
		CSAction *action = CSActionLoader::INSTANCE.load((char *)stateData->defaultActionName);
		x = action->getMaxX(); if (maxX < x) maxX =x;
		y = action->getMaxY(); if (maxY < y) maxY =y;
		bundle->mActions->push_back(action);
		// pointer to the default action will allways be available at once!
		bundle->mDefaultAction = action;

		actionMap.insert(ActionBundleMap::value_type(bundle->stateId, bundle));
	}
	
	mWidth = maxX;
	mHeight = maxY;
	mId = strdup(data.id);
	mType = data.type;
	mSubtype = data.subtype;

	mState = buildState();
	LOG_EXIT
}

CSSprite::~CSSprite()
{
	static char *functionName="~CSSprite";
	LOG_ENTER 
	ActionBundleMap::iterator iter = actionMap.begin();
	while (iter != actionMap.end())
	{
		delete iter->second;
		iter++;
	}
	actionMap.clear();

	if (mId != 0)
	{
		free(mId);
	}
	LOG_EXIT
}

void CSSprite::setSecPerFrame(float secPerFrame)
{
	static char *functionName="setSecPerFrame";
	LOG_ENTER 
	for (ActionBundleMap::iterator iter2 = actionMap.begin(); iter2 !=actionMap.end(); iter2++)
	{
		for (CSActions::iterator iter = iter2->second->mActions->begin(); iter != iter2->second->mActions->end(); iter++)
		{
			CSAction *action = *iter;
			action->setSecPerFrame(secPerFrame);
		}
	}
	LOG_EXIT
}

void CSSprite::setAlpha(int alpha)
{
	static char *functionName="setAlpha";
	LOG_ENTER 
	for (ActionBundleMap::iterator iter2 = actionMap.begin(); iter2 !=actionMap.end(); iter2++)
	{
		for (CSActions::iterator iter = iter2->second->mActions->begin(); iter != iter2->second->mActions->end(); iter++)
		{
			CSAction *action = *iter;
			action->setAlpha(alpha);
		}
	}
	LOG_EXIT
}

bool CSSprite::changeStateTo(int state)
{
	static char *functionName="changeStateTo";
	ActionBundleMap::iterator iter = actionMap.find(state);

	if (iter != actionMap.end())
	{
		mState.mCurrentBundleId = state;
		mState.mCurrentBundle = (iter->second);
		
		for (CSActions::iterator iter = mState.mCurrentBundle->mActions->begin(); iter != mState.mCurrentBundle->mActions->end(); iter++)
		{
			CSAction *action = *iter;
			if (action->getEnvoker() == mState.mCurrentAction->getEnvoker())
			{

				mState.mCurrentAction = action;
				// next display NEXT round
				// since we are in eventHandling now!
				//wrong time to start drawing!
				mState.mCurrentAction->startNextAction(mState.mActionState);
				return true;
			}
		}
		mState.mCurrentAction = mState.mCurrentBundle->mDefaultAction;
		mState.mCurrentAction->startNextAction(mState.mActionState);
		return true;
	}
	return false;
}


// build an initial state for this sprite, sets variables to defaults
SpriteState CSSprite::buildState()
{
	static char *functionName="buildState";
	SpriteState state;
	state.mCurrentBundleId = mDefaultBundle;

	ActionBundleMap::iterator iter = actionMap.find(state.mCurrentBundleId);
	if (iter != actionMap.end())
	{
		state.mCurrentBundle = (iter->second);
		state.mCurrentAction = state.mCurrentBundle->mDefaultAction;
	}
	else
	{
		state.mCurrentBundle = 0;
		state.mCurrentAction = 0;
	}
			
	if (state.mCurrentAction)
	{
		state.mActionState = state.mCurrentAction->buildState();
	}
	return state;
}

// resets a state to default values
// doesn't  update mDisplayParams! 
// (this position and mDisplayPicture do not change!)
void CSSprite::resetState(SpriteState &state)
{
	static char *functionName="resetState";
	state.mCurrentBundleId = mDefaultBundle;
	ActionBundleMap::iterator iter = actionMap.find(state.mCurrentBundleId);
	if (iter != actionMap.end())
	{
		state.mCurrentBundle = iter->second;
		state.mCurrentAction = state.mCurrentBundle->mDefaultAction;
	}
	else
	{
		state.mCurrentBundle = 0;
		state.mCurrentAction = 0;
	}
	if (state.mCurrentAction)
	{
		state.mCurrentAction->resetState(state.mActionState);
	}
}
void CSSprite::resetNavigator() 
{
	static char *functionName="resetNavigator";
	if (mNavigator!=0) 
	{
		mNavigator->reset();
	}
}

// does not display anything, but prepares (update) for next displaying
// calls right at the beginning navigateor->navigate() if set.
void CSSprite::next()
{
	static char *functionName="next";
	if (mNavigator != 0)
	{
		mNavigator->navigate(&(mState.mActionState.mAnimationState.mDisplayParams));
	}

	if (!mActive)
	{
		return;
	}

	if (mState.mCurrentAction == 0)
	{
		return;
	}

	if (!mState.mCurrentAction->next(mState.mActionState))
	{
		sendMessage(MESSAGE_SPRITE_ACTION_CHANGE);
		if (mState.mCurrentAction->getDefaultAction() != 0)
		{
			for (CSActions::iterator iter = mState.mCurrentBundle->mActions->begin(); iter != mState.mCurrentBundle->mActions->end(); iter++)
			{
				CSAction *action = *iter;
				if (action->isAction(mState.mCurrentAction->getDefaultAction()))
				{
					mState.mCurrentAction = action;
					mState.mCurrentAction->startAction(mState.mActionState);
					next();// next action invoked at once
					return;
				}
			}
		}
		mState.mCurrentAction = mState.mCurrentBundle->mDefaultAction;
		mState.mCurrentAction->startAction(mState.mActionState);
		next(); 
	}
}

// display sprite with current display values (changed by next())
void CSSprite::display()
{
	static char *functionName="display";
	if (!mActive)
	{
		return;
	}
	if (mState.mCurrentAction == 0)
	{
		return;
	}
	mState.mCurrentAction->display(mState.mActionState);
}

// where on the screen is 0,0 of our displaying	
// called from CSWorld
void CSSprite::setDisplayOffset(int xOffset, int yOffset)
{
	static char *functionName="setDisplayOffset";
	mState.mActionState.mAnimationState.mDisplayParams.mXDisplayStart = xOffset;
	mState.mActionState.mAnimationState.mDisplayParams.mYDisplayStart = yOffset;
}

// where in the world will we start drawing the screen
// called from CSWorld
void CSSprite::setWorldPosition(int x, int y)
{
	static char *functionName="setWorldPosition";
	mState.mActionState.mAnimationState.mDisplayParams.mXWorldStart = x;
	mState.mActionState.mAnimationState.mDisplayParams.mYWorldStart = y;
}

void CSSprite::setNavigator(CSNavigator *navigator)
{
	static char *functionName="setNavigator";
	mNavigator = navigator;
	navigator->setSprite(this);
}
// overrides values in animation
void CSSprite::setSpeedY(int speed)
{
	static char *functionName="setSpeedY";
	mState.mActionState.mAnimation->setSpeedY(speed,mState.mActionState.mAnimationState);
}

// overrides values in animation
void CSSprite::setSpeedX(int speed)
{
	static char *functionName="setSpeedX";
	mState.mActionState.mAnimation->setSpeedX(speed,mState.mActionState.mAnimationState);	
}

// overrides values in animation
void CSSprite::adjustSpeed(int speed)
{
	static char *functionName="adjustSpeed";
	mState.mActionState.mAnimation->adjustSpeed(speed,mState.mActionState.mAnimationState);
}

// returns to speed values of animation (default)
void CSSprite::resetSpeed() 
{
	static char *functionName="resetSpeed";
	mState.mActionState.mAnimation->resetSpeed(mState.mActionState.mAnimationState);
}

void CSSprite::setPosition(int x, int y, unsigned int z)
{
	static char *functionName="setPosition";
	mWorldPos = z;
	mState.mActionState.mAnimationState.mDisplayParams.mXPos = x;
	mState.mActionState.mAnimationState.mDisplayParams.mYPos = y;
	mState.mActionState.mAnimationState.mXPos = x;
	mState.mActionState.mAnimationState.mYPos = y;
}

unsigned int CSSprite::checkCollisionPixels(CSSprite *other)
{
	static char *functionName="checkCollisionPixels";
	// values of this
	int myX, myY, myW,myH;
	CSPicture *myPicture;
	
	// values of other
	int oX,oY, oW,oH;
	CSPicture *oPicture;

	myPicture = mState.mActionState.mAnimationState.mDisplayPicture;
	if (myPicture == 0) 
	{
		return 0; // sprite was never displayed!
	}
	myX = mState.mActionState.mAnimationState.mDisplayParams.mXPos;
	myY = mState.mActionState.mAnimationState.mDisplayParams.mYPos;
	myW = myPicture->getMaxX();
	myH = myPicture->getMaxY();

	oPicture = other->mState.mActionState.mAnimationState.mDisplayPicture;
	if (oPicture == 0) 
	{
		return 0; // sprite was never displayed!
	}
	oX = other->mState.mActionState.mAnimationState.mDisplayParams.mXPos;
	oY = other->mState.mActionState.mAnimationState.mDisplayParams.mYPos;
	oW = oPicture->getMaxX();
	oH = oPicture->getMaxY();

	if ((myX < oX) && (myX + myW < oX)) 
	{
		return 0;	// i am x smaller
	}
	if ((myX > oX) && (myX > oX + oW)) 
	{
		return 0;	// i am x greater
	}
	if ((myY < oY) && (myY + myH < oY)) 
	{
		return 0;	// i am y smaller
	}
	if ((myY > oY) && (myY > oY + oH)) 
	{
		return 0;	// i am y greater
	}

	if ((myPicture->getPixelProjectionCount() == 0) || (oPicture->getPixelProjectionCount() == 0))
	{
		return 0;
	}

	// sprites overlap in some way - now count the pixels
	// use pixel projection of current displayPicture(s)
	unsigned char **myProjection = myPicture->getProjection();
	unsigned char **oProjection = oPicture->getProjection();

	// now get the "rectangle" where the to sprites overlap
	// nad check only there!
	int xStart, xEnd;
	int yStart, yEnd;
	int offsetOx;
	int offsetOy;
	unsigned int count = 0;
	if (myX < oX) xStart = oX-myX;
	else xStart = 0;
	if (myX+myW > oX+oW) xEnd = myW - ((myX+myW) - (oX+oW));
	else xEnd = myW;

	if (myY < oY) yStart = oY-myY;
	else yStart = 0;
	if (myY+myH > oY+oH) yEnd = myH - ((myY+myH) - (oY+oH));
	else yEnd = myH;

	offsetOx = myX-oX;
	offsetOy = myY-oY;


	/*
	printf("\nyStart: %i", yStart);
	printf("\nxStart: %i", xStart);
	printf("\nyEnd: %i", yEnd);
	printf("\nxEnd: %i", xEnd);
	printf("\noffsetOy: %i", offsetOy);
	printf("\noffsetOx: %i", offsetOx);
	*/

	for (int y = yStart; y < yEnd; y++)
	{
		for (int x = xStart; x < xEnd; x++)
		{
			if ((myProjection[y][x] == 1) && (oProjection[y+offsetOy][x+offsetOx] == 1))
			{
				count++;
			}
		}
	}
/*
	printf("\n");
	for (y = yStart; y < yEnd; y++)
	{
		for (int x = xStart; x < xEnd; x++)
		{
			if (myProjection[y][x] == 1) 
				printf("1");
			else
				printf("0");
		}
		printf("\n");
	}

	printf("\n");
	for (y = yStart; y < yEnd; y++)
	{
		for (int x = xStart; x < xEnd; x++)
		{
			if (oProjection[y+offsetOy][x+offsetOx] == 1) 
				printf("1");
			else
				printf("0");
		}
		printf("\n");
	}
*/
	return count;
}

unsigned int CSSprite::checkCollisionPixels(CSTileMap *map)
{
	static char *functionName="checkCollisionPixels";
	// first get a list of all tiles, that are touched by the sprite
	int count = 0;
	int myX, myY, myW,myH;
	CSPicture *myPicture;
	myPicture = mState.mActionState.mAnimationState.mDisplayPicture;
	if (myPicture == 0) 
	{
		return 0; // sprite was never displayed!
	}
	myX = mState.mActionState.mAnimationState.mDisplayParams.mXPos;
	myY = mState.mActionState.mAnimationState.mDisplayParams.mYPos;
	myW = myPicture->getMaxX();
	myH = myPicture->getMaxY();

	CSTiles tiles = map->getTouchingTiles(myX,myY,myW,myH);
	for (CSTiles::iterator iter = tiles.begin(); iter != tiles.end(); iter++)
	{
	// than check them one by one
		count += checkCollisionPixels(*iter);
	}
	return count;
}

unsigned int CSSprite::checkCollisionPixels(CSTile *tile)
{
	static char *functionName="checkCollisionPixels";
	unsigned int count = 0;

	// values of this
	int myX, myY, myW,myH;
	CSPicture *myPicture;
	
	// values of tile
	myPicture = mState.mActionState.mAnimationState.mDisplayPicture;
	if (myPicture == 0) 
	{
		return 0; // sprite was never displayed!
	}
	myX = mState.mActionState.mAnimationState.mDisplayParams.mXPos;
	myY = mState.mActionState.mAnimationState.mDisplayParams.mYPos;
	myW = myPicture->getMaxX();
	myH = myPicture->getMaxY();

	int oX,oY, oW,oH;
	CSPicture *oPicture;
	oPicture = tile->getAnimationState()->mPicture;
	if (oPicture == 0) 
	{
		return 0; // sprite was never displayed!
	}
	oY = tile->getY();
	oH = tile->getHeight();
	oX = tile->getX();
	oW = tile->getWitdh();

	if ((myPicture->getPixelProjectionCount() == 0) || (oPicture->getPixelProjectionCount() == 0))
	{
		return 0;
	}

	// pictures overlap in some way - now count the pixels
	// use pixel projection of current displayPicture(s)
	unsigned char **myProjection = myPicture->getProjection();
	unsigned char **oProjection = oPicture->getProjection();

	// now get the "rectangle" where the to pictures overlap
	// and check only there!
	int xStart, xEnd;
	int yStart, yEnd;
	int offsetOx;
	int offsetOy;
	if (myX < oX) xStart = oX-myX;
	else xStart = 0;
	if (myX+myW > oX+oW) xEnd = myW - ((myX+myW) - (oX+oW));
	else xEnd = myW;

	if (myY < oY) yStart = oY-myY;
	else yStart = 0;
	if (myY+myH > oY+oH) yEnd = myH - ((myY+myH) - (oY+oH));
	else yEnd = myH;

	offsetOx = myX-oX;
	offsetOy = myY-oY;
	/*
	printf("\nyStart: %i", yStart);
	printf("\nxStart: %i", xStart);
	printf("\nyEnd: %i", yEnd);
	printf("\nxEnd: %i", xEnd);
	printf("\noffsetOy: %i", offsetOy);
	printf("\noffsetOx: %i", offsetOx);
	*/

	for (int y = yStart; y < yEnd; y++)
	{
		for (int x = xStart; x < xEnd; x++)
		{
			if ((myProjection[y][x] == 1) && (oProjection[y+offsetOy][x+offsetOx] == 1))
			{
				count++;
			}
		}
	}
/*
	printf("\n");
	for (y = yStart; y < yEnd; y++)
	{
		for (int x = xStart; x < xEnd; x++)
		{
			if (myProjection[y][x] == 1) 
				printf("1");
			else
				printf("0");
		}
		printf("\n");
	}

	printf("\n");
	for (y = yStart; y < yEnd; y++)
	{
		for (int x = xStart; x < xEnd; x++)
		{
			if (oProjection[y+offsetOy][x+offsetOx] == 1) 
				printf("1");
			else
				printf("0");
		}
		printf("\n");
	}
*/ 
	return count;
}

bool CSSprite::checkCollision(CSSprite *other)
{
	static char *functionName="checkCollision";
	// values of this
	int myX, myY, myW,myH;
	CSPicture *myPicture;

	myPicture = mState.mActionState.mAnimationState.mDisplayPicture;
	if (myPicture == 0) return 0; 
	{
		return 0; // sprite was never displayed!
	}
	myX = mState.mActionState.mAnimationState.mDisplayParams.mXPos;
	myY = mState.mActionState.mAnimationState.mDisplayParams.mYPos;
	myW = myPicture->getMaxX();
	myH = myPicture->getMaxY();

	// values of other
	int oX,oY, oW,oH;
	CSPicture *oPicture;
	oPicture = other->mState.mActionState.mAnimationState.mDisplayPicture;
	if (oPicture == 0) 
	{
		return 0; // sprite was never displayed!
	}
	oX = other->mState.mActionState.mAnimationState.mDisplayParams.mXPos;
	oY = other->mState.mActionState.mAnimationState.mDisplayParams.mYPos;
	oW = oPicture->getMaxX();
	oH = oPicture->getMaxY();

	if ((myX < oX) && (myX + myW < oX)) 
	{
		return 0;	// i am x smaller
	}
	if ((myX > oX) && (myX > oX + oW)) 
	{
		return 0;	// i am x greater
	}
	if ((myY < oY) && (myY + myH < oY)) 
	{
		return 0;	// i am y smaller
	}
	if ((myY > oY) && (myY > oY + oH)) 
	{
		return 0;	// i am y greater
	}

	if ((myPicture->getPixelProjectionCount() == 0) || (oPicture->getPixelProjectionCount() == 0))
	{
		return false;
	}

	// sprites overlap in some way - check if collide
	// use pixel projection of current displayPicture(s)
	unsigned char **myProjection = myPicture->getProjection();
	unsigned char **oProjection = oPicture->getProjection();

	// now get the "rectangle" where the to sprites overlap
	// and check only there!
	int xStart, xEnd;
	int yStart, yEnd;
	int offsetOx;
	int offsetOy;
	if (myX < oX) xStart = oX-myX;
	else xStart = 0;
	if (myX+myW > oX+oW) xEnd = myW - ((myX+myW) - (oX+oW));
	else xEnd = myW;

	if (myY < oY) yStart = oY-myY;
	else yStart = 0;
	if (myY+myH > oY+oH) yEnd = myH - ((myY+myH) - (oY+oH));
	else yEnd = myH;

	offsetOx = myX-oX;
	offsetOy = myY-oY;

	for (int y = yStart; y < yEnd; y++)
	{
		for (int x = xStart; x < xEnd; x++)
		{
			if ((myProjection[y][x] == 1) && (oProjection[y+offsetOy][x+offsetOx] == 1))
			{
				return true;
			}
		}
	}
	return false;
}

bool CSSprite::checkCollision(CSTileMap *map)
{
	static char *functionName="checkCollision";
	// first get a list of all tiles, that are touched by the sprite
	int myX, myY, myW,myH;
	CSPicture *myPicture;
	myPicture = mState.mActionState.mAnimationState.mDisplayPicture;
	if (myPicture == 0) 
	{
		return 0; // sprite was never displayed!
	}
	myX = mState.mActionState.mAnimationState.mDisplayParams.mXPos;
	myY = mState.mActionState.mAnimationState.mDisplayParams.mYPos;
	myW = myPicture->getMaxX();
	myH = myPicture->getMaxY();

	CSTiles tiles = map->getTouchingTiles(myX,myY,myW,myH);
	for (CSTiles::iterator iter = tiles.begin(); iter != tiles.end(); iter++)
	{
		if (checkCollision(*iter))
		{
			return true;
		}
	}
	return false;
}

bool CSSprite::checkCollision(CSTile *tile)
{
	static char *functionName="checkCollision";
	// values of this
	CSPicture *myPicture;
	int myX, myY, myW,myH;
	
	myPicture = mState.mActionState.mAnimationState.mDisplayPicture;
	if (myPicture == 0) 
	{
		return 0; // sprite was never displayed!
	}
	myX = mState.mActionState.mAnimationState.mDisplayParams.mXPos;
	myY = mState.mActionState.mAnimationState.mDisplayParams.mYPos;
	myW = myPicture->getMaxX();
	myH = myPicture->getMaxY();

	// values of tile
	CSPicture *oPicture;
	int oX,oY, oW,oH;
	oPicture = tile->getAnimationState()->mPicture;
	if (oPicture == 0) 
	{
		return 0; // sprite was never displayed!
	}
	oY = tile->getY();
	oH = tile->getHeight();
	oX = tile->getX();
	oW = tile->getWitdh();

	if ((myPicture->getPixelProjectionCount() == 0) || (oPicture->getPixelProjectionCount() == 0))
	{
		return false;
	}

	// sprites overlap in some way - check if collide
	// use pixel projection of current displayPicture(s)
	unsigned char **myProjection = myPicture->getProjection();
	unsigned char **oProjection = oPicture->getProjection();

	// now get the "rectangle" where the to sprites overlap
	// and check only there!
	int xStart, xEnd;
	int yStart, yEnd;
	int offsetOx;
	int offsetOy;
	if (myX < oX) xStart = oX-myX;
	else xStart = 0;
	if (myX+myW > oX+oW) xEnd = myW - ((myX+myW) - (oX+oW));
	else xEnd = myW;

	if (myY < oY) yStart = oY-myY;
	else yStart = 0;
	if (myY+myH > oY+oH) yEnd = myH - ((myY+myH) - (oY+oH));
	else yEnd = myH;

	offsetOx = myX-oX;
	offsetOy = myY-oY;

	for (int y = yStart; y < yEnd; y++)
	{
		for (int x = xStart; x < xEnd; x++)
		{
			if ((myProjection[y][x] == 1) && (oProjection[y+offsetOy][x+offsetOx] == 1))
			{
				return true;
			}
		}
	}
	return false;
}

CSTiles CSSprite::getCollidingTiles(CSTileMap *map)
{
	static char *functionName="getCollidingTiles";
	// first get a list of all tiles, that are touched by the sprite
	int myX, myY, myW,myH;
	CSPicture *myPicture;
	myPicture = mState.mActionState.mAnimationState.mDisplayPicture;
	if (myPicture == 0) 
	{
		return (CSTiles) 0; // sprite was never displayed!
	}
	myX = mState.mActionState.mAnimationState.mDisplayParams.mXPos;
	myY = mState.mActionState.mAnimationState.mDisplayParams.mYPos;
	myW = myPicture->getMaxX();
	myH = myPicture->getMaxY();

	CSTiles tiles = map->getTouchingTiles(myX,myY,myW,myH);
	CSTiles::iterator iter = tiles.begin();
	while (iter != tiles.end())
	{
		if (!checkCollision(*iter))
		{
			iter = tiles.erase(iter);
		}
		else
		{
			iter++;
		}
	}
	return tiles;
}
		
void CSSprite::reactOnMessage(CSMessage *message)
{
	static char *functionName="reactOnMessage";
	switch (message->getType())
	{
		case SPRITE_MESSAGE:
		{
			if (message->getSubtype() == SPRITE_STATE_CHANGE_MESSAGE)
			{
				SpriteMessage *sMessage = (SpriteMessage *) message;
				sMessage->setSuccess(changeStateTo(sMessage->state));
			}

			break;
		}
		case ACTION_MESSAGE:
		{
			// look for an action that
			// has as its envoker the subtype of the message!
			if (message->getSubtype() == DEFAULT_MESSAGE)
			{
				if (mState.mCurrentAction->getDefaultAction() != 0)
				{
					for (CSActions::iterator iter = mState.mCurrentBundle->mActions->begin(); iter != mState.mCurrentBundle->mActions->end(); iter++)
					{
						CSAction *action = *iter;
						if (action->isAction(mState.mCurrentAction->getDefaultAction()))
						{
							// next display NEXT round
							// since we are in eventHandling now!
							// wrong time to start drawing!
							mState.mCurrentAction = action;
							mState.mCurrentAction->startNextAction(mState.mActionState);
							return;
						}
					}
				}
				return;
			}
			if (mState.mCurrentAction->isBreakable(mState.mActionState))
			{
				for (CSActions::iterator iter = mState.mCurrentBundle->mActions->begin(); iter != mState.mCurrentBundle->mActions->end(); iter++)
				{
					CSAction *action = *iter;
					if (action->getEnvoker() == message->getSubtype())
					{
						if (action->isAction(mState.mCurrentAction->getId()))
						{
							return;
						}
						mState.mCurrentAction = action;
						// next display NEXT round
						// since we are in eventHandling now!
						//wrong time to start drawing!
						mState.mCurrentAction->startNextAction(mState.mActionState);
						return;
					}
				}
			}
		}
	}
}

void CSSprite::loadSpriteData(const std::string &filename, CSSpriteData &data)
{
	static char *functionName="loadSpriteData";
	LOG_ENTER 
	char tmp[10];
	char tmp2[10];
	int i=-1,j=-1;
	
	CSXMLHelper xmlSupport(filename, "SPRITE");
	try
	{
		if (xmlSupport.getError())
		{
			LOG_EXIT
			throw "error";
		}

		data.id = strdup(xmlSupport.getString("ID").c_str());
		data.defaultBundle = xmlSupport.getInt("/SPRITE/DEFAULT_STATE");
		data.layer_position = xmlSupport.getInt("/SPRITE/LAYER_POSITION");
		data.y_position = xmlSupport.getInt("/SPRITE/Y_POSITION");

		data.type = xmlSupport.getInt("/SPRITE/TYPE");
		data.subtype = xmlSupport.getInt("/SPRITE/SUBTYPE");
		
	
		int stateCount = xmlSupport.getInt("count(/SPRITE/STATE)");
		for (i=0; i<stateCount; i++)
		{
			StateData *stateData = new StateData();
			// i+1, predicates in xpath start with 1
			itoa(i+1,tmp,10);
			stateData->stateId = xmlSupport.getInt((std::string)"/SPRITE/STATE["+tmp+"]/STATE_ID");
			stateData->defaultActionName = strdup(xmlSupport.getString((std::string)"/SPRITE/STATE["+tmp+"]/DEFAULT_ACTION").c_str());

			int actionCount = xmlSupport.getInt((std::string)"count(/SPRITE/STATE["+tmp+"]/ACTION)");

			for (j=0; j<actionCount; j++)
			{
				// j+1, predicates in xpath start with 1
				itoa(j+1,tmp2,10);
				std::string t = (std::string)"/SPRITE/STATE["+tmp+"]/ACTION["+tmp2+"]";
				std::string a = xmlSupport.getString(t);
				stateData->actionNames.push_back(strdup(a.c_str()));
			}

			data.stateDatas.push_back(stateData);
		}
	}
	catch(...)
	{
		char tmp3[100];
		sprintf(tmp3,"i=%i, j=%i",i,j);
		
		LOG_EXIT
		SDLMain::shutdown((std::string)"XML error \"" + filename + "\": "+tmp3 + xmlSupport.getErrorMessage().c_str(), 1);
	}
	LOG_EXIT
}

