#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

#include "CSTile.h"
#include "CSXMLHelper.h"

CSTileLoader CSTileLoader::INSTANCE;
const char *CSTile::CLASS = "CSTile";

CSTile::CSTile(const std::string &filename)
{
	static char *functionName="CSTile";
	LOG_ENTER 
	CSTileData data;
	loadTileData(filename, data);
	initialize(data);
	LOG_EXIT
}

CSTile::CSTile(const CSTile &tile)
{
	static char *functionName="CSTile";
	LOG_ENTER 
	initialize();

	// since all Animations are singletons, we can reuse the animation from our parent
	mAnimation = tile.mAnimation;
	mAnimationState = tile.mAnimationState;
	mId = strdup(tile.mId);
	mIdChar = tile.mIdChar;
	mIsSolid = tile.mIsSolid;
	mType = tile.mType;
	mSubtype = tile.mSubtype;
	LOG_EXIT
}

void CSTile::initialize(void)
{
	static char *functionName="initialize";
	LOG_ENTER 
	mId = 0;
	mType = 0;
	mSubtype = 0;
	mAnimation = 0;
	mMapPosX = 0;
	mMapPosY = 0;
	mIsSolid = false;
	mChanged = false;
	LOG_EXIT
}

void CSTile::initialize(const CSTileData &data)
{
	static char *functionName="initialize";
	LOG_ENTER 
	initialize();
	mAnimation = CSAnimationLoader::INSTANCE.load(data.animationName);
	mAnimation->resetState(mAnimationState);
	mId = strdup(data.id);
	mChanged = false;
	mIdChar = data.idChar;
	mType = data.type;
	mSubtype = data.subtype;
	LOG_EXIT
}

CSTile::~CSTile()
{
	static char *functionName="~CSTile";
	LOG_ENTER 
	// since all Animations are singeltons!
	mAnimation = 0;
	if (mId != 0)
	{
		free(mId);
		mId = 0;
	}
	LOG_EXIT
}

void CSTile::setSecPerFrame(float secPerFrame)
{
	static char *functionName="setSecPerFrame";
	LOG_ENTER 
	mAnimation->setSecPerFrame(secPerFrame);
	LOG_EXIT
}

void CSTile::setSolid(bool b)
{
	static char *functionName="setSolid";
	LOG_ENTER 
	mIsSolid = b;
	mAnimation->setSolid(mIsSolid);
	LOG_EXIT
}

void CSTile::loadTileData(const std::string &filename, CSTileData &data)
{
	static char *functionName="loadTileData";
	LOG_ENTER 
	CSXMLHelper xmlSupport(filename, "TILE");
	try
	{
		if (xmlSupport.getError())
		{
			LOG_EXIT
			throw "error";
		}

		data.id = strdup(xmlSupport.getString("ID").c_str());
		data.idChar = xmlSupport.getString("ID_CHAR").c_str()[0];
		data.animationName = strdup(xmlSupport.getString("ANIMATION").c_str());
		data.type = xmlSupport.getInt("TYPE");
		data.subtype = xmlSupport.getInt("SUBTYPE");
	}
	catch(...)
	{
		LOG_EXIT
		SDLMain::shutdown((std::string)"XML error \"" + filename + "\": " + xmlSupport.getErrorMessage().c_str(), 1);
	}
	LOG_EXIT
}

void CSTile::scale(int width, int tilesX, int height, int tilesY)
{
	static char *functionName="scale";
	LOG_ENTER 
	mAnimation->scale(width, tilesX, height, tilesY);
	LOG_EXIT
}
