/**
*/

#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

#include "CSTileMap.h"
#include "CSXMLHelper.h"

CSTileMapLoader CSTileMapLoader::INSTANCE;
const char *CSTileMap::CLASS = "CSTileMap";

CSTileMap::CSTileMap(const std::string &filename)
{
	static char *functionName="CSTileMap";
	LOG_ENTER 
	CSTileMapData data;
	loadTileMapData(filename, data);
	initialize(data);
	LOG_EXIT
}

// not Done!
CSTileMap::CSTileMap(const CSTileMap &tileMap)
{
	static char *functionName="CSTileMap";
	LOG_ENTER 
	initialize();
	mId = strdup(tileMap.mId);
	LOG_EXIT
}

void CSTileMap::initialize(void)
{
	static char *functionName="initialize";
	LOG_ENTER 
	mScaleDisplay = false;
	mR = -1;
	mG = -1;
	mB = -1;
	mROrg = mR;
	mBOrg = mB;
	mGOrg = mG;
	mFactor = 0;
	mIsChangeable = false;
	mWorldNo = 0;
	mScaledMap = 0;
	mIsSolid = false;
	mId = 0;
	mTileSet = 0;
	mMap = 0;
	mRows = 0;
	mHeight = 0;
	mWidth = 0;
	mIsBackground = false;
	mIsParallax = false;
	mDisplayWidth = 0;
	mDisplayHeight = 0;
	mDisplayParams.mXDisplayStart = 0;
	mDisplayParams.mYDisplayStart = 0;
	mDisplayParams.mXWorldStart = 0;
	mDisplayParams.mYWorldStart = 0;
	mDisplayParams.mXPos = 0;
	mDisplayParams.mYPos = 0;
	mActive = false;
	LOG_EXIT
}

void CSTileMap::initialize(const CSTileMapData &data)
{
	static char *functionName="initialize";
	LOG_ENTER 
	int xPos = 0;
	int yPos = 0;
	mXSize = 0;
	mYSize = 0;
	initialize();
	mId = strdup(data.id);
	mIsChangeable = data.isChangebale;

	mTileSet = CSTileSetLoader::INSTANCE.load(data.tileSet);
	mRows = new StringVector();
	StringVector::iterator iter;
	for (iter = data.rows->begin(); iter != data.rows->end(); iter++)
	{
		char *row = (char *) *iter;
		mRows->push_back(strdup(row));
		if (mXSize < strlen(row))
		{
			mXSize = strlen(row);
		}

		mYSize++;
	}

	mMap = (CSTile ***) new void *[mYSize];
	
	iter = mRows->begin(); 
	for (int y=0; y< mYSize; y++)
	{
		char *row = (char *) *iter;
		mMap[y] = (CSTile **) new void* [mXSize];
		int yh = 0;
		xPos = 0;
		for (int x = 0; x < mXSize; x++)
		{
			if (x < strlen(row))
			{
				if (row[x] == 0)
				{
					LOG_EXIT
					SDLMain::shutdown((std::string)"TileMap includes characters not found in TileSet! ", 1);
				}
				mMap[y][x] = mTileSet->getTile(row[x]);
				mMap[y][x]->setChanged(true);
				mMap[y][x]->setX(xPos);
				mMap[y][x]->setY(yPos);
				mMap[y][x]->setMapPosition(x,y);
				xPos += mMap[y][x]->getWitdh();
				if (yh < mMap[y][x]->getHeight())
				{
					yh = mMap[y][x]->getHeight();
				}
			}
			else
			{
				mMap[y][x] = 0;
			}
		}
		yPos += yh;
		iter++;
	}
	mHeight = yPos;
	mWidth = xPos; // assumes all tiles have the same size -> which is not a MUST!
	mWorldHeight = mHeight;
	mWorldWidth = mWidth;
	if (!(mMap[0][0]->getAnimationState()->mPicture->isSolid()))
	{
		mMap[0][0]->getAnimationState()->mPicture->getRGB(mR,mB,mG);
		mROrg = mR;
		mBOrg = mB;
		mGOrg = mG;
	}
	LOG_EXIT
}

void CSTileMap::setOffset(int offsetX, int offsetY)
{
	static char *functionName="setOffset";
	LOG_ENTER 
	mOffsetX = offsetX;
	mOffsetY = offsetY;

	int xPos = 0;
	int yPos = 0;

	for (int y=0; y< mYSize; y++)
	{
		int yh = 0;
		for (int x = 0; x < mXSize; x++)
		{
			if (mMap[y][x] != 0)
			{
				mMap[y][x]->setX(xPos + mOffsetX);
				mMap[y][x]->setY(yPos + mOffsetY);
				xPos += mMap[y][x]->getWitdh();
				if (yh < mMap[y][x]->getHeight())
				{
					yh = mMap[y][x]->getHeight();
				}
			}
		}
		yPos += yh;
		xPos = 0;
	}
	LOG_EXIT
}

void CSTileMap::setSolid(bool b)
{
	static char *functionName="setSolid";
	LOG_ENTER 
	mIsSolid = b;
	mTileSet->setSolid(b);
	if (b)
	{
		mR=mG=mB = -1;
	}
	else
	{
		mR = mROrg;
		mB = mBOrg;
		mG = mGOrg;
	}
	LOG_EXIT
}

void CSTileMap::setSecPerFrame(float secPerFrame)
{
	static char *functionName="setSecPerFrame";
	LOG_ENTER 
	mTileSet->setSecPerFrame(secPerFrame);
	LOG_EXIT
}

CSTileMap::~CSTileMap()
{
	static char *functionName="~CSTileMap";
	LOG_ENTER 
	if (mScaledMap != 0)
	{
		SDLMain::freeSurface(mScaledMap);
		mScaledMap = 0;
	}

	if (mId != 0)
	{
		free(mId);
		mId = 0;
	}
	if (mTileSet != 0)
	{
		// TileSet are Singletons!
		mTileSet = 0;
	}
	if (mRows != 0)
	{
		for (StringVector::iterator iter = mRows->begin(); iter != mRows->end(); iter++)
		{
			char *help = (char *) *iter;
			free(help);
		}
		free(mRows);
		mRows = 0;
	}

	for (int y=0; y< mYSize; y++)
	{
		for (int x = 0; x < mXSize; x++)
		{
			if (mMap[y][x] != 0)
			{
				delete mMap[y][x];
			}
		}
		delete[] mMap[y];
	}
	delete[] mMap;
	LOG_EXIT
}

// this is more an exchangeTile(), since it only does something, if there is allready a tile
// on the x,y position!
// deletes old tile at position!
void CSTileMap::setTile(unsigned int x, unsigned int y, char idChar)
{
	static char *functionName="setTile";
	if (!mIsChangeable)
	{
		return;
	}
	if (mMap[y][x] != 0)
	{
		int xPos = mMap[y][x]->getX();
		int yPos = mMap[y][x]->getY();
		delete mMap[y][x];
		mMap[y][x] = mTileSet->getTile(idChar);
		mMap[y][x]->setChanged(true);
		mMap[y][x]->setX(xPos);
		mMap[y][x]->setY(yPos);
		mMap[y][x]->setMapPosition(x,y);
	}
}

// this is more an exchangeTile(), since it only does something, if there is allready a tile
// on the x,y position!
// does not delete old tile at position!
// if center is true AND the tileset is set to "IsOneSize" the tile that
// is set, is "centered" at the position "regardless" of its own size.
void CSTileMap::setTile(unsigned int x, unsigned int y, CSTile *tile, bool center)
{
	static char *functionName="setTile";
	if (!mIsChangeable)
	{
		return;
	}
	if (mMap[y][x] != 0)
	{
		int xPos;
		int yPos;
		if ((center) && (mTileSet->isOneSize()))
		{
			xPos = x * mTileSet->getTileXSize() + mOffsetX;
			yPos = y * mTileSet->getTileYSize() + mOffsetY;
			
			int nh = tile->getHeight();
			int nw = tile->getWitdh();
			xPos += (mTileSet->getTileXSize()-nw)/2;
			yPos += (mTileSet->getTileYSize()-nh)/2;
		}
		else
		{
		xPos = mMap[y][x]->getX();
		yPos = mMap[y][x]->getY();
		}
		mMap[y][x] = tile;
		mMap[y][x]->setChanged(true);
		mMap[y][x]->setX(xPos);
		mMap[y][x]->setY(yPos);
		mMap[y][x]->setMapPosition(x,y);
	}
}

CSTile *CSTileMap::getTileAtPixel(unsigned int tx, unsigned int ty)
{
	static char *functionName="getTileAtPixel";
	int x, y;
	CSTile *tile;
	for (y=0; y< mYSize; y++)
	{
		tile = mMap[y][0];
		if (ty > tile->getY() + tile->getHeight()) continue;	// i am y greater

		for (x = 0; x < mXSize; x++)
		{
			if (mMap[y][x] != 0)
			{
				tile = mMap[y][x];
				if (tx > tile->getX() + tile->getWitdh()) continue;	// i am x greater
				return mMap[y][x];
			}
		}
	}
	return 0;
}

CSTiles CSTileMap::getTouchingTiles(int tx,int ty, int tw, int th)
{
	static char *functionName="getTouchingTiles";
	// assumes all tiles have the same size!
	CSTiles tiles;
	for (int y=0; y< mYSize; y++)
	{
		
		CSTile *tile = mMap[y][0];
		int oY = tile->getY();
		int oH = tile->getHeight();
		if ((ty < oY) && (ty + th < oY)) 
		{
			return tiles;	// i am y smaller
		}
		if ((ty > oY) && (ty > oY + oH)) continue;	// i am y greater

		for (int x = 0; x < mXSize; x++)
		{
			if (mMap[y][x] != 0)
			{
				CSTile *tile = mMap[y][x];
				int oX = tile->getX();
				int oW = tile->getWitdh();

				if ((tx < oX) && (tx + tw < oX)) break;	// i am x smaller
				if ((tx > oX) && (tx > oX + oW)) continue;	// i am x greater
		
				tiles.push_back(mMap[y][x]);
			}
		}
	}
	return tiles;
}

void CSTileMap::display()
{
	static char *functionName="display";
	int y,x;
	CSTile *tile;
	
	for (y=sy; y< ey; y++)
	{
		for (x = sx; x < ex; x++)
		{
			tile = mMap[y][x];
			if (tile)
			{
				tile->display();
			}
		}
	}
	CSSprites::iterator iter = mSprites.begin();
	while (iter != mSprites.end())
	{
		CSSprite *sprite = *iter;
		if (mWorldNo == sprite->getWorldPos())
		{
			sprite->display();
		}
		iter++;
	}
}

void CSTileMap::update()
{
	static char *functionName="update";
	if (mActive)
	{
		int y,x;
		CSTile *tile;
		sx = 0;
		sy = 0;
		ex = mXSize;
		ey = mYSize;
		int startX = mDisplayParams.mXWorldStart;
		int startY = mDisplayParams.mYWorldStart;
		int endX = startX + mDisplayWidth;
		int endY = startY + mDisplayHeight;

		// assuming tiles have appr. same size
		for (y = 0; ((y<mYSize) && (startY > (int) mMap[y][0]->getY()+(int)mMap[y][0]->getHeight())); y++);
		for (x = 0; ((x<mXSize) && (startX > (int) mMap[0][x]->getX()+(int)mMap[0][x]->getWitdh())); x++);

		sx = x;
		sy = y;

		for (; ((y<mYSize) && (endY > (int) mMap[y][0]->getY())); y++);
		for (; ((x<mXSize) && (endX > (int) mMap[0][x]->getX())); x++);
		ex = x; // changed from x;
		ey = y; // changed from y;

		for (y=sy; y< ey; y++)
		{
			for (x = sx; x < ex; x++)
			{
				tile = mMap[y][x];
				if (tile)
				{
					tile->next(mDisplayParams);
				}
			}
		}
	}
	CSSprites::iterator iter = mSprites.begin();
	while (iter != mSprites.end())
	{
		CSSprite *sprite = *iter;
		if (mWorldNo == sprite->getWorldPos())
		{
			sprite->next();
		}
		iter++;
	}
}

void CSTileMap::setMapPosition(int px, int py)
{
	static char *functionName="setMapPosition";
	if (mIsBackground)
	{
		px = 0;
		py = 0;
	}
	else if (mIsParallax)
	{
		/*
		1
		float scalex = (float) (((float) mWidth-mDisplayWidth) )/ ((float)(mWorldWidth-mDisplayWidth)) ;
		float scaley = (float)(((float) mHeight-mDisplayHeight)) / ((float)(mWorldHeight-mDisplayHeight)) ;
		px = mWorldWidth - px;
		py = mWorldHeight - py;
		px = ((float) px) * scalex;
		py = ((float) py) * scaley;
		*/

		px = (float)(((float) mWidth-mDisplayWidth) * (float)((float)( ((float) px) / ((float)(mWorldWidth-mDisplayWidth)) )));
		py = (float)(((float) mHeight-mDisplayHeight) * (float)((float)( ((float) py) / ((float)(mWorldHeight-mDisplayHeight)) )));
	}
	mDisplayParams.mXWorldStart = px;
	mDisplayParams.mYWorldStart = py;
}

void CSTileMap::setDisplayOffset(int xOffset, int yOffset)
{
	static char *functionName="setDisplayOffset";
	mDisplayParams.mXDisplayStart = xOffset;
	mDisplayParams.mYDisplayStart = yOffset;
	CSSprites::iterator iter = mSprites.begin();
	while (iter != mSprites.end())
	{
		CSSprite *sprite = *iter;
		sprite->setDisplayOffset(xOffset, yOffset);
		iter++;
	}
}

void CSTileMap::addSprite(CSSprite *sprite)
{
	static char *functionName="addSprite";
	LOG_ENTER 
	// savety meassure, not to add twice!
	removeSprite(sprite);
	mSprites.push_back(sprite);
	sprite->setDisplayOffset(mDisplayParams.mXDisplayStart, mDisplayParams.mYDisplayStart);
	LOG_EXIT
}

void CSTileMap::removeSprite(CSSprite *sprite)
{
	static char *functionName="removeSprite";
	LOG_ENTER 
	CSSprites::iterator iter = mSprites.begin();
	while (iter != mSprites.end())
	{
		if ((*iter) == sprite)
		{
			mSprites.erase(iter);
			iter = mSprites.begin();	// restart!
		}
		else
		{
			iter++;
		}
	}
	LOG_EXIT
}

void CSTileMap::loadTileMapData(const std::string &filename, CSTileMapData &data)
{
	static char *functionName="loadTileMapData";
	LOG_ENTER 
	char tmp[10];
	CSXMLHelper xmlSupport(filename, "TILEMAP");
	try
	{
		if (xmlSupport.getError())
		{
			LOG_EXIT
			throw "error";
		}

		data.id = strdup(xmlSupport.getString("ID").c_str());
		data.tileSet = strdup(xmlSupport.getString("SET").c_str());

		data.isChangebale = xmlSupport.getInt("CHANGEABLE") != 0;
		int rowCount = xmlSupport.getInt("count(ROWS/ROW)");
		for (int i=0; i<rowCount; i++)
		{
			// i+1, predicates in xpath start with 1
			std::string a = xmlSupport.getString((std::string)"ROWS/ROW["+itoa(i+1,tmp,10)+"]");
			data.rows->push_back(strdup(a.c_str()));
		}
	}
	catch(...)
	{
		LOG_EXIT
		SDLMain::shutdown((std::string)"XML error \"" + filename + "\": " + xmlSupport.getErrorMessage().c_str(), 1);
	}
	LOG_EXIT
}

void CSTileMap::setBackground(bool b) 
{
	static char *functionName="setBackground";
	LOG_ENTER 
	mIsBackground = b;
	LOG_EXIT
}

void CSTileMap::scaleMapToFit(int width, int height)
{
	static char *functionName="scaleMapToFit";
	LOG_ENTER 
	mTileSet->scaleToFit(width, mXSize,height,mYSize);
	mHeight = height;
	mWidth = width;
	mWorldHeight = mHeight;
	mWorldWidth = mWidth;
	LOG_EXIT
}

void CSTileMap::resetScaledMap()
{
	static char *functionName="resetScaledMap";
	LOG_ENTER 
	if (mScaledMap != 0)
	{
		SDLMain::freeSurface(mScaledMap);
		mScaledMap = 0;
	}
	for (int y=0; y< mYSize; y++)
	{
		int yh = 0;
		for (int x = 0; x < mXSize; x++)
		{
			if (mMap[y][x] != 0)
			{
				mMap[y][x]->setChanged(true);
			}
		}
	}
	LOG_EXIT
}

// scaled Map has color key of only the first tile!
// parallax in scaled map not supported
SDL_Surface *CSTileMap::getScaledMap(double factor, bool isSolid) 
{
	static char *functionName="getScaledMap";
	if ((!mIsChangeable) && (mFactor == factor) && (mScaledMap != 0))
	{
		return mScaledMap;
	}

	if (!((mFactor == factor) && (mScaledMap != 0)))
	{
		if (mScaledMap != 0)
		{
			SDLMain::freeSurface(mScaledMap);
			mScaledMap = 0;
		}
 		int width = ((double) mWorldWidth)*factor;
		int height = ((double) mWorldHeight)*factor;
		int flags = SDLMain::getScreen()->flags;
		
		// don't know why this is needed! Windows?
		if ((flags & SDL_HWSURFACE) == SDL_HWSURFACE)
		{
			flags -= SDL_HWSURFACE;
		}
		mScaledMap = SDL_CreateRGBSurface(flags, 
								   width, 
								   height, 
								   SDLMain::getScreen()->format->BitsPerPixel, 
								   SDLMain::getScreen()->format->Rmask, 
								   SDLMain::getScreen()->format->Gmask, 
								   SDLMain::getScreen()->format->Bmask, 
								   SDLMain::getScreen()->format->Amask);

		if ((!(mMap[0][0]->getAnimationState()->mPicture->isSolid())) && (!isSolid))
		{
			if (mR+mG+mB != -3)
			{
				int mColorKey = SDL_MapRGB(mScaledMap->format, mR, mG, mB);
				SDL_SetColorKey(mScaledMap, SDL_SRCCOLORKEY, mColorKey);
			}
		}
	}
	mFactor = factor;

	SDL_Rect dstrect;

	int y,x,h;
	int ox = (double)(((double)(mOffsetX))*factor);
	int oy = (double)(((double)(mOffsetY))*factor);

	dstrect.y = oy;
	for (y=0; y< mYSize; y++)
	{
		dstrect.x = ox;
		h = 0;
		for (x = 0; x < mXSize; x++)
		{
			SDL_Surface *src = mMap[y][x]->getScaledTile(factor);
			dstrect.w = src->w;
			dstrect.h = src->h;
			if (dstrect.h > h)
			{
				h = dstrect.h;
			}
			
			// during a scale at runtime-> all tiles are
			// probably a unchanged, but we need an 
			// update of all tiles -> so we better don't even ask!
			// if (mMap[y][x]->isChanged())
			{
				//mMap[y][x]->setChanged(false);
				if (mR+mG+mB != -3)
				{
					int mColorKey = SDL_MapRGB(mScaledMap->format, mR, mG, mB);
					SDL_FillRect(mScaledMap, &dstrect, mColorKey);
				}

				SDL_BlitSurface(src, 0, mScaledMap, &dstrect);
			}
			dstrect.x += dstrect.w;
		}
		dstrect.y += h;
	}
	return mScaledMap;
}

void CSTileMap::addScaledSprites(double factor, const SDL_Rect &offsetRect, SDL_Surface *destination, const SDL_Rect &displayRect)
{
	static char *functionName="addScaledSprites";
	CSSprites::iterator iter = mSprites.begin();
	SDL_Rect dstrect;
	while (iter != mSprites.end())
	{
		CSSprite *sprite = *iter;
		if (mWorldNo == sprite->getWorldPos())
		{
			SDL_Surface *src = sprite->getScaledSprite(factor);
			register int x = sprite->getState().mActionState.mAnimationState.mDisplayParams.mXPos;
			register int y = sprite->getState().mActionState.mAnimationState.mDisplayParams.mYPos;
			dstrect.x = - offsetRect.x  + displayRect.x + (double)(((double)(x))*factor);
			dstrect.y = - offsetRect.y + displayRect.y + (double)(((double)(y))*factor);
			dstrect.w = src->w;
			dstrect.h = src->h;

			SDL_BlitSurface(src, 0, destination, &dstrect);
		}
		iter++;
	}
}

