#ifndef CSTILEMAP_H
#define CSTILEMAP_H

// USES SDL_Surface
// USES SDL_Rect 

#include <string>
#include <vector>
#include "CSTypes.h"
#include "SDLMain.h"
#include "CSTileSet.h"
#include "CSSprite.h"
#include "CSLoadable.h"
#include "CSLog.h"

class CSTileMapLoader;
class CSTileMap;
typedef std::vector<CSTileMap *> CSTileMaps;

class CSTileMapData
{
	public:
		char *id;					// defining 
		char *tileSet;				// defining 
		StringVector *rows;
		bool isChangebale;

		CSTileMapData()
		{
			id = 0;
			isChangebale = false;
			tileSet = 0;
			rows = new StringVector();
		}

		~CSTileMapData()
		{
			if (id != 0)
			{
				free (id);
				id = 0;
			}
			if (tileSet != 0)
			{
				free (tileSet);
				tileSet = 0;
			}
			if (rows != 0)
			{
				for (StringVector::iterator iter = rows->begin(); iter != rows->end(); iter++)
				{
					char *help = (char *) *iter;
					free(help);
				}
				free(rows);
			}
		}
};

class CSTileMap
{
	friend CSTileMapLoader;
	private:
		int sx;
		int sy;
		int ex;
		int ey;

		CSSprites mSprites;
		SDL_Surface *mScaledMap;
		
		char *mId;							// defining 
		CSTileSet *mTileSet;
		CSTile ***mMap;
		StringVector *mRows;
		unsigned int mXSize;	// in Tiles
		unsigned int mYSize;	// in Tiles
		unsigned int mHeight;	// in Pixel
		unsigned int mWidth;	// in Pixel
		unsigned int mWorldHeight;// in Pixel
		unsigned int mWorldWidth;// in Pixel
		bool mActive;
		bool mScaleDisplay;

		int mOffsetX;
		int mOffsetY;
		bool mIsChangeable;
		double mFactor;
		bool mIsBackground;
		bool mIsParallax;
		unsigned int mDisplayWidth;
		unsigned int mDisplayHeight;
		unsigned int mWorldNo;
		bool mIsSolid;
		int mR;
		int mG;
		int mB;
		int mROrg;
		int mBOrg;
		int mGOrg;
		CSDisplayParams mDisplayParams;

		// constructor using "*.act" file
		CSTileMap(const std::string &filename);
		static void loadTileMapData(const std::string &filename, CSTileMapData &data);

		void initialize(const CSTileMapData &data);
		void initialize();
	
	public:
		CSTileMap(const CSTileMap &tileMap);
		virtual ~CSTileMap();
		static const char *CLASS;
		virtual std::string getType() {return (std::string) CLASS;}

		void update();
		void display();
		void setSolid(bool b);
		void setChangeable(bool b) {mIsChangeable = b;}
		bool isChangeable(void) {return mIsChangeable;}
		bool isSolid() {return mIsSolid;}
		void setSecPerFrame(float secPerFrame);
		
		void setWorldNo(unsigned int no){mWorldNo = no;}
		void setDisplayOffset(int xOffset, int yOffset);
		void setMapPosition(int x, int y);
		void addSprite(CSSprite *sprite);
		void removeSprite(CSSprite *sprite);
		CSTiles getTouchingTiles(int x,int y, int w, int h);

		void setScaleDisplay(bool b){mScaleDisplay = b;}
		bool isScaleDisplay(void){return mScaleDisplay;}
		void addScaledSprites(double factor, const SDL_Rect &srcRect, SDL_Surface *destination, const SDL_Rect &displayRect);
		SDL_Surface *getScaledMap(double factor, bool isSolid);
		void resetScaledMap();

		void setOffset(int offsetX, int offsetY);
		unsigned int getHeight(){return mHeight;}
		unsigned int getWidth(){return mWidth;}
		unsigned int getTileHeight(){return mYSize;}
		unsigned int getTileWidth(){return mXSize;}
		bool isBackground(void) {return mIsBackground;}
		void setBackground(bool b);
		void scaleMapToFit(int width, int height);
		void setParallax(bool b) {mIsParallax = b;}
		void setWorldMax(unsigned int width, unsigned int height) {mWorldWidth = width;mWorldHeight = height;}
		void setDisplaySize(unsigned int w, unsigned int h) {mDisplayWidth = w;mDisplayHeight = h;}
		void setTile(unsigned int x, unsigned int y, char idChar); // in Map Position old tile on position deleted
		void setTile(unsigned int x, unsigned int y, CSTile *tile, bool center = true); // in Map Position old tile on position NOT deleted! (is overwritten!)
		bool isBorderTile(CSTile *tile) 
		{
		
			if ((tile->getX()==0) || (tile->getY()==0)) return true;
			if ((tile->getX()==mXSize-1) || (tile->getY()==mYSize-1)) return true;
			return false;
		}
		CSTile *getTileAtPixel(unsigned int x, unsigned int y); // x, y in pixel!
		CSTile *getTile(unsigned int tx, unsigned int ty) {return mMap[ty][tx];}
		CSTileSet *getTileSet() {return mTileSet;}
		
		void setActive(bool active) 
		{
			mActive = active;
			CSSprites::iterator iter = mSprites.begin();
			while (iter != mSprites.end())
			{
				CSSprite *sprite = *iter;
				if (mWorldNo == sprite->getWorldPos())
				{
					sprite->setActive(mActive) ;
				}
				iter++;
			}
		}
};		

class CSTileMapLoader : public Loadable<CSTileMap>
{
	protected:
		virtual bool isSingleCreate(void) const
		{
			return false;
		}

		virtual CSTileMap *create(const std::string &filename)
		{
			return new CSTileMap(filename);
		}

	public:
		static CSTileMapLoader INSTANCE;
};

#endif // CSTILEMAP_H
