#ifndef CSTILESET_H
#define CSTILESET_H

#include <string>
#include <vector>
#include "CSTypes.h"
#include "SDLMain.h"
#include "CSTile.h"
#include "CSLoadable.h"
#include "CSLog.h"

class CSTileSetLoader;
class CSTileSet;
typedef std::vector<CSTileSet *> CSTileSets;

class CSTileSetData
{
	public:
		char *id;					// defining 
		StringVector *tileNames;
		bool onesize;
		int width;
		int height;

		CSTileSetData()
		{
			id = 0;
			onesize=false;
			width=0;
			height=0;
			tileNames = new StringVector();
		}

		~CSTileSetData()
		{
			if (id != 0)
			{
				free (id);
				id = 0;
			}

			if (tileNames != 0)
			{
				for (StringVector::iterator iter = tileNames->begin(); iter != tileNames->end(); iter++)
				{
					char *help = (char *) *iter;
					free(help);
				}
				free(tileNames);
			}
		}
};

const int MAX_TILE = 256;
class CSTileSet
{
	friend CSTileSetLoader;
	private:
		CSTile *mTiles[MAX_TILE];
		int mTileCount;
		char *mId;							// defining 
		bool mIsSolid;
		bool mIsOneSize;
		int mXSize;
		int mYSize;
		float mSecPerFrame;
		// constructor using "*.xml" file
		CSTileSet(const std::string &filename);
		static void loadTileSetData(const std::string &filename, CSTileSetData &data);

		void initialize(const CSTileSetData &data);
		void initialize();
	
	public:
		CSTileSet(const CSTileSet &tileSet);
		virtual ~CSTileSet();
		static const char *CLASS;
		virtual std::string getType() {return (std::string) CLASS;}

		CSTile *getTile(unsigned char c) 
		{
			if (mTiles[c] == 0)
			{
				SDLMain::shutdown((std::string)"TileSet characters not found in TileSet!", 1);
			}
			return new CSTile(*mTiles[c]);
		}	// return a new Tile -> must be freed by outside!
		void setSolid(bool b);
		void setSecPerFrame(float secPerFrame);
		int getTileCount() {return mTileCount;}
		void scaleToFit(int width, int tilesX, int height, int tilesY);
		int getFreeTile();
		int getMaxTileSize() {return MAX_TILE;}
		void setTile(CSTile* tile, unsigned char c);
		bool isOneSize() {return mIsOneSize;}
		int getTileXSize() {return mXSize;}
		int getTileYSize() {return mYSize;}
	
};		

class CSTileSetLoader : public Loadable<CSTileSet>
{
	protected:
		virtual CSTileSet *create(const std::string &filename)
		{
			return new CSTileSet(filename);
		}

	public:
		static CSTileSetLoader INSTANCE;
};

#endif // CSTILESET_H
