/**
*/

#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

#include "CSWorld.h"
#include "CSXMLHelper.h"
#include "CSMusic.h"

CSWorldLoader CSWorldLoader::INSTANCE;
const char *CSWorld::CLASS = "CSWorld";

CSWorld::CSWorld(const std::string &filename)
{
	static char *functionName="CSWorld";
	LOG_ENTER 
	CSWorldData data;
	loadWorldData(filename, data);
	initialize(data);
	LOG_EXIT
}

// not Done!
CSWorld::CSWorld(const CSWorld &world)
{
	static char *functionName="CSWorld";
	LOG_ENTER 
	initialize();
	mId = strdup(world.mId);
	mLayers = world.mLayers;
	LOG_EXIT
}

void CSWorld::initialize(void)
{
	static char *functionName="initialize";
	LOG_ENTER 
	mFactor = 0;
	mId = 0;
	mScaleMapCount = 0;
	mWorld = 0;
	mLayers = 0;
	mMapArea.x = 0;
	mMapArea.y = 0;
	mMapArea.w = 0;
	mMapArea.h = 0;
	mXOffset = 0;
	mYOffset = 0;
	mHeight = 0;
	mWidth = 0;
	mXPos = 0;
	mYPos = 0;
	mWorldNo = 0;
	mMusic = 0;
	mActive = false;
	mScaledWorldDestination.x = 0;
	mScaledWorldDestination.y = 0;
	mScaledWorldDestination.w = 0;
	mScaledWorldDestination.h = 0;
	LOG_EXIT
}

void CSWorld::scaleBackgroundTiles(int w, int h)
{
	static char *functionName="scaleBackgroundTiles";
	LOG_ENTER 
	if (mWorld != 0)
	{
		for (int l=0; l< mLayers; l++)
		{
			if (mWorld[l]->isBackground())
			{
				mWorld[l]->scaleMapToFit(w, h);
			}
		}
	}
	LOG_EXIT
}

void CSWorld::initialize(const CSWorldData &data)
{
	static char *functionName="initialize";
	LOG_ENTER 
	initialize();
	mId = strdup(data.id);
	mLayers = data.layers;
	
	mWorld = (CSTileMap **) new void *[data.layers];
	
	for (int l=0; l< data.layers; l++)
	{
		MapData *mapData = data.maps->at(l);
		CSTileMap *map = CSTileMapLoader::INSTANCE.load(mapData->name);
		map->setOffset(mapData->offsetX, mapData->offsetY);
		map->setBackground(mapData->isBackground);

		map->setParallax(mapData->isParallax);
		if (l==0)
		{
			map->setSolid(true);
		}
		else
		{
			map->setSolid(false);
		}
		mWorld[l] = map;
		if (!mapData->isBackground)
		{
			if (map->getHeight() > mHeight) mHeight = map->getHeight();
			if (map->getWidth() > mWidth) mWidth = map->getWidth();
		}
	}


	if (mWorld != 0)
	{
		for (int l=0; l< mLayers; l++)
		{
			mWorld[l]->setWorldMax(mWidth, mHeight);
		}
	}
	if (data.musicName != 0)
	{
		mMusic = CSMusicLoader::INSTANCE.load(data.musicName);
		mMusic->play();
	}
	LOG_EXIT
}

CSWorld::~CSWorld()
{
	static char *functionName="~CSWorld";
	LOG_ENTER 
	if (mId != 0)
	{
		free(mId);
		mId = 0;
	}
	
	if (mWorld != 0)
	{
		for (int l=0; l< mLayers; l++)
		{
			delete mWorld[l];
		}
		delete[] mWorld;
		mWorld = 0;
	}
	if (mMusic != 0)
	{
		// delete mMusic; is a singleton!
		mMusic->stop();
		mMusic = 0;
	}
	LOG_EXIT
}

void CSWorld::display()
{
	static char *functionName="display";
	SDL_SetClipRect(SDLMain::getScreen(), &mMapArea);

	if (mWorld != 0)
	{
		for (int l=0; l< mLayers; l++)
		{
			mWorld[l]->display();
		}
	}
	SDL_SetClipRect(SDLMain::getScreen(), 0);
}

void CSWorld::setWorldNo(unsigned int no) 
{
	static char *functionName="setWorldNo";
	LOG_ENTER 
	mWorldNo = no;
	if (mWorld != 0)
	{
		for (int l=0; l< mLayers; l++)
		{
			mWorld[l]->setWorldNo(no);
		}
	}
	LOG_EXIT
}

void CSWorld::setTile(unsigned int x, unsigned int y, unsigned int z, char idChar)
{
	static char *functionName="setTile";
	mWorld[z]->setTile(x, y, idChar);
}

void CSWorld::setTile(unsigned int x, unsigned int y, unsigned int z, CSTile *tile)
{
	static char *functionName="setTile";
	mWorld[z]->setTile(x, y, tile);
}

void CSWorld::update()
{
	static char *functionName="update";
	if (mWorld != 0)
	{
		for (int l=0; l< mLayers; l++)
		{
			mWorld[l]->update();
		}
	}
}

void CSWorld::setDisplayArea(SDL_Rect area)
{
	static char *functionName="setDisplayArea";
	LOG_ENTER 
	mMapArea = area;
	mXOffset = area.x;
	mYOffset = area.y;

	if (mWorld != 0)
	{
		for (int l=0; l< mLayers; l++)
		{
			mWorld[l]->setDisplayOffset(mXOffset, mYOffset);
			mWorld[l]->setDisplaySize(mMapArea.w, mMapArea.h);
		}
	}
	LOG_EXIT
}

void CSWorld::setWorldPosition(int x, int y)
{
	static char *functionName="setWorldPosition";
	mXPos = x;
	mYPos = y;
	if (mWorld != 0)
	{
		for (int l=0; l< mLayers; l++)
		{
			mWorld[l]->setMapPosition(x,y);
		}
	}

	CSSprites::iterator iter = mSprites.begin();
	while (iter != mSprites.end())
	{
		CSSprite *sprite = *iter;
		sprite->setWorldPosition(x, y);
		iter++;
	}
}

void CSWorld::setSecPerFrame(float secPerFrame)
{
	static char *functionName="setSecPerFrame";
	LOG_ENTER 
	if (mWorld != 0)
	{
		for (int l=0; l< mLayers; l++)
		{
			mWorld[l]->setSecPerFrame(secPerFrame);
		}
	}
	CSSprites::iterator iter = mSprites.begin();
	while (iter != mSprites.end())
	{
		CSSprite *sprite = *iter;
		sprite->setSecPerFrame(secPerFrame);
		iter++;
	}
	LOG_EXIT
}

void CSWorld::loadWorldData(const std::string &filename, CSWorldData &data)
{
	static char *functionName="loadWorldData";
	LOG_ENTER 
	char tmp[10];
	CSXMLHelper xmlSupport(filename, "WORLD");
	try
	{
		if (xmlSupport.getError())
		{
			LOG_EXIT
			throw "error";
		}

		data.id = strdup(xmlSupport.getString("ID").c_str());
		int mapCount = xmlSupport.getInt("count(MAPS/MAP)");
		for (int i=0; i<mapCount; i++)
		{
			MapData *map = new MapData();
			// i+1, predicates in xpath start with 1
			std::string a = xmlSupport.getString((std::string)"MAPS/MAP["+itoa(i+1,tmp,10)+"]/NAME");
			map->name =  strdup(a.c_str());

			int hasOffsetData = xmlSupport.getInt((std::string)"count(MAPS/MAP["+itoa(i+1,tmp,10)+"]/OFFSET_X)");
			if (hasOffsetData == 1)
			{
				map->offsetX = xmlSupport.getInt((std::string)"MAPS/MAP["+itoa(i+1,tmp,10)+"]/OFFSET_X");
				map->offsetY = xmlSupport.getInt((std::string)"MAPS/MAP["+itoa(i+1,tmp,10)+"]/OFFSET_Y");
			}
			map->isBackground = xmlSupport.getInt((std::string)"count(MAPS/MAP["+itoa(i+1,tmp,10)+"]/BACKGROUND)") == 1;
			map->isParallax = xmlSupport.getInt((std::string)"count(MAPS/MAP["+itoa(i+1,tmp,10)+"]/PARALLAX)") == 1;
			data.maps->push_back(map);
		}
		data.layers = mapCount;
		bool hasMusic = xmlSupport.getInt((std::string)"count(MUSIC)") == 1;
		if (hasMusic)
		{
			data.musicName = strdup(xmlSupport.getString("MUSIC").c_str());
		}
	}
	catch(...)
	{
		LOG_EXIT
		SDLMain::shutdown((std::string)"XML error \"" + filename + "\": " + xmlSupport.getErrorMessage().c_str(), 1);
	}
	LOG_EXIT
}

// sprites are displayed by there corresponding layer!
void CSWorld::addSprite(CSSprite *sprite)
{
	static char *functionName="addSprite";
	LOG_ENTER 
	// savety meassure, not to add twice!
	removeSprite(sprite);
	mSprites.push_back(sprite);

	if (mWorld != 0)
	{
		if (sprite->getLayer() < 0)
		{
			// add to topmost layer!
			mWorld[mLayers-1]->addSprite(sprite);
			return;
			LOG_EXIT
		}
		if (sprite->getLayer() < mLayers)
		{
			mWorld[sprite->getLayer()]->addSprite(sprite);
		}
		else
		{
			LOG_EXIT
			SDLMain::shutdown((std::string)"Sprite-Layer-Error!", 1);
		}
	}
	LOG_EXIT
}

void CSWorld::removeSprite(CSSprite *sprite)
{
	static char *functionName="removeSprite";
	LOG_ENTER 
	CSSprites::iterator iter = mSprites.begin();
	while (iter != mSprites.end())
	{
		if ((*iter) == sprite)
		{
			mSprites.erase(iter);
			iter = mSprites.begin();	// restart!
		}
		else
		{
			iter++;
		}
	}
	if (mWorld != 0)
	{
		if (sprite->getLayer() < 0)
		{
			// add to topmost layer!
			mWorld[mLayers-1]->removeSprite(sprite);
			LOG_EXIT
			return;
		}
		if (sprite->getLayer() < mLayers)
		{
			mWorld[sprite->getLayer()]->removeSprite(sprite);
		}
		else
		{
			LOG_EXIT
			SDLMain::shutdown((std::string)"Sprite-Layer-Error!", 1);
		}
	}
	LOG_EXIT
}

void CSWorld::buildScaledWorld(double factor, const SDL_Rect &sourceRect, SDL_Surface *destination, const SDL_Rect &displayRect)
{
	static char *functionName="buildScaledWorld";
	bool isNew = false;
	mFactor = factor;
	SDL_Rect destRect;
	SDL_Rect srcRect;
	bool first = true;
	if (mWorld != 0)
	{
		for (int l=0; l< mLayers; l++)
		{
			if (mWorld[l]->isScaleDisplay())
			{
				destRect.x = displayRect.x;
				destRect.y = displayRect.y;
				destRect.w = displayRect.w;
				destRect.h = displayRect.h;
				srcRect.x = sourceRect.x;
				srcRect.y = sourceRect.y;
				srcRect.w = sourceRect.w;
				srcRect.h = sourceRect.h;
				SDL_Surface *src = mWorld[l]->getScaledMap(factor, first);
				first = false;
				
				if (SDL_BlitSurface(src, &srcRect, destination, &destRect) < 0)
				{
					SDLMain::shutdown((std::string)"Display Format error (buildScaledWorld()): " + SDL_GetError(), 1);
				}
			}
		}
	}

	if (mWorld != 0)
	{
		for (int l=0; l< mLayers; l++)
		{
			mWorld[l]->addScaledSprites(factor, sourceRect, destination, displayRect);
		}
	}
}

void CSWorld::resetScaledWorld()
{
	static char *functionName="resetScaledWorld";
	LOG_ENTER 
	if (mWorld != 0)
	{
		for (int l=0; l< mLayers; l++)
		{
			mWorld[l]->resetScaledMap();
		}
	}
	LOG_EXIT
}

void CSWorld::displayScaled(CSSprite *centerSprite)
{
	static char *functionName="displayScaled";
	if ((mScaledWorldDestination.w == 0) || (mScaledWorldDestination.h == 0) || (mFactor == 0))
	{
		return;
	}
	int x = ((double)centerSprite->getXPos() + centerSprite->getMaxWidth()/2) * mFactor;
	int y = ((double)centerSprite->getYPos() + centerSprite->getMaxHeight()/2) * mFactor;
	int xOffset;
	int yOffset;
	int thresholdX = mScaledWorldDestination.w/2;
	int thresholdY = mScaledWorldDestination.h/2;

	SDL_Rect srcRect;
	srcRect.w = mScaledWorldDestination.w;
	srcRect.h = mScaledWorldDestination.h;

	xOffset = x-thresholdX;
	yOffset = y-thresholdY;
	
	if (xOffset + mScaledWorldDestination.w > ((double)mWidth) * mFactor)
	{
		xOffset = ((double) (((double)mWidth) * mFactor)) - mScaledWorldDestination.w;
	}
	if (yOffset + mScaledWorldDestination.h >((double)mHeight) * mFactor)
	{
		yOffset = ((double) (((double)mHeight) * mFactor)) - mScaledWorldDestination.h;
	}

	if (xOffset < 0)
	{
		xOffset = 0;
	}
	if (yOffset < 0)
	{
		yOffset = 0;
	}

	srcRect.x = xOffset; 
	srcRect.y = yOffset;
	
	SDL_SetClipRect(SDLMain::getScreen(), &mScaledWorldDestination);
	SDL_Rect destRect = mScaledWorldDestination;

	buildScaledWorld(mFactor, srcRect, SDLMain::getScreen(),destRect);
	SDL_SetClipRect(SDLMain::getScreen(), 0);
}

void CSWorld::setScaleDisplayArea(SDL_Rect area)
{
	static char *functionName="setScaleDisplayArea";
	LOG_ENTER 
	mScaledWorldDestination = area;
	LOG_EXIT
}
