#include "CSXMLHelper.h"

#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

#ifdef USE_TINYXML

#include <vector>

const char *CSXMLHelper::CLASS = "CSXMLHelper";

void cstoupper(char * s)
{
	int i = strlen(s);
	for (int e =0; e < i; e++)
	{
		s[e] = toupper(s[e]);
	}
}

void CSXMLHelper::addEntitiesFromFile(const std::string &filename, const std::string &parentFilename)
{
	static char *functionName="addEntitiesFromFile";
	LOG_ENTER
	std::string fullName;
	//////////////////////////////
	// Build filename + path
	//////////////////////////////
	int posStart = parentFilename.find_last_of("\\", parentFilename.size());
	if (posStart != -1)
	{
		fullName = parentFilename.substr(0, posStart+1);
	}
	fullName += filename;

	////////////
	// open File
	////////////
	FILE *handle = fopen(fullName.c_str(), "rt");
	if (handle == 0)
	{
		LOG_DEBUG_MESSAGE(((std::string)"Entity file: \"") + fullName+"\" cannot be opened!") 
		LOG_EXIT
		return;
	}
	static char tmp[1024];

	////////////////
	// seek entities
	////////////////
	while (1)
	{
		char *res = fgets(tmp, 1024, handle);
		std::string strValue = tmp;
		if (res == 0)
		{
			break;
		}
		if (strstr(tmp, "<!") != 0)
		{
			if (strstr(tmp, "SYSTEM") != 0)
			{
				// another include found!
				int posStart = strValue.find_first_of('\"');
				int posEnd = strValue.find_first_of('\"', posStart+1);
				std::string dtdName = strValue.substr(posStart+1, (posEnd-1) - (posStart));
				if (parentFilename.size() != 0)
				{
					addEntitiesFromFile(dtdName, parentFilename);
				}
				else
				{
					addEntitiesFromFile(dtdName, filename);
				}
			}
			else
			{
				if (strstr(tmp, "!ENTITY") != 0)
				{	
					int posStart = strValue.find_first_of("!ENTITY");
					posStart = strValue.find_first_of(" \t", posStart);
					posStart = strValue.find_first_not_of(" \t", posStart);
					int posEnd = strValue.find_first_of(" \t", posStart);
					std::string key = strValue.substr(posStart, posEnd - posStart);
					key = "&" + key + ";";

					posStart = strValue.find_first_of('\"');
					posEnd = strValue.find_first_of('\"', posStart+1);

					std::string value = strValue.substr(posStart+1, (posEnd-1) - (posStart));
					mEntityMap.insert(EntityMap::value_type(key, value));
				}
			}
		}
	}
	
	fclose(handle);
	LOG_EXIT
}


CSXMLHelper::CSXMLHelper(const std::string &xmlFilename, const std::string &contextNode)
{
	static char *functionName="CSXMLHelper";
	mError = CSXML_OK;
	mXMLdoc = 0;

	mXMLdoc = new TiXmlDocument(xmlFilename);
	mXMLdoc->setIgnoreNoEndTag(false);
	mXMLdoc->SetCondenseWhiteSpace(true); 

	if (!mXMLdoc)
	{
		mError = CSXML_CONTEXT_LOADING_ERROR;
		std::string errorMessageString;
		errorMessageString = std::string("Error - while loading context \"" + contextNode +"\"!");
		mErrorMessageString = mErrorMessageString + "\n"+ errorMessageString;
	}

	addEntitiesFromFile(xmlFilename, "");

	if (!mXMLdoc->LoadFile() )
	{
		mError = CSXML_DOCUMENT_LOADING_ERROR;
		std::string errorMessageString;
		errorMessageString = std::string("Error - while loading document \"" + xmlFilename +"\"!");
		mErrorMessageString = mErrorMessageString + "\n"+ errorMessageString;
		return;			
	}
	mXMLsource = mXMLdoc->FirstChildElement(contextNode);
}

CSXMLHelper::~CSXMLHelper()
{
	static char *functionName="~CSXMLHelper";
	LOG_ENTER
	delete (mXMLdoc);
	mXMLdoc = 0;
	mXMLsource = 0;
	mEntityMap.clear();
	LOG_EXIT
}

int CSXMLHelper::getInt(const std::string &xpath)
{
	static char *functionName="getInt";
	LOG_ENTER
	int ret = 0;
	if (mError != 0)
	{
		LOG_WARN_MESSAGE("Error")
		LOG_EXIT
		throw "error";
	}
	try
	{
		TiXmlNode *node = getXPathResult(xpath);
		if (node == 0)
		{
			LOG_EXIT
			return 0;
		}
		if (node->Type() == TiXmlNode::FUNCTION)
		{
			ret = toInt(*node);
			delete node;
			node = 0;
		}
		else
		{
			TiXmlText *element = node->ToText();
			if (element == 0)
			{
				LOG_DEBUG_MESSAGE("XPath \"" + xpath + "\" doesn't represent a text element!") 
			}
			ret = toInt(*element);
		}
	}
	catch(...)
	{
		mError = CSXML_XPATH_ERROR;
		std::string errorMessageString;
		errorMessageString = "tinyXMLException";
		mErrorMessageString = mErrorMessageString + "\n"+ errorMessageString;
		LOG_WARN_MESSAGE(mErrorMessageString)
		LOG_EXIT
		throw "error";
	}
	LOG_EXIT
	return ret;
}

std::string CSXMLHelper::getString(const std::string &xpath)
{
	static char *functionName="getString";
	LOG_ENTER
	std::string cstr;
	if (mError != 0)
	{
		LOG_WARN_MESSAGE("XML! Not properly initialized!")
		LOG_EXIT
		throw std::string("XML! Not properly initialized!");
	}
	try
	{
		TiXmlNode *node = getXPathResult(xpath);
		if (node == 0)
		{
			LOG_EXIT
			return "";
		}
		if (node->Type() == TiXmlNode::FUNCTION)
		{
			cstr = toAscii(*node);
			delete node;
			node = 0;
		}
		else
		{
			TiXmlText *element = node->ToText();
			if (element == 0)
			{
				LOG_DEBUG_MESSAGE("XPath \"" + xpath + "\" doesn't represent a text element!") 
			}
			cstr = toAscii(*element);
		}
		
	}
	catch(...)
	{
		mError = CSXML_XPATH_ERROR;
		std::string errorMessageString;
		errorMessageString = "tinyXMLException";
		mErrorMessageString = mErrorMessageString + "\n"+ errorMessageString;
		LOG_WARN_MESSAGE(mErrorMessageString)
		LOG_EXIT
		throw "error";
	}
	LOG_EXIT
	return cstr;
}

ElementDescription CSXMLHelper::getElementDescription(std::string &xpath)
{
	static char *functionName="getElementDescription";
	LOG_ENTER
	ElementDescription desc;
	int posStart;
	int posEnd;
	LOG_DEBUG_MESSAGE("current xpath: "+ xpath) 

	/////////////////////
	// check for function
	/////////////////////
	const std::string FKT_COUNT = "count(";
	if (xpath.compare(0, FKT_COUNT.size(), FKT_COUNT) == 0)
	{
		LOG_DEBUG_MESSAGE("Function count() encountered: ") 
		// strip xpath of function
		// assumes closing bracket (")") is last element of string
		xpath = xpath.substr(FKT_COUNT.size(), xpath.size() - FKT_COUNT.size()-1);
		desc.count = true;
		desc.function = true;
	}

	///////////////////////////
	// look if we start at root
	///////////////////////////
	posStart = xpath.find_first_of("/");
	if (posStart == 0)
	{
		// yes, than we have a leading slash, remember that
		desc.leadingSlash = true;
		posStart = 1;
	}
	else
	{
		// we start right at the beginning
		posStart = 0;
	}
	
	///////////////////////////
	// seperate current element
	///////////////////////////
	// this element, till we find a path-seperator "/"
	posEnd = xpath.find_first_of("/", posStart);
	if (posEnd == -1)
	{
		// now Path-seperator found - this ist the "last" element!
		posEnd = xpath.size();
	}
	// the element ist the one up to the next "/" or the end of the path!
	desc.elementName = xpath.substr(posStart, posEnd - posStart);
	LOG_DEBUG_MESSAGE("path element name: "+ desc.elementName) 

	// next we cut the current path element from xpath
	// if this element was not the last - than we cut the current element from xpath
	if (xpath.size() - posEnd > 0)
	{
		xpath = xpath.substr(posEnd, xpath.size() - posEnd);
	
		// skip next path seperator -> so we won't start from root!
		posStart = xpath.find_first_of("/");
		if (posStart == 0)
		{
			if (xpath.size() - 1 > 0)
			{
				xpath = xpath.substr(1, xpath.size() - 1);
			}
			else
			{
				// last element ended with a trailing "/"
				xpath = "";
			}
		}
	}
	else
	{
		// if this was the last element -> xpath is empty
		xpath = "";
	}

	//////////////////////////////
	// check for predicate information
	//////////////////////////////
	posStart = desc.elementName.find_first_of("[", posStart);
	if (posStart != -1)
	{
		// predicate information found
		// now extract predicate information
		std::string predicateValue = desc.elementName.substr(posStart+1, desc.elementName.size() - (posStart+1+1));
		desc.elementName = desc.elementName.substr(0, posStart);
		desc.predicatePos = atoi(predicateValue.c_str()) - 1; // since predicates start in xpath with 1
	}

	LOG_DEBUG_MESSAGE("next xpath: "+ xpath) 
	LOG_EXIT
	return desc;
}

TiXmlNode *CSXMLHelper::getNextNode(TiXmlNode *sourceElement, std::string &xpath)
{
	static char *functionName="getNextNode";
	LOG_ENTER
	TiXmlNode *result = sourceElement;
	
	////////////////////
	// recursion ending!
	////////////////////
	if (xpath.size() == 0)
	{
		LOG_EXIT
		return result;
	}

	////////////////////////
	// get next element name
	////////////////////////
	ElementDescription elementDescription = getElementDescription(xpath);

	///////////////////
	// is root element?
	///////////////////
	if (elementDescription.leadingSlash)
	{
		sourceElement = mXMLdoc;
	}

	//////////////
	// get element
	//////////////
	TiXmlNode *nextSourceElement = sourceElement->FirstChildElement(elementDescription.elementName);
	if (nextSourceElement == 0)
	{
		LOG_EXIT
		return 0;
	}

	/////////////////////////
	// has predicate information?
	/////////////////////////
	if (elementDescription.predicatePos != -1)
	{
		LOG_DEBUG_MESSAGE("Predicate encountered!") 
		std::string name = nextSourceElement->Value();
		int count = 0;
		char tmp[10];
		sprintf(tmp,"%i", elementDescription.predicatePos + 1); // +1 to correct predicate value
		while ((nextSourceElement != 0) && (count != elementDescription.predicatePos))
		{
			nextSourceElement = nextSourceElement->NextSibling(name);
			if (nextSourceElement)
			{
				count++;
			}
		}
		if (result == 0)
		{
			LOG_DEBUG_MESSAGE(name+"["+tmp+"] not found!") 
			LOG_EXIT
			return 0;
		}
		else
		{
			LOG_DEBUG_MESSAGE(name+"["+tmp+"] found!") 
		}
	}
	
	result = getNextNode(nextSourceElement, xpath);
	if (result == 0)
	{
		LOG_EXIT
		return 0;
	}
	std::string name = result->Value();

	////////////////////////
	// has a xpath function?
	////////////////////////
	if (elementDescription.function)
	{
		if (elementDescription.count)
		{
			char tmp[10];
			int count = 1;
			LOG_DEBUG_MESSAGE(((std::string)"Counting Node: ")+ result->Value()) 

			while (result != 0)
			{
				result = result->NextSibling(name);
				if (result)
				{
					count++;
				}
			}
			sprintf(tmp,"%i", count);
			LOG_DEBUG_MESSAGE(name +" found "+ tmp +" times.") 
			
			TiFunctionReturn *ret = new TiFunctionReturn(tmp);
			result = ret;
			// a node with a value  of count :-)
		}
	}

	LOG_EXIT
	return result;
}

TiXmlNode *CSXMLHelper::getXPathResult(const std::string &xpath)
{
	static char *functionName="getXPathResult";
	static char h[1025];
	LOG_ENTER
	h[0] = 0;
	strcat(h, "normalize-space(");
	strcat(h, xpath.c_str());
	strcat(h, ")");

//	LOG_DEBUG_MESSAGE("------------------- Start XPath -------------------") 

	std::string path = xpath;

	TiXmlNode *result = 0;
	result = getNextNode(mXMLsource, path);
	if (result==0)
	{
//		LOG_DEBUG_MESSAGE(((std::string)"Result: null")) 
		if (!xpath.find("count"))
		{
			LOG_DEBUG_MESSAGE(xpath + "->0" )
		}
		else
		{
			LOG_DEBUG_MESSAGE(xpath + "->" ) 
		}
		LOG_EXIT
		return 0;
	}

	if (!result->NoChildren()) // than most probably a function
	{
		result = result->FirstChild();
	}
	LOG_DEBUG_MESSAGE(xpath + "->" + lookUp(result->Value()) ) 

	LOG_EXIT
	return result;
}

int CSXMLHelper::getError(void)
{
	return mError;
}

std::string CSXMLHelper::getErrorMessage(void)
{
	return mErrorMessageString;
}

std::string CSXMLHelper::toAscii(const TiXmlNode &node)
{
	static char *functionName="toAscii";
	return lookUp(node.Value());
}

int CSXMLHelper::toInt(const TiXmlNode &node)
{
	static char *functionName="toInt";
	std::string value = node.Value();
	int result;
	result = atoi(lookUp(value).c_str());
	
	return result;
}

std::string replace(const std::string &key, const std::string &predicate, const std::string &value);

// Ersetze in dem bergeben Ausdruck alle Entities <-> mit ihren Werten (Recursiv)
std::string CSXMLHelper::lookUp(const std::string &key)
{
	static char *functionName="lookUp";
	LOG_ENTER
	std::string newKey = key;
	std::string p = findPredicate(key);
	if (p.size() == 0)
	{
		LOG_EXIT
		return newKey;
	}

	do
	{
		std::string value;
		EntityMap::iterator iter = mEntityMap.find(p);
		if (iter != mEntityMap.end())
		{
			value = iter->second;
		}
		newKey = replace(newKey, p,  value);
		p = findPredicate(newKey);
	} while (p.size() > 0);
		
	LOG_EXIT
	return newKey;
}

std::string CSXMLHelper::findPredicate(const std::string &key)
{
	EntityMap::iterator iter = mEntityMap.begin();
	while (iter != mEntityMap.end())
	{
		if (strstr(key.c_str(), iter->first.c_str()))
		{
			return iter->first;
		}
		iter++;
	}
	return "";
}

// wo, was, womit
std::string replace(const std::string &key, const std::string &predicate, const std::string &value)
{
	std::string newKey;
	std::string rest;
	int posStart = strstr(key.c_str(), predicate.c_str()) - key.c_str();
	int len = predicate.size();
	if (posStart != 0)
	{
		newKey = key.substr(0, posStart);
	}
	else
	{
		newKey = key.substr(len, key.size()-len);
	}
	newKey = newKey +value;
	
	if (posStart+len < key.size())
	{
		newKey = newKey + key.substr(posStart + len, key.size() - (posStart + len) );
	}

	return newKey;
}

#endif // USE_TINYXML
