#ifndef CSXMLHelperTinyXML_H
#define CSXMLHelperTinyXML_H

#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

#include "CSLog.h"
#include <map>
#include <string>
//#include <iostream.h>

#include "tinyxml.h"

typedef std::map<std::string, std::string> EntityMap;

class TiFunctionReturn : public TiXmlText 
{
	public:
		TiFunctionReturn (const char * initValue) : TiXmlText (initValue)
		{
			setType(TiXmlNode::FUNCTION);
		}
};

class ElementDescription
{
	public:
		std::string elementName;
		bool function;
		bool count;
		bool leadingSlash;
		int predicatePos;
		ElementDescription()
		{
			elementName = "";
			function = false;
			count = false;
			leadingSlash = false;
			predicatePos = -1;
		}

};

class XPathDescription
{
	public:
		std::string id;
};

class CSXMLHelper
{
	private:
		EntityMap mEntityMap;

		std::string mErrorMessageString;
		TiXmlDocument	*mXMLdoc;
		TiXmlNode *mXMLsource;
		
		int	mError;
		ElementDescription getElementDescription(std::string &xpath);
		TiXmlNode *getNextNode(TiXmlNode *sourceElement, std::string &xpath);
		void addEntitiesFromFile(const std::string &filename, const std::string &parentFilename);

		TiXmlNode *getXPathResult(const std::string &xpath);
		std::string toAscii(const TiXmlNode &node);
		int toInt(const TiXmlNode &node);
		std::string lookUp(const std::string &key);
		std::string findPredicate(const std::string &key);

	public:
		CSXMLHelper(const std::string &xmlFilename, const std::string &contextNode);
		virtual ~CSXMLHelper();
		static const char *CLASS;
		virtual std::string getType() {return (std::string) CLASS;}

		int getError(void);
		std::string getErrorMessage(void);
		int getInt(const std::string &xpath);
		std::string getString(const std::string &xpath);
};

#endif CSXMLHelperTinyXML_H