#include "CSXMLHelper.h"

#ifdef USE_TINYXPATH

#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

#include <vector>

const char *CSXMLHelper::CLASS = "CSXMLHelper";

CSXMLHelper::CSXMLHelper(const std::string &xmlFilename, const std::string &contextNode)
{
	static char *functionName="CSXMLHelper";
	mError = CSXML_OK;
	mXMLdoc = 0;
	
	mXMLdoc = new TiXmlDocument(xmlFilename);
	if (!mXMLdoc->LoadFile() )
	{
		mError = CSXML_DOCUMENT_LOADING_ERROR;
		std::string errorMessageString;
		errorMessageString = std::string("Error - while loading document \"" + xmlFilename +"\"!");
		mErrorMessageString = mErrorMessageString + "\n"+ errorMessageString;
		return;			
	}
	mXMLsource = mXMLdoc->FirstChildElement(contextNode);
	if (!mXMLsource)
	{
		mError = CSXML_CONTEXT_LOADING_ERROR;
		std::string errorMessageString;
		errorMessageString = std::string("Error - while loading context \"" + contextNode +"\"!");
		mErrorMessageString = mErrorMessageString + "\n"+ errorMessageString;
	}
}

CSXMLHelper::~CSXMLHelper()
{
	static char *functionName="~CSXMLHelper";
	LOG_ENTER
	delete (mXMLdoc);
	mXMLdoc = 0;
	mXMLsource = 0;
	LOG_EXIT
}

int CSXMLHelper::getInt(const std::string &xpath)
{
	static char *functionName="getInt";
	int ret = 0;
	if (mError != 0)
	{
		LOG_WARN_MESSAGE("Error")
		throw "error";
	}
	try
	{
		XObjectPtr xml = getXPathResult(xpath);
		ret = DOMStringToInt(xml->str());
	}
	catch(const SAXException&)
	{
		mError = CSXML_XPATH_ERROR;
		std::string errorMessageString;
		errorMessageString = std::string("SAXException") + mErrorMessageString;
		mErrorMessageString = mErrorMessageString + "\n"+ errorMessageString;
		LOG_WARN_MESSAGE(mErrorMessageString)
		throw "error";
	}
	catch(const XMLException&)
	{
		mError = CSXML_XPATH_ERROR;
		std::string errorMessageString;
		errorMessageString = std::string("XMLException") + mErrorMessageString;
		mErrorMessageString = mErrorMessageString + "\n"+ errorMessageString;
		LOG_WARN_MESSAGE(mErrorMessageString)
		throw "error";
	}
	catch(const XalanDOMException&)
	{
		mError = CSXML_XPATH_ERROR;
		std::string errorMessageString;
		errorMessageString = std::string("XalanDOMException") + mErrorMessageString;
		mErrorMessageString = mErrorMessageString + "\n"+ errorMessageString;
		LOG_WARN_MESSAGE(mErrorMessageString)
		throw "error";
	}
	return ret;
}

std::string CSXMLHelper::getString(const std::string &xpath)
{
	static char *functionName="getString";
	std::string cstr;
	if (mError != 0)
	{
		LOG_WARN_MESSAGE("XML! Not properly initialized!")
		throw std::string("XML! Not properly initialized!");
	}
	try
	{
		XObjectPtr xml = getXPathResult(xpath);
		cstr = toAscii(xml);
	}
	catch(const SAXException&)
	{
		mError = CSXML_XPATH_ERROR;
		std::string errorMessageString;
		errorMessageString = std::string("SAXException") + mErrorMessageString;
		mErrorMessageString = mErrorMessageString + "\n"+ errorMessageString;
		LOG_WARN_MESSAGE(mErrorMessageString)
		throw "error";
	}
	catch(const XMLException&)
	{
		mError = CSXML_XPATH_ERROR;
		std::string errorMessageString;
		errorMessageString = std::string("XMLException") + mErrorMessageString;
		mErrorMessageString = mErrorMessageString + "\n"+ errorMessageString;
		LOG_WARN_MESSAGE(mErrorMessageString)
		throw "error";
	}
	catch(const XalanDOMException&)
	{
		mError = CSXML_XPATH_ERROR;
		std::string errorMessageString;
		errorMessageString = std::string("XalanDOMException") + mErrorMessageString;
		mErrorMessageString = mErrorMessageString + "\n"+ errorMessageString;
		LOG_WARN_MESSAGE(mErrorMessageString)
		throw "error";
	}
	return cstr;
}


XObjectPtr CSXMLHelper::getXPathResult(const std::string &xpath)
{
	static char *functionName="getXPathResult";
	static char h[1025];
	h[0] = 0;
	strcat(h, "normalize-space(");
	strcat(h, xpath.c_str());
	strcat(h, ")");

	xfsp_engine = new xpath_from_source(mXMLsource, xpath);
    xfsp_engine -> v_apply_xpath (cp_test_name, Fp_html);
    delete xfsp_engine;

	const XObjectPtr	theResult(
		theEvaluator->evaluate(
				*theDOMSupport,
				theContextNode,
				XalanDOMString(h).c_str(),
				theDocument->getDocumentElement())
				);
	if (theResult.null() != false)
	{
		mError = CSXML_XPATH_ERROR;
		std::string errorMessageString;
		errorMessageString = std::string("General XPATH-Result = 0") + mErrorMessageString;
		mErrorMessageString = mErrorMessageString + "\n"+ errorMessageString;
		throw "error";
	}
	return theResult;
}

std::string CSXMLHelper::toAscii(const XObjectPtr &xObjectPtr)
{
	static char *functionName="toAscii";
	std::string cstr;
	const XalanDOMString xmlString = xObjectPtr->str();
	return toAscii(xmlString);
}
	
#endif // USE_TINYXPATH
