#ifndef SDLMain_h
#define SDLMain_h

#include <SDL.h>
#include "CSLog.h"
#include "SDL_mixer.h"
#include "smpeg.h"
#include <string>

#include "CSMessageDispatchable.h"
#include "CSMessage.h"

#define COLOR(src, c) SDL_MapRGB(((src)->format),(((c)&0xff0000)>>16),(((c)&0xff00)>>8),(((c)&0xff)))
const int NUM_UPDATE_RECTS = 500;

int convert(const char *start_string, int radix);
int _putPixelAlpha(SDL_Surface * surface, Sint16 x, Sint16 y, Uint32 color, Uint8 alpha);

class SDLMain : public CSMessageDispatchable
{
	private:
		SDLMessage MESSAGE_SCREEN_CHANGED;
		static SDLMain *mInstance;
		static SDL_Surface *mScreen; 
		static SDL_Rect *mRects;
		static int mCurrentCount;
		static int mNoPages;
		static int mPage;
		static int mWidth;
		static int mHeight;
		static int mDepth;
		static unsigned int mMode;
		static bool mSDLInit;

		static bool mIsHardware;
		static bool mIsDoublebuffer;
		static bool mIsFullscreen;
		static bool mIsAnyFormat;
		static bool mUpdateWholeScreen;
		
		static unsigned int mAudioFormat;
		static unsigned int mAudioChannels;
		static unsigned int mAudioBuffers;
		static unsigned int mAudioRate;
		SDLMain();

	public:
		~SDLMain();
		static SDLMain *getInstance();

		static const char *CLASS;
		virtual std::string getType() {return (std::string) CLASS;}

		static inline SDL_Surface *getScreen(){return mScreen;};
		static void shutdown();
		static void shutdown(std::string, unsigned int errorNo);
		static void clearScreen( int red, int green, int blue );

		static void addUpdateRect(const SDL_Rect &saveRegion);
		static void addUpdateRect(signed short x, signed short y, unsigned short w, unsigned short h);
		static int getPageNo();
		static int getCurrentPage();
		static void updateScreen();
		static unsigned int getMode();
		static int getScreenWidth() {return mWidth;}
		static int getScreenHeight() {return mHeight;}
		static int getScreenDepth() {return mDepth;}
		static void setScreenWidth(int w);
		static void setScreenHeight(int h);
		static void setScreenDepth(int d);

		static bool isHardwareSurface() {return mIsHardware;}
		static bool isDoubleBufferSurface() {return mIsDoublebuffer;}
		static bool isFullscreenSurface() {return mIsFullscreen;}
		static bool isUseScreenAnyhowSurface() {return mIsAnyFormat;}
		static bool isInitialized() {return mSDLInit;}

		static void setHardwareSurface(bool h);
		static void setDoubleBufferSurface(bool d);
		static void setFullscreenSurface(bool f);
		static void setUseScreenAnyhowSurface(bool a);
		static void toggleFullscreen();
		static void resetVideo();
		static Uint32 getPixel(SDL_Surface *Surface, Sint32 X, Sint32 Y);
		static void setPixel(SDL_Surface *surface, Sint32 X, Sint32 Y, Uint32 color);
		static void setUpdateWholeScreen(bool b) {mUpdateWholeScreen = b;}

		static SDL_Surface *scale(SDL_Surface *surface, int width, int height);
		static SDL_Surface *scale(SDL_Surface *surface, int width, int divideX, int height, int divideY);
		static SDL_Surface *scale(SDL_Surface *surface, double factor);
		static int scaleX(int width, int divideX, int x);
		static int scaleY(int height, int divideY, int y);
		static int line(SDL_Surface * dst, Sint16 x1, Sint16 y1, Sint16 x2, Sint16 y2, Uint32 color);
		static void freeSurface(SDL_Surface *surface);
		static void adjustClipping(SDL_Rect &newRect, SDL_Rect &oldRect);

		// must lock surface if used in fullscrren!
		static void DrawPixel(SDL_Surface *surface, Sint32 x, Sint32 y, Uint32 color)
		{
			if ((x < 0) || (x > surface->w) || (y<0) || (y>surface->h)) return;
			setPixel(surface, x, y, color);
		}
		static void DrawRoundRectNorth(SDL_Surface * screen, int x0, int y0, int w, int h, Uint32 color, int corner, int thickness=1)
		{
			DrawRoundRect(screen, x0, y0, w, h, color, corner, thickness, true,false,false,false);
		}
		static void DrawRoundRectSouth(SDL_Surface * screen, int x0, int y0, int w, int h, Uint32 color, int corner, int thickness=1)
		{
			DrawRoundRect(screen, x0, y0, w, h, color, corner, thickness, false,true,false,false);
		}
		static void DrawRoundRectEast(SDL_Surface * screen, int x0, int y0, int w, int h, Uint32 color, int corner, int thickness=1)
		{
			DrawRoundRect(screen, x0, y0, w, h, color, corner, thickness, false,false,true,false);
		}
		static void DrawRoundRectWest(SDL_Surface * screen, int x0, int y0, int w, int h, Uint32 color, int corner, int thickness=1)
		{
			DrawRoundRect(screen, x0, y0, w, h, color, corner, thickness, false,false,false,true);
		}
		static void DrawRoundRect(SDL_Surface * screen, SDL_Rect *rect, Uint32 color, int corner = 3, int thickness=1);
		static void DrawRoundRect(SDL_Surface * screen, int x0, int y0, int w, int h, Uint32 color, int corner, int thickness=1, bool north = true, bool south = true, bool east = true, bool west = true);
		static void DrawRoundRectFill(SDL_Surface * screen, int x0, int y0, int w, int h, Uint32 color, int corner=5);
		static void vertgradient(SDL_Surface * surf, SDL_Rect & gradRect, int c1, int c2, Uint8 alpha);
		static void horizgradient(SDL_Surface * surf, SDL_Rect & gradRect, int c1, int c2, Uint8 alpha);

};

#endif // SDLMain_h
