#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

#include "CSBar.h"
#include "CSBorder.h"
#include "CSDesktop.h"

const char *CSBar::CLASS = "CSBar";
/** Create a new \a CSBar.
	
	If \a autoPlacing is enabled the to be added elements are positioned automatically.
	If bar is of type TYPE_HORIZONTAL, they are added from left to right (centered vertically).
	If bar is of type TYPE_VERTICAL, they are added from upper to lower (centered horizontally.

	\param type either \a TYPE_HORIZONTAL or \a TYPE_VERTICAL
	\param autoPlacing true if components are to be placed automatically (by CSBAR)
*/
CSBar::CSBar(int type) : CSGrafikElement(0, 0)
{
	static char *functionName="CSBar";
	LOG_ENTER 
	mAutoPlacingType = type;
	initBar();
	LOG_EXIT
}

CSBar::~CSBar()
{
	static char *functionName="~CSBar";
}

void CSBar::initBar()
{
	static char *functionName="initBar";
	LOG_ENTER 
	if (mAutoPlacingType == TYPE_HORIZONTAL)
	{
		CSLayoutData layout;
		setLayoutManager(new CSLayoutManagerStackHorizontal());
		layout.setPosition(POSITION_ORDER_TOP);
		layout.setPackedVertical(true);
		layout.setStretchedHorizontal(true);
		layout.setSpacing(0,2);
		setLayoutData(layout);
		setInset(CSInset(0,0,1,1));
	}
	else
	{
		CSLayoutData layout;
		setLayoutManager(new CSLayoutManagerStackVertical());
		layout.setPosition(POSITION_ORDER_LEFT);
		layout.setPackedHorizontal(true);
		layout.setStretchedVertical(true);
		layout.setSpacing(2,0);
		setLayoutData(layout);
		setInset(CSInset(1));
	}
	LOG_EXIT
}

//! the added element will be freed by this instance of Bar
//! first pos is 0
void CSBar::addElementAt(CSGrafikElement *element, int place)
{
	static char *functionName="addElementAt";
	LOG_ENTER 
		
	CSGrafikElements elements;

	CSGrafikElements::iterator iter = mElements.begin();
	while (iter != mElements.end())
	{
		CSGrafikElement *element = *iter;
		elements.push_back(element);
		removeElement(element);
		iter = mElements.begin();
	}

	iter = elements.begin();
	int count=0;
	CSLayoutData layoutData;
	if (mAutoPlacingType == TYPE_HORIZONTAL)
	{
		layoutData = CSLayoutData(POSITION_WEST);
	}
	else
	{
		layoutData = CSLayoutData(POSITION_NORTH);
	}
	layoutData.setSpacing(getVerticalElementSpacing(), getHorizontalElementSpacing());

	while (iter != elements.end())
	{
		CSGrafikElement *oldElement = *iter;
		if (count == place)
		{
			element->setLayoutData(layoutData);
			addElement(element);
		}
		oldElement->setLayoutData(layoutData);
		addElement(oldElement); 
		count++;
		iter++;
	}
	if (count <= place)
	{
		element->setLayoutData(layoutData);
		addElement(element);
	}
	LOG_EXIT
}

void CSBar::layoutSetupBar()
{
	static char *functionName="layoutSetupBar";
	LOG_ENTER 
	LOG_EXIT
}


