#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

#include "CSBorder.h"
#include "CSLAF.h"
#include "CSHelper.h"
#include "CSGrafikElement.h"

const char *CSBorder::CLASS = "CSBorder";
const char *CSRoundedBorder::CLASS = "CSRoundedBorder";

CSBorder::CSBorder()
{
	static char *functionName="CSBorder";
	LOG_ENTER 
	mBorderOuterEdgeWidth = 0;
	mBorderInnerEdgeWidth = 0;
	mBorderInterEdgeWidth = 0;
	mState = BORDER_STATE_DEFAULT;
	LOG_EXIT
}

void CSBorder::paint(SDL_Surface *destination, SDL_Rect *area)
{
	static char *functionName="paint";
	CSLAF *laf = CSLAF::getCurrentLAF(); 
	int type = mType;
	if ((!(mState & BORDER_STATE_VISIBLE)) || (type == BORDER_TYPE_NONE))
	{
		return;
	}

	mShadeBrighter = CSHelper::colorPercent(laf->getBorderDefaultBrightestColor(), 100);
	mShadeBright = CSHelper::colorPercent(laf->getBorderDefaultBrightestColor(), 70);
	mShadeDark = CSHelper::colorPercent(laf->getBorderDefaultBrightestColor(), 50);
	mShadeDarker = CSHelper::colorPercent(laf->getBorderDefaultBrightestColor(), 20);

	// if type up
	if (type & BORDER_TYPE_UP)
	{
		// and pressed
		if (mState & BORDER_STATE_PRESSED)
		{
			// draw pressed border
			type = type & (~BORDER_TYPE_UP);
		}
		else
		{
			// if hovering enabled 
			if (mState & BORDER_STATE_HOVERING)
			{
				// and hovering happens
				if (type & BORDER_TYPE_HOVER)
				{
					// draw on hovering Type up
					type = type | BORDER_TYPE_UP;
				}
				else
				{
					// if overing enabeld and not hovering, draw nothing
					return;
					//type = type & (~BORDER_TYPE_UP);
				}
			}
			else // if hovering disabled
			{
				// and not pressed, than draw border up on pressed
				type = type | BORDER_TYPE_UP;
			}
		}
	}
	else
	{
	}


	SDL_Rect copy;
	int o = 0;
	if (type & BORDER_TYPE_UP)
	{
		if (mBorderOuterEdgeWidth)
		{
			SDL_Rect a = *area;
			a.x += o;
			a.y += o;
			a.h = mBorderOuterEdgeWidth;
			a.w = a.w - 2*o;
			if (a.w > 60000) a.w = 0;
			if (mType & BORDER_TYPE_SINGLE_COLOR)
			{
				copy = a;
				SDL_FillRect(destination, &copy, COLOR(destination, mShadeDarker));
				a.y = area->h+area->y - (mBorderOuterEdgeWidth+o);
				copy = a;
				SDL_FillRect(destination, &copy, COLOR(destination, mShadeDarker));
			}
			else
			{
				copy = a;
				SDL_FillRect(destination, &copy, COLOR(destination, mShadeBrighter));
				a.y = area->h+area->y - (mBorderOuterEdgeWidth+o);
				copy = a;
				SDL_FillRect(destination, &copy, COLOR(destination, mShadeDarker));
			}

			a = *area;
			a.x += o;
			a.y += o;
			a.w = mBorderOuterEdgeWidth;
			a.h = a.h - 2*o;
			if (a.h > 60000) a.h = 0;
			if (mType & BORDER_TYPE_SINGLE_COLOR)
			{
				copy = a;
				SDL_FillRect(destination, &copy, COLOR(destination, mShadeDarker));
				a.x = area->w+area->x - (mBorderOuterEdgeWidth +o);
				copy = a;
				SDL_FillRect(destination, &copy, COLOR(destination, mShadeDarker));
			}
			else
			{
				copy = a;
				SDL_FillRect(destination, &copy, COLOR(destination, mShadeDarker));
				a.x = area->w+area->x - (mBorderOuterEdgeWidth +o);
				copy = a;
				SDL_FillRect(destination, &copy, COLOR(destination, mShadeBrighter));
			}
		}	
		o+=mBorderOuterEdgeWidth;
		if (type & BORDER_TYPE_DOUBLE)
		{

			if (mBorderInnerEdgeWidth)
			{
				SDL_Rect a = *area;
				a.x += o;
				a.y += o;
				a.h = mBorderInnerEdgeWidth;
				a.w = a.w - 2*o;
				if (a.w > 60000) a.w = 0;
				if (mType & BORDER_TYPE_SINGLE_COLOR)
				{
					copy = a;
					SDL_FillRect(destination, &copy, COLOR(destination, mShadeDark));
					a.y = area->h+area->y - (mBorderInnerEdgeWidth+o);
					copy = a;
					SDL_FillRect(destination, &copy, COLOR(destination, mShadeDark));
				}
				else
				{
					copy = a;
					SDL_FillRect(destination, &copy, COLOR(destination, mShadeBright));
					a.y = area->h+area->y - (mBorderInnerEdgeWidth+o);
					copy = a;
					SDL_FillRect(destination, &copy, COLOR(destination, mShadeDark));
				}

				a = *area;
				a.x += o;
				a.y += o;
				a.w = mBorderInnerEdgeWidth;
				a.h = a.h - 2*o;
				if (a.h > 60000) a.h = 0;
				if (mType & BORDER_TYPE_SINGLE_COLOR)
				{
					copy = a;
					SDL_FillRect(destination, &copy, COLOR(destination, mShadeDark));
					a.x = area->w+area->x - (mBorderInnerEdgeWidth +o);
					copy = a;
					SDL_FillRect(destination, &copy, COLOR(destination, mShadeDark));
				}
				else
				{
					copy = a;
					SDL_FillRect(destination, &copy, COLOR(destination, mShadeDark));
					a.x = area->w+area->x - (mBorderInnerEdgeWidth +o);
					copy = a;
					SDL_FillRect(destination, &copy, COLOR(destination, mShadeBright));
				}
			}	
		}
		o+=mBorderInnerEdgeWidth;
		if (type & BORDER_TYPE_INTER)
		{
			SDL_Rect a = *area;
			a.x += o + mBorderInterEdgeWidth;
			a.y += o + mBorderInterEdgeWidth;
			a.w = a.w - 2*o - (2*mBorderInterEdgeWidth);
			a.h = a.h - 2*o - (2*mBorderInterEdgeWidth);
			if (a.w > 60000) a.w = 0;
			if (a.h > 60000) a.h = 0;
			
			int typeSave = type;
			type = type - BORDER_TYPE_INTER - BORDER_TYPE_UP;
			paint(destination, &a);
			type = typeSave;
		}
	}	
	else // down
	{
		if (mBorderOuterEdgeWidth)
		{
			SDL_Rect a = *area;
			a.x += o;
			a.y += o;
			a.h = mBorderOuterEdgeWidth;
			a.w = a.w - 2*o;
			if (a.w > 60000) a.w = 0;
			copy = a;
			SDL_FillRect(destination, &copy, COLOR(destination, mShadeDarker));
			a.y = area->h+area->y - (mBorderOuterEdgeWidth+o);
			copy = a;
			SDL_FillRect(destination, &copy, COLOR(destination, mShadeBrighter));

			a = *area;
			a.x += o;
			a.y += o;
			a.w = mBorderOuterEdgeWidth;
			a.h = a.h - 2*o;
			if (a.h > 60000) a.h = 0;
			copy = a;
			SDL_FillRect(destination, &copy, COLOR(destination, mShadeBrighter));
			a.x = area->w+area->x - (mBorderOuterEdgeWidth +o);
			copy = a;
			SDL_FillRect(destination, &copy, COLOR(destination, mShadeDarker));
		}	
		o+=mBorderOuterEdgeWidth;
		if (type & BORDER_TYPE_DOUBLE)
		{
			if (mBorderInnerEdgeWidth)
			{
				SDL_Rect a = *area;
				a.x += o;
				a.y += o;
				a.h = mBorderInnerEdgeWidth;
				a.w = a.w - 2*o;
				if (a.w > 60000) a.w = 0;
				copy = a;
				SDL_FillRect(destination, &copy, COLOR(destination, mShadeDark));
				a.y = area->h+area->y - (mBorderInnerEdgeWidth+o);
				copy = a;
				SDL_FillRect(destination, &copy, COLOR(destination, mShadeBright));

				a = *area;
				a.x += o;
				a.y += o;
				a.w = mBorderInnerEdgeWidth;
				a.h = a.h - 2*o;
				if (a.h > 60000) a.h = 0;
				copy = a;
				SDL_FillRect(destination, &copy, COLOR(destination, mShadeBright));
				a.x = area->w+area->x - (mBorderInnerEdgeWidth +o);
				copy = a;
				SDL_FillRect(destination, &copy, COLOR(destination, mShadeDark));
			}
		}
		if (type & BORDER_TYPE_INTER)
		{
			SDL_Rect a = *area;
			a.x += o + mBorderInterEdgeWidth;
			a.y += o + mBorderInterEdgeWidth;
			a.w = a.w - 2*o - (2*mBorderInterEdgeWidth);
			a.h = a.h - 2*o - (2*mBorderInterEdgeWidth);
			if (a.w > 60000) a.w = 0;
			if (a.h > 60000) a.h = 0;

			int typeSave = type;
			type = type - BORDER_TYPE_INTER - BORDER_TYPE_UP;
			paint(destination, &a);
			type = typeSave;
		}
		o+=mBorderInnerEdgeWidth;
	}
}

int CSBorder::getTotalWidth()
{
	return getSizeEast() + getSizeWest();
}

int CSBorder::getTotalHeight()
{
	return getSizeSouth() + getSizeNorth();
}

int CSBorder::getSizeSouth()
{
	int size = 0;
	if (mType == BORDER_TYPE_NONE)
	{
		return 0;
	}
	if (mType & BORDER_TYPE_INTER)
	{
		size += mBorderInterEdgeWidth;
	}
	if (mType & BORDER_TYPE_DOUBLE)
	{
		size += mBorderInnerEdgeWidth;
	}
	size += mBorderOuterEdgeWidth;
	return size;
}

int CSBorder::getSizeNorth()
{
	int size = 0;
	if (mType == BORDER_TYPE_NONE)
	{
		return 0;
	}
	if (mType & BORDER_TYPE_INTER)
	{
		size += mBorderInterEdgeWidth;
	}
	if (mType & BORDER_TYPE_DOUBLE)
	{
		size += mBorderInnerEdgeWidth;
	}
	size += mBorderOuterEdgeWidth;
	return size;
}

int CSBorder::getSizeEast()
{
	int size = 0;
	if (mType == BORDER_TYPE_NONE)
	{
		return 0;
	}
	if (mType & BORDER_TYPE_INTER)
	{
		size += mBorderInterEdgeWidth;
	}
	if (mType & BORDER_TYPE_DOUBLE)
	{
		size += mBorderInnerEdgeWidth;
	}
	size += mBorderOuterEdgeWidth;
	return size;
}

int CSBorder::getSizeWest()
{
	int size = 0;
	if (mType == BORDER_TYPE_NONE)
	{
		return 0;
	}
	if (mType & BORDER_TYPE_INTER)
	{
		size += mBorderInterEdgeWidth;
	}
	if (mType & BORDER_TYPE_DOUBLE)
	{
		size += mBorderInnerEdgeWidth;
	}
	size += mBorderOuterEdgeWidth;
	return size;
}

CSBorder *CSBorder::getBorder(int type)
{
	CSBorder *border = new CSBorder(); 
	border->mType = type;
	if (border->mType == BORDER_TYPE_NONE)
	{
		border->mBorderOuterEdgeWidth = 0;
		border->mBorderInnerEdgeWidth = 0;
		border->mBorderInterEdgeWidth = 0;
	}
	else
	{
		border->mBorderOuterEdgeWidth = 1;
		border->mBorderInnerEdgeWidth = 1;
		border->mBorderInterEdgeWidth = 2;
	}
	return border;
}

void CSRoundedBorder::paint(SDL_Surface *destination, SDL_Rect *area)
{
	static char *functionName="paint";
	CSLAF *laf = CSLAF::getCurrentLAF(); 
	int type = mType;
	if ((!(mState & BORDER_STATE_VISIBLE)) || (type == BORDER_TYPE_NONE))
	{
		return;
	}

	mShadeBrighter = CSHelper::colorPercent(laf->getBorderDefaultBrightestColor(), 100);
	mShadeBright = CSHelper::colorPercent(laf->getBorderDefaultBrightestColor(), 70);
	mShadeDark = CSHelper::colorPercent(laf->getBorderDefaultBrightestColor(), 50);
	mShadeDarker = CSHelper::colorPercent(laf->getBorderDefaultBrightestColor(), 20);

	// if type up
	if (type & BORDER_TYPE_UP)
	{
		// and pressed
		if (mState & BORDER_STATE_PRESSED)
		{
			// draw pressed border
			type = type & (~BORDER_TYPE_UP);
		}
		else
		{
			// if hovering enabled 
			if (mState & BORDER_STATE_HOVERING)
			{
				// and hovering happens
				if (type & BORDER_TYPE_HOVER)
				{
					// draw on hovering Type up
					type = type | BORDER_TYPE_UP;
				}
				else
				{
					// if overing enabeld and not hovering, draw nothing
					return;
					//type = type & (~BORDER_TYPE_UP);
				}
			}
			else // if hovering disabled
			{
				// and not pressed, than draw border up on pressed
				type = type | BORDER_TYPE_UP;
			}
		}
	}
	else
	{
	}


	SDL_Rect copy;
	int o = 0;
	if (type & BORDER_TYPE_UP)
	{
		if (mBorderOuterEdgeWidth)
		{
			SDL_Rect a = *area;
			a.x += o;
			a.y += o;
			a.h = area->h;//mBorderOuterEdgeWidth;
			a.w = a.w - 2*o;
			if (a.w > 60000) a.w = 0;
			copy = a;
			if (mType & BORDER_TYPE_SINGLE_COLOR)
			{
				SDLMain::DrawRoundRectNorth(destination, copy.x, copy.y, copy.w, copy.h, COLOR(destination, mShadeDarker), mCorner, mBorderOuterEdgeWidth);
				SDLMain::DrawRoundRectSouth(destination, copy.x, copy.y, copy.w, copy.h, COLOR(destination, mShadeDarker), mCorner, mBorderOuterEdgeWidth);
				SDLMain::DrawRoundRectWest(destination, copy.x, copy.y, copy.w, copy.h, COLOR(destination, mShadeDarker), mCorner, mBorderOuterEdgeWidth);
				SDLMain::DrawRoundRectEast(destination, copy.x, copy.y, copy.w, copy.h, COLOR(destination, mShadeDarker), mCorner, mBorderOuterEdgeWidth);
			}
			else
			{
				SDLMain::DrawRoundRectNorth(destination, copy.x, copy.y, copy.w, copy.h, COLOR(destination, mShadeBrighter), mCorner, mBorderOuterEdgeWidth);
				SDLMain::DrawRoundRectSouth(destination, copy.x, copy.y, copy.w, copy.h, COLOR(destination, mShadeDarker), mCorner, mBorderOuterEdgeWidth);
				SDLMain::DrawRoundRectWest(destination, copy.x, copy.y, copy.w, copy.h, COLOR(destination, mShadeDarker), mCorner, mBorderOuterEdgeWidth);
				SDLMain::DrawRoundRectEast(destination, copy.x, copy.y, copy.w, copy.h, COLOR(destination, mShadeBrighter), mCorner, mBorderOuterEdgeWidth);
			}
		}	
		o+=mBorderOuterEdgeWidth;
		if (type & BORDER_TYPE_DOUBLE)
		{
			if (mBorderInnerEdgeWidth)
			{
				SDL_Rect a = *area;
				a.x += o;
				a.y += o;
				a.h = a.h - 2*o;
				a.w = a.w - 2*o;
				if (a.w > 60000) a.w = 0;
				copy = a;
				if (mType & BORDER_TYPE_SINGLE_COLOR)
				{
					SDLMain::DrawRoundRectNorth(destination, copy.x, copy.y, copy.w, copy.h, COLOR(destination, mShadeDark), mCorner, mBorderInnerEdgeWidth);
					SDLMain::DrawRoundRectSouth(destination, copy.x, copy.y, copy.w, copy.h, COLOR(destination, mShadeDark), mCorner, mBorderInnerEdgeWidth);
					SDLMain::DrawRoundRectWest(destination, copy.x, copy.y, copy.w, copy.h, COLOR(destination, mShadeDark), mCorner, mBorderInnerEdgeWidth);
					SDLMain::DrawRoundRectEast(destination, copy.x, copy.y, copy.w, copy.h, COLOR(destination, mShadeDark), mCorner, mBorderInnerEdgeWidth);
				}
				else
				{
					SDLMain::DrawRoundRectNorth(destination, copy.x, copy.y, copy.w, copy.h, COLOR(destination, mShadeBright), mCorner, mBorderInnerEdgeWidth);
					SDLMain::DrawRoundRectSouth(destination, copy.x, copy.y, copy.w, copy.h, COLOR(destination, mShadeDark), mCorner, mBorderInnerEdgeWidth);
					SDLMain::DrawRoundRectWest(destination, copy.x, copy.y, copy.w, copy.h, COLOR(destination, mShadeDark), mCorner, mBorderInnerEdgeWidth);
					SDLMain::DrawRoundRectEast(destination, copy.x, copy.y, copy.w, copy.h, COLOR(destination, mShadeBright), mCorner, mBorderInnerEdgeWidth);
				}
			}	
		}

		o+=mBorderInnerEdgeWidth;
		if (type & BORDER_TYPE_INTER)
		{
			SDL_Rect a = *area;
			a.x += o + mBorderInterEdgeWidth;
			a.y += o + mBorderInterEdgeWidth;
			a.w = a.w - 2*o - (2*mBorderInterEdgeWidth);
			a.h = a.h - 2*o - (2*mBorderInterEdgeWidth);
			if (a.w > 60000) a.w = 0;
			if (a.h > 60000) a.h = 0;
			
			int typeSave = type;
			type = type - BORDER_TYPE_INTER - BORDER_TYPE_UP;
			paint(destination, &a);
			type = typeSave;
		}
	}	
	else // down
	{
		if (mBorderOuterEdgeWidth)
		{
			SDL_Rect a = *area;
			a.x += o;
			a.y += o;
			a.h = area->h;//mBorderOuterEdgeWidth;
			a.w = a.w - 2*o;
			if (a.w > 60000) a.w = 0;
			copy = a;
			SDLMain::DrawRoundRectNorth(destination, copy.x, copy.y, copy.w, copy.h, COLOR(destination, mShadeDarker), mCorner, mBorderOuterEdgeWidth);
			SDLMain::DrawRoundRectSouth(destination, copy.x, copy.y, copy.w, copy.h, COLOR(destination, mShadeBrighter), mCorner, mBorderOuterEdgeWidth);
			SDLMain::DrawRoundRectWest(destination, copy.x, copy.y, copy.w, copy.h, COLOR(destination, mShadeBrighter), mCorner, mBorderOuterEdgeWidth);
			SDLMain::DrawRoundRectEast(destination, copy.x, copy.y, copy.w, copy.h, COLOR(destination, mShadeDarker), mCorner, mBorderOuterEdgeWidth);
		}	
		o+=mBorderOuterEdgeWidth;
		if (type & BORDER_TYPE_DOUBLE)
		{
			if (mBorderInnerEdgeWidth)
			{
				SDL_Rect a = *area;
				a.x += o;
				a.y += o;
				a.h = a.h - 2*o;
				a.w = a.w - 2*o;
				if (a.w > 60000) a.w = 0;
				copy = a;
				SDLMain::DrawRoundRectNorth(destination, copy.x, copy.y, copy.w, copy.h, COLOR(destination, mShadeDark), mCorner, mBorderOuterEdgeWidth);
				SDLMain::DrawRoundRectSouth(destination, copy.x, copy.y, copy.w, copy.h, COLOR(destination, mShadeBright), mCorner, mBorderOuterEdgeWidth);
				SDLMain::DrawRoundRectWest(destination, copy.x, copy.y, copy.w, copy.h, COLOR(destination, mShadeBright), mCorner, mBorderOuterEdgeWidth);
				SDLMain::DrawRoundRectEast(destination, copy.x, copy.y, copy.w, copy.h, COLOR(destination, mShadeDark), mCorner, mBorderOuterEdgeWidth);
			}
		}
		if (type & BORDER_TYPE_INTER)
		{
			SDL_Rect a = *area;
			a.x += o + mBorderInterEdgeWidth;
			a.y += o + mBorderInterEdgeWidth;
			a.w = a.w - 2*o - (2*mBorderInterEdgeWidth);
			a.h = a.h - 2*o - (2*mBorderInterEdgeWidth);
			if (a.w > 60000) a.w = 0;
			if (a.h > 60000) a.h = 0;

			int typeSave = type;
			type = type - BORDER_TYPE_INTER - BORDER_TYPE_UP;
			paint(destination, &a);
			type = typeSave;
		}
		o+=mBorderInnerEdgeWidth;
	}
}


CSRoundedBorder *CSRoundedBorder::getBorder(int type)
{
	CSRoundedBorder *border = new CSRoundedBorder(); 
	border->mType = type;
	if (border->mType == BORDER_TYPE_NONE)
	{
		border->mBorderOuterEdgeWidth = 0;
		border->mBorderInnerEdgeWidth = 0;
		border->mBorderInterEdgeWidth = 0;
	}
	else
	{
		border->mBorderOuterEdgeWidth = 1;
		border->mBorderInnerEdgeWidth = 1;
		border->mBorderInterEdgeWidth = 2;
	}
	return border;
}
