#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

#include "CSButton.h"
#include "CSLAF.h"

const char *CSButton::CLASS = "CSButton";
const char *CSDragButton::CLASS = "CSDragButton";

// button frees Icon!
CSButton::CSButton(CSIcon *icon) : CSGrafikElement(0, 0)
{
	static char *functionName="CSButton";
	LOG_ENTER 
	mIcon = icon;
	initButton();
	LOG_EXIT
}

CSButton::CSButton(const std::string &text)  : CSGrafikElement(0, 0)
{
	static char *functionName="CSButton";
	LOG_ENTER 
	mIcon = CSTextIcon::buildIcon(text);
	initButton();

	LOG_EXIT
}

CSButton::~CSButton()
{
	if (mIcon)
	{
		delete(mIcon);
		mIcon = 0;
	}
}

void CSButton::initButton()
{
	// Button layout
	getLayoutDataInternal()->setPackedHorizontal(true);
	getLayoutDataInternal()->setPackedVertical(true);
	getLayoutDataInternal()->setCenteredHorizontal(true);
	getLayoutDataInternal()->setCenteredVertical(true);
	
	setLayoutManager(new CSLayoutManagerStackVertical());

	// Icon layout
	CSLayoutData layoutData = CSLayoutData(POSITION_CENTER, true);

	if (mIcon)
	{
		layoutData.setPackedHorizontal(true);
		layoutData.setPackedVertical(true);
		layoutData.setCenteredHorizontal(true);
		layoutData.setCenteredVertical(true);
		layoutData.setSpacing(0,0);
		mIcon->setLayoutData(layoutData);

		addElement(mIcon);
		mIcon->addMessageListener(this, GUI_MESSAGE);
	}
	mBorderState = BUTTON_DEFAULT;
	MESSAGE_BUTTON_PRESSED.setSubtype(BUTTON_PRESSED_MESSAGE);
	MESSAGE_BUTTON_RELEASED.setSubtype(BUTTON_RELEASED_MESSAGE);
	layoutSetup();
	getLayoutManager()->pack(this);
}

void CSButton::reactOnMessage(CSMessage *message)
{
	reactOnMessageButton(message);
}

void CSButton::reactOnMessageButton(CSMessage *message)
{
	static char *functionName="reactOnMessage";
	if (message->mIsHandled)
	{
		reactOnMessageGrafikElement(message);
		return;
	}

	if (!getEnabled()) return;
	if (message->getType() == GUI_MESSAGE)
	{
		GuiMessage *gm = (GuiMessage *) message;
		
		if (isChild(gm->receiver))
		{
			switch (message->getSubtype())
			{
				case MOUSE_BUTTON_RELEASED_MESSAGE:
				{
					mBorderState = getButtonState() & (~BUTTON_PRESSED);
					MESSAGE_BUTTON_RELEASED.actionId = mActionId;
					MESSAGE_BUTTON_RELEASED.receiver = 0;
					MESSAGE_BUTTON_RELEASED.mIsHandled = false;
//event is sent in response to mouse click event
//listener gets tzwo evebnts this and mouse click -> endless look
					sendMessage(MESSAGE_BUTTON_RELEASED);
					gm->mIsHandled = true;
					break;
				}

				case MOUSE_BUTTON_PRESSED_MESSAGE:
				{
					if (getButtonState() & BUTTON_ENABLED)
					{
						mBorderState = getButtonState() | BUTTON_PRESSED;
						MESSAGE_BUTTON_PRESSED.actionId = mActionId;
						MESSAGE_BUTTON_PRESSED.receiver = 0;
						MESSAGE_BUTTON_PRESSED.mIsHandled = false;
						sendMessage(MESSAGE_BUTTON_PRESSED);
					}
					gm->mIsHandled = true;
					break;
				}
			}
		}
		else
		{
			// if somewhere else released
			// at least reset the state of the button
			// (but don't tell anyone about it)
			if (message->getSubtype() ==  MOUSE_BUTTON_RELEASED_MESSAGE)
			{
				mBorderState = getButtonState() & (~BUTTON_PRESSED);
				gm->mIsHandled = true;
			}
		}
	}
	reactOnMessageGrafikElement(message);
}

void CSButton::layoutSetupButton()
{
	static char *functionName="layoutSetupButton";
	LOG_ENTER 
	if (mIcon)
	{
		mMinHeight = mIcon->getMinHeight();
		mMinWidth = mIcon->getMinWidth();
	}
	else
	{
		mMinHeight = 0;
		mMinWidth = 0;
	}
	LOG_EXIT
}

// button frees Icon!
CSDragButton::CSDragButton(CSIcon *icon) : CSButton(icon)
{
	static char *functionName="CSDragButton";
	LOG_ENTER 
	mDragging=false;
	MESSAGE_BUTTON_DRAGGED.setSubtype(BUTTON_DRAGGED_MESSAGE);
	getLayoutDataInternal()->setPackedHorizontal(false);
	getLayoutDataInternal()->setPackedVertical(false);
	LOG_EXIT
}

void CSDragButton::reactOnMessage(CSMessage *message)
{
	reactOnMessageDragButton(message);
}

void CSDragButton::reactOnMessageDragButton(CSMessage *message)
{
	static char *functionName="reactOnMessage";
	if (message->mIsHandled)
	{
		if (message->getType() == GUI_MESSAGE)
		{
			GuiMessage *gm = (GuiMessage *) message;
			if (message->getSubtype() ==  MOUSE_BUTTON_RELEASED_MESSAGE)
			{
				mDragging = false;
				gm->mIsHandled = false;
			}
		}
		reactOnMessageButton(message);
		return;
	}
	if (message->getType() == GUI_MESSAGE)
	{
		GuiMessage *gm = (GuiMessage *) message;
		if (message->getSubtype() ==  MOUSE_MOTION_MESSAGE)
		{
			if (mDragging)
			{
				int deltaX = gm->screenX - mDragStartX;
				int deltaY = gm->screenY - mDragStartY;

				MESSAGE_BUTTON_DRAGGED.screenX = gm->screenX;
				MESSAGE_BUTTON_DRAGGED.screenY = gm->screenY;
				MESSAGE_BUTTON_DRAGGED.deltaX = deltaX;
				MESSAGE_BUTTON_DRAGGED.deltaY = deltaY;
				MESSAGE_BUTTON_DRAGGED.actionId = mActionId;
				MESSAGE_BUTTON_DRAGGED.mIsHandled = false;
				MESSAGE_BUTTON_DRAGGED.receiver = 0;
				sendMessage(MESSAGE_BUTTON_DRAGGED);

				mDragStartX = MESSAGE_BUTTON_DRAGGED.deltaX;
				mDragStartY = MESSAGE_BUTTON_DRAGGED.deltaY;
			}
			gm->mIsHandled = true;
		}
		if (message->getSubtype() ==  MOUSE_BUTTON_RELEASED_MESSAGE)
		{
			mDragging = false;
			gm->mIsHandled = false;
		}
		if ((gm->receiver == this) || ( gm->receiver == mIcon))
		{
			switch (message->getSubtype())
			{
				case MOUSE_BUTTON_PRESSED_MESSAGE:
				{
					mDragging = true;
					mDragStartX = gm->screenX;
					mDragStartY = gm->screenY;
					gm->mIsHandled = true;
					break;
				}
			}
		}
	}
	reactOnMessageButton(message);
}
