#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

#include "CSCheckBox.h"
#include "CSBorder.h"
#include "CSFont.h"
#include "CSLAF.h"

const char *CSCheckBox::CLASS = "CSCheckBox";

CSCheckBox::CSCheckBox(void) : CSGrafikElement(0, 0)
{
	static char *functionName="CSCheckBox";
	LOG_ENTER 
	initCheckBox();
	LOG_EXIT
}

void CSCheckBox::initCheckBox()
{
	mCheckState = false;
	mState = BUTTON_DEFAULT;
	setLayoutManager(new CSLayoutManagerXY());
	MESSAGE_CHECKBOX_CHANGED.setSubtype(CHECKBOX_CHANGED_MESSAGE);
	layoutSetup();
}

void CSCheckBox::paint(SDL_Surface *destination, SDL_Rect *parentViewport)
{
	static char *functionName="paint";
	SDL_Rect elementArea = getElementArea();
	elementArea.x += parentViewport->x;
	elementArea.y += parentViewport->y;
	
	if (mCheckState)
	{
		int w = getWidth() - getBorder()->getTotalWidth() -1;
		int x = getBorder()->getSizeWest() + mInset.getSizeWest();
		int y = getBorder()->getSizeNorth() + mInset.getSizeNorth();
		line(destination, &elementArea,x,y+2*w/3,x+w/3,y+w,0x000000);
		line(destination, &elementArea,x+w/3,y+w,x+w,y,0x000000);
		x++;
		line(destination, &elementArea,x,y+2*w/3,x+w/3,y+w,0x000000);
		line(destination, &elementArea,x+w/3,y+w,x+w,y,0x000000);
	}
}

void CSCheckBox::reactOnMessage(CSMessage *message)
{
	reactOnMessageCheckBox(message);
}

void CSCheckBox::reactOnMessageCheckBox(CSMessage *message)
{
	static char *functionName="reactOnMessage";
	if (message->mIsHandled)
	{
		reactOnMessageGrafikElement(message);
		return;
	}
	if (!getEnabled()) return;
	if (message->getType() == GUI_MESSAGE)
	{
		GuiMessage *gm = (GuiMessage *) message;
		if (gm->receiver != this)
		{
			if (message->getSubtype() ==  MOUSE_BUTTON_RELEASED_MESSAGE)
			{
				mState = mState & (~BUTTON_PRESSED);
				gm->mIsHandled = true;
			}
		}
		else // receiver == this
		{
			switch (message->getSubtype())
			{
				case MOUSE_BUTTON_RELEASED_MESSAGE:
				{
					mState = mState & (~BUTTON_PRESSED);
					MESSAGE_CHECKBOX_CHANGED.actionId = mActionId;
					MESSAGE_CHECKBOX_CHANGED.newCheckBoxState = mCheckState;
					MESSAGE_CHECKBOX_CHANGED.mIsHandled = false;
					sendMessage(MESSAGE_CHECKBOX_CHANGED);
					gm->mIsHandled = true;
					break;
				}

				case MOUSE_BUTTON_PRESSED_MESSAGE:
				{
					if (mState & BUTTON_ENABLED)
					{
						mState = mState | BUTTON_PRESSED;
						mCheckState = !mCheckState;

						// callback
					}
					gm->mIsHandled = true;
					break;
				}
			}
		}
	}
	reactOnMessageGrafikElement(message);
}

void CSCheckBox::layoutSetupCheckBox()
{
	static char *functionName="layoutSetupCheckBox";
	LOG_ENTER 
	CSLAF *laf = CSLAF::getCurrentLAF(); // make sure laf is loaded/initialized at least once

	mMinHeight = laf->getCheckBoxSize() - mInset.getTotalHeight() - getBorder()->getTotalHeight();
	mMinWidth = mMinHeight;	// build a square
	
	mHeight = mMinHeight + mInset.getTotalHeight() + getBorder()->getTotalHeight();
	mWidth = mHeight; // build a square

	LOG_EXIT
}
