#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

#include "CSCombobox.h"
#include "CSFont.h"
#include "CSDesktop.h"
#include "CSLAF.h"
#include "CSButton.h"
#include "CSListbox.h"
const char *CSCombobox::CLASS = "CSCombobox";

CSCombobox::CSCombobox(int width) : CSGrafikElement(0, width)
{
	static char *functionName="CSCombobox";
	LOG_ENTER 
	init("");
	LOG_EXIT
}

CSCombobox::~CSCombobox()
{
}

void CSCombobox::init(const std::string &text)
{
	static char *functionName="init";
	LOG_ENTER 
	mString = text;
	CSLAF *laf = CSLAF::getCurrentLAF();
	
	mTickTimeNext = 0;
	mCursorRate = laf->getCursorRate(); // 500 milliseconds
	mCursorShown = false;

	mListboxActive = false;
	mIsEditable = true;
	//setEnabled(false);
	mListbox = 0;
	mMore = 0;
	mTextfield = 0;
	mListbox = 0;
	MESSAGE_COMBOBOX_ITEM_SELECTED.setSubtype(COMBOBOX_ITEM_SELECTED_MESSAGE);
	setLayoutManager(new CSLayoutManagerBorder());
	CSLayoutData layoutData = getLayoutData();
	layoutData.setPackedVertical(true);
	setLayoutData(layoutData);
	
	rebuildElement();
	layoutSetup();
	LOG_EXIT
}

void CSCombobox::putString(SDL_Surface *destination, SDL_Rect *parentViewport, int x1, int y1, const std::string &string)
{
	static char *functionName="putString";
	
	x1 += parentViewport->x + getX();
	y1 += parentViewport->y + getY();
	getFont()->putString(destination, x1, y1, getTextColor(), string);
}

//! \todo build a combobox from a textfield, a button and a listbos
//! that is, remove custom textfield and reuse the common textfield!
void CSCombobox::paint(SDL_Surface *destination, SDL_Rect *parentViewport)
{
	static char *functionName="paint";
	SDL_Rect rectSave;
	SDL_GetClipRect(destination, &rectSave);

	SDL_Rect rectClip = getViewportArea();
	rectClip.x += parentViewport->x + getX();
	rectClip.y += parentViewport->y + getY();

	SDLMain::adjustClipping(rectClip, rectSave);
	SDL_SetClipRect(destination, &rectClip);

	int xOffset = getBorder()->getSizeWest() + getInset().getSizeWest() + mComboBoxInset.getSizeWest();
	int yOffset = getBorder()->getSizeNorth() + getInset().getSizeNorth() + mComboBoxInset.getSizeNorth();

	if ((getFocused()) && (getActive()))
	{
		float currentTick = SDL_GetTicks() + mCursorRate;
		if (currentTick > mTickTimeNext)
		{
			mTickTimeNext = currentTick + mCursorRate;
			mCursorShown = !mCursorShown;
		}

		putString(destination, parentViewport, xOffset, yOffset + mTextOffset,  mWorkingString);
		if (mCursorShown)
		{
			int witdh = getFont()->getWidth(mWorkingString);
			putString(destination, parentViewport, xOffset + witdh, yOffset + mTextOffset, "_");
		}
	}
	else
	{
		putString(destination, parentViewport, xOffset, yOffset + mTextOffset, mString);
	}
	SDL_SetClipRect(destination, &rectSave);
	paintChildren(destination, parentViewport);
}

CSStringlist CSCombobox::getItems()
{
	if (!mListbox)
	{
		return CSStringlist();
	}
	return mListbox->getItems();
}

void CSCombobox::setItems(const CSStringlist &list)
{
	if (mListbox)
	{
		mListbox->setItems(list);
	}
}

std::string CSCombobox::getSelectedString()
{
	if (!mListbox)
	{
		return std::string();
	}
	
	return mListbox->getItems().getString(mListbox->getSelected());
}


void *CSCombobox::getSelectedObject()
{
	if (!mListbox)
	{
		return 0;
	}
	
	return mListbox->getItems().getObject(mListbox->getSelected());
}

void CSCombobox::setSelected(int i)
{
	if (!mListbox)
	{
		return ;
	}
	mListbox->setSelected(i);
}

void CSCombobox::reactOnMessage(CSMessage *message)
{
	reactOnMessageCombobox(message);
}

void CSCombobox::reactOnMessageCombobox(CSMessage *message)
{
	static char *functionName="reactOnMessage";
	if (message->mIsHandled)
	{
		reactOnMessageGrafikElement(message);
		return;
	}
	if (!getEnabled()) return;
	
	if (message->getType() == GUI_MESSAGE)
	{
		GuiMessage *gm = (GuiMessage *) message;
		if (gm->receiver != this)
		{
			if (gm->getSubtype() == BUTTON_PRESSED_MESSAGE)
			{
				if (gm->actionId == COMBOBOX_BUTTON_MORE_ACTION_ID)
				{
					setListActive(!mListboxActive);
				}
				else
				{
					setListActive(false);
				}
				gm->mIsHandled = true;
			}

			if (gm->getSubtype() == LIST_ITEM_SELECTED_MESSAGE)
			{
				setListActive(true);
				setListActive(false);
				getDesktop()->setFocusedComponent(0);
				mString = getSelectedString();

				MESSAGE_COMBOBOX_ITEM_SELECTED.selected = gm->selected;
				MESSAGE_COMBOBOX_ITEM_SELECTED.actionId = mActionId;
				MESSAGE_COMBOBOX_ITEM_SELECTED.mIsHandled = false;
				sendMessage(MESSAGE_COMBOBOX_ITEM_SELECTED);
				gm->mIsHandled = true;
			}
			if (gm->getSubtype() == FOCUS_LOST_MESSAGE)
			{
				if (getActive())
				{
					mString = mWorkingString;
					setActive(false);
				}
				setListActive(false);
				gm->mIsHandled = true;
			}
		}
		else // receiver == this
		{
			switch (message->getSubtype())
			{
				case FOCUS_GAINED_MESSAGE:
				{
					if (mIsEditable)
					{
						mWorkingString = mString;
						setActive(true);
						setListActive(!mListboxActive);
					}
					gm->mIsHandled = true;
					break;
				}
				
				case FOCUS_LOST_MESSAGE:
				{
					if (getActive())
					{
						mString = mWorkingString;
						setActive(false);
					}
					setListActive(false);
					gm->mIsHandled = true;
					break;
				}

				case MOUSE_BUTTON_PRESSED_MESSAGE:
				{
					setListActive(true);
					gm->mIsHandled = true;
					break;
				}

				case KEY_PRESSED_MESSAGE:
				{
					if (!getActive())
					{
						break;
					}
					int ch=gm->keyUnicodeChar;

					if ((gm->keySymbolic == SDLK_RETURN) || 
						(gm->keySymbolic == SDLK_KP_ENTER) || 
						(gm->keySymbolic == SDLK_TAB))
					{
						getDesktop()->focusedNextComponent();
						gm->mIsHandled = true;
						break;
					}
					if (((ch>31)||(ch=='\b')) && (ch<128)) 
					{
						int len = mWorkingString.length();
						if (ch=='\b')
						{
							if (len>0)
							{
								mWorkingString.resize(len-1);
							}
						}
						else
						{
							mWorkingString += (char) ch;
							
							len = mWorkingString.length();
							int witdh = getFont()->getWidth(mWorkingString) + getFont()->getWidth("_");
							if (witdh > getWidth() - (getBorder()->getSizeWest()))
							{
								mWorkingString.resize(len-1);
							}
						}
						gm->mIsHandled = true;
					}
					break;
				}
			}
		}
	}
	reactOnMessageGrafikElement(message);
}

void CSCombobox::setListActive(bool active)
{
	if (active == mListboxActive)
	{
		return;
	}
	mListboxActive = active;
	
	if (!mListboxActive)
	{
		mListboxActive = false;
		mListbox->setVisible(false);
		CSDesktop *desktop = getDesktop();
		desktop->removeFrontElement(mListbox);
	}
	else
	{
		mListboxActive = true;
		mListbox->setVisible(true);

		int x = 0;
		int y = getHeight();
		translateToDesktop(x, y);
		CSDesktop *desktop = getDesktop();
		desktop->addFrontElement(mListbox, x, y);
	}
}

void CSCombobox::rebuildElement()
{
	static char *functionName="rebuildElement";
	CSLAF *laf = CSLAF::getCurrentLAF();
	if (mMore)
	{
		mMore->removeMessageListener(this);
		removeElement(mMore);
		delete (mMore);
		mMore = 0;
	}

	mMore = new CSButton(laf->getIcon(LAF_ICON_TYPE_DOWN));

	CSLayoutData layoutData = mMore->getLayoutData();
	layoutData.setSpacing(0,0);
	layoutData.setCenteredHorizontal(false);
	mMore->setLayoutData(layoutData);
	mMore->setActionId(COMBOBOX_BUTTON_MORE_ACTION_ID);
	mMore->addMessageListener(this, GUI_MESSAGE);
	addElement(mMore, POSITION_EAST);

	if (mListbox)
	{
		mListbox->removeMessageListener(this);
		removeElement(mListbox);
		delete (mListbox);
		mListbox = 0;
	}
	mListbox = new CSListbox(laf->getComboboxItemsShown(), mWidth);
	mListbox->setFocusable(false);
	mListbox->addMessageListener(this, GUI_MESSAGE);
	mListbox->setItemsShown(laf->getComboboxItemsShown());
	mListbox->setVisible(false);
	mListbox->setHoverSelection(true);
}

void CSCombobox::layoutSetupCombobox()
{
	static char *functionName="layoutSetupCombobox";
	CSLAF *laf = CSLAF::getCurrentLAF();
	mComboBoxInset = CSInset(2);

	mTextOffset = 0; 
	mHeight = getFont()->getHeight() + mComboBoxInset.getTotalHeight();
	mMinHeight = getFont()->getHeight(); // changed 07.01.2005
	if (mHeight < mMore->getHeight())
	{
		mHeight = mMore->getHeight();
		mTextOffset = (mMore->getHeight() - mHeight)/2;
		mMinHeight = mMore->getHeight(); // changed 07.01.2005
	}
//	mMinHeight = mHeight;
	mHeight += getBorder()->getTotalHeight() + mInset.getTotalHeight();
	//mMinWidth = mWidth - getBorder()->getTotalWidth() + mInset.getTotalWidth();;
	mMinWidth = mWidth; // changed 07.01.2005

	
	

}


