#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

#include "CSDesktop.h"
#include "CSWindow.h"
#include "CSLabel.h"
#include "CSFont.h"
#include "CSArea.h"
#include "CSBorder.h"
#include "CSPanel.h"

#include <vector>
#include <string>

#include "CSMouseCursor.h"


const char *CSDesktop::CLASS = "CSDesktop";

CSDesktop::CSDesktop() : CSGrafikElement(0, 0)
{
	static char *functionName="CSDesktop";
	LOG_ENTER 
	mHeight = SDLMain::getScreenHeight();
	mWidth = SDLMain::getScreenWidth();
	mScreenArea.x = 0;
	mScreenArea.y = 0;
	mScreenArea.h = mHeight;
	mScreenArea.w = mWidth;
	mFocusedComponent = 0;
	mCurrentUnderMouseComponent = 0;
	mStartDown = 0;
	mRequestLayout = true;
	init(mHeight, mWidth);
	
	mNorth = 0;
	mWest = 0;
	mSouth = 0;
	mEast = 0;
	mCenter = 0;
	setMainLayoutManager(new CSLayoutManagerBorder());

	mNorth = new CSPanel();
	mNorth->setLayoutManager(new CSLayoutManagerStackVertical());
	mWest = new CSPanel();
	mWest->setLayoutManager(new CSLayoutManagerStackHorizontal());
	mSouth = new CSPanel();
	mSouth->setLayoutManager(new CSLayoutManagerStackVertical());
	mEast = new CSPanel();
	mEast->setLayoutManager(new CSLayoutManagerStackHorizontal());
	mCenter = new CSPanel();
	
	CSLayoutData layout = CSLayoutData(POSITION_NORTH);
	layout.setStretchedHorizontal(true);
	layout.setPackedVertical(true);
	mNorth->setLayoutData(layout);
	addMainElement(mNorth);

	layout.setPosition(POSITION_SOUTH);
	mSouth->setLayoutData(layout);
	addMainElement(mSouth);

	layout.setPosition(POSITION_CENTER);
	layout.setStretchedVertical(true);
	mCenter->setLayoutData(layout);
	addMainElement(mCenter);
	setMainElement(mCenter);
	
	layout.setPackedHorizontal(true);
	layout.setPosition(POSITION_WEST);
	mWest->setLayoutData(layout);
	addMainElement(mWest);
	
	layout.setPosition(POSITION_EAST);
	mEast->setLayoutData(layout);
	addMainElement(mEast);
	
	mLastTimePainted = -1;
	mLastCursorX = -1;
	mLastCursorY = -1;
	mModalComponent = 0;
	mTooltipPanel = new CSPanel(0, 0);
	mTooltipPanel->setLayoutManager(new CSLayoutManagerStackVertical());
	layout = CSLayoutData(0,0);
	layout.setPackedVertical(true);
	layout.setPackedHorizontal(true);
	mTooltipPanel->setLayoutData(layout);
	mTooltipLabel = new CSLabel("");
	mTooltipLabel->setLayoutData(CSLayoutData(POSITION_ORDER_TOP));
	mTooltipPanel->addElement(mTooltipLabel);
	addMainElement(mTooltipPanel);
	LOG_EXIT
}

CSDesktop::CSDesktop(SDL_Rect area) : CSGrafikElement(area.h, area.w)
{
	static char *functionName="CSDesktop";
	LOG_ENTER 
	mX = area.x;
	mY = area.y;
	mViewportX = area.x;
	mViewportY = area.y;
	
	mFocusedComponent = 0;
	mCurrentUnderMouseComponent = 0;
	mStartDown = 0;
	LOG_EXIT
}

CSDesktop::~CSDesktop()
{
	static char *functionName="~CSDesktop";
	LOG_ENTER 
	LOG_EXIT
}

void CSDesktop::showToolTip()
{
	int receiverX, receiverY;
	CSGrafikElement *element = getElement(mCursorX, mCursorY, &mScreenArea, receiverX, receiverY);
	std::string text = element->getTooltipText();
	if (text.length() > 0)
	{
		if (!mTooltipPanel->getVisible())
		{
			mTooltipLabel->setText(text);
			mTooltipPanel->setPosition(mCursorX, mCursorY);
			toFront(mTooltipPanel);
			mTooltipPanel->setVisible(true);
		}
	}
}

void CSDesktop::paint(SDL_Surface *destination, SDL_Rect *parentViewport)
{
	static char *functionName="paint";
	CSLAF *laf = CSLAF::getCurrentLAF();
	long mCurrentTime = SDL_GetTicks();

	if (mRequestLayout)
	{
		// every change should be communicated now!	
		doLayout(); 
		mRequestLayout = false;
	}

	if ((mLastCursorX == mCursorX) && (mLastCursorY == mCursorY))
	{
		long t = mCurrentTime - mLastTimePainted;
		if (t >= laf->tooltipThreshold)
		{
			showToolTip();
		}
	}
	else
	{
		mTooltipPanel->setVisible(false);
		mLastTimePainted = mCurrentTime;
	}
	mLastCursorX = mCursorX;
	mLastCursorY = mCursorY;

	SDL_Rect thisArea = mElementArea;
	if (parentViewport)
	{
		thisArea.x += parentViewport->x;
		thisArea.y += parentViewport->y;
	}

	SDL_Rect thisAreaCopy = thisArea;

	SDL_SetClipRect(destination, &thisAreaCopy);
	if (parentViewport)
	{
		paintChildren(destination, parentViewport);
	}
	else
	{
		paintChildren(destination, &mScreenArea);
	}

	int receiverX, receiverY;
	CSGrafikElement *receiver = getElement(mCursorX, mCursorY, &mScreenArea, receiverX, receiverY);
	if (receiver)
	{
		receiver->getCursor()->setPosition(mCursorX, mCursorY);
		receiver->getCursor()->paint(destination, &thisArea);
	}
	else
	{
		getCursor()->setPosition(mCursorX, mCursorY);
		getCursor()->paint(destination, &thisArea);
	}
	SDL_SetClipRect(destination, 0);
}

void CSDesktop::addElementNorth(CSGrafikElement *element)
{
	mNorth->addElement(element);
}

void CSDesktop::addElementSouth(CSGrafikElement *element)
{
	mSouth->addElement(element);
}

void CSDesktop::addElementEast(CSGrafikElement *element)
{
	mEast->addElement(element);
}

void CSDesktop::addElementWest(CSGrafikElement *element)
{
	mWest->addElement(element);
}

void CSDesktop::onMouseMotion(int x, int y)
{
	static char *functionName="onMouseMotion";
	mCursorX = x;
	mCursorY = y;
	MESSAGE_MOUSE_MOTION.receiver = getElement(x, y, &mScreenArea, MESSAGE_MOUSE_MOTION.receiverX, MESSAGE_MOUSE_MOTION.receiverY);

	if ((mModalComponent) && (!mModalComponent->isChild(MESSAGE_MOUSE_MOTION.receiver)))
	{
		if ((mCurrentUnderMouseComponent!=0) && (mCurrentUnderMouseComponent != MESSAGE_MOUSE_MOTION.receiver))
		{
			if (mStartDown == mCurrentUnderMouseComponent)
			{
				MESSAGE_MOUSE_MOTION.screenX = x;
				MESSAGE_MOUSE_MOTION.screenY = y;
				MESSAGE_MOUSE_MOTION.sender = this;
				MESSAGE_MOUSE_MOTION.mIsHandled = false;
				MESSAGE_MOUSE_MOTION.receiver = mStartDown;
				mStartDown->sendMessage(MESSAGE_MOUSE_MOTION);
				return;
			}
		}
		return;
	}
	if ((mCurrentUnderMouseComponent!=0) && (mCurrentUnderMouseComponent != MESSAGE_MOUSE_MOTION.receiver))
	{
		if (mStartDown == mCurrentUnderMouseComponent)
		{
			MESSAGE_MOUSE_MOTION.screenX = x;
			MESSAGE_MOUSE_MOTION.screenY = y;
			MESSAGE_MOUSE_MOTION.sender = this;
			MESSAGE_MOUSE_MOTION.mIsHandled = false;
			MESSAGE_MOUSE_MOTION.receiver = mStartDown;
//delete all clear startdown and mCurrentUnderMouseComponent
			mStartDown->sendMessage(MESSAGE_MOUSE_MOTION);
			return;
		}
		else
		{
			// old
			MESSAGE_MOUSE_MOTION_LOST.screenX = x;
			MESSAGE_MOUSE_MOTION_LOST.screenY = y;
			MESSAGE_MOUSE_MOTION_LOST.sender = this;
			MESSAGE_MOUSE_MOTION_LOST.mIsHandled = false;
			MESSAGE_MOUSE_MOTION_LOST.receiver = mCurrentUnderMouseComponent;
			mCurrentUnderMouseComponent->sendMessage(MESSAGE_MOUSE_MOTION_LOST);
		}
	}
	MESSAGE_MOUSE_MOTION.screenX = x;
	MESSAGE_MOUSE_MOTION.screenY = y;
	MESSAGE_MOUSE_MOTION.sender = this;
	MESSAGE_MOUSE_MOTION.mIsHandled = false;
	MESSAGE_MOUSE_MOTION.receiver = getElement(x, y, &mScreenArea, MESSAGE_MOUSE_MOTION.receiverX, MESSAGE_MOUSE_MOTION.receiverY);
	mCurrentUnderMouseComponent = MESSAGE_MOUSE_MOTION.receiver;
	if (mStartDown)
	{
		mStartDown->sendMessage(MESSAGE_MOUSE_MOTION);
	}
	else
	{
		if (MESSAGE_MOUSE_MOTION.receiver)
		{
			MESSAGE_MOUSE_MOTION.receiver->sendMessage(MESSAGE_MOUSE_MOTION);
		}
	}
}

void CSDesktop::onMouseDown(int x, int y)
{
	static char *functionName="onMouseDown";
	bool focusedComponentMessaged = false;

	LOG_ENTER 
	MESSAGE_MOUSE_PRESSED.screenX = x;
	MESSAGE_MOUSE_PRESSED.screenY = y;
	MESSAGE_MOUSE_PRESSED.sender = this;
	MESSAGE_MOUSE_PRESSED.mIsHandled = false;
	MESSAGE_MOUSE_PRESSED.receiver = getElement(x, y, &mScreenArea, MESSAGE_MOUSE_PRESSED.receiverX, MESSAGE_MOUSE_PRESSED.receiverY);
	if ((mModalComponent) && (!mModalComponent->isChild(MESSAGE_MOUSE_PRESSED.receiver)))
	{
		return;
	}
	if (MESSAGE_MOUSE_PRESSED.receiver)
	{
		// during focus lost, the receiver can be discarded
		// sounds strange, but happens e.g. with menus on a menuBar
		// the discarded component is not element of the desktop
		// but it still can (and must) acknowledge the mouse down event
		// 
		// a strikt NO GO is deleting a element while within focus lost event!
		if (MESSAGE_MOUSE_PRESSED.receiver->getFocusable())
		{
			setFocusedComponent(0);
			focusedComponentMessaged = true;
		}
		int receiverX, receiverY; 
		CSGrafikElement *receiver = getElement(x, y, &mScreenArea, receiverX, receiverY);
		clearLostElements();
		MESSAGE_MOUSE_PRESSED.receiver->sendMessage(MESSAGE_MOUSE_PRESSED);

		// was element deleted while send message?
		if ((!isLostElement(receiver)) && (receiver == MESSAGE_MOUSE_PRESSED.receiver))
		{
			if (receiver->getFocusable())
			{
				setFocusedComponent(receiver);
				focusedComponentMessaged = true;
			}
			mStartDown = receiver;
		}
		else
		{
			mStartDown = 0;
			setFocusedComponent(0);
			focusedComponentMessaged = true;
		}
	}
	else
	{
		setFocusedComponent(0);
		focusedComponentMessaged = true;
	}
	LOG_EXIT
}

void CSDesktop::onMouseUp(int x, int y)
{
	static char *functionName="onMouseUp";
	LOG_ENTER 
	MESSAGE_MOUSE_RELEASED.screenX = x;
	MESSAGE_MOUSE_RELEASED.screenY = y;
	MESSAGE_MOUSE_RELEASED.sender = this;
	MESSAGE_MOUSE_RELEASED.mIsHandled = false;
	MESSAGE_MOUSE_RELEASED.receiver = getElement(x, y, &mScreenArea, MESSAGE_MOUSE_RELEASED.receiverX, MESSAGE_MOUSE_RELEASED.receiverY);
	MESSAGE_MOUSE_RELEASED.downReceiver = mStartDown;
	if ((mModalComponent) && (!mModalComponent->isChild(MESSAGE_MOUSE_RELEASED.receiver)))
	{
		return;
	}

	if ((MESSAGE_MOUSE_RELEASED.receiver) && (MESSAGE_MOUSE_RELEASED.receiver == mStartDown))
	{
		MESSAGE_MOUSE_RELEASED.receiver->sendMessage(MESSAGE_MOUSE_RELEASED);
	}
	if (mStartDown != MESSAGE_MOUSE_RELEASED.receiver)
	{
		if (mStartDown)
		{
			mStartDown->sendMessage(MESSAGE_MOUSE_RELEASED);
		}
	}
	mStartDown = 0;
	LOG_EXIT
}

void CSDesktop::setFocusedComponent(CSGrafikElement *element)
{
	static char *functionName="setFocusedComponent";
	LOG_ENTER 
	if (mFocusedComponent == element)
	{
		return;
	}
	if (mFocusedComponent)
	{
		MESSAGE_FOCUS_LOST.screenX = 0;
		MESSAGE_FOCUS_LOST.screenY = 0;
		MESSAGE_FOCUS_LOST.sender = this;
		MESSAGE_FOCUS_LOST.mIsHandled = false;
		MESSAGE_FOCUS_LOST.receiver = mFocusedComponent;
		mFocusedComponent->setFocused(false);

		CSGrafikElement *component = mFocusedComponent;
		while ((!MESSAGE_FOCUS_LOST.mIsHandled) && (component != 0))
		{
			component->sendMessage(MESSAGE_FOCUS_LOST);
			component = component->getParent();
		}
	}

	if (element)
	{
		if (element->getFocusable())
		{
			MESSAGE_FOCUS_GAINED.screenX = 0;
			MESSAGE_FOCUS_GAINED.screenY = 0;
			MESSAGE_FOCUS_GAINED.sender = this;
			MESSAGE_FOCUS_GAINED.mIsHandled = false;
			MESSAGE_FOCUS_GAINED.receiver = element;
			mFocusedComponent = element;
			mFocusedComponent->setFocused(true);
			mFocusedComponent->sendMessage(MESSAGE_FOCUS_GAINED);
		}
		else
		{
			mFocusedComponent = 0;
		}
	}
	else
	{
		mFocusedComponent = element;
	}
	LOG_EXIT
}

void CSDesktop::focusedNextComponent()
{
	static char *functionName="focusedNextComponent";
	LOG_ENTER 
	if (mFocusedComponent == 0)
	{
		LOG_EXIT
		return;
	}
	CSGrafikElement *component = 0;
	// todo seek next component
	setFocusedComponent(component);
	LOG_EXIT
}

bool CSDesktop::dispatchKeyEvent(const SDL_Event &event, int state)
{
	static char *functionName="dispatchKeyEvent";
	LOG_ENTER 
	GuiMessage *message = &MESSAGE_KEY_PRESSED;
	if (state == SDL_KEYUP)
	{
		message = &MESSAGE_KEY_RELEASED;
	}

	message->screenX = 0;
	message->screenY = 0;
	message->sender = this;
	message->mIsHandled = false;
	message->keyUnicodeChar = event.key.keysym.unicode;
	message->keySymbolic = event.key.keysym.sym;

	if (mFocusedComponent)
	{
		message->receiver = mFocusedComponent;
	}
	else
	{
		message->receiver = this;
	}
	message->receiver->sendMessage(*message);

	LOG_EXIT
	return message->mIsHandled;
}

void CSDesktop::runModalDesktop(CSGrafikElement *modalElement, SDL_Surface *destination)
{
	static char *functionName = "runModalDesktop";
	LOG_ENTER 
	SDL_Surface *mBack = 0;
	SDL_SetClipRect(destination, 0);
	mModalComponent	= modalElement;
	int flags = destination->flags;
	if ((flags & SDL_HWSURFACE) == SDL_HWSURFACE)
	{
		flags -= SDL_HWSURFACE;
	}
	if ((flags & SDL_DOUBLEBUF) == SDL_DOUBLEBUF)
	{
		flags -= SDL_DOUBLEBUF;
	}
	if ((flags & SDL_FULLSCREEN) == SDL_FULLSCREEN)
	{
		flags -= SDL_FULLSCREEN;
	}
	
	mBack = SDL_CreateRGBSurface(
					flags, 
					destination->w,
					destination->h,
					destination->format->BitsPerPixel, 
					destination->format->Rmask, 
					destination->format->Gmask, 
					destination->format->Bmask, 
					destination->format->Amask);
	if (mBack == 0)
	{
		LOG_EXIT
		SDLMain::shutdown((std::string)"Display Format error: " + SDL_GetError(), 1);
	}
	if (SDL_BlitSurface(destination, 0, mBack, 0) < 0)
	{
		LOG_EXIT
		SDLMain::shutdown((std::string)"BlitSurface error: " + SDL_GetError(), 1);
	}
	do
	{
		handleEvents();
		doLayout(); 

		if (SDL_BlitSurface(mBack, 0, destination, 0) < 0)
		{
			SDLMain::shutdown((std::string)"BlitSurface error: " + SDL_GetError(), 1);
		}
		SDL_Rect viewportAreaCopy = mViewportArea;
		SDL_Rect areaCopy = mElementArea;
		modalElement->paintArea(destination, &viewportAreaCopy);
		modalElement->paint(destination, &viewportAreaCopy);
		modalElement->paintBorder(destination, &viewportAreaCopy);
		
		int receiverX, receiverY;
		CSGrafikElement *receiver = getElement(mCursorX, mCursorY, &mScreenArea, receiverX, receiverY);
		if (receiver)
		{
			receiver->getCursor()->setPosition(mCursorX, mCursorY);
			receiver->getCursor()->paint(destination, &areaCopy);
		}
		else
		{
			getCursor()->setPosition(mCursorX, mCursorY);
			getCursor()->paint(destination, &areaCopy);
		}

		SDLMain::updateScreen();
	} while(modalElement->getVisible());
	
	if (mBack)
	{
		SDL_FreeSurface(mBack);
		mBack = 0;
	}
	mModalComponent	= 0;
	LOG_EXIT
}

void CSDesktop::onMouseMotion(SDL_MouseMotionEvent event)
{
	static char *functionName="onMouseMotion";
	if (isIn(event.x,event.y, &mElementArea))
	{
		onMouseMotion(event.x,event.y);
	}
}

void CSDesktop::onMouseButtonDown(SDL_MouseButtonEvent event)
{
	static char *functionName="onMouseButtonDown";
	LOG_ENTER 
	if (isIn(event.x,event.y, &mElementArea))
	{
		onMouseDown(event.x,event.y);
	}
	LOG_EXIT 
}

void CSDesktop::onMouseButtonUp(SDL_MouseButtonEvent event)
{
	static char *functionName="onMouseButtonUp";
	LOG_ENTER 
	if (isIn(event.x,event.y, &mElementArea))
	{
		onMouseUp(event.x,event.y);
	}
	LOG_EXIT 
}

void CSDesktop::handleEvents()
{
	static char *functionName="handleEvents";
	SDL_Event event;

	if (SDL_PeepEvents(&event, 1, SDL_PEEKEVENT,SDL_MOUSEMOTIONMASK)==1)
	{
		do 
		{
			SDL_PeepEvents(&event,1, SDL_GETEVENT,SDL_MOUSEMOTIONMASK);
		} while (SDL_PeepEvents(&event,1 , SDL_PEEKEVENT,SDL_MOUSEMOTIONMASK)==1);
		
		onMouseMotion(event.motion);
	}
	
	if (SDL_PollEvent(&event))
	{
		switch(event.type)
		{
			case SDL_MOUSEMOTION:
			{
				onMouseMotion(event.motion); 
				break;
			}
			case SDL_MOUSEBUTTONDOWN:
			{
				onMouseButtonDown(event.button); 
				break;
			}
			case SDL_MOUSEBUTTONUP:
			{
				onMouseButtonUp(event.button); 
				break;
			}
			case SDL_KEYDOWN:
			{
				dispatchKeyEvent(event, SDL_KEYDOWN);
			}
			case SDL_KEYUP:
			{
				dispatchKeyEvent(event, SDL_KEYUP);
			}
			break;
		}
	}
}

void CSDesktop::layoutSetupDesktop()
{
	static char *functionName="layoutSetupDesktop";
	LOG_ENTER 
	LOG_EXIT
}

void CSDesktop::correctElementHandling(CSGrafikElement *element)
{
	if (mFocusedComponent == element)
	{
		mFocusedComponent = 0;
	}
	if (mStartDown == element)
	{
		mStartDown = 0;
	}
	if (mCurrentUnderMouseComponent == element)
	{
		mCurrentUnderMouseComponent = 0;
	}
	if (mModalComponent == element)
	{
		mModalComponent = 0;
	}
}
