#ifndef CSDESKTOP_h
#define CSDESKTOP_h

// USES SDL_Surface
// USES SDL_Rect 
// USES SDL_Events

#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

#include <vector>
#include <string>

#include "CSGrafikElement.h"
#include "CSLog.h"

class CSMouseCursor;
class CSGame;
class CSLabel;
class CSPanel;

class CSDesktop : public CSGrafikElement
{
	friend CSGame;
	friend CSGrafikElement; // for modal desktop only
	private:
		CSGrafikElement *mFocusedComponent;
		CSGrafikElement *mStartDown;
		CSGrafikElement *mCurrentUnderMouseComponent;
		CSGrafikElement *mModalComponent;
		CSPanel *mTooltipPanel;
		CSLabel *mTooltipLabel;
		CSPanel *mNorth;
		CSPanel *mWest;
		CSPanel *mSouth;
		CSPanel *mEast;
		CSPanel *mCenter;
		int mCursorX;
		int mCursorY;
		long mLastTimePainted;
		int mLastCursorX;
		int mLastCursorY;
		SDL_Rect mScreenArea;	// mX, mY, mHeight, mWidth (of screen)
		bool mRequestLayout;

		void onMouseMotion(int x, int y);
		void onMouseDown(int x, int y);
		void onMouseUp(int x, int y);
		bool dispatchKeyEvent(const SDL_Event &event, int state);

		// following privates are for modal desktop only
		void runModalDesktop(CSGrafikElement *modalElement, SDL_Surface *destination);
		void onMouseMotion(SDL_MouseMotionEvent event);
		void onMouseButtonDown(SDL_MouseButtonEvent event);
		void onMouseButtonUp(SDL_MouseButtonEvent event);
		void handleEvents();
		void showToolTip();

	protected:
		void layoutSetupDesktop();

	public:
		static const char *CLASS;
		virtual std::string getType() {return (std::string) CLASS;}
		CSDesktop();
		CSDesktop(SDL_Rect area);
		~CSDesktop();
		virtual void paint(SDL_Surface *destination, SDL_Rect *parentViewport);
		void setFocusedComponent(CSGrafikElement *element);
		CSGrafikElement *getFocusedComponent() {return mFocusedComponent;}
		void focusedNextComponent();
		void addElementNorth(CSGrafikElement *element);
		void addElementSouth(CSGrafikElement *element);
		void addElementEast(CSGrafikElement *element);
		void addElementWest(CSGrafikElement *element);
		virtual void layoutSetup() {layoutSetupDesktop();}
		void correctElementHandling(CSGrafikElement *element);
		void requestLayout() {mRequestLayout = true;}

};

#endif // CSDESKTOP_h
