#ifndef CSGUITESTCLASSES
#define CSGUITESTCLASSES

// USES SDL_Surface

#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

class CSBorderLayoutPanel : public CSPanel
{
	protected:
		CSPanel *north1;
		CSPanel *south1;
		CSPanel *west1;
		CSPanel *east1;
		CSPanel *center1;

	public:
		CSBorderLayoutPanel(int w=200, int h=200)
		{
			setHeight(h);
			setWidth(w);
			setBorder(CSBorder::getBorder(BORDER_TYPE_DOUBLE));
			setLayoutManager(new CSLayoutManagerBorder());
			setVisible(true);

			north1 = new CSPanel(20,100);
			south1 = new CSPanel(20,100);
			west1 = new CSPanel(100,20);
			east1 = new CSPanel(100,20);
			center1 = new CSPanel(100,100);

			north1->setLayoutManager(new CSLayoutManagerBorder());
			south1->setLayoutManager(new CSLayoutManagerBorder());
			west1->setLayoutManager(new CSLayoutManagerBorder());
			east1->setLayoutManager(new CSLayoutManagerBorder());
			center1->setLayoutManager(new CSLayoutManagerBorder());

			north1->setArea( new CSArea());
			north1->setBackgroundColor(0xff0000);
			CSLayoutData layout1 = north1->getLayoutData();
			layout1.setStretchedHorizontal(true);
			layout1.setStretchedVertical(true);
			north1->setLayoutData(layout1);

			south1->setArea( new CSArea());
			south1->setBackgroundColor(0x00ff00);
			layout1 = south1->getLayoutData();
			layout1.setStretchedHorizontal(true);
			layout1.setStretchedVertical(true);
			south1->setLayoutData(layout1);

			east1->setArea( new CSArea());
			east1->setBackgroundColor(0x0000ff);
			layout1 = east1->getLayoutData();
			layout1.setStretchedVertical(true);
			layout1.setStretchedHorizontal(true);
			east1->setLayoutData(layout1);

			west1->setArea( new CSArea());
			west1->setBackgroundColor(0x00ffff);
			layout1 = west1->getLayoutData();
			layout1.setStretchedVertical(true);
			layout1.setStretchedHorizontal(true);
			west1->setLayoutData(layout1);

			center1->setArea( new CSArea());
			center1->setBackgroundColor(0xffffff);
			layout1 = center1->getLayoutData();
			layout1.setStretchedVertical(true);
			layout1.setStretchedHorizontal(true);
			center1->setLayoutData(layout1);

			CSLabel *n=new CSLabel("North");
			layout1 = n->getLayoutData();
			layout1.setPosition(POSITION_CENTER);
			layout1.setCenteredHorizontal(true);
			layout1.setCenteredVertical(true);
			n->setLayoutData(layout1);
			north1->addElement(n);

			CSLabel *s=new CSLabel("South");
			layout1 = s->getLayoutData();
			layout1.setPosition(POSITION_CENTER);
			layout1.setCenteredHorizontal(true);
			layout1.setCenteredVertical(true);
			s->setLayoutData(layout1);
			south1->addElement(s);

			CSLabel *e=new CSLabel("East");
			layout1 = e->getLayoutData();
			layout1.setPosition(POSITION_CENTER);
			layout1.setCenteredHorizontal(true);
			layout1.setCenteredVertical(true);
			e->setLayoutData(layout1);
			east1->addElement(e);

			CSLabel *we=new CSLabel("West");
			layout1 = we->getLayoutData();
			layout1.setPosition(POSITION_CENTER);
			layout1.setCenteredHorizontal(true);
			layout1.setCenteredVertical(true);
			we->setLayoutData(layout1);
			west1->addElement(we);

			CSLabel *c=new CSLabel("Center");
			layout1 = c->getLayoutData();
			layout1.setPosition(POSITION_CENTER);
			layout1.setCenteredHorizontal(true);
			layout1.setCenteredVertical(true);
			c->setLayoutData(layout1);
			center1->addElement(c);


			addElement(north1, POSITION_NORTH);
			addElement(south1, POSITION_SOUTH);
			addElement(west1, POSITION_WEST);
			addElement(east1, POSITION_EAST);
			addElement(center1, POSITION_CENTER);
		}

		~CSBorderLayoutPanel()
		{
			delete north1;
			delete south1;
			delete east1;
			delete west1;
			delete center1;
		}

		void setNorthElement(CSGrafikElement *element)
		{
			removeElement(north1);
			addElement(element, POSITION_NORTH);
		}
		void setSouthElement(CSGrafikElement *element)
		{
			removeElement(south1);
			addElement(element, POSITION_SOUTH);
		}
		void setEastElement(CSGrafikElement *element)
		{
			removeElement(east1);
			addElement(element, POSITION_EAST);
		}
		void setWestElement(CSGrafikElement *element)
		{
			removeElement(west1);
			addElement(element, POSITION_WEST);
		}
		void setCenterElement(CSGrafikElement *element)
		{
			removeElement(center1);
			addElement(element, POSITION_CENTER);
		}

		void centerSelf(bool b1=true, bool b2=true)
		{
			CSLayoutData layout1 = getLayoutData();
			layout1.setCenteredVertical(b1);
			layout1.setCenteredHorizontal(b2);
			setLayoutData(layout1);
		}
		void expandSelf(bool b1=true, bool b2=true)
		{
			CSLayoutData layout1 = getLayoutData();
			layout1.setStretchedHorizontal(b1);
			layout1.setStretchedVertical(b2);
			setLayoutData(layout1);
		}
		void packSelf(bool b1=true, bool b2=true)
		{
			CSLayoutData layout1 = getLayoutData();
			layout1.setPackedHorizontal(b1);
			layout1.setPackedVertical(b2);
			setLayoutData(layout1);
		}
};

class CSBorderLayoutPanelCentered : public CSBorderLayoutPanel
{
	public:
		CSBorderLayoutPanelCentered(int w=200, int h=200)
		{
			CSLayoutData layout1 = north1->getLayoutData();
			layout1.setCenteredVertical(true);
			layout1.setCenteredHorizontal(true);
			north1->setLayoutData(layout1);

			layout1.setPosition(POSITION_SOUTH);
			south1->setLayoutData(layout1);
			layout1.setPosition(POSITION_WEST);
			west1->setLayoutData(layout1);
			layout1.setPosition(POSITION_EAST);
			east1->setLayoutData(layout1);
			layout1.setPosition(POSITION_CENTER);
			center1->setLayoutData(layout1);

		}
};

class CSHorizontalStackLayoutPanel : public CSPanel
{
			CSPanel *p1;
			CSPanel *p2;
			CSPanel *p3;
			CSPanel *p4;
			CSPanel *p5;
	public:
		CSHorizontalStackLayoutPanel(int w=200, int h=200)
		{
			setHeight(h);
			setWidth(w);
			setBorder(CSBorder::getBorder(BORDER_TYPE_DOUBLE));
			setLayoutManager(new CSLayoutManagerStackHorizontal());
			setVisible(true);

			p1 = new CSPanel(20,100);
			p2 = new CSPanel(20,100);
			p3 = new CSPanel(100,20);
			p4 = new CSPanel(100,20);
			p5 = new CSPanel(100,100);

			p1->setLayoutManager(new CSLayoutManagerBorder());
			p2->setLayoutManager(new CSLayoutManagerBorder());
			p3->setLayoutManager(new CSLayoutManagerBorder());
			p4->setLayoutManager(new CSLayoutManagerBorder());
			p5->setLayoutManager(new CSLayoutManagerBorder());

			CSLayoutData layout;

			p1->setArea( new CSArea());
			p2->setArea( new CSArea());
			p3->setArea( new CSArea());
			p4->setArea( new CSArea());
			p5->setArea( new CSArea());

			p1->setBackgroundColor(0xff0000);
			p2->setBackgroundColor(0x00ff00);
			p3->setBackgroundColor(0x0000ff);
			p4->setBackgroundColor(0x00ffff);
			p5->setBackgroundColor(0xffffff);

			layout = p1->getLayoutData();
			layout.setStretchedHorizontal(true);
			layout.setStretchedVertical(true);
			p1->setLayoutData(layout);

			p2->setLayoutData(layout);
			p3->setLayoutData(layout);
			p4->setLayoutData(layout);
			p5->setLayoutData(layout);


			CSLabel *l1=new CSLabel("1");
			layout = l1->getLayoutData();
			layout.setPosition(POSITION_CENTER);
			layout.setCenteredHorizontal(true);
			layout.setCenteredVertical(true);
			l1->setLayoutData(layout);
			p1->addElement(l1);

			CSLabel *l2=new CSLabel("2");
			l2->setLayoutData(layout);
			p2->addElement(l2);

			CSLabel *l3=new CSLabel("3");
			l3->setLayoutData(layout);
			p3->addElement(l3);

			CSLabel *l4=new CSLabel("4");
			l4->setLayoutData(layout);
			p4->addElement(l4);

			CSLabel *l5=new CSLabel("5");
			l5->setLayoutData(layout);
			p5->addElement(l5);

			addElement(p1,POSITION_WEST);
			addElement(p2,POSITION_WEST);
			addElement(p3,POSITION_WEST);
			addElement(p4,POSITION_WEST);
			addElement(p5,POSITION_WEST);
		}
		void centerSelf(bool b1=true, bool b2=true)
		{
			CSLayoutData layout1 = getLayoutData();
			layout1.setCenteredVertical(b1);
			layout1.setCenteredHorizontal(b2);
			setLayoutData(layout1);
		}
		void expandSelf(bool b1=true, bool b2=true)
		{
			CSLayoutData layout1 = getLayoutData();
			layout1.setStretchedHorizontal(b1);
			layout1.setStretchedVertical(b2);
			setLayoutData(layout1);
		}
		void packSelf(bool b1=true, bool b2=true)
		{
			CSLayoutData layout1 = getLayoutData();
			layout1.setPackedHorizontal(b1);
			layout1.setPackedVertical(b2);
			setLayoutData(layout1);
		}

		void addElementWest(CSGrafikElement *element)
		{
			addElement(element, POSITION_WEST);
		}
		void addElementEast(CSGrafikElement *element)
		{
			addElement(element, POSITION_EAST);
		}
};


class CSVerticalStackLayoutPanel : public CSPanel
{
			CSPanel *p1;
			CSPanel *p2;
			CSPanel *p3;
			CSPanel *p4;
			CSPanel *p5;
	public:
		CSVerticalStackLayoutPanel(int w=200, int h=200)
		{
			setHeight(h);
			setWidth(w);
			setBorder(CSBorder::getBorder(BORDER_TYPE_DOUBLE));
			setLayoutManager(new CSLayoutManagerStackVertical());
			setVisible(true);

			p1 = new CSPanel(20,100);
			p2 = new CSPanel(20,100);
			p3 = new CSPanel(100,20);
			p4 = new CSPanel(100,20);
			p5 = new CSPanel(100,100);

			p1->setLayoutManager(new CSLayoutManagerBorder());
			p2->setLayoutManager(new CSLayoutManagerBorder());
			p3->setLayoutManager(new CSLayoutManagerBorder());
			p4->setLayoutManager(new CSLayoutManagerBorder());
			p5->setLayoutManager(new CSLayoutManagerBorder());

			CSLayoutData layout;

			p1->setArea( new CSArea());
			p2->setArea( new CSArea());
			p3->setArea( new CSArea());
			p4->setArea( new CSArea());
			p5->setArea( new CSArea());

			p1->setBackgroundColor(0xff0000);
			p2->setBackgroundColor(0x00ff00);
			p3->setBackgroundColor(0x0000ff);
			p4->setBackgroundColor(0x00ffff);
			p5->setBackgroundColor(0xffffff);

			layout = p1->getLayoutData();
			layout.setStretchedHorizontal(true);
			layout.setStretchedVertical(true);
//			layout.setCenteredHorizontal(true);
//			layout.setCenteredVertical(true);
			p1->setLayoutData(layout);
			p2->setLayoutData(layout);
			p3->setLayoutData(layout);
			p4->setLayoutData(layout);
			p5->setLayoutData(layout);


			CSLabel *l1=new CSLabel("1");
			layout = l1->getLayoutData();
			layout.setPosition(POSITION_CENTER);
			layout.setCenteredHorizontal(true);
			layout.setCenteredVertical(true);

			l1->setLayoutData(layout);
			p1->addElement(l1);

			CSLabel *l2=new CSLabel("2");
			l2->setLayoutData(layout);
			p2->addElement(l2);

			CSLabel *l3=new CSLabel("3");
			l3->setLayoutData(layout);
			p3->addElement(l3);

			CSLabel *l4=new CSLabel("4");
			l4->setLayoutData(layout);
			p4->addElement(l4);

			CSLabel *l5=new CSLabel("5");
			l5->setLayoutData(layout);
			p5->addElement(l5);

			addElement(p1,POSITION_NORTH);
			addElement(p2,POSITION_NORTH);
			addElement(p3,POSITION_NORTH);
			addElement(p4,POSITION_NORTH);
//			addElement(p5,POSITION_NORTH);
		}
		void centerSelf(bool b1=true, bool b2=true)
		{
			CSLayoutData layout1 = getLayoutData();
			layout1.setCenteredVertical(b1);
			layout1.setCenteredHorizontal(b2);
			setLayoutData(layout1);
		}
		void expandSelf(bool b1=true, bool b2=true)
		{
			CSLayoutData layout1 = getLayoutData();
			layout1.setStretchedHorizontal(b1);
			layout1.setStretchedVertical(b2);
			setLayoutData(layout1);
		}
		void packSelf(bool b1=true, bool b2=true)
		{
			CSLayoutData layout1 = getLayoutData();
			layout1.setPackedHorizontal(b1);
			layout1.setPackedVertical(b2);
			setLayoutData(layout1);
		}

		void addElementNorth(CSGrafikElement *element)
		{
			addElement(element, POSITION_NORTH);
		}
		void addElementSouth(CSGrafikElement *element)
		{
			addElement(element, POSITION_SOUTH);
		}
};

#endif // CSGUITESTCLASSES
