
// next todo:
// try to display only the stuff that might actually go on the screen
// (Performance upgrade tha is)


// html only with grafik elements -> than table!

#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

#include "CSHTMLElement.h"
#include "CSFont.h"
#include "CSDesktop.h"
#include "CSWindow.h"
#include "CSLAF.h"
#include "CSScrollbar.h"
#include "CSHelper.h"

const char *CSHTMLElement::CLASS = "CSHTMLElement";

CSHTMLElement::CSHTMLElement(CSHTMLNode node)
{
	mNode = node;
	mParent = 0;
	mX = 0;
	mY = 0;
	CSHTMLNode child = mNode.firstChild();
	while (!child.isEmpty())
	{
		CSHTMLElement * nextChild = new CSHTMLElement(child);
		nextChild->setParent(this);
		mElements.push_back(nextChild);
		child = mNode.nextChild();
	}

	// we all listen to screen changes, since images must be updated!
	// the compilation information is corrupt, when some 
	// instance have no compilation (all or nothing), (BODY TAG e.g. is not
	// set anymore, if first tag is an image and only images are cleared!)

	// if this is switched on, upon toggle screen -> the program
	// crashes -> unkown!!!
//CSA SDLMain::getInstance()->addMessageListener(this, SDL_MESSAGE);
}

CSHTMLElement::~CSHTMLElement()
{
	clearCompiled();
	CSHTMLElements::iterator iter = mElements.begin();
	while (iter != mElements.end())
	{
		CSHTMLElement *element = *iter;
		mElements.erase(iter);
		delete element;
		iter = mElements.begin();
	}
}

void CSHTMLElement::clearCompiled()
{
}

CSHTMLElement *CSHTMLElement::getAncestorType(int type)
{
	CSHTMLElement *parent = getParent();
	while (parent)
	{
		if (parent->getTagType() == type)
		{
			return parent;
		}
		parent = parent->getParent();
	}
	return 0;
}

int CSHTMLElement::getTagType()
{
	return getTagType(mNode);
}

// all Tag types are mapped here from their string representation to a
// numerical value, see "CSHTMLarea.h" for their definition
// static function - therefor "mHTMLTagMap" is never freed!
HTMLTagMap *CSHTMLElement::mHTMLTagMap = 0;

int CSHTMLElement::getTagType(CSHTMLNode &node)
{
	if (mHTMLTagMap == 0)
	{
		mHTMLTagMap = new HTMLTagMap();
		mHTMLTagMap->insert(HTMLTagMap::value_type("HTML", HTML_HTML));
		mHTMLTagMap->insert(HTMLTagMap::value_type("HEAD", HTML_HEAD));
		mHTMLTagMap->insert(HTMLTagMap::value_type("TITLE", HTML_TITLE));
		mHTMLTagMap->insert(HTMLTagMap::value_type("BODY", HTML_BODY));
		mHTMLTagMap->insert(HTMLTagMap::value_type("H1", HTML_H1));
		mHTMLTagMap->insert(HTMLTagMap::value_type("H2", HTML_H2));
		mHTMLTagMap->insert(HTMLTagMap::value_type("H3", HTML_H3));
		mHTMLTagMap->insert(HTMLTagMap::value_type("H4", HTML_H4));
		mHTMLTagMap->insert(HTMLTagMap::value_type("H5", HTML_H5));
		mHTMLTagMap->insert(HTMLTagMap::value_type("P", HTML_P));
		mHTMLTagMap->insert(HTMLTagMap::value_type("BR", HTML_BR));
		mHTMLTagMap->insert(HTMLTagMap::value_type("HL", HTML_HL));
		mHTMLTagMap->insert(HTMLTagMap::value_type("PRE", HTML_PRE));
		mHTMLTagMap->insert(HTMLTagMap::value_type("IMG", HTML_IMG));
		mHTMLTagMap->insert(HTMLTagMap::value_type("A", HTML_A));
		mHTMLTagMap->insert(HTMLTagMap::value_type("B", HTML_B));
		mHTMLTagMap->insert(HTMLTagMap::value_type("I", HTML_I));
		mHTMLTagMap->insert(HTMLTagMap::value_type("DD", HTML_DD));
		mHTMLTagMap->insert(HTMLTagMap::value_type("DL", HTML_DL));
		mHTMLTagMap->insert(HTMLTagMap::value_type("DT", HTML_DT));
		mHTMLTagMap->insert(HTMLTagMap::value_type("UL", HTML_UL));
		mHTMLTagMap->insert(HTMLTagMap::value_type("OL", HTML_OL));

		mHTMLTagMap->insert(HTMLTagMap::value_type("TABLE", HTML_TABLE));
		mHTMLTagMap->insert(HTMLTagMap::value_type("TR", HTML_TR));
		mHTMLTagMap->insert(HTMLTagMap::value_type("TH", HTML_TH));
		mHTMLTagMap->insert(HTMLTagMap::value_type("TD", HTML_TD));
	}
	std::string tag = node.getTag();
	if (tag.size() != 0)
	{
		HTMLTagMap::iterator iter = mHTMLTagMap->find(tag);
		if (iter != mHTMLTagMap->end())
		{
			return iter->second;
		}
	}
	return 0;
}

// tokenify text in handable parts (only ASCII)
// token seperator is " ", "\n" and "\t"
// static function!
StringVector *CSHTMLElement::tokenifyText(const std::string &text)
{
	StringVector *tokens = new StringVector();
	char *copy = strdup(text.c_str());
	char *tmp = copy;
	
	char *pointerToWord = strtok( copy, " \n\t");
	while (pointerToWord != 0)
	{
		tokens->push_back(strdup(pointerToWord));
		pointerToWord = strtok( 0, " \n\t");
	}
	free(tmp);
	return tokens;
}

// returns for an html tag all children in form as text only
// not really needed, just for convenience for the /TITLE/ TAG
std::string CSHTMLElement::getChildrenAsText()
{
	std::string text="";
	CSHTMLElements::iterator iter;
	for (iter = mElements.begin(); iter != mElements.end(); iter++)
	{
		CSHTMLElement *element = *iter;
		if (element->mNode.isText())
		{
			text = text + element->mNode.getValue();
		}
		text = text + element->getChildrenAsText();
	}
	return text;
}

// get a child that has a given type, a given attribute and a given attribute value
// used to get to html ankers
// if either (or both) the attribute parameters are empty, the
// first child node of the kind "type" is returned (used to get the first BODY tag...)
CSHTMLElement *CSHTMLElement::getChildType(int type, std::string attributeName, std::string attributeValue)
{
	CSHTMLElements::iterator iter;
	for (iter = mElements.begin(); iter != mElements.end(); iter++)
	{
		CSHTMLElement *element = *iter;
		if (element->getTagType() == type)
		{
			if ((attributeName.size() == 0) || (attributeValue.size() == 0))
			{
				return element;
			}
			std::string elementAttributeValue = element->getAttributValue(attributeName);
			if (elementAttributeValue.compare(attributeValue) == 0)
			{
				return element;
			}
		}
		CSHTMLElement *childResult = element->getChildType(type, attributeName, attributeValue);
		if (childResult)
		{
			return childResult;
		}
	}
	return 0;
}

// yet to be done... for now only straight and correkt "files" are handled!
std::string CSHTMLElement::formatURL(const std::string &rawURL)
{
	return rawURL;
}

// get the value of an attribute for this given node
std::string CSHTMLElement::getAttributValue(std::string attributeName)
{
	return mNode.attributValue(attributeName);
}

// translates strings of kind "#f0f0f0" into an integer value
// convert is a helper function to be found in CSHelper
int CSHTMLElement::getColorFromString(std::string acolor)
{
	if (acolor.size()<= 0)
	{
		return -1;
	}
	int posStart = strstr(acolor.c_str(), "#") - acolor.c_str()+1;
	std::string v = acolor.substr(posStart, acolor.size()-posStart);
	int value = CSHelper::convert(v.c_str(), 16);
	return value;
}

// if destination is 0, the cursor only counts the
// space needed for displaying with the current given viewport!
// when first displayed (destination != 0)
// each node is "compiled" into a structure for easier (and faster)
// accessability
HTMLCursor CSHTMLElement::buildDisplay(HTMLCursor cursor, CSGrafikElement *destination, int parentViewportWidth)
{
	bool goDeeper = true;	
	std::string text = "";
	std::string tag = mNode.getTag();
	HTMLCursor returnCursor = cursor;
	bool newLine = false;
	mY = cursor.mYPos;

	// TAG BEGIN
	// switch thru all known tags
	// adjust cursor setting accordingly!
	switch (getTagType(mNode))
	{
		case HTML_PRE:
		{
			CSLAF *laf = CSLAF::getCurrentLAF();
			cursor.mInPre = true;
			
			// PRE allways starts at a new line
			cursor.mXPos = 0;
			cursor.mXPos += cursor.mAddX;
			cursor.mYPos += cursor.nextAdd;
			cursor.nextAdd = cursor.mFont->getHeight(); 
			cursor.mFont = laf->getFont("MonoSpaced"); // Mono spaced font! 
			break;
		}
		case HTML_HTML:
		{
			break;
		}
		case HTML_BODY:
		{
			// if there are - rememeber the colors!
			std::string bgColor = mNode.attributValue("BGCOLOR");
			std::string textColor = mNode.attributValue("TEXT");
			std::string linkColor = mNode.attributValue("LINK");

			int t = getColorFromString(textColor);
			if (t != -1) cursor.mPenColor = t;

			t = getColorFromString(bgColor);
			if (t != -1) cursor.mBackgroundColor = t;
			
			t = getColorFromString(linkColor);
			if (t != -1) cursor.mLinkColor = t;

			// only tags within body are displayed!
			cursor.mInBody = true;
			break;
		}
		case HTML_TITLE:
		{
			//quick Hack, since possible in Title there might be (stupid!!!) other tags!			
			std::string title = getChildrenAsText();
			if (destination)
			{
				destination->getParentWindow()->setTitle(title);
			}
			goDeeper = false;
			break;
		}
		case HTML_LI:
		{
			// list counter for <OL> - List get garbled when
			// Lists are used in lists - not fixed yet
			// (not even thought about it...)

			// counter == -1 is an unordered list,
			// everything else is the counter for the ordered list
			if (cursor.mListCounter != -1)
			{
				// produce some text output as counters for the ordered list
				char helper[10];
				cursor.mListCounter++; 
				sprintf(helper, "%i. ", cursor.mListCounter);
				text = std::string(helper);
			}
			else
			{
				// nothing fancy for not ordered Lists, just an asterix
				text = std::string("* ");
			}
			newLine = true;
		}
		case HTML_DT:
		{
			cursor.mXPos = 0;
			cursor.mYPos += cursor.nextAdd;
			cursor.nextAdd = cursor.mFont->getHeight(); 
			cursor.mXPos += cursor.mAddX;
			break;
		}
		case HTML_OL:
		{
			// start the ordered list counter at 0
			cursor.mListCounter = 0; 
		}
		case HTML_UL:
		case HTML_DD:
		{
			// and do some indenting for the lists
			cursor.mAddX += 3*cursor.mFont->getWidth(" "); 
			break;
		}

		// well just the ol' carriage return
		case HTML_BR:
		// haven't done any real Headlines's yet
		case HTML_H1:
		{
			cursor.mXPos = 0;
			cursor.mXPos += cursor.mAddX;
			cursor.mYPos += cursor.nextAdd;
			cursor.nextAdd = cursor.mFont->getHeight(); 
			newLine = true;
			break;
		}

		// two lines might actually be to much - but what the heck
		// not done the whole attribute staff here
		case HTML_P:
		{
			cursor.mXPos = 0;
			cursor.mXPos += cursor.mAddX;
			cursor.mYPos += cursor.nextAdd;
			cursor.nextAdd = cursor.mFont->getHeight(); 
			cursor.mYPos += cursor.nextAdd;
			newLine = true;
			break;
		}
		case HTML_A:
		{
			// just for show - the attributes are needed some other time!
			// std::string rawURL = mNode.attributValue("HREF");
			// std::string formattedURL = formatURL(rawURL);
			// std::string ankerName = mNode.attributValue("NAME");
			if (mNode.attributValue("HREF").size() != 0 )
			{
				cursor.mStyle = cursor.mStyle | FONT_STYLE_UNDERLINE;
				cursor.mPenColor = cursor.mLinkColor;
				cursor.mIsLink = true;
			}
			break;
		}
		case HTML_B:
		{
			cursor.mStyle = cursor.mStyle | FONT_STYLE_BOLD;
			break;
		}
		case HTML_I:
		{
			// some day I might use TTF - but until than this is just for the show
			cursor.mStyle = cursor.mStyle | FONT_STYLE_ITALIC;
			break;
		}

		case HTML_TABLE:
		{
			int x = cursor.mXPos;
			int y = cursor.mYPos;
			CSPanel *panel = new CSPanel(100,100);
panel->setParentLayoutUpdate(false);
			panel->setLayoutManager(new CSLayoutManagerStackVertical());
			CSLayoutData layoutData = panel->getLayoutData();
//			layoutData.setStretchedHorizontal(true);
//			layoutData.setPackedVertical(true);
			panel->setLayoutData(layoutData);
			panel->setBorder(CSBorder::getBorder(BORDER_TYPE_UP));

			CSHTMLElements::iterator iter;
			for (iter = mElements.begin(); iter != mElements.end(); iter++)
			{
				CSHTMLElement *element = *iter;
				cursor = element->buildDisplay(cursor, panel, 100);
			}

if (destination != 0)
{
	destination->addElement(panel, x, y);
	printf("Adding to destination!\n");
	flushall();
}
else
{
	delete panel;
	panel = 0;
}
//			goDeeper = false;

			break;
		}
		case HTML_TR:
		{

			int x = cursor.mXPos;
			int y = cursor.mYPos;
			CSPanel *panel = new CSPanel(20,100);
panel->setParentLayoutUpdate(false);
			panel->setLayoutManager(new CSLayoutManagerStackHorizontal());
			CSLayoutData layoutData = panel->getLayoutData();
//			layoutData.setStretchedHorizontal(true);
//			layoutData.setPackedVertical(true);
			panel->setLayoutData(layoutData);
			panel->setBorder(CSBorder::getBorder(BORDER_TYPE_BUTTON));

			CSHTMLElements::iterator iter;
			for (iter = mElements.begin(); iter != mElements.end(); iter++)
			{
				CSHTMLElement *element = *iter;
				cursor = element->buildDisplay(cursor, panel, 100);
			}
			
if (destination != 0)
{
	destination->addElement(panel, POSITION_SOUTH);
	printf("Adding to destination 2!\n");
	flushall();
}
else
{
	delete panel;
	panel = 0;
}
			goDeeper = false;

			break;
		}
		case HTML_TH:
		case HTML_TD:
		{
			int x = cursor.mXPos;
			int y = cursor.mYPos;
			CSPanel *panel = new CSPanel(20,10);
panel->setParentLayoutUpdate(false);

			CSLayoutData layoutData = panel->getLayoutData();
//			layoutData.setStretchedHorizontal(true);
//			layoutData.setPackedVertical(true);
			panel->setLayoutData(layoutData);
			panel->setBorder(CSBorder::getBorder(BORDER_TYPE_TEXTFIELD));

			CSHTMLElements::iterator iter;
			for (iter = mElements.begin(); iter != mElements.end(); iter++)
			{
				CSHTMLElement *element = *iter;
				cursor = element->buildDisplay(cursor, panel, 20);
			}
			
if (destination != 0)
{
	destination->addElement(panel, POSITION_EAST);
	printf("Adding to destination 3!\n");
	flushall();
}
else
{
	delete panel;
	panel = 0;
}
			goDeeper = false;

			break;
		}

		case HTML_IMG:
		{
			// load a picture!
			cursor.mXPos = 0;
			cursor.mXPos += cursor.mAddX;
			std::string rawURL = mNode.attributValue("SRC");
			std::string formattedURL = formatURL(rawURL);

			// just to rememeber
			// cursor.mXPos								is the position within the viewport
			// parentViewport->x						is the position on screen of the viewport
			// cursor.mStartHorizontalDisplayAtPixel	is the offset produced by scrolling withing the viewport
			int x = cursor.mXPos;
			int y = cursor.mYPos;

			CSIcon *icon = CSPictureIcon::buildIcon(CSPictureDirectLoader::INSTANCE.load(formattedURL));
			if (icon == 0)
			{
				text = std::string()+"[Image not found: "+formattedURL+"]";
				icon = CSTextIcon::buildIcon(text);
			}
			if (icon != 0)
			{
				cursor.mXPos += icon->getWidth();
				cursor.nextAdd = icon->getHeight();
				if (cursor.mXPos > cursor.mXPosMax)
				{
					cursor.mXPosMax = cursor.mXPos;
				}

				if (destination)
				{
					destination->addElement(icon, x, y);
					if (cursor.mIsLink)
					{
						addLink(icon, destination);
					}
				}
				else
				{
					delete icon;
				}
			}
		}
	}

	// if node has text - get it
	if (mNode.isText())
	{
		text = mNode.getValue();
	}

	// even none text Nodes can have "textuall" effects - see above

	// display text with current cursor settings
	// (if in Body of html page)
	if (cursor.mInBody == true)
	{
		if (text.size() != 0)
		{
			// pre is bad...
			// we display (sort of)
			// every letter by itself - so we don't miss "carriage return"
			if (cursor.mInPre)
			{
				std::string textOut = "";
				for (const char *pointerToText = text.c_str(); (*pointerToText);pointerToText++)
				{
					if (*pointerToText != '\n')
					{
						char letter[2];
						letter[0] = *pointerToText;
						letter[1] = 0;
						textOut += letter;
					}
					else
					{
						// if there was a carriage return - display the collected letters
						if (destination)
						{
							int x = cursor.mXPos;
							int y = cursor.mYPos;

							CSLabel *label = new CSLabel();
label->setParentLayoutUpdate(false);
							label->setFont(cursor.mFont);
							label->setStyle(cursor.mStyle);
							label->setTextColor(cursor.mPenColor);
							label->setText(textOut);
							destination->addElement(label, x, y);
							if (cursor.mIsLink)
							{
								addLink(label, destination);
							}
						}
						cursor.mXPos += cursor.mFont->getWidth(textOut);
						textOut = "";
						if (cursor.mXPos > cursor.mXPosMax)
						{
							cursor.mXPosMax = cursor.mXPos;
						}
						
						cursor.mXPos = 0;
						cursor.mXPos += cursor.mAddX;
						cursor.mYPos += cursor.nextAdd;
						cursor.nextAdd = cursor.mFont->getHeight(); 
					}
				}
			
				// done with the tag, but nor carriage return found
				// than display the collected letters without it...
				if (textOut.size() > 0)
				{
					if (destination)
					{
						int x = cursor.mXPos;
						int y = cursor.mYPos;
						CSLabel *label = new CSLabel();
label->setParentLayoutUpdate(false);
						label->setFont(cursor.mFont);
						label->setStyle(cursor.mStyle);
						label->setTextColor(cursor.mPenColor);
						label->setText(textOut);
						destination->addElement(label, x, y);
						if (cursor.mIsLink)
						{
							addLink(label, destination);
						}
					}
					cursor.mXPos += cursor.mFont->getWidth(textOut);
					textOut = "";
					if (cursor.mXPos > cursor.mXPosMax)
					{
						cursor.mXPosMax = cursor.mXPos; 
					}
				}
			}
			else
			{
				// Non Pre - displaying is easier now (sort of)
				// First build tokens for ech word
				// build lines that fill the space we have got as good as possible
				StringVector::iterator iter;
				StringVector *tokens = tokenifyText(text);
				int width = 0;
				// build a textline
				std::string line="";

				// go thru all tokens
				for (iter = tokens->begin(); iter != tokens->end();)
				{
					char *word = *iter;
					
					width = cursor.mFont->getWidth(line+" "+word);
					// not wider than the viewport - keep on collecting
					if (cursor.mXPos + width < parentViewportWidth)
					{
						if (line.size()>0)
						{
							line = line +" "+ word;
						}
						else
						{
							line = line + word;
						}
					}
					else
					{
						// if wider than the viewport - display it an build up a
						// new line
						if (destination)
						{
							int x = cursor.mXPos;
							int y = cursor.mYPos;
							CSLabel *label = new CSLabel();
label->setParentLayoutUpdate(false);
							label->setFont(cursor.mFont);
							label->setStyle(cursor.mStyle);
							label->setTextColor(cursor.mPenColor);
							label->setText(line);
							destination->addElement(label, x, y);
							if (cursor.mIsLink)
							{
								addLink(label, destination);
							}
						}
						cursor.mXPos += cursor.mFont->getWidth(line);
						if (cursor.mXPos > cursor.mXPosMax)
						{
							cursor.mXPosMax = cursor.mXPos;
						}
						cursor.mXPos = 0;
						cursor.mXPos += cursor.mAddX;
						cursor.mYPos += cursor.nextAdd;
						cursor.nextAdd = cursor.mFont->getHeight(); 
						line = word;
					}
					iter++;
				}

				// done, but line was not full
				// so we display what we have got...
				if (destination)
				{
					int x = cursor.mXPos;
					int y = cursor.mYPos;
					CSLabel *label = new CSLabel();
label->setParentLayoutUpdate(false);
					label->setFont(cursor.mFont);
					label->setStyle(cursor.mStyle);
					label->setTextColor(cursor.mPenColor);
					label->setText(line);
					destination->addElement(label, x, y);
					if (cursor.mIsLink)
					{
						addLink(label, destination);
					}
				}
				if (line.size()>0)
				{
					line = line + " ";
				}
				cursor.mXPos += cursor.mFont->getWidth(line);
				if (cursor.mXPos > cursor.mXPosMax)
				{
					cursor.mXPosMax = cursor.mXPos;
				}

				for (iter = tokens->begin(); iter != tokens->end();iter++)
				{
					char *word = *iter;
					free(word);
				}
				tokens->clear();
				delete (tokens);
				tokens = 0;
			}
		}
	}

	// was the tag one that can have other tags inside? (nearly all!)
	// than go deeper (recursively)
	if (goDeeper == true)
	{
		CSHTMLElements::iterator iter;
		for (iter = mElements.begin(); iter != mElements.end(); iter++)
		{
			CSHTMLElement *element = *iter;
			cursor = element->buildDisplay(cursor, destination, parentViewportWidth);
		}
	}
	
/*
	if (newLine)
	{
		returnCursor.mYPos = cursor.mYPos + cursor.nextAdd;
		returnCursor.mXPos = 0;
		returnCursor.nextAdd = cursor.nextAdd+cursor.nextAdd; 
		returnCursor.mXPosMax = cursor.mXPosMax;
	}
	else
*/
	{
		// at the end of a Tag "reset" the cursor to its origin 
		// (only position information is kept!)
		returnCursor.mYPos = cursor.mYPos;
		returnCursor.mXPos = cursor.mXPos;
		returnCursor.nextAdd = cursor.nextAdd; 
		returnCursor.mXPosMax = cursor.mXPosMax;
	}

	
	// for more than one list(cascading), the list counting is not correct!
	// can at the moment not be bothered to correct it...
	returnCursor.mListCounter = cursor.mListCounter; 
	return returnCursor;
}

// get the y starting position of this node
// used for Anker - hopping
int CSHTMLElement::getFirstYPostion()
{
	return mY;
}

void CSHTMLElement::addLink(CSGrafikElement *element, CSGrafikElement *listener)
{
	element->addMessageListener(listener, GUI_MESSAGE);
	element->setActionId(HTML_LINK);
	element->setActionObject(this);
}
