#ifndef CSHTMLElement_h
#define CSHTMLElement_h

// USES SDL_Surface

#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

#include <vector>
#include <string>
#include <map>
#include "CSTypes.h"

#include "CSGrafikElement.h"
#include "CSLog.h"
#include "CSTypes.h"
#include "CSStringlist.h"
#include "CSIcon.h"
#include "CSHTMLHelper.h"
#include "CSPicture.h"
#include "CSPanel.h"
#include "CSMessageListener.h"
#include "CSMessageBox.h"

class HTMLCursor;
class CSHTMLElement;

typedef std::vector<CSHTMLElement *> CSHTMLElements;
typedef std::map<std::string, int> HTMLTagMap;

const int HTML_HTML = 1;
const int HTML_HEAD = 2;
const int HTML_TITLE = 3;
const int HTML_BODY = 4;
const int HTML_H1 = 5;
const int HTML_H2 = 6;
const int HTML_H3 = 7;
const int HTML_H4 = 8;
const int HTML_H5 = 9;
const int HTML_P = 10;
const int HTML_BR = 11;
const int HTML_HL = 12;
const int HTML_PRE = 13;
const int HTML_IMG = 14;
const int HTML_A = 15;
const int HTML_B = 16;
const int HTML_I = 17;
const int HTML_DL = 18;
const int HTML_DD = 19;
const int HTML_DT = 20;
const int HTML_OL = 21;
const int HTML_UL = 22;
const int HTML_LI = 23;
const int HTML_TABLE = 24;
const int HTML_TR = 25;
const int HTML_TH = 26;
const int HTML_TD = 27;

const int HTML_OUTPUT_NONE = 0;
const int HTML_OUTPUT_IMAGE = 1;
const int HTML_OUTPUT_TEXT = 2;
const int HTML_OUTPUT_PANEL = 3;


const int HTML_LINK = 12345;

class HTMLCompiled;
class CSHTMLPanel;

class HTMLCursor	// Stylesheet ;-)
{
	friend CSHTMLElement;
	friend CSHTMLPanel;
	private:
		CSFont *mFont;
		int mFontSize; // ?
		int mStyle;		// ?
		
		int mXPos;			// current cursor position in viweport
		int mYPos;			// current cursor position in viweport
		int mXPosMax;		// maximal X width of the whole html page (in relation to the current viewport)
		int mCurrentStartX; // indenting for <LI> 
		int mAddX;			// this where a carriage return leads, the start of the x axis
		int nextAdd;		// nextADD is the amount of pixel that will be added upon the next line to the y axis
		int mPenColor;
		int mBackgroundColor;
		int mLinkColor;
		int mTextalignment;
		int mListCounter;	// starts at 0, -1 means not active, otherwise counted upwords!
		bool mInBody;
		bool mInPre;
		bool mIsLink;

	public:
		HTMLCursor()
		{
			mInBody = false;
			mIsLink = false;
			mInPre = false;
			mPenColor = 0xffffff;
			mLinkColor = 0x3030d0;
			mBackgroundColor = 0xd0d0d0;
			mXPosMax = 0;
			mAddX = 0;
			mListCounter = -1;
			mStyle = 0;
		}
};

class CSHTMLElement
{
	private:
		static HTMLTagMap *mHTMLTagMap;
		static int getTagType(CSHTMLNode &node);
		static StringVector *tokenifyText(const std::string &text);

		CSGrafikElements mGrafikElements;
		CSHTMLElements mElements;
		CSHTMLElement *mParent;

		int mX;						// starting with zero
		int mY;
		int mWidth;
		int mHeight;
		int mXOffset;				// offset von destination, ab dem etwas "geschrieben" werden darf
		int mYOffset;			
		CSHTMLNode mNode;
		std::string getChildrenAsText();
		std::string formatURL(const std::string &rawURL);
		void addPostion(int x, int y, int w, int h);
		void setParent(CSHTMLElement *parent){mParent = parent;}
		CSHTMLElement *getParent(){return mParent;}
		void addLink(CSGrafikElement *element, CSGrafikElement *listener);

	public:
		static const char *CLASS;
		static int getColorFromString(std::string acolor);
		virtual std::string getType() {return (std::string) CLASS;}

		CSHTMLElement(CSHTMLNode node);
		~CSHTMLElement();				// free subElements
		HTMLCursor buildDisplay(HTMLCursor Cursor, CSGrafikElement *destination, int parentViewportWidth);
		int getX() {return mX;}
		int getY() {return mY;}
		int getWidth() {return mWidth;}
		int getHeight() {return mHeight;}
		void clearCompiled();
		std::string getText()
		{
			if (mNode.isText()) return mNode.getValue();
			return std::string();
		}

		int getTagType();
		CSHTMLElement *getAncestorType(int type);
		CSHTMLElement *getChildType(int type, std::string attributName, std::string attributeValue);
		std::string getAttributValue(std::string attributeName);
		int getFirstYPostion();
};


#endif // CSHTMLElement_h
