/*
scrollpane
tabbed pane
table in html
*/

// next todo:
// try to display only the stuff that might actually go on the screen
// (Performance upgrade tha is)

#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

#include "CSHTMLPanel.h"
#include "CSFont.h"
#include "CSDesktop.h"
#include "CSWindow.h"
#include "CSLAF.h"
#include "CSHelper.h"
#include "CSHTMLElement.h"

const char *CSHTMLPanel::CLASS = "CSHTMLPanel";

// htmlPanel
// to possible uses - 
// 1. as a "static" sized "texarea"-like field within some other component
//    eg: //	CSHTMLPanel *html2 = new CSHTMLPanel(100, 100, "index.html");
//				window->addElement(html2, 200,20);
//    a mini sized htmlarea (100x100)
//
// 2. or autosizing within a window
//    	CSHTMLPanel *html = new CSHTMLPanel(150, 300, "index.html");
//		html->setElementAutoSizing(true);
//		window->addElement(html, POSITION_CENTER);
//
//      (The given size in the constructor is ignored!)
//		(Remember to switch autosizing on!)
//		(Remember also to tell the window the kind of posotioning, absolut
//		 positioning is not supported with autosizing)
//
// Only Files on a local system are supported
// no http: ... 
// without protokoll, just plain and simple the filename!
CSHTMLPanel::CSHTMLPanel(int height, int width, const std::string &filename) 
: CSGrafikElement(height, width)
{
	static char *functionName="CSHTMLPanel";
	LOG_ENTER 
	initHTMLPanel();
	setPage(filename, true);
	LOG_EXIT
}

CSHTMLPanel::~CSHTMLPanel() 
{
	if (mMessageBox)
	{
		delete mMessageBox;
		mMessageBox = 0;
	}
	deleteOldPage();
}

void CSHTMLPanel::initHTMLPanel()
{
	static char *functionName="initHTMLPanel";
	LOG_ENTER 
	CSLAF *laf = CSLAF::getCurrentLAF();
	mSetPageLater = false;
	mSetPageLaterName = "";
	mMessageBox = 0;
	mBaseDirectory = "";
	mAnker = "";
	mPageKickoff = false;
	mHelper = 0;
	mHTMLHeightInPixel = 0;
	mHTMLWidthInPixel = 0;
	addMessageListener(this, GUI_MESSAGE);

	CSLayoutManagerXY *layoutManager = new CSLayoutManagerXY();

	
	// only accept changes if component is resized
	// all other changes are discarded
	// adding components does not trigger resize!
	// therefore we can build html elements
	// inside without triggering changes
	// which would result in an etnernal changes - loop
	layoutManager->setPropagateSizeChangesOnly(true);
	setLayoutManager(layoutManager);
	
	rebuildElement();

	CSLayoutData layout = getLayoutData();
	layout.setCenteredHorizontal(true);
	layout.setCenteredVertical(true);
	layout.setStretchedHorizontal(true);
	layout.setStretchedVertical(true);
	layout.setPosition(POSITION_CENTER);
	setLayoutData(layout);
	setInset(CSInset(0));
	LOG_EXIT
}

void CSHTMLPanel::deleteOldPage()
{
	CSHTMLElements::iterator iter;
	for (iter = mHTMLElements.begin(); iter != mHTMLElements.end();)
	{
		CSHTMLElement *element = *iter;
		mHTMLElements.erase(iter);
		delete element;
		iter = mHTMLElements.begin();
	}
	deleteElements();
	if (mHelper != 0)
	{
		delete mHelper;
		mHelper = 0;
	}
	mHTMLHeightInPixel = 0;
	mHTMLWidthInPixel = 0;
}

// extracts leading PATHs from a filename and rememebers the
// path (for relative links to work correctly)
std::string CSHTMLPanel::getBaseDirectory(const std::string &filename)
{
	// searching for "/"
	const char *posStart = strrchr(filename.c_str(), '/');
	if (posStart != 0)
	{
		std::string newText = filename.substr(0, (posStart - filename.c_str()+1));
		return newText;
	}
	posStart = strrchr(filename.c_str(), '\\');
	if (posStart != 0)
	{
		std::string newText = filename.substr(0, (posStart - filename.c_str()+1));
		return newText;
	}
	return "";
}

// extract the anker from a given filename (startinng with # after the filename)
// the filename is truncated!
std::string CSHTMLPanel::extractAnker(std::string &filename)
{
	// searching for "#"
	const char *posStart = strrchr(filename.c_str(), '#');
	if (posStart != 0)
	{
		std::string newFilename = filename.substr(0, (posStart - filename.c_str()));
		std::string anker = filename.substr((posStart - filename.c_str())+1, filename.size());
		filename = newFilename;
		return anker;
	}
	return "";
}

// load a new page
// or keep the old one upon an error
void CSHTMLPanel::setPage(const std::string &filename, bool now)
{
	static char *functionName="setPage";
	
	if (!now)
	{
		mSetPageLater = true;
		mSetPageLaterName = filename;
		layoutChanged(true);
		return;
	}

	// filename handling
	// Anker and Bases...
	CSHTMLHelper *helper;
	std::string newFilename = filename;
	mAnker = extractAnker(newFilename);
	mPageKickoff = true;
	helper = new CSHTMLHelper(newFilename);
	if (helper->getError() != CSHTML_OK)
	{
		delete helper;
		helper = 0;
		
		newFilename = mBaseDirectory + filename;
		mAnker = extractAnker(newFilename);
		helper = new CSHTMLHelper(newFilename);
		if (helper->getError() != CSHTML_OK)
		{
			delete helper;
			helper = 0;
			if (mMessageBox)
			{
				removeFrontElement(mMessageBox);
				delete mMessageBox;
				mMessageBox = 0;
			}
			mMessageBox = new CSMessageBox("Error loading Page:\n"+filename, "Error loading html page!", ERROR_ICON);
			if (mMessageBox)
			{
				addFrontElementCenter(mMessageBox);
				mMessageBox->setVisible(true);
			}
			// load old page instead
			if (mFilename.compare(newFilename) == 0 )
			{
				// already seond try - giving up
				return;
			}
			setPage(mFilename, now);
			return;
		}
	}
	mBaseDirectory = getBaseDirectory(newFilename);
	deleteOldPage();
	mHelper = helper;
	mFilename = filename;
	CSHTMLNode node = mHelper->getRootNode();

	// from the root node
	// all other nodes are recursivly created!
	CSHTMLElement *element = new CSHTMLElement(node);
	mHTMLElements.push_back(element);
	mHTMLHeightInPixel = 0;
	mHTMLWidthInPixel = 0;
	if (!mSetPageLater)
	{
		layoutChanged(true);
	}
	mSetPageLater = false;
	mSetPageLaterName = "";
}

//! \todo html area only painting in the specified region, not everything and clippin!
void CSHTMLPanel::paint(SDL_Surface *destination, SDL_Rect *parentViewport)
{
	static char *functionName="paint";
	mPageKickoff = false;
	paintChildren(destination, parentViewport);
}

void CSHTMLPanel::reactOnMessage(CSMessage *message)
{
	reactOnMessageHTMLPanel(message);
}

void CSHTMLPanel::reactOnMessageHTMLPanel(CSMessage *message)
{
	static char *functionName="reactOnMessage";
	if (message->mIsHandled)
	{
		reactOnMessageGrafikElement(message);
		return;
	}
	
	switch (message->getType())
	{
		case GUI_MESSAGE:
		{
			GuiMessage *gm = (GuiMessage *) message;
			if (message->getSubtype() == MOUSE_BUTTON_PRESSED_MESSAGE)
			{
				if (gm->receiver->getActionId() == HTML_LINK)
				{
					void *object = gm->receiver->getActionObject();
					if (!object)
					{
						return;
					}
					CSHTMLElement *element = (CSHTMLElement *) object;
					// clicked on a link?
					CSHTMLElement *linkNode = element->getAncestorType(HTML_A);
					if (linkNode)
					{
						// yop - try trying to get it!
						// open new page
						std::string linkValue = linkNode->getAttributValue("HREF");
						if (linkValue.size() != 0)
						{
							message->mIsHandled = true;
							setPage(linkValue, false);
							break;
						}
					}
				}
			}
		}
	}
	reactOnMessageGrafikElement(message);
}

// search thru all nodes for the body and extact
// the background color information
// not REALLY nice - but have you got an better idea?
// -1 is not set
// -2 is error in BODY (or not found at all)
int CSHTMLPanel::getBackgroundColorHTML()
{
	CSHTMLElement *bodyElement = 0;
	int value = -1;
	CSHTMLElements::iterator iter;
	for (iter = mHTMLElements.begin(); iter != mHTMLElements.end(); iter++)
	{
		CSHTMLElement *element = *iter;
		bodyElement = element->getChildType(HTML_BODY, "", "");
		if (bodyElement)
		{
			std::string bgColor = bodyElement->getAttributValue("BGCOLOR");
			value = bodyElement->getColorFromString(bgColor);
			break;
		}
	}
	if (value == -1)
	{
		CSLAF *laf = CSLAF::getCurrentLAF();
		value = laf->getBackgroundColorEnabled(CSWindow::CLASS);
	}
	return value;
}

// sat the scrollbar position so that we start at the anker!
void CSHTMLPanel::startAtAnker(const std::string &anker)
{
	CSHTMLElement *ankerElement = 0;

	CSHTMLElements::iterator iter;
	for (iter = mHTMLElements.begin(); iter != mHTMLElements.end(); iter++)
	{
		CSHTMLElement *element = *iter;
		ankerElement = element->getChildType(HTML_A, "NAME", anker);
		if (ankerElement)
		{
			break;
		}
	}

	CSWindow *window = getParentWindow();
	if ((ankerElement) && (window))
	{
		int y = ankerElement->getFirstYPostion();
		if (y != -1)
		{
			window->setVerticalDisplayStart(y);
			window->setHorizontalDisplayStart(0);
		}
	}
}

// in pixel, since there might be pictures as well!
// we do a "dummy" paint of the complete site and
// look how much virtual space we might have used!
// this is quite time expansive
// therefor the information is kept in two 
// variables:
//		mHTMLWidthInPixel;
//		mHTMLHeightInPixel;
// only when these are set to 0 (either or both)
//
// the width an the height will be recalculated
void CSHTMLPanel::getHTMLSize(int &w, int &h)
{
	static char *functionName="getVerticalHTMLSize";
//	if ((mHTMLHeightInPixel != 0) && (mHTMLWidthInPixel != 0))
//	{
//		w = mHTMLWidthInPixel;
//		h = mHTMLHeightInPixel;
//		return;
//	}
//	deleteElements();
	CSLAF *laf = CSLAF::getCurrentLAF();

	int x = 0;
	int y = 0;

	// area without parent offsets is ok, since we don't actually put anything on the screen
	// just the area is interesting!
	SDL_Rect thisViewportArea = getViewportArea();
	CSGrafikElement *parent = getParent();
	if (parent)
	{
		thisViewportArea = parent->getViewportArea();
		thisViewportArea.w-=30; // 30 is dirty, should be size of scrollbar!
		thisViewportArea.h-=30;

		// if < 0 than reset to 0
		if (thisViewportArea.w > 60000)
		{
			thisViewportArea.w = 0;
		}
		if (thisViewportArea.h > 60000)
		{
			thisViewportArea.h = 0;
		}
	}

	HTMLCursor htmlCursor;
	htmlCursor.mXPos = 0;
	htmlCursor.mYPos = 0;
	htmlCursor.mStyle = 0;
	htmlCursor.mFont = getFont();
	htmlCursor.mXPosMax = 0;
	htmlCursor.nextAdd = htmlCursor.mFont->getHeight(); 
	CSHTMLElements::iterator iter;
	for (iter = mHTMLElements.begin(); iter != mHTMLElements.end(); iter++)
	{
		CSHTMLElement *element = *iter;
		// paint with 0 surface means no painting, just counting!
		htmlCursor = element->buildDisplay(htmlCursor, 0, thisViewportArea.w);
	}

	// if the parent viewport is smaller than the largest html element
	// the parent viewport is not the correct value for width of layout
	// relayout the page with the correct value
	// parent window will add a scrollbar!
	if (thisViewportArea.w < htmlCursor.mXPosMax)
	{
		thisViewportArea.w = htmlCursor.mXPosMax-30;
		deleteElements();
		htmlCursor.mXPos = 0;
		htmlCursor.mYPos = 0;
		htmlCursor.mStyle = 0;
		htmlCursor.mFont = getFont();
		htmlCursor.mXPosMax = 0;
		htmlCursor.nextAdd = htmlCursor.mFont->getHeight(); 
		for (iter = mHTMLElements.begin(); iter != mHTMLElements.end(); iter++)
		{
			CSHTMLElement *element = *iter;
			// paint with 0 surface means no painting, just counting!
			htmlCursor = element->buildDisplay(htmlCursor, 0, thisViewportArea.w);
		}
	}
	
	h = mHTMLHeightInPixel = htmlCursor.mYPos + htmlCursor.mFont->getHeight();
	w = mHTMLWidthInPixel = htmlCursor.mXPosMax;
}

void CSHTMLPanel::rebuildElement()
{
	// befor we calculate the new viewport size
	// we have to remove the scrollbars
	// otherwise the viewport width might calculated wrongly
	// (when setHeight/setWidth of parent ist called, the scrollbar sizes are
	//  added (when removed) to the new size - but the new size never knew
	//  of the scrollbars in first place)

	// the size of the viewport changes if a scrollbar is added -> every size might change...
	// looks like ping pong - but is a must :-(
}

// if an element is autosizing -> it is notified by calling changeDisplayLayout!
// that means we have something to do if we are autosizing!
void CSHTMLPanel::layoutSetupHTMLPanel()
{
	static char *functionName="layoutSetupHTMLPanel";
	LOG_ENTER 
	CSLAF *laf = CSLAF::getCurrentLAF();

	if (mSetPageLater)
	{
		setPage(mSetPageLaterName, true);
	}

	// clear all compiled or collected information - 
	// we have to start anew
	getHTMLSize(mMinWidth, mMinHeight);
	deleteElements();
	
	// set up the starting cursor!
	HTMLCursor htmlCursor;
	htmlCursor.mBackgroundColor = getBackgroundColor();
	htmlCursor.mXPos = 0;
	htmlCursor.mYPos = 0;
	htmlCursor.mXPosMax = 0;

	htmlCursor.mFont = getFont();
	htmlCursor.nextAdd = htmlCursor.mFont->getHeight(); 


// TODO!	
/*
	im layout drfen keine elemente erschaffen oder gelscht werden...
	oder das layouten nach oben und unten muss fr die komponenten aus
	geschaltet werden
-> HTML nicht mehr mit GraphicElements erzwingen sondern auf eine Surface "malen"... dann keine Probleme mehr!
  in der update round wird immer die surface auf den bildschirm gebracht, bei einem neuen Layout wird die surface neu gezeichnet
  bei einer grennderung muss eine neue surface erstellt und bezeichnet werden...

  bei einer html seite beineinflusst die gre des viewports, die gre der betrachteten "Welt" - blde
*/	
	// and paint all elements recursivly
	// Insets not respected -> doesn't matter if Insets stay at 0
	CSHTMLElements::iterator iter;
	for (iter = mHTMLElements.begin(); iter != mHTMLElements.end(); iter++)
	{
		CSHTMLElement *element = *iter;
		htmlCursor = element->buildDisplay(htmlCursor, this, getViewportWidth());
	}
		
	setBackgroundColor(getBackgroundColorHTML());
	CSWindow *window = getParentWindow();

	// if we have an anker we start there
	// by setting the scroll offsets likewise
	if (mPageKickoff)
	{
		if (mAnker.size()>0)
		{
			startAtAnker(mAnker);
		}
		else
		{
			if (window)
			{
				window->setVerticalDisplayStart(0);
				window->setHorizontalDisplayStart(0);
			}
		}
	}
}
 