#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

#include "CSIcon.h"
#include "CSFont.h"
#include "CSDesktop.h"
#include "CSBorder.h"
#include "CSLAF.h"

const char *CSIcon::CLASS = "CSIcon";
const char *CSPictureIcon::CLASS = "CSPictureIcon";
const char *CSTextIcon::CLASS = "CSTextIcon";

CSPictureIcon::~CSPictureIcon()
{
	static char *functionName="~CSPictureIcon";
	LOG_ENTER 
	mEnabledPicture = 0;
	mDisabledPicture = 0;
	LOG_EXIT
}

bool CSPictureIcon::initIcon(CSPicture *picture)
{
	static char *functionName="initIcon";
	LOG_ENTER 
	mEnabledPicture = picture;
	mDisabledPicture = picture;
	if (mEnabledPicture && mDisabledPicture)
	{
		init(mEnabledPicture->getMaxY(), mEnabledPicture->getMaxX());
	}
	else
	{
		LOG_EXIT
		return false;
	}
	LOG_EXIT
	return true;
}

bool CSPictureIcon::initIcon(std::string enabledPictureFile, std::string disabledPictureFile)
{
	static char *functionName="initIcon";
	LOG_ENTER 
	mEnabledPicture = CSPictureLoader::INSTANCE.load(enabledPictureFile);
	mDisabledPicture = CSPictureLoader::INSTANCE.load(disabledPictureFile);
	if (mEnabledPicture && mDisabledPicture)
	{
		if ((mEnabledPicture->getHeight() != mDisabledPicture->getHeight()) || (mEnabledPicture->getWidth() != mDisabledPicture->getWidth()))
		{
			LOG_EXIT
			return false;
		}
		init(mEnabledPicture->getHeight(), mEnabledPicture->getWidth());
	}
	else
	{
		LOG_EXIT
		return false;
	}
	LOG_EXIT
	return true;
}

//! \todo display picture is allways on screen! -> must change!
void CSPictureIcon::paint(SDL_Surface *destination, SDL_Rect *parentViewport)
{
	static char *functionName="paint";
	int state = getState();
	if (state == ICON_STATE_ENABLED)
	{
		setEnabled(true);

		
		mEnabledPicture->display(
				getElementArea().x + parentViewport->x + getBorder()->getSizeWest() + getInsetInternal()->getSizeWest(), 
				getElementArea().y + parentViewport->y + getBorder()->getSizeNorth()  + getInsetInternal()->getSizeNorth()
				);
	}
	if (state == ICON_STATE_DISABLED)
	{
		setEnabled(false);
		mDisabledPicture->display(
				getElementArea().x + parentViewport->x + getBorder()->getSizeWest() + getInsetInternal()->getSizeWest(), 
				getElementArea().y + parentViewport->y + getBorder()->getSizeNorth()  + getInsetInternal()->getSizeNorth()
				);
	}
}

void CSPictureIcon::layoutSetupPictureIcon() 
{
	if (mEnabledPicture)
	{
		// mMinHeight = mHeight = mEnabledPicture->getHeight();
		// mMinWidth = mWidth = mEnabledPicture->getWidth();
		mMinHeight = mEnabledPicture->getHeight();
		mMinWidth = mEnabledPicture->getWidth();
	}
}

/*************************************************************************************************************/
/*************************************************************************************************************/
/*************************************************************************************************************/

CSTextIcon::~CSTextIcon() 
{
	static char *functionName="~CSTextIcon";
	LOG_ENTER 
	LOG_EXIT
}

bool CSTextIcon::initIcon(const std::string &text)
{
	static char *functionName="initIcon";
	LOG_ENTER 
	mText = text;
	init(getFont()->getHeight(), getFont()->getWidth(mText.c_str()));
	LOG_EXIT
	return true;
}

void CSTextIcon::paint(SDL_Surface *destination, SDL_Rect *parentViewport)
{
	static char *functionName="paint";
	int state = getState();

	if (state == ICON_STATE_ENABLED)
	{
		setEnabled(true);
	}
	if (state == ICON_STATE_DISABLED)
	{
		setEnabled(false);
	}

	putString(destination, parentViewport, 
		getX() + getBorder()->getSizeWest() + getInsetInternal()->getSizeWest(), 
		getY() + getBorder()->getSizeNorth()  + getInsetInternal()->getSizeNorth(), 
		getTextColor(), mText.c_str());
}

void CSTextIcon::layoutSetupTextIcon()
{
	static char *functionName="layoutSetupTextIcon";
	LOG_ENTER 
	mMinHeight = mHeight = getFont()->getHeight();
	mMinWidth = mWidth = getFont()->getWidth(mText.c_str());
	LOG_EXIT
}

