#ifndef CSIcon_h
#define CSIcon_h

// USES SDL_Surface

#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

#include <vector>
#include <string>

#include "CSGrafikElement.h"
#include "CSPicture.h"
#include "CSLog.h"
#include "CSInset.h"

class CSIcon;
typedef std::vector<CSIcon *> CSIcons;

const int ICON_KIND_UNKOWN = -1;
const int ICON_KIND_PICTURE = 0;
const int ICON_KIND_TEXT  = 1;
const int ICON_KIND_PAINT   = 2;

const int ICON_STATE_UNKOWN = -1;
const int ICON_STATE_ENABLED = 0;
const int ICON_STATE_DISABLED  = 1;
const int ICON_STATE_ACTIVATED   = 2;

class CSIcon : public CSGrafikElement
{
	private:
		int mKind;
		int mState;

	protected:
		CSIcon(int kind) : CSGrafikElement(0,0)
		{
			mKind = kind;
			mState = ICON_STATE_ENABLED;
		}

		int getKind() {return mKind;}

	public:
		static const char *CLASS;
		virtual std::string getType() {return (std::string) CLASS;}
		virtual ~CSIcon() {}
		void setState(int state) 
		{
			if ((state <= ICON_STATE_UNKOWN) || (state >= ICON_STATE_ACTIVATED))
				return;
			mState = state;
		}

		int getState() {return mState;}
		virtual bool isFocusable() {return false;}
		virtual void paint(SDL_Surface *destination, SDL_Rect *parentViewport) = 0;
};

class CSPictureIcon : public CSIcon
{
	private:
		CSPicture *mEnabledPicture;
		CSPicture *mDisabledPicture;

	protected:
		CSPictureIcon() : CSIcon(ICON_KIND_PICTURE){}
		void layoutSetupPictureIcon();

	public:
		static const char *CLASS;
		virtual std::string getType() {return (std::string) CLASS;}
		static CSPictureIcon* buildIcon(std::string enabledPictureFile, std::string disabledPictureFile)
		{
			CSPictureIcon* icon = 0;
			try
			{
				icon = new CSPictureIcon();
				if (!icon->initIcon(enabledPictureFile, disabledPictureFile))
				{
					delete(icon);
					icon = 0;
				}
			}
			catch (...) {}
			icon->layoutChanged();
			return icon;
		}
		static CSPictureIcon* buildIcon(CSPicture *picture)
		{
			CSPictureIcon* icon = 0;
			try
			{
				icon = new CSPictureIcon();
				if (!icon->initIcon(picture))
				{
					delete(icon);
					icon = 0;
				}
			}
			catch (...) {}
			icon->layoutChanged();
			return icon;
		}

		static CSPictureIcon* buildIcon(std::string enabledPictureFile)
		{
			CSPictureIcon* icon = 0;
			try
			{
				icon = new CSPictureIcon();
				if (!icon->initIcon(enabledPictureFile, enabledPictureFile))
				{
					delete(icon);
					icon = 0;
				}
			}
			catch (...) {}
			icon->layoutChanged();
			return icon;
		}
		virtual ~CSPictureIcon();
		bool initIcon(std::string enabledPictureFile, std::string disabledPictureFile);
		bool initIcon(CSPicture *picture);
		virtual void paint(SDL_Surface *destination, SDL_Rect *parentViewport);
		virtual void layoutSetup() {layoutSetupPictureIcon();}
};

class CSTextIcon : public CSIcon
{
	private:
		std::string mText;

	protected:
		CSTextIcon() : CSIcon(ICON_KIND_TEXT) {}
		void layoutSetupTextIcon();

	public:
		static const char *CLASS;
		virtual std::string getType() {return (std::string) CLASS;}

		static CSTextIcon* buildIcon(const std::string &text)
		{
			CSTextIcon* icon = 0;
			try
			{
				icon = new CSTextIcon();
				if (!icon->initIcon(text))
				{
					delete(icon);
					icon = 0;
				}
			}
			catch (...) {}
			icon->layoutChanged();
			return icon;
		}
		virtual ~CSTextIcon();
		bool initIcon(const std::string &text);
		void setText(const std::string &text) 
		{
			if (text.compare(mText) != 0)
			{
				mText = text;layoutChanged();
			}
		}
		virtual void paint(SDL_Surface *destination, SDL_Rect *parentViewport);
		virtual void layoutSetup() {layoutSetupTextIcon();}
};

#endif // CSIcon_h

