#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

#include "CSLayoutManager.h"
#include "CSGrafikElement.h"
#include "CSBorder.h"

const char *CSLayoutManager::CLASS = "CSLayoutManager";

void CSLayoutData::init()
{
	mX = 0;
	mY = 0;
	mPosition = POSITION_NOT_SET;
	mPackedHorizontal = false;
	mPackedVertical = false;
	mCenteredHorizontal = false;
	mCenteredVertical = false;
	mStretchedVertical = false;
	mStretchedHorizontal = false;
	mOversize = true;
	mVerticalSpacing = 0;
	mHorizontalSpacing = 0;
	mSpacingSet = false;

}
CSGrafikElements *CSLayoutManager::getElements(CSGrafikElement *element)
{
	if (element)
	{
		return element->getElements();
	}
	return 0;
}

void CSLayoutManager::layout()
{
	CSGrafikElement *host = getHostingElement();
	if (host == 0) 
	{
		return;
	}
	buildArea(host);
	layoutElements();
}

bool CSLayoutManager::setLayout(CSGrafikElement *visitor, int x, int y, int width ,int height)
{
	bool pos = false;
	bool size = false;

	// don't set positions if any is -1
	if ((x != -1) && (y != -1))
	{
		if ((visitor->mX != x) || (visitor->mY != y))
		{
			pos = true;
		}
		if (x != -1)
		{
			visitor->mX = x;
		}
		if (y != -1)
		{
			visitor->mY = y;
		}
	}
	if (width != -1)
	{
		if (width < visitor->mMinWidth) 
		{
			width = visitor->mMinWidth;
		}
		if (visitor->mWidth != width)
		{
			size = true;
			visitor->mWidth = width;
			visitor->mWidthSet = true;
		}
	}
	else
	{
		// not to be changed from layout!
		// visitor->mWidthSet = false;
	}
	if (height != -1)
	{
		if (height < visitor->mMinHeight) 
		{
			height = visitor->mMinHeight;
		}
		if (visitor->mHeight != height)
		{
			size = true;
			visitor->mHeight = height;
			visitor->mHeightSet = true;
		}
	}
	else
	{
		// not to be changed from layout!
		// visitor->mHeightSet = false;
	}

	if (pos | size)
	{
		buildArea(visitor);
	}
	return pos | size;
}

CSLayoutData *CSLayoutManager::getLayoutData(CSGrafikElement *visitor) 
{
	return visitor->getLayoutDataInternal();
}

CSGrafikElement *CSLayoutManager::getParent(CSGrafikElement *element)
{
	element = element->getParent();
	return element;
}

int CSLayoutManager::getParentWidth(CSGrafikElement *element)
{
	element = element->getParent();
	if (element)
	{
		return element->getViewportWidth();
	}
	return 0;
}

int CSLayoutManager::getParentHeight(CSGrafikElement *element)
{
	element = element->getParent();
	if (element)
	{
		return element->getViewportHeight();
	}
	return 0;
}

int CSLayoutManager::getViewportWidth(CSGrafikElement *element)
{
	return element->getViewportWidth();
}

int CSLayoutManager::getViewportHeight(CSGrafikElement *element)
{
	return element->getViewportHeight();
}

int CSLayoutManager::getWidth(CSGrafikElement *element)
{
	return element->getWidth();
}

int CSLayoutManager::getHeight(CSGrafikElement *element)
{
	return element->getHeight();
}

int CSLayoutManager::getMinWidth(CSGrafikElement *element)
{
	return element->getMinWidth();
}

int CSLayoutManager::getMinHeight(CSGrafikElement *element)
{
	return element->getMinHeight();
}

int CSLayoutManager::getBorderWidth(CSGrafikElement *element)
{
	return element->getBorder()->getTotalWidth();
}

int CSLayoutManager::getBorderHeight(CSGrafikElement *element)
{
	return element->getBorder()->getTotalHeight();
}

int CSLayoutManager::getInsetWidth(CSGrafikElement *element)
{
	return element->getInsetInternal()->getTotalWidth();
}

int CSLayoutManager::getInsetHeight(CSGrafikElement *element)
{
	return element->getInsetInternal()->getTotalHeight();
}

void CSLayoutManager::pack(CSGrafikElement *element)
{
	static char *functionName="pack";

	if (element->mHeight < element->getLayoutManager()->getMinimumHeight(element) + getBorderHeight(element) + getInsetHeight(element)) 
	{
		// changed to -> csa 17,11,2011
		element->mHeight = element->getLayoutManager()->getMinimumHeight(element) + getBorderHeight(element) + getInsetHeight(element);
		element->mHeight = element->getLayoutManager()->getMinimumHeight(element);
	}
	if (element->mWidth < element->getLayoutManager()->getMinimumWidth(element) + getBorderWidth(element) + getInsetWidth(element)) 
	{
		int m1 = element->getLayoutManager()->getMinimumWidth(element);
		int m2 = getBorderWidth(element);
		int m3 = getInsetWidth(element);
		
		element->mWidth = element->getLayoutManager()->getMinimumWidth(element) + getBorderWidth(element) + getInsetWidth(element);
		// changed to -> csa 17,11,2011
		element->mWidth = element->getLayoutManager()->getMinimumWidth(element);
	}
	buildArea(element);
}

/**
	(Re) build the sizes of element, including viewports. Takes into
	account that children mght be non viewported and thus influence
	the sise of \a this viewoirt...
*/
// only input is:
//	mX;
//	mY;
//	mHeight;
//	mWidth;
void CSLayoutManager::buildArea(CSGrafikElement *element)
{
	static char *functionName="buildArea";
	if (element->mHeight < getBorderHeight(element) + getInsetHeight(element)) 
	{
		element->mHeight = getBorderHeight(element) + getInsetHeight(element);
	}
	if (element->mWidth < getBorderWidth(element) + getInsetWidth(element)) 
	{
		element->mWidth = getBorderWidth(element) + getInsetWidth(element);
	}


	if (!getLayoutData(element)->getOversize())
	{
		CSGrafikElement *parent = element->mParent;
		if (parent)
		{
			if ( (element->mHeight>parent->getViewportHeight()) && (parent->getViewportHeight()!=0))
			{
				element->mHeight = parent->getViewportHeight();
			}
			if ((element->mWidth>parent->getViewportWidth()) && (parent->getViewportWidth()!=0))
			{
				element->mWidth = parent->getViewportWidth();
			}
		}
	}

	
	element->mElementArea.x = element->mX;
	element->mElementArea.y = element->mY;
	element->mElementArea.h = element->mHeight;
	element->mElementArea.w = element->mWidth;

	element->mViewportX = element->getBorder()->getSizeWest() + element->getInsetInternal()->getSizeWest();
	element->mViewportY = element->getBorder()->getSizeNorth()  + element->getInsetInternal()->getSizeNorth();
	element->mViewportHeight = element->getHeight() - getBorderHeight(element) - getInsetHeight(element);
	element->mViewportWidth = element->getWidth() - getBorderWidth(element)  - getInsetWidth(element);

	if (element->mViewportHeight < 0) element->mViewportHeight = 0;
	if (element->mViewportWidth < 0) element->mViewportWidth = 0;

	element->mViewportArea.x = element->mViewportX;
	element->mViewportArea.y = element->mViewportY;
	element->mViewportArea.h = element->mViewportHeight;
	element->mViewportArea.w = element->mViewportWidth;

	element->getLayoutDataInternal()->setX(element->mX);
	element->getLayoutDataInternal()->setY(element->mY);



}

bool CSLayoutManager::isNextUnstretchedParentHorizontalFixed(CSGrafikElement *element)
{
	CSGrafikElement *parent = element->mParent;
	while (parent != 0)
	{
		if (parent->getLayoutDataInternal()->getPackedHorizontal())
		{
			return false;
		}
		if (parent->getLayoutDataInternal()->getStretchedHorizontal())
		{
			parent = parent->getParent();
			continue;
		}
		return true;		
	}
	return false;
}

bool CSLayoutManager::isNextUnstretchedParentVerticalFixed(CSGrafikElement *element)
{
	CSGrafikElement *parent = element->mParent;
	while (parent != 0)
	{
		if (parent->getLayoutDataInternal()->getPackedVertical())
		{
			return false;
		}
		if (parent->getLayoutDataInternal()->getStretchedVertical())
		{
			parent = parent->getParent();
			continue;
		}
		return true;		
	}
	return false;
}

