#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

#include "CSLayoutManager.h"
#include "CSGrafikElement.h"
#include "CSBorder.h"

const char *CSLayoutManagerBorder::CLASS = "CSLayoutManagerBorder";

/**
	if one border is missing center is always extended first
	north and south is extend each other if center is missing
	north and south are equal
	east and west are equal

	CSPanel* panelNV = new CSPanel(200,200);
	l = CSLayoutData(POSITION_NORTH);
	l.setStretchedHorizontal(true);
	mDesktop->addElement(panelNV, l);

	CSPanel* panelSV = new CSPanel(200,200);
	l = CSLayoutData(POSITION_SOUTH);
	l.setStretchedHorizontal(true);
	mDesktop->addElement(panelSV, l);

	CSPanel* panelEV = new CSPanel(200,200);
	l = CSLayoutData(POSITION_EAST);
	l.setStretchedVertical(true);
	mDesktop->addElement(panelEV, l);

	CSPanel* panelWV = new CSPanel(200,200);
	l = CSLayoutData(POSITION_WEST);
	l.setStretchedVertical(true);
	mDesktop->addElement(panelWV, l);

*/

void CSLayoutManagerBorder::layoutElements()
{
	CSGrafikElement *host = getHostingElement();
	if (host == 0) 
	{
		return;
	}

	// stretching of components within borderlayout is a bit invonvinient
	// perhaps I implemented it a bit to complicated - for now it seems to work as I want it - anyway

	// first initialize all layout properties to false
	bool northXStretch = false;
	bool northXPack = false;
	bool northYPack = false;
	bool northYStretch = false;
	bool southXStretch = false;
	bool southYStretch = false;
	bool southXPack = false;
	bool southYPack = false;
	bool eastXStretch = false;
	bool eastYStretch = false;
	bool eastXPack = false;
	bool eastYPack = false;
	bool westXStretch = false;
	bool westYStretch = false;
	bool westXPack = false;
	bool westYPack = false;
	bool centerXStretch = false;
	bool centerYStretch = false;
	bool centerXPack = false;
	bool centerYPack = false;

	// Space is the least space of the corresponding component will use
	// we sum it up an build the differenz between the some and
	// the actual viewport size
	// the result is the maximum space we can stretch the components
	// each component can stretch equally (the is the so calculated stretch size is divided
	// by the count of stretchable components)
	int nYSpace = 0;
	int sYSpace = 0;
	int cYSpace = 0;
	int wYSpace = 0;
	int eYSpace = 0;
	int nXSpace = 0;
	int sXSpace = 0;
	int cXSpace = 0;
	int wXSpace = 0;
	int eXSpace = 0;
	
	// number of components that will stretch for a given direction
	int xStretchCount = 0;
	int yStretchCount = 0;

	// space already used for stretching
	int xStretchUsed = 0;
	int yStretchUsed = 0;

	// current count of components that already are stretched
	int yStretchCurrentCount=0;
	int xStretchCurrentCount=0;

	int verticalSpacing;
	int horizontalSpacing;
	CSLayoutData *hostLayoutData = getLayoutData(host);
	hostLayoutData->getSpacing(verticalSpacing, horizontalSpacing);

	int width = getViewportWidth(host);
	int height = getViewportHeight(host);

	int centerStartY = 0;
	int centerEndY = height;
	int centerStartX = 0;
	int centerEndX = width;

	CSGrafikElements *elements = getElements(host);
	CSGrafikElements::iterator iter = elements->begin();
	CSGrafikElement *north = 0;
	CSGrafikElement *south = 0;
	CSGrafikElement *east = 0;
	CSGrafikElement *west = 0;
	CSGrafikElement *center = 0;

	// get the five cinstituing components!
	while (iter != elements->end())
	{
		CSGrafikElement *visitor = *iter;
		if (!visitor->getVisible())
		{
			iter++;
			continue;
		}
		CSLayoutData *visitorLayoutData = getLayoutData(visitor);
		// now, lay it out
		switch (visitorLayoutData->getPosition())
		{
			case POSITION_NORTH:
			{
				north = visitor;
				northXStretch = getLayoutData(north)->getStretchedHorizontal();
				northYStretch = getLayoutData(north)->getStretchedVertical();
				northXPack = getLayoutData(north)->getPackedHorizontal();
				northYPack = getLayoutData(north)->getPackedVertical();
				if (northYStretch) 
					yStretchCount++;

				// minimum size of a stretching component is min-Size,
				// the added stretching size is done later
				if (northYPack || northYStretch) 
					nYSpace = north->getLayoutManager()->getMinimumHeight(north);
				else 
					nYSpace = north->getHeight();
				if (northXPack || northXStretch) 
					nXSpace = north->getLayoutManager()->getMinimumWidth(north);
				else 
					nXSpace = north->getWidth();
				break;
			}
			case POSITION_SOUTH:
			{
				south = visitor;
				southXStretch = getLayoutData(south)->getStretchedHorizontal();
				southYStretch = getLayoutData(south)->getStretchedVertical();
				southXPack = getLayoutData(south)->getPackedHorizontal();
				southYPack = getLayoutData(south)->getPackedVertical();
				if (southYStretch) 
					yStretchCount++;
				if (southYPack || southYStretch) 
					sYSpace = south->getLayoutManager()->getMinimumHeight(south);
				else 
					sYSpace = south->getHeight();
				if (southXPack || southXStretch) 
					sXSpace = south->getLayoutManager()->getMinimumWidth(south);
				else 
					sXSpace = south->getWidth();
				break;
			}
			case POSITION_EAST:
			{
				east = visitor;
				eastXStretch = getLayoutData(east)->getStretchedHorizontal();
				eastYStretch = getLayoutData(east)->getStretchedVertical();
				eastXPack = getLayoutData(east)->getPackedHorizontal();
				eastYPack = getLayoutData(east)->getPackedVertical();
				if (eastXStretch) 
					xStretchCount++;
				if (eastYPack || eastYStretch) 
					eYSpace = east->getLayoutManager()->getMinimumHeight(east);
				else 
					eYSpace = east->getHeight();
				if (eastXPack || eastXStretch) 
					eXSpace = east->getLayoutManager()->getMinimumWidth(east);
				else 
					eXSpace = east->getWidth();
				break;
			}
			case POSITION_WEST:
			{
				west = visitor;
				westXStretch = getLayoutData(west)->getStretchedHorizontal();
				westYStretch = getLayoutData(west)->getStretchedVertical();
				westXPack = getLayoutData(west)->getPackedHorizontal();
				westYPack = getLayoutData(west)->getPackedVertical();
				if (westXStretch) 
					xStretchCount++;
				if (westYPack || westYStretch) 
					wYSpace = west->getLayoutManager()->getMinimumHeight(west);
				else 
					wYSpace = west->getHeight();
				if (westXPack || westXStretch) 
					wXSpace = west->getLayoutManager()->getMinimumWidth(west);
				else 
					wXSpace = west->getWidth();
				break;
			}
			case POSITION_CENTER:
			{
				center = visitor;
				centerXStretch = getLayoutData(center)->getStretchedHorizontal();
				centerYStretch = getLayoutData(center)->getStretchedVertical();
				centerXPack = getLayoutData(center)->getPackedHorizontal();
				centerYPack = getLayoutData(center)->getPackedVertical();
				if (centerXStretch) 
					xStretchCount++;
				if (centerYStretch) 
					yStretchCount++;
				if (centerYPack || centerYStretch) 
					cYSpace = center->getLayoutManager()->getMinimumHeight(center);
				else 
					cYSpace = center->getHeight();
				if (centerXPack || centerXStretch) 
					cXSpace = center->getLayoutManager()->getMinimumWidth(center);
				else 
					cXSpace = center->getWidth();
				break;
			}
			default:
			{
				break;
			}
		}
		iter++;
	}

	// the center components (Y-direction) are counted as one Y-component in respect to stretching
	// collectData
	if (centerYStretch || eastYStretch || westYStretch) 
		yStretchCount++;

	// get y - stretch - space! (center stretch is maximum of east, west and center)
	if (cYSpace < eYSpace) 
		cYSpace = eYSpace;
	if (cYSpace < wYSpace) 
		cYSpace = wYSpace;
	// stretch space is the viewport height - the minimum space needed alltogether
	int yStretchSpace = height - nYSpace - sYSpace - cYSpace;

	// and don't forget the spacings between the components
	if ((north) && ((center) || (east) || (west)) && (south))
	{
		yStretchSpace -= verticalSpacing;
		yStretchSpace -= verticalSpacing;
	}
	else if ((north) && ((center) || (east) || (west)))
	{
		yStretchSpace -= verticalSpacing;
	}
	else if (((center) || (east) || (west)) && (south))
	{
		yStretchSpace -= verticalSpacing;
	}
	else if ((north) && (south))
	{
		yStretchSpace -= verticalSpacing;
	}

	// get x - stretch - space!
	cXSpace = cXSpace + wXSpace + eXSpace;
	if (cXSpace < nXSpace) 
		cXSpace = nXSpace;
	if (cXSpace < sXSpace) 
		cXSpace = sXSpace;
	int xStretchSpace = width - cXSpace;
	if ((west) && (center) && (east))
	{
		xStretchSpace -= horizontalSpacing;
		xStretchSpace -= horizontalSpacing;
	}
	else if ((west) && (center))
	{
		xStretchSpace -= horizontalSpacing;
	}
	else if ((center) && (east))
	{
		xStretchSpace -= horizontalSpacing;
	}
	else if ((west) && (east))
	{
		xStretchSpace -= horizontalSpacing;
	}

	if (north)
	{
   		CSLayoutData *layoutData = getLayoutData(north);
		int x = 0;
		int y = 0;
		int w = north->getWidth();
		int h = north->getHeight();
		
		if (layoutData->getStretchedHorizontal())
		{
			// easy
			w = width;
			if (w < north->getLayoutManager()->getMinimumWidth(north))
			{
				w = north->getLayoutManager()->getMinimumWidth(north);
			}
		}
		else if (layoutData->getPackedHorizontal())
		{
			w = north->getLayoutManager()->getMinimumWidth(north);
		}
		if (layoutData->getCenteredHorizontal())
		{
			x = (width / 2) - w/2;
			if (x < 0)
			{
				x = 0;
			}
		}

		if (layoutData->getStretchedVertical())
		{
			h = north->getLayoutManager()->getMinimumHeight(north);
			// StretchUsed is the sum of streches we already did
			// used to compare to yStretchSpace
			// if modulo stretchspace != 0 we can add the differenz to the last stretched component
			// so we don't miss a pixel!
			yStretchUsed += yStretchSpace / yStretchCount;
			h += yStretchSpace / yStretchCount;
			yStretchCurrentCount++;
		}
		else if (layoutData->getPackedVertical())
		{
			h = north->getLayoutManager()->getMinimumHeight(north);
		}
		setLayout(north, x, y, w, h);
		centerStartY = y + h + verticalSpacing;
	}
	if (south)
	{
		CSLayoutData *layoutData = getLayoutData(south);
		bool southPackedVertical = false;
		int x = 0;
		int y;
		int w = south->getWidth();
		int h = south->getHeight();
		
		if (layoutData->getStretchedHorizontal())
		{
			w = width;
			if (w < south->getLayoutManager()->getMinimumWidth(south))
			{
				w = south->getLayoutManager()->getMinimumWidth(south);
			}
		}
		else if (layoutData->getPackedHorizontal())
		{
			w = south->getLayoutManager()->getMinimumWidth(south);
		}
		if (layoutData->getCenteredHorizontal())
		{
			x = (width / 2) - w/2;
			if (x < 0)
			{
				x = 0;
			}
		}
		if (layoutData->getStretchedVertical())
		{
			h = south->getLayoutManager()->getMinimumHeight(south);
			// StretchUsed is the sum of streches we already did
			// used to compare to yStretchSpace
			yStretchUsed += yStretchSpace / yStretchCount;
			h += yStretchSpace / yStretchCount;
			yStretchCurrentCount++;

			// if modulo stretchspace != 0 we can add the differenz to the last stretched component
			// so we don't miss a pixel!
			if (yStretchCurrentCount == yStretchCount)
			{
				if (yStretchUsed != yStretchSpace)
				{
					h += yStretchSpace - yStretchUsed;
					yStretchUsed = yStretchSpace;
				}
			}
		}
		else 
		{
			if (layoutData->getPackedVertical())
			{
				h = south->getLayoutManager()->getMinimumHeight(south);
				southPackedVertical = true;
			}
		}
		y = height - h;
		if (y < 0) 
		{
			y = 0;
		}
		setLayout(south, x, y, w, h);
		centerEndY = y - verticalSpacing;
	}
	
	if (west)
	{
		CSLayoutData *layoutData = getLayoutData(west);
		int x = 0;
		int y = centerStartY;
		int w = west->getWidth();
		int h = west->getHeight();
		
		if (layoutData->getStretchedHorizontal())
		{
			w = west->getLayoutManager()->getMinimumWidth(west);
			// + anteile am stretch
			xStretchUsed += xStretchSpace / xStretchCount;
			w += xStretchSpace / xStretchCount;
			xStretchCurrentCount++;
		}
		else 
		{
			if (layoutData->getPackedHorizontal())
			{
				w = west->getLayoutManager()->getMinimumWidth(west);
			}
		}

		if (layoutData->getStretchedVertical())
		{
			h = centerEndY - centerStartY;
			if (h < west->getLayoutManager()->getMinimumHeight(west))
			{
				h = west->getLayoutManager()->getMinimumHeight(west);
			}
		}
		else 
		{
			if (layoutData->getPackedVertical())
			{
				h = west->getLayoutManager()->getMinimumHeight(west);
			}
		}
		if (layoutData->getCenteredVertical())
		{
			y += ((centerEndY - centerStartY)/2) - h/2;
			if (y < centerStartY)
			{
				y = centerStartY;
			}
		}
		setLayout(west, x, y, w , h);
		centerStartX = x + w + horizontalSpacing;
	}

	if (east)
	{
		CSLayoutData *layoutData = getLayoutData(east);
		int x;
		int y = centerStartY;
		int w = east->getWidth();
		int h = east->getHeight();
		if (layoutData->getStretchedHorizontal())
		{
			w = east->getLayoutManager()->getMinimumWidth(east);
			// + anteile am stretch
			xStretchUsed += xStretchSpace / xStretchCount;
			w += xStretchSpace / xStretchCount;
			xStretchCurrentCount++;
			if (xStretchCurrentCount == xStretchCount)
			{
				if (xStretchUsed != xStretchSpace)
				{
					w += xStretchSpace - xStretchUsed;
					xStretchUsed = xStretchSpace;
				}
			}
		}
		else 
		{
			if (layoutData->getPackedHorizontal())
			{
				w = east->getLayoutManager()->getMinimumWidth(east);
			}
		}
		x = width  - w;
		if (x < 0)  
		{
			x = 0;
		}
		if (layoutData->getStretchedVertical())
		{
			h = centerEndY - centerStartY;
			if (h < east->getLayoutManager()->getMinimumHeight(east))
			{
				h = east->getLayoutManager()->getMinimumHeight(east);
			}
		}
		else 
		{
			if (layoutData->getPackedVertical())
			{
				h = east->getLayoutManager()->getMinimumHeight(east);
			}
		}
		if (layoutData->getCenteredVertical())
		{
			y += ((centerEndY - centerStartY)/2) - h/2;
			if (y < centerStartY)
			{
				y = centerStartY;
			}
		}
		setLayout(east, x, y, w , h);
		centerEndX = x - horizontalSpacing;
	}

	if (center)
	{
		CSLayoutData *layoutData = getLayoutData(center);
		int x = centerStartX;
		int y = centerStartY;
		int w = center->getWidth();
		int h = center->getHeight();
		if (layoutData->getStretchedHorizontal())
		{
			// nice and easy, if center is stretched, we just take all space that is left over!
			w = centerEndX - centerStartX;
			if (w < center->getLayoutManager()->getMinimumWidth(center))
			{
				w = center->getLayoutManager()->getMinimumWidth(center);
			}
		}
		else 
		{
			if (layoutData->getPackedHorizontal())
			{
				w = center->getLayoutManager()->getMinimumWidth(center);
			}
		}
		if (layoutData->getCenteredHorizontal())
		{
			x +=  ((centerEndX - centerStartX) / 2) - w/2;
		}
			
		if (layoutData->getStretchedVertical())
		{
			// nice and easy, if center is stretched, we just take all space that is left over!
			h = centerEndY - centerStartY;
			if (h < center->getLayoutManager()->getMinimumHeight(center))
			{
				h = center->getLayoutManager()->getMinimumHeight(center);
			}
		}
		else 
		{
			if (layoutData->getPackedVertical())
			{
				h = center->getLayoutManager()->getMinimumHeight(center);
			}
		}
		if (layoutData->getCenteredVertical())
		{
			y +=  ((centerEndY - centerStartY) / 2) - h/2;
		}
		setLayout(center, x, y, w , h);
	}
}

int CSLayoutManagerBorder::getMinimumHeight(CSGrafikElement *element)
{
	if (element == 0)
	{
		return 0;
	}
	if (!element->getVisible())
	{
		return 0;
	}
	CSLayoutData *layoutData = getLayoutData(element);
	
	if (layoutData->isFixSized())
	{
		return element->getHeight();
	}


	int height = getMinHeight(element);

	CSGrafikElements *elements = getElements(element);
	CSGrafikElements::iterator iter = elements->begin();

	CSGrafikElement *north = 0;
	CSGrafikElement *south = 0;
	CSGrafikElement *east = 0;
	CSGrafikElement *west = 0;
	CSGrafikElement *center = 0;

	while (iter != elements->end())
	{
		CSGrafikElement *visitor = *iter;
		if (!visitor->getVisible())
		{
			iter++;
			continue;
		}
		CSLayoutData *visitorLayoutData = getLayoutData(visitor);
		switch (visitorLayoutData->getPosition())
		{
			case POSITION_NORTH:
			{
				north = visitor;
				break;
			}
			case POSITION_SOUTH:
			{
				south = visitor;
				break;
			}
			case POSITION_EAST:
			{
				east = visitor;
				break;
			}
			case POSITION_WEST:
			{
				west = visitor;
				break;
			}
			case POSITION_CENTER:
			{
				center = visitor;
				break;
			}
			default:
			{
				break;
			}
		}
		iter++;
	}

	int verticalSpacing;
	int horizontalSpacing;
	layoutData->getSpacing(verticalSpacing, horizontalSpacing);

	int neededHeight = 0;
	if ((north) && ((center) || (east) || (west)) && (south))
	{
		neededHeight += verticalSpacing;
		neededHeight += verticalSpacing;
	}
	else if ((north) && ((center) || (east) || (west)))
	{
		neededHeight += verticalSpacing;
	}
	else if (((center) || (east) || (west)) && (south))
	{
		neededHeight += verticalSpacing;
	}
	else if ((north) && (south))
	{
		neededHeight += verticalSpacing;
	}

	if (north)
	{
		neededHeight += north->getLayoutManager()->getMinimumHeight(north);
	}
	if ((center) || (east) || (west))
	{
		int maxHeight = 0;
		int centerHeight = 0;
		int eastHeight = 0;
		int westHeight = 0;

		if (center)
		{
			centerHeight = center->getLayoutManager()->getMinimumHeight(center);
		}
		if (east)
		{
			eastHeight = east->getLayoutManager()->getMinimumHeight(east);
		}
		if (west)
		{
			westHeight = west->getLayoutManager()->getMinimumHeight(west);
		}

		maxHeight = centerHeight;
		if (eastHeight>maxHeight)
		{
			maxHeight = eastHeight;
		}
		if (westHeight>maxHeight)
		{
			maxHeight = westHeight;
		}

		neededHeight += maxHeight;
	}
	if (south)
	{
		neededHeight += south->getLayoutManager()->getMinimumHeight(south);
	}

	if (neededHeight > height)
	{
		height = neededHeight;
	}

	height += getInsetHeight(element) + getBorderHeight(element);
	return height;
}

int CSLayoutManagerBorder::getMinimumWidth(CSGrafikElement *element)
{
	if (element == 0)
	{
		return 0;
	}
	if (!element->getVisible())
	{
		return 0;
	}

	CSLayoutData *layoutData = getLayoutData(element);

	if (layoutData->isFixSized())
	{
		return element->getWidth();
	}
	
	int width = getMinWidth(element);
	CSGrafikElements *elements = getElements(element);
	CSGrafikElements::iterator iter = elements->begin();

	CSGrafikElement *north = 0;
	CSGrafikElement *south = 0;
	CSGrafikElement *east = 0;
	CSGrafikElement *west = 0;
	CSGrafikElement *center = 0;

	while (iter != elements->end())
	{
		CSGrafikElement *visitor = *iter;
		if (!visitor->getVisible())
		{
			iter++;
			continue;
		}
		CSLayoutData *visitorLayoutData = getLayoutData(visitor);
		switch (visitorLayoutData->getPosition())
		{
			case POSITION_NORTH:
			{
				north = visitor;
				break;
			}
			case POSITION_SOUTH:
			{
				south = visitor;
				break;
			}
			case POSITION_EAST:
			{
				east = visitor;
				break;
			}
			case POSITION_WEST:
			{
				west = visitor;
				break;
			}
			case POSITION_CENTER:
			{
				center = visitor;
				break;
			}
			default:
			{
				break;
			}
		}
		iter++;
	}

	int verticalSpacing;
	int horizontalSpacing;

	layoutData->getSpacing(verticalSpacing, horizontalSpacing);
	
	int neededWidth = 0;
	int centralWidth = 0;
	int maxWidth = 0;
	int northWidth = 0;
	int southWidth = 0;

	if ((west) && (center) && (east))
	{
		centralWidth += horizontalSpacing;
		centralWidth += horizontalSpacing;
	}
	else if ((west) && (center))
	{
		centralWidth += horizontalSpacing;
	}
	else if ((center) && (east))
	{
		centralWidth += horizontalSpacing;
	}
	else if ((west) && (east))
	{
		centralWidth += horizontalSpacing;
	}
	if (west)
	{
		centralWidth += west->getLayoutManager()->getMinimumWidth(west);
	}
	if (center)
	{
		centralWidth += center->getLayoutManager()->getMinimumWidth(center);
	}
	if (east)
	{
		centralWidth += east->getLayoutManager()->getMinimumWidth(east);
	}

	if (north)
	{
		northWidth += north->getLayoutManager()->getMinimumWidth(north);
	}
	if (south)
	{
		southWidth += south->getLayoutManager()->getMinimumWidth(south);
	}

	maxWidth = centralWidth;
	if (northWidth >maxWidth)
	{
		maxWidth = northWidth;
	}
	if (southWidth >maxWidth)
	{
		maxWidth = southWidth;
	}

	if (maxWidth > width)
	{
		width = maxWidth;
	}

	width += getInsetWidth(element) + getBorderWidth(element);
	return width;
}
