#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

#include "CSLayoutManager.h"
#include "CSGrafikElement.h"
#include "CSBorder.h"

const char *CSLayoutManagerStackHorizontal::CLASS = "CSLayoutManagerStackHorizontal";

/**
	Only layout elements, which have position: POSITION_ORDER_LEFT, or POSITION_ORDER_RIGHT
	Only vertical Center / stretch allowed! (is must)

    Elements stacked from East:
    E1 E2 E3
    Elements stacked from West:
    W3 W2 W1
 
    together:
    E1 E2 E3 W3 W2 W1

    addElementWest(W4):
    E1 E2 E3 W4 W3 W2 W1

    addElementEast(E4):
    E1 E2 E3 E4 W4 W3 W2 W1


*/
void CSLayoutManagerStackHorizontal::layoutElements()
{
	CSGrafikElement *host = getHostingElement();
	if (host == 0) 
	{
		return;
	}
	CSGrafikElements *elements = getElements(host);

	int verticalSpacing;
	int horizontalSpacing;
	CSLayoutData *layoutData = getLayoutData(host);
	layoutData->getSpacing(verticalSpacing, horizontalSpacing);
	int width;
	int height;
	width = getViewportWidth(host);
	height = getViewportHeight(host);

	int x;
	int y = 0;
	int w, h;
	int stretchCounter = 0;
	int nonStretchSize = 0;

	int packCounter = 0;
	int packSize = 0;

	int verticalMinimum = 0;
	int horizontalMinimum = 0;
	int horizontalSpacingCollect = 0;
	int count = 0;
	CSGrafikElements::iterator iter = elements->begin();
	while (iter != elements->end())
	{
		CSGrafikElement *visitor = *iter;
		if (!visitor->getVisible())
		{
			iter++;
			continue;
		}
		CSLayoutData *visitorLayoutData = getLayoutData(visitor);
		if ((visitorLayoutData->getPosition() != POSITION_ORDER_LEFT) && (visitorLayoutData->getPosition() != POSITION_ORDER_RIGHT))
		{
			iter++;
			continue;
		}

		w = visitor->getWidth();
		if (visitorLayoutData->getStretchedHorizontal())
		{
			stretchCounter++;
		}
		else if (visitorLayoutData->getPackedHorizontal())
		{
			packSize += visitor->getLayoutManager()->getMinimumWidth(visitor);
		}
		else
		{
			nonStretchSize += w;
		}

		horizontalMinimum += visitor->getLayoutManager()->getMinimumWidth(visitor);

		if (verticalMinimum < visitor->getLayoutManager()->getMinimumHeight(visitor))
		{
			verticalMinimum = visitor->getLayoutManager()->getMinimumHeight(visitor);
		}
	
		if (count != 0)
		{
			horizontalMinimum += horizontalSpacing;
			horizontalSpacingCollect += horizontalSpacing;
		}
		count++;
		iter++;
	}

	if (height < verticalMinimum)
	{
		height = verticalMinimum;
	}
	if (width < horizontalMinimum)
	{
		width = horizontalMinimum;
	}
	int oneStretchedSize = 0;
	int xMax;
	int xMin;
	xMin = getViewportWidth(host);
	xMax = 0;
	if (stretchCounter)
	{
		oneStretchedSize = (width-horizontalSpacingCollect-nonStretchSize-packSize) / stretchCounter;
	}
	int stretchCounterNew = 0;
	count = 0;
	iter = elements->begin();
	while (iter != elements->end())
	{
		CSGrafikElement *visitor = *iter;
		if (!visitor->getVisible())
		{
			iter++;
			continue;
		}
		CSLayoutData *visitorLayoutData = getLayoutData(visitor);
		if ((visitorLayoutData->getPosition() != POSITION_ORDER_LEFT) && (visitorLayoutData->getPosition() != POSITION_ORDER_RIGHT))
		{
			iter++;
			continue;
		}
		y = 0;
		w = visitor->getWidth();
		h = visitor->getHeight();
		if (visitorLayoutData->getStretchedHorizontal())
		{
			w = oneStretchedSize;
			stretchCounterNew++;
			
			// align to stretchcounter divide
			if (stretchCounterNew < (width-horizontalSpacingCollect-nonStretchSize-packSize) - (oneStretchedSize * stretchCounter))
			{
				w++;
			}
		}
		else if (visitorLayoutData->getPackedHorizontal())
		{
			w = visitor->getLayoutManager()->getMinimumWidth(visitor);
		}
		else 
		{
			// minimum even if not packed!
			int w2 = visitor->getLayoutManager()->getMinimumWidth(visitor);
			if (w2 > w)
			{
				w = w2;
			}
		}

		if (visitorLayoutData->getStretchedVertical())
		{
			h = height;
		}
		else if (visitorLayoutData->getPackedVertical())
		{
			h = visitor->getLayoutManager()->getMinimumHeight(visitor);
		}
		else 
		{
			// minimum even if not packed!
			int h2 = visitor->getLayoutManager()->getMinimumHeight(visitor);
			if (h2 > h)
			{
				h = h2;
			}
		}
		if (visitorLayoutData->getCenteredVertical())
		{
			y = height/2 - h/2;
		}
		if (visitorLayoutData->getPosition() == POSITION_ORDER_LEFT)
		{
			if (count != 0)
			{
				xMax += horizontalSpacing;
			}
			x = xMax;
			xMax += w;
		}
		else if (visitorLayoutData->getPosition() == POSITION_ORDER_RIGHT)
		{
			xMin -= w;
			x = xMin;
			if (count != 0)
			{
				xMin -= horizontalSpacing;
			}
		}
		setLayout(visitor, x, y, w, h);
		count++;
		iter++;
	}
}

int CSLayoutManagerStackHorizontal::getMinimumHeight(CSGrafikElement *element)
{
	if (element == 0)
	{
		return 0;
	}
	if (!element->getVisible())
	{
		return 0;
	}
	CSLayoutData *layoutData = getLayoutData(element);

	if (layoutData->isFixSized())
	{
		return element->getHeight();
	}

	int height = getMinHeight(element);
	CSGrafikElements *elements = getElements(element);

	if (!elements == 0)
	{
		CSGrafikElements::iterator iter = elements->begin();
		while (iter != elements->end())
		{
			int neededHeight;
			CSGrafikElement *visitor = *iter;
			neededHeight = visitor->getLayoutManager()->getMinimumHeight(visitor);

			if (height < neededHeight)
			{
				height = neededHeight;
			}
			iter++;
		}
	}
	height += getInsetHeight(element) + getBorderHeight(element);
	return height;
}


int CSLayoutManagerStackHorizontal::getMinimumWidth(CSGrafikElement *element)
{
	if (element == 0)
	{
		return 0;
	}
	if (!element->getVisible())
	{
		return 0;
	}
	CSLayoutData *layoutData = getLayoutData(element);

	if (layoutData->isFixSized())
	{
		return element->getWidth();
	}
	
	int width = getMinWidth(element);
	CSGrafikElements *elements = getElements(element);

	int verticalSpacing;
	int horizontalSpacing;

	layoutData->getSpacing(verticalSpacing, horizontalSpacing);

	if (!elements == 0)
	{
		int neededWidth = 0;
		int count = 0;
		CSGrafikElements::iterator iter = elements->begin();
		while (iter != elements->end())
		{
			CSGrafikElement *visitor = *iter;
			if (!visitor->getVisible())
			{
				iter++;
				continue;
			}
			CSLayoutData *visitorLayoutData = getLayoutData(visitor);

			int visitorWidth = visitor->getLayoutManager()->getMinimumWidth(visitor);
			if (count != 0)
			{
				visitorWidth += horizontalSpacing; 
			}

			neededWidth += visitorWidth;
			iter++;
			count++;
		}
		if (neededWidth > width)
		{
			width = neededWidth;
		}
	}
	width += getInsetWidth(element) + getBorderWidth(element);
	return width;
}

