#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

#include "CSLayoutManager.h"
#include "CSGrafikElement.h"
#include "CSBorder.h"

const char *CSLayoutManagerStackVertical::CLASS = "CSLayoutManagerStackVertical";

/**
	Only layout elements, which have position: POSITION_ORDER_TOP, or POSITION_ORDER_BOTTOM
	Only horizontal Center / 
	horizontal/vertical stretch allowed!
	// Center Vertical NOT respected!
*/
void CSLayoutManagerStackVertical::layoutElements()
{
	CSGrafikElement *host = getHostingElement();
	if (host == 0) 
	{
		return;
	}
	CSGrafikElements *elements = getElements(host);

	int verticalSpacing;
	int horizontalSpacing;

	CSLayoutData *layoutData = getLayoutData(host);
	layoutData->getSpacing(verticalSpacing, horizontalSpacing);
	int width;
	int height;
	width = getViewportWidth(host);
	height = getViewportHeight(host);

	int x = 0;
	int y;
	int w, h;
	int stretchCounter = 0;
	int nonStretchSize = 0;

	int packCounter = 0;
	int packSize = 0;

	int verticalMinimum = 0;
	int horizontalMinimum = 0;
	int verticalSpacingCollect = 0;
	int count = 0;
	CSGrafikElements::iterator iter = elements->begin();
	while (iter != elements->end())
	{
		CSGrafikElement *visitor = *iter;
		if (!visitor->getVisible())
		{
			iter++;
			continue;
		}
		CSLayoutData *visitorLayoutData = getLayoutData(visitor);
		if ((visitorLayoutData->getPosition() != POSITION_ORDER_TOP) && (visitorLayoutData->getPosition() != POSITION_ORDER_BOTTOM))
		{
			iter++;
			continue;
		}
		
		h = visitor->getHeight();
		if (visitorLayoutData->getStretchedVertical())
		{
//			if (isNextUnstretchedParentVerticalFixed(visitor) == true)
//			{
				stretchCounter++;
//			}
//			else
//			{
//				packSize += visitor->getLayoutManager()->getMinimumHeight(visitor);
//			}
		}
		else if (visitorLayoutData->getPackedVertical())
		{
			packSize += visitor->getLayoutManager()->getMinimumHeight(visitor);
		}
		else
		{
			nonStretchSize += h;
		}

		verticalMinimum += visitor->getLayoutManager()->getMinimumHeight(visitor);
		if (horizontalMinimum < visitor->getLayoutManager()->getMinimumWidth(visitor))
		{
			horizontalMinimum = visitor->getLayoutManager()->getMinimumWidth(visitor);
		}
		if (count != 0)
		{
			verticalMinimum += verticalSpacing;
			verticalSpacingCollect += verticalSpacing;
		}
		count++;
		iter++;
	}

	if (height < verticalMinimum)
	{
		height = verticalMinimum;
	}
	if (width < horizontalMinimum)
	{
		width = horizontalMinimum;
	}
	int oneStretchedSize = 0;
	int yMax;
	int yMin;
	yMin = getViewportHeight(host);
	yMax = 0;
	if (stretchCounter)
	{
		oneStretchedSize = ((height-verticalSpacingCollect)-nonStretchSize-packSize) / stretchCounter;
	}
	int stretchCounterNew = 0;
	count = 0;
	iter = elements->begin();
	while (iter != elements->end())
	{
		CSGrafikElement *visitor = *iter;
		if (!visitor->getVisible())
		{
			iter++;
			continue;
		}
		CSLayoutData *visitorLayoutData = getLayoutData(visitor);
		if ((visitorLayoutData->getPosition() != POSITION_ORDER_TOP) && (visitorLayoutData->getPosition() != POSITION_ORDER_BOTTOM))
		{
			iter++;
			continue;
		}

		x = 0;
		w = visitor->getWidth();
		h = visitor->getHeight();
		if (visitorLayoutData->getStretchedVertical())
		{
//			if (isNextUnstretchedParentVerticalFixed(visitor) == true)
//			{
				h = oneStretchedSize;
				stretchCounterNew++;
				if (stretchCounterNew < (height- verticalSpacingCollect-nonStretchSize-packSize) - oneStretchedSize * stretchCounter)
				{
					h++;
				}
//			}
//			else
//			{
//				h = visitor->getLayoutManager()->getMinimumHeight(visitor);
//			}
		}
		else if (visitorLayoutData->getPackedVertical())
		{
			h = visitor->getLayoutManager()->getMinimumHeight(visitor);
		}
		else
		{
			int h2 = visitor->getLayoutManager()->getMinimumHeight(visitor);
			if (h < h2)
			{
				h = h2;
			}
		}
		
		if (visitorLayoutData->getStretchedHorizontal())
		{
			w = width;
		}
		else if (visitorLayoutData->getPackedHorizontal())
		{
			w = visitor->getLayoutManager()->getMinimumWidth(visitor);
		}
		else 
		{
			int w2 = visitor->getLayoutManager()->getMinimumWidth(visitor);
			if (w < w2)
			{
				w = w2;
			}
		}
		if (visitorLayoutData->getCenteredHorizontal())
		{
			x = width/2 - w/2;
		}
		if (visitorLayoutData->getPosition() == POSITION_ORDER_TOP)
		{
			if (count)
			{
				yMax += verticalSpacing;
			}
			y = yMax;
			yMax += h;
		}
		else if (visitorLayoutData->getPosition() == POSITION_ORDER_BOTTOM)
		{
			yMin -= h;
			y = yMin;
			if (count)
			{
				yMin -= verticalSpacing;
			}
		}
		setLayout(visitor, x, y, w, h);
		count++;
		iter++;
	}
}

int CSLayoutManagerStackVertical::getMinimumHeight(CSGrafikElement *element)
{
	if (element == 0)
	{
		return 0;
	}
	if (!element->getVisible())
	{
		return 0;
	}

	CSLayoutData *layoutData = getLayoutData(element);

	if (layoutData->isFixSized())
	{
		return element->getHeight();
	}
	
	int height = getMinHeight(element);
	CSGrafikElements *elements = getElements(element);

	int verticalSpacing;
	int horizontalSpacing;

	layoutData->getSpacing(verticalSpacing, horizontalSpacing);

	if (!elements == 0)
	{
		int neededHeight = 0;
		int count = 0;
		CSGrafikElements::iterator iter = elements->begin();
		while (iter != elements->end())
		{
			CSGrafikElement *visitor = *iter;
			if (!visitor->getVisible())
			{
				iter++;
				continue;
			}
			CSLayoutData *visitorLayoutData = getLayoutData(visitor);
			if ((visitorLayoutData->getPosition() != POSITION_ORDER_TOP) && (visitorLayoutData->getPosition() != POSITION_ORDER_BOTTOM))
			{
				iter++;
				continue;
			}

			int visitorHeight = visitor->getLayoutManager()->getMinimumHeight(visitor);
			if (count != 0)
			{
				visitorHeight += verticalSpacing; 
			}

			neededHeight += visitorHeight;
			iter++;
			count++;
		}
		if (neededHeight > height)
		{
			height = neededHeight;
		}
	}

	height += getInsetHeight(element) + getBorderHeight(element);
	return height;
}

int CSLayoutManagerStackVertical::getMinimumWidth(CSGrafikElement *element)
{
	if (element == 0)
	{
		return 0;
	}
	if (!element->getVisible())
	{
		return 0;
	}
	CSLayoutData *layoutData = getLayoutData(element);

	if (layoutData->isFixSized())
	{
		return element->getWidth();
	}

	int width = getMinWidth(element);
	CSGrafikElements *elements = getElements(element);

	int verticalSpacing;
	int horizontalSpacing;

	layoutData->getSpacing(verticalSpacing, horizontalSpacing);

	if (!elements == 0)
	{
		int neededWidth = 0;
		CSGrafikElements::iterator iter = elements->begin();
		while (iter != elements->end())
		{
			CSGrafikElement *visitor = *iter;
			if (!visitor->getVisible())
			{
				iter++;
				continue;
			}

			CSLayoutData *visitorLayoutData = getLayoutData(visitor);
			if ((visitorLayoutData->getPosition() != POSITION_ORDER_TOP) && (visitorLayoutData->getPosition() != POSITION_ORDER_BOTTOM))
			{
				iter++;
				continue;
			}
			int visitorWidth = visitor->getLayoutManager()->getMinimumWidth(visitor);

			if (visitorWidth > neededWidth)
			{
				neededWidth = visitorWidth;
			}
			iter++;
		}
		if (neededWidth > width)
		{
			width = neededWidth;
		}
	}

	width += getInsetWidth(element) + getBorderWidth(element);
	return width;
}
