#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

#include "CSLayoutManager.h"
#include "CSGrafikElement.h"
#include "CSBorder.h"

const char *CSLayoutManagerXY::CLASS = "CSLayoutManagerXY";

/**
	cannot be stretched or centered!
	horizontal/vertical spacing don't make sense with XY Layout -> settings are ignored!
*/
void CSLayoutManagerXY::layoutElements()
{
	CSGrafikElement *host = getHostingElement();
	if (host == 0) 
	{
		return;
	}
	
	CSLayoutData *layoutData = getLayoutData(host);
	
	CSGrafikElements *elements = getElements(host);
	if (elements != 0)
	{
		CSGrafikElements::iterator iter = elements->begin();
		while (iter != elements->end())
		{
			CSGrafikElement *visitor = *iter;
			if (!visitor->getVisible())
			{
				iter++;
				continue;
			}
			CSLayoutData *visitorLayoutData = getLayoutData(visitor);
			int x = visitorLayoutData->getX();
			int y = visitorLayoutData->getY();
			int w = -1;
			int h = -1;
			
			if (visitorLayoutData->getPackedVertical())
			{
				h = visitor->getLayoutManager()->getMinimumHeight(visitor);
			}
			if (visitorLayoutData->getPackedHorizontal())
			{
				w = visitor->getLayoutManager()->getMinimumWidth(visitor);
			}
			setLayout(visitor, x , y, w, h);
			iter++;
		}
	}
}

int CSLayoutManagerXY::getMinimumHeight(CSGrafikElement *element)
{
	if (element == 0)
	{
		return 0;
	}
	if (!element->getVisible())
	{
		return 0;
	}
	int height = getMinHeight(element);
	CSLayoutData *layoutData = getLayoutData(element);
	if (layoutData->isFixSized())
	{
		return element->getHeight();
	}
	CSGrafikElements *elements = getElements(element);


	if (elements != 0)
	{
		CSGrafikElements::iterator iter = elements->begin();
		while (iter != elements->end())
		{
			CSGrafikElement *visitor = *iter;
			if (!visitor->getVisible())
			{
				iter++;
				continue;
			}
			CSLayoutData visitorLayoutData = visitor->getLayoutData();
			int h = visitor->getLayoutManager()->getMinimumHeight(visitor);
			if (!mComponentSizeOnly)
			{
				h += visitorLayoutData.getY();
			}
			
			
			if ( h > height)
			{
				height = h;
			}
			iter++;
		}
	}
	height += getInsetHeight(element) + getBorderHeight(element);
	return height;
}
#include "CSScrollbar.h"

int CSLayoutManagerXY::getMinimumWidth(CSGrafikElement *element)
{
	if (element == 0)
	{
		return 0;
	}
	if (element->getType().compare(CSScrollbar::CLASS) == 0)
	{
		int wissdth = getMinWidth(element);
	}
	if (!element->getVisible())
	{
		return 0;
	}
	int width = getMinWidth(element);

	CSLayoutData *layoutData = getLayoutData(element);
	if (layoutData->isFixSized())
	{
		return element->getWidth();
	}

	CSGrafikElements *elements = getElements(element);

	if (elements != 0)
	{
		CSGrafikElements::iterator iter = elements->begin();
		while (iter != elements->end())
		{
			CSGrafikElement *visitor = *iter;
			if (!visitor->getVisible())
			{
				iter++;
				continue;
			}
			CSLayoutData visitorLayoutData = visitor->getLayoutData();
			int w = visitor->getLayoutManager()->getMinimumWidth(visitor);
			if (!mComponentSizeOnly)
			{
				w += visitorLayoutData.getX();
			}
			if ( w > width)
			{
				width = w;
			}
			iter++;
		}
	}
	width += getInsetWidth(element) + getBorderWidth(element);
	return width;
}


