#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

#include "CSListbox.h"
#include "CSIcon.h"
#include "CSLAF.h"
#include "CSDesktop.h"
#include "CSMessage.h"
#include "CSFont.h"
#include "CSBorder.h"

const char *CSListbox::CLASS = "CSListbox";

CSListbox::CSListbox(int numberOfElementsDisplayed, int width) : CSGrafikElement(numberOfElementsDisplayed*16, width) // * 16 is dummy, size is calculated in changeDisplayLayout()
{
	static char *functionName="CSListbox";
	LOG_ENTER 
	mNumberOfElementsDisplayed = numberOfElementsDisplayed;
	initListbox();
	LOG_EXIT
}

CSListbox::~CSListbox()
{
	static char *functionName="~CSListbox";
	LOG_ENTER 
	if (mScrollbar != 0) 
	{
		removeElement(mScrollbar);
		delete(mScrollbar);
		mScrollbar = 0;
	}
	LOG_EXIT
}

void CSListbox::initListbox()
{
	static char *functionName="initListbox";
	LOG_ENTER 
	MESSAGE_LIST_ITEM_SELECTED.setSubtype(LIST_ITEM_SELECTED_MESSAGE);
	setLayoutManager(new CSLayoutManagerStackHorizontal());
	mSelectedItem = -1; // not item selected
	mStart = 0;	// first item to be displayed
	mEnd = 0;
	mDisplayLength = 0;
	mScrollbar = 0;
	mHoverSelection = false;
	mSelectionBorder = 0;
	
	rebuildElement();
	layoutSetup();
	LOG_EXIT
}

void CSListbox::setItemsShown(int numberOfElementsDisplayed)
{
	static char *functionName="setItemsShown";
	LOG_ENTER 
	mNumberOfElementsDisplayed = numberOfElementsDisplayed;
	layoutChanged();
	LOG_EXIT
}

void CSListbox::reactOnMessage(CSMessage *message)
{
	reactOnMessageListbox(message);
}

void CSListbox::reactOnMessageListbox(CSMessage *message)
{
	static char *functionName="reactOnMessage";
	if (message->mIsHandled)
	{
		reactOnMessageGrafikElement(message);
		return;
	}
	if (!getEnabled()) return;
	switch (message->getType())
	{
		case GUI_MESSAGE:
		{
			GuiMessage *gm = (GuiMessage *) message;
			if (gm->receiver == this)
			{
				if (gm->getSubtype() == MOUSE_BUTTON_RELEASED_MESSAGE)
				{
					if (mHoverSelection)
					{
						MESSAGE_LIST_ITEM_SELECTED.selected = mSelectedItem;
						MESSAGE_LIST_ITEM_SELECTED.mIsHandled = false;
						sendMessage(MESSAGE_LIST_ITEM_SELECTED);
					}
					gm->mIsHandled = true;
				}
			}
			if (gm->receiver != this)
			{
				if (gm->getSubtype() == SCROLLBAR_POSITION_CHANGED_MESSAGE)
				{
					mStart = gm->start;
					mEnd = gm->end;
					gm->mIsHandled = true;
				}
			}
			else // receiver == this
			{
				switch (message->getSubtype())
				{
					case MOUSE_MOTION_MESSAGE:
					{
						if (mHoverSelection)
						{
							int x,y;
							x = gm->receiverX;
							y = gm->receiverY;
							setGrafikCursor(x,y);
						}
						gm->mIsHandled = true;
						break;
					}

					case MOUSE_BUTTON_PRESSED_MESSAGE:
					{
						if (!mHoverSelection)
						{
							int x,y;
							x = gm->receiverX;
							y = gm->receiverY;
							setGrafikCursor(x,y);
						}
						gm->mIsHandled = true;
						break;
					}
				}
			}
		}
	}
	reactOnMessageGrafikElement(message);
}

// Coordinates in relation to listbox (elementArea)
void CSListbox::setGrafikCursor(int x, int y)
{
	static char *functionName="CSListbox::setGrafikCursor(int x, int y)";

	int offsetX = getBorder()->getSizeWest() + mListInset.getSizeWest();
	int offsetY = getBorder()->getSizeNorth() + mListInset.getSizeNorth();
	int yAdd = getFont()->getHeight() + getVerticalElementSpacing();
	x -= offsetX;
	y -= offsetY;
	mSelectedItem = y/yAdd;
	mSelectedItem += mStart;
	if (!mHoverSelection)
	{
		MESSAGE_LIST_ITEM_SELECTED.selected = mSelectedItem;
		MESSAGE_LIST_ITEM_SELECTED.mIsHandled = false;
		sendMessage(MESSAGE_LIST_ITEM_SELECTED);
	}
}

void CSListbox::paint(SDL_Surface *destination, SDL_Rect *parentViewport)
{
	static char *functionName="CSListbox::paint(SDL_Surface *destination, SDL_Rect *parentViewport)";
	SDL_Rect rectSave;
	SDL_GetClipRect(destination, &rectSave);

	SDL_Rect rectClip = getViewportArea();
	rectClip.x += parentViewport->x + getX();
	rectClip.y += parentViewport->y + getY();

	SDLMain::adjustClipping(rectClip, rectSave);
	
	SDL_SetClipRect(destination, &rectClip);

	SDL_Rect viewArea = getViewportArea();
	SDL_Rect elementArea = getElementArea();
	SDL_Rect printArea = viewArea;
	printArea.x += parentViewport->x + elementArea.x;
	printArea.y += parentViewport->y + elementArea.y;

	int x = mListInset.getSizeWest();
	int y = mListInset.getSizeNorth();

	int yAdd = getFont()->getHeight() + getVerticalElementSpacing();

	for (int i = mStart; ((i<mEnd) && (i<mItems.getSize())); i++)
	{
		if (mItems.getString(i).size())
		{
			std::string t = mItems.getString(i);
	 
				putString(destination, &printArea, x, y, getTextColor(), t);

//	listboxColorEnabled = 0xffffff;
//	listboxColorDisabled = 0xa0a0a0;


			if (i == mSelectedItem)
			{
				SDL_Rect selectedItemArea = printArea;
				selectedItemArea.x;
				selectedItemArea.y += y - mListInset.getSizeNorth();
				
				selectedItemArea.h = getFont()->getHeight() + getVerticalElementSpacing();
				if (mScrollbar->getVisible())
				{
					selectedItemArea.w -= mScrollbar->getWidth();
				}
				mSelectionBorder->paint(destination, &selectedItemArea);
				mSelectionBorder->setState(BORDER_STATE_VISIBLE);
			}
		}
		y += yAdd;
	}

	SDL_SetClipRect(destination, &rectSave);
	paintChildren(destination, parentViewport);
}

void CSListbox::setSelected(int i)
{
	mSelectedItem = i;
}

void CSListbox::rebuildElement()
{
	static char *functionName="rebuildElement";
	LOG_ENTER 
	CSLAF *laf = CSLAF::getCurrentLAF();
	if (mSelectionBorder != 0)
	{
		delete mSelectionBorder;
		mSelectionBorder = 0;
	}
	mSelectionBorder = CSBorder::getBorder(BORDER_TYPE_SELECTED);

	if (mScrollbar != 0)
	{
		mScrollbar->removeMessageListener(this);
		removeElement(mScrollbar);
		delete (mScrollbar);
		mScrollbar = 0;
	}
		
	mScrollbar = new CSScrollbar(TYPE_VERTICAL);
	mScrollbar->addMessageListener(this, GUI_MESSAGE);

	mScrollbar->setFocusable(getFocusable());
	addElement(mScrollbar, POSITION_EAST);
	LOG_EXIT
}

void CSListbox::layoutSetupListbox()
{
	static char *functionName="layoutSetupListbox";
	LOG_ENTER 
	CSLAF *laf = CSLAF::getCurrentLAF();
	mListInset = CSInset(2);

	mHeight =	  mNumberOfElementsDisplayed * (getFont()->getHeight()) 
				+ (mNumberOfElementsDisplayed-1) * (getVerticalElementSpacing()) 
				+ getBorder()->getTotalHeight()  
				+ getInset().getTotalHeight() 
				+ mListInset.getTotalHeight();

	mMinHeight = mHeight - getBorder()->getTotalHeight() - getInset().getTotalHeight();
	mMinWidth = mWidth - getBorder()->getTotalWidth() - getInset().getTotalWidth(); //+ getHorizontalElementSpacing()?

	int yAdd = getFont()->getHeight() + getVerticalElementSpacing();
	mDisplayLength = (mMinHeight) / yAdd;
	
	mEnd = mStart + mDisplayLength;
	mScrollbar->setDisplayStart(mStart);
	mScrollbar->setDisplayEnd(mEnd);
	mScrollbar->setDisplayLength(mDisplayLength);
	mScrollbar->setStart(0);
	mScrollbar->setEnd(mItems.getSize());

	if (mDisplayLength < mItems.getSize())
	{
		mScrollbar->setVisible(true);
	}
	else
	{
		mScrollbar->setVisible(false);
	}
	LOG_EXIT
}
