#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

#include "CSMenu.h"
#include "CSPanel.h"
#include "CSIcon.h"
#include "CSLAF.h"
#include "CSDesktop.h"
#include "CSMenuItem.h"
#include "CSMenuBar.h"
#include "CSMessage.h"
#include "CSFont.h"
#include "CSLayoutManager.h"

const char *CSMenu::CLASS = "CSMenu";

CSMenu::CSMenu(const std::string &menuTitle) : CSButton(CSTextIcon::buildIcon(menuTitle))
{
	static char *functionName="CSMenu";
	LOG_ENTER 
	mTitle = menuTitle;
	initMenu();
	LOG_EXIT
}

CSMenu::~CSMenu()
{
	static char *functionName="~CSMenu";
	if (mMenuPanel != 0)
	{
		delete (mMenuPanel);
		mMenuPanel = 0;
	}
}

void CSMenu::initMenu()
{
	mMenuBar = 0;
	mMenuIsOpen = false;
	getLayoutDataInternal()->setPosition(POSITION_WEST);
	mMenuPanel = new CSPanel(0, 0);
	mMenuPanel->setVisible(true);
	mMenuPanel->setLayoutManager(new CSLayoutManagerStackVertical());
	CSLayoutData layout = mMenuPanel->getLayoutData();
	layout.setPackedHorizontal(true);
	layout.setPackedVertical(true);
	mMenuPanel->setLayoutData(layout);
	mMenuPanel->setGUIType("MenuWindow");
	this->setFocusable(false);
	mBorderState = 0;
}

void CSMenu::reactOnMessage(CSMessage *message)
{
	reactOnMessageMenu(message);
}

void CSMenu::reactOnMessageMenu(CSMessage *message)
{
	static char *functionName="reactOnMessage";
	if (message->mIsHandled)
	{
		reactOnMessageButton(message);
		return;
	}
	if (message->getType() == GUI_MESSAGE)
	{
		GuiMessage *gm = (GuiMessage *) message;
		if (isChild(gm->receiver))
		{
			switch (message->getSubtype())
			{
				case MOUSE_MOTION_LOST_MESSAGE:
				{
					mBorderState = mBorderState & (~BUTTON_VISIBLE);
					gm->mIsHandled = true;
					break;
				}
				case MOUSE_MOTION_MESSAGE:
				{
					mBorderState = mBorderState | BUTTON_VISIBLE;
					mBorderState = mBorderState | BUTTON_HOVERING;
					if (getMenuBar() != 0)
					{
						if ((getMenuBar()->getMenuState(this) & BUTTON_PRESSED) == BUTTON_PRESSED)
						{
							if (getMenuBar()->getActiveMenu() != this)
							{
								mBorderState = mBorderState | BUTTON_PRESSED;
								getMenuBar()->setActiveMenu(this);
								getMenuBar()->closeLastMenu();
								openMenu();
							}
						}
					}
					gm->mIsHandled = true;
					break;
				}
				case MOUSE_BUTTON_PRESSED_MESSAGE:
				{
					mBorderState = mBorderState | BUTTON_PRESSED;
					if (getMenuBar() != 0)
					{
						getMenuBar()->setMenuState(mBorderState);
						getMenuBar()->setActiveMenu(this);
						openMenu();
					}
					gm->mIsHandled = true;
					break;
				}
			}
		}
		else // receiver is not a child
		{
			switch (message->getSubtype())
			{
				case FOCUS_LOST_MESSAGE:
				{
					if (getMenuBar() != 0)
					{
						CSGrafikElement *lastActiveMenu = 0;
						// check whether we just switched menus in the same menuBar
						// if the lastActiveMenu was ourselfs AND the component that lost
						// the focus was we ourself too -> than there was just a switch
						// in menus (if another outsider component had received the focus
						// we would still be the active menu!)
						if (getMenuBar()->getLastActiveMenu())
						{
							lastActiveMenu = getMenuBar()->getLastActiveMenu()->mMenuPanel;
						}
						CSGrafikElement *lastFocusedComponent =  gm->receiver;
						if (lastActiveMenu != lastFocusedComponent)
						{
							getMenuBar()->resetMenu();
							mBorderState = mBorderState & (~BUTTON_VISIBLE);
							mBorderState = mBorderState & (~(BUTTON_PRESSED | BUTTON_HOVERING));
						}

						getMenuBar()->setMenuState(mBorderState);
						getMenuBar()->setActiveMenu(0);
					}
					closeMenu();

					gm->mIsHandled = true;
				}
			}
		}
	}
}

int CSMenu::getButtonState() 
{
	if (getMenuBar() != 0)
	{
		return getMenuBar()->getMenuState(this);
	}
	return mBorderState;
}

// MenuItems added are freed by CSMenu
void CSMenu::addMenuItem(CSMenuItem *item)
{
	CSLayoutData layoutData(POSITION_ORDER_TOP);
	layoutData.setStretchedHorizontal(true);
	layoutData.setStretchedVertical(true);
	item->setLayoutData(layoutData);
	mMenuPanel->addElement(item);
}

void CSMenu::removeMenuItem(CSMenuItem *item)
{
	mMenuPanel->removeElement(item);
}

void CSMenu::closeMenu()
{
	CSDesktop *desktop = getDesktop();
	if (desktop == 0)
	{
		return;
	}
	// desktop->removeElement(mMenuPanel);
	desktop->removeFrontElement(mMenuPanel);
	mMenuIsOpen = false;
}

void CSMenu::openMenu()
{
	if (mMenuIsOpen)
	{
		return;
	}
	if (getMenuBar() != 0)
	{
		getMenuBar()->closeLastMenu();
	}
	CSDesktop *desktop = getDesktop();
	if (desktop)
	{
		CSLAF *laf = CSLAF::getCurrentLAF();

		if ((getMenuBar() != 0) && (getMenuBar()->getPosition() == POSITION_SOUTH))
		{
			int x = laf->getMenuOpenXOffset();
			int y = getY() - laf->getMenuOpenYOffset() - mMenuPanel->getHeight();
			translateToDesktop(x, y);
			desktop->addFrontElement(mMenuPanel, x, y);

		}
		else
		{
			int x = laf->getMenuOpenXOffset();
			int y = laf->getMenuOpenYOffset() + getHeight();
			translateToDesktop(x, y);
			desktop->addFrontElement(mMenuPanel, x, y);
		}
		mMenuPanel->getLayoutManager()->pack(mMenuPanel);
		desktop->setFocusedComponent(mMenuPanel);
		mMenuPanel->addMessageListener(this, GUI_MESSAGE);
		mMenuIsOpen = true;
	}
}

void CSMenu::setMenuBar(CSMenuBar *bar) 
{
	mMenuBar=bar;
}	

CSMenuBar *CSMenu::getMenuBar() 
{
	static char *functionName="setParent";
	LOG_ENTER 
	if (mMenuBar != 0)
	{
		return mMenuBar;
	}

	CSGrafikElement *parent = getParent();
	while (parent != 0)
	{
		std::string type = parent->getType();
		if (type.compare(CSMenuBar::CLASS) == 0)
		{
			mMenuBar = (CSMenuBar *) parent;
			mMenuBar->setMenuState(mBorderState);
			break;
		}
		parent = parent->getParent();
	}
	LOG_EXIT
	return mMenuBar;
}

void CSMenu::layoutSetupMenu()
{
	static char *functionName="layoutSetupMenu";
	LOG_ENTER 
	layoutSetupButton();
	LOG_EXIT
}
