#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

#include "CSScrollbar.h"
#include "CSFont.h"
#include "CSLAF.h"

const char *CSScrollbar::CLASS = "CSScrollbar";

CSScrollbar::CSScrollbar(int kind) : CSGrafikElement(0, 0)
{
	static char *functionName="CSScrollbar";
	LOG_ENTER 
	mKind = kind;
	mStart = 0;
	mEnd = 0;
	mDisplayStart = 0;
	mDisplayEnd = 0;
	mDisplayLength = 0;
	CSLayoutManagerXY *layoutManager = new CSLayoutManagerXY();
	layoutManager->setComponentSizeOnly(true);
	layoutManager->setPropagateSizeChangesOnly(true);
	setLayoutManager(layoutManager);
	if (mKind == TYPE_HORIZONTAL)
	{
		CSLayoutData layoutData = CSLayoutData(POSITION_SOUTH);
		layoutData.setStretchedHorizontal(true);
		setLayoutData(layoutData);
	}
	else if (mKind == TYPE_VERTICAL)
	{
		CSLayoutData layoutData = CSLayoutData(POSITION_EAST);
		layoutData.setStretchedVertical(true);
		setLayoutData(layoutData);
	}

	mLess = 0;
	mMore = 0;
	mHandle = 0;

	MESSAGE_SCROLLBAR_CHANGED.setSubtype(SCROLLBAR_POSITION_CHANGED_MESSAGE);
	rebuildElement();
	layoutSetup();
	LOG_EXIT
}

void CSScrollbar::positionChanged()
{
	static char *functionName="positionChanged";
	MESSAGE_SCROLLBAR_CHANGED.sender = this;
	MESSAGE_SCROLLBAR_CHANGED.start = mDisplayStart;
	MESSAGE_SCROLLBAR_CHANGED.end = mDisplayEnd;
	MESSAGE_SCROLLBAR_CHANGED.mIsHandled = false;
	sendMessage(MESSAGE_SCROLLBAR_CHANGED);
}

//! \todo change position settings of handles and buttons not in paint!
//! it takes some time (lafChanges()...) to set position!!!
void CSScrollbar::paint(SDL_Surface *destination, SDL_Rect *parentViewport)
{
	static char *functionName="paint";
	CSGrafikElement *parent = getParent();
	if (!parent)
	{
		return;
	}
	int pw = mWidth;
	int ph = mHeight;

	int overallLength = mEnd - mStart;
	int currentLength = mDisplayEnd - mDisplayStart;
	bool mustCallPositioning = false;

	float handleLength = 1;
	if (!overallLength)
	{
		overallLength = mDisplayLength;
	}
	if (overallLength > mDisplayLength)
	{
		handleLength = (float) ((float) (currentLength) / (float) (overallLength));
	}

	if (mKind == TYPE_HORIZONTAL)
	{
		mLess->setPosition(0, 0);
		mMore->setPosition(mWidth - mLess->getWidth(), 0);
		mHandle->setHeight(mLess->getHeight());

		int w = (mWidth - 2*mLess->getWidth())*handleLength;
		if (w < 3) w = 3; // at least three Pixel Size!
		if (w < 4) w = 4; // at least three Pixel Size!
		if (mHandle->getWidth() != w)
		{
			mustCallPositioning = true;
		}
		mHandle->setWidth( w );
		if (mDisplayEnd == mEnd)
		{
			mHandle->setPosition(pw-mLess->getWidth()-mHandle->getWidth(), 0);
		}
		else
		{
			int handleSlideWay = mWidth - mLess->getWidth() - mMore->getWidth();
			float s = ((float)(handleSlideWay) / (float) overallLength);
			if (!s)
			{
				s=1;
			}
			mHandle->setPosition(mLess->getWidth() + ((float)((float)s) * ((float)mDisplayStart)), 0);
		}
	}

	if (mKind == TYPE_VERTICAL)
	{
		mLess->setPosition(0, 0);
		mMore->setPosition(0, mHeight - mLess->getHeight());

		mHandle->setWidth(mLess->getWidth());

		int h = (mHeight - 2*mLess->getHeight())*handleLength;
		if (h < 3) h = 3; // at least three Pixel Size!
		if (h < 4) h = 4; // at least three Pixel Size!
		if (mHandle->getHeight() != h)
		{
			mustCallPositioning = true;
		}
		mHandle->setHeight( h );
		if (mDisplayEnd == mEnd)
		{
			mHandle->setPosition(0, ph-mLess->getHeight()-mHandle->getHeight());
		}
		else
		{
			int handleSlideWay = mHeight - mLess->getHeight() - mMore->getHeight();
			float s = ((float)(handleSlideWay) / (float) overallLength);
			if (!s)
			{
				s=1;
			}
			mHandle->setPosition(0, mLess->getHeight() + ((float)((float)s) * ((float)mDisplayStart)));
		}
	}

	paintChildren(destination, parentViewport);

	// this can happen if parent gets resized!
	if (mustCallPositioning)
	{
		positionChanged();
	}
}

void CSScrollbar::reactOnMessage(CSMessage *message)
{
	reactOnMessageScrollbar(message);
}

void CSScrollbar::reactOnMessageScrollbar(CSMessage *message)
{
	static char *functionName="reactOnMessage";
	if (message->mIsHandled)
	{
		reactOnMessageGrafikElement(message);
		return;
	}
	if (message->getType() == GUI_MESSAGE)
	{
		GuiMessage *gm = (GuiMessage *) message;
		if (gm->receiver != this)
		{
			if (gm->getSubtype() == BUTTON_PRESSED_MESSAGE)
			{
				if (gm->actionId == SCROLL_BUTTON_LESS_ACTION_ID)
				{
					if (mDisplayStart > 0)
					{
						mDisplayStart--;
						mDisplayEnd = mDisplayStart+mDisplayLength;
						if (mDisplayEnd>mEnd)
						{
							mDisplayEnd = mEnd;
						}
						positionChanged();
					}
				}

				if (gm->actionId == SCROLL_BUTTON_MORE_ACTION_ID)
				{
					if (mDisplayEnd < mEnd)
					{
						mDisplayEnd++;
						mDisplayStart=mDisplayEnd-mDisplayLength;
						if (mDisplayStart<0)
						{
							mDisplayStart = 0;
						}
						positionChanged();
					}
				}
			}

			if (gm->getSubtype() == BUTTON_DRAGGED_MESSAGE)
			{
				int overallLength = mEnd - mStart;
				if (gm->actionId == SCROLL_BUTTON_DRAG_ACTION_ID)
				{

					if (mKind == TYPE_VERTICAL)
					{
						int handleSlideWay = mHeight - mLess->getHeight() - mMore->getHeight();
						float s = ((float)(handleSlideWay) / (float) overallLength);
						if (!s)
						{
							s=1;
						}
						gm->deltaX = gm->screenX;

						if (gm->deltaY >= s)
						{
							mDisplayEnd+=gm->deltaY/s;
							if (mDisplayEnd>mEnd)
							{
								mDisplayEnd = mEnd;
							}
							mDisplayStart = mDisplayEnd - mDisplayLength;
							if (mDisplayStart<0)
							{
								mDisplayStart = 0;
							}
							gm->deltaY = gm->screenY;
							positionChanged();
						}
						else if (-(gm->deltaY) >= s)
						{
							mDisplayStart+=gm->deltaY/s;
							if (mDisplayStart<0)
							{
								mDisplayStart = 0;
							}

							mDisplayEnd = mDisplayStart + mDisplayLength;
							if (mDisplayEnd>mEnd)
							{
								mDisplayEnd = mEnd;
							}
							gm->deltaY = gm->screenY;
							positionChanged();
						}
						else
						{
							gm->deltaY = gm->screenY - gm->deltaY;
						}
					}
					else if (mKind == TYPE_HORIZONTAL)
					{
						int handleSlideWay = mWidth - mLess->getWidth() - mMore->getWidth();
						float s = ((float)(handleSlideWay) / (float) overallLength);

						gm->deltaY = gm->screenY;
						if (gm->deltaX >= s)
						{
							mDisplayEnd+=gm->deltaX/s;
							if (mDisplayEnd>mEnd)
							{
								mDisplayEnd = mEnd;
							}
							mDisplayStart = mDisplayEnd - mDisplayLength;
							if (mDisplayStart<0)
							{
								mDisplayStart = 0;
							}
							gm->deltaX = gm->screenX;
							positionChanged();
						}
						else if (-(gm->deltaX) >= s)
						{
							mDisplayStart+=gm->deltaX/s;
							if (mDisplayStart<0)
							{
								mDisplayStart = 0;
							}

							mDisplayEnd = mDisplayStart + mDisplayLength;
							if (mDisplayEnd>mEnd)
							{
								mDisplayEnd = mEnd;
							}
							gm->deltaX = gm->screenX;
							positionChanged();
						}
						else
						{
							gm->deltaX = gm->screenX - gm->deltaX;
						}
					}
				}
			}
		}
		else // receiver == this
		{
			switch (message->getSubtype())
			{
				case MOUSE_BUTTON_PRESSED_MESSAGE:
				{
					int x = gm->receiverX;
					int y = gm->receiverY;

					if (mKind == TYPE_VERTICAL)
					{
						if (y <mHandle->getY())
						{
							mDisplayStart -= mDisplayLength;
							mDisplayEnd = mDisplayStart + mDisplayLength;
							
							if (mDisplayStart<0)
							{
								mDisplayStart = 0;
								mDisplayEnd = mDisplayStart + mDisplayLength;
							}
							positionChanged();
						}
						else
						{
							mDisplayStart += mDisplayLength;
							mDisplayEnd = mDisplayStart + mDisplayLength;
							
							if (mDisplayEnd>mEnd)
							{
								mDisplayEnd = mEnd;
								mDisplayStart = mEnd - mDisplayLength;
							}
							positionChanged();
						}
					}
					else if (mKind == TYPE_HORIZONTAL)
					{
						if (x <mHandle->getX())
						{
							mDisplayStart -= mDisplayLength;
							mDisplayEnd = mDisplayStart + mDisplayLength;
							
							if (mDisplayStart<0)
							{
								mDisplayStart = 0;
								mDisplayEnd = mDisplayStart + mDisplayLength;
							}
							positionChanged();
						}
						else
						{
							mDisplayStart += mDisplayLength;
							mDisplayEnd = mDisplayStart + mDisplayLength;
							
							if (mDisplayEnd>mEnd)
							{
								mDisplayEnd = mEnd;
								mDisplayStart = mEnd - mDisplayLength;
							}
							positionChanged();
						}
					}
					gm->mIsHandled = true;
					break;
				}
			}
		}
	}
	reactOnMessageGrafikElement(message);
}

void CSScrollbar::rebuildElement()
{
	static char *functionName="rebuildElement";
	LOG_ENTER 
	CSLAF *laf = CSLAF::getCurrentLAF();

	if (mMore != 0)
	{
		mMore->removeMessageListener(this);
		removeElement(mMore);
		delete (mMore);
		mMore = 0;
	}

	if (mLess != 0)
	{
		mLess->removeMessageListener(this);
		removeElement(mLess);
		delete (mLess);
		mLess = 0;
	}

	if (mHandle != 0)
	{
		mHandle->removeMessageListener(this);
		removeElement(mHandle);
		delete (mHandle);
		mHandle = 0;
	}

	mHandle = new CSDragButton(0);
	if (mKind == TYPE_HORIZONTAL)
	{
		mLess = new CSButton(laf->getIcon(LAF_ICON_TYPE_LEFT));
		mMore = new CSButton(laf->getIcon(LAF_ICON_TYPE_RIGHT));

		CSLayoutData layout = mHandle->getLayoutData();
		layout.setStretchedVertical(true);
		mHandle->setLayoutData(layout);

	}
	if (mKind == TYPE_VERTICAL)
	{
		mLess = new CSButton(laf->getIcon(LAF_ICON_TYPE_UP));
		mMore = new CSButton(laf->getIcon(LAF_ICON_TYPE_DOWN));

		CSLayoutData layout = mHandle->getLayoutData();
		layout.setStretchedHorizontal(true);
		mHandle->setLayoutData(layout);

	}
	addElement(mLess, 0,0);
	addElement(mMore, 0,0);
	addElement(mHandle,0,0);

	mMore->setFocusable(getFocusable());
	mLess->setFocusable(getFocusable());
	mHandle->setFocusable(getFocusable());
	mHandle->setInset(CSInset(0));

	
	// position doesnt matter here - gets calculated during paint()
	// but position should be 0,0 nonetheless, so during evaluating the size of the component
	// (XYLayout) - the size (of the handle) is calculated correctly
	// (calculated from the size of the mLess and mMore)
	// (is calculated by width = x+minWidth)


	mLess->setActionId(SCROLL_BUTTON_LESS_ACTION_ID);
	mLess->addMessageListener(this, GUI_MESSAGE);
	mMore->setActionId(SCROLL_BUTTON_MORE_ACTION_ID);
	mMore->addMessageListener(this, GUI_MESSAGE);
	mHandle->setActionId(SCROLL_BUTTON_DRAG_ACTION_ID);
	mHandle->addMessageListener(this, GUI_MESSAGE);
	
	LOG_EXIT
}

void CSScrollbar::layoutSetupScrollbar()
{
	static char *functionName="layoutSetupScrollbar";
	LOG_ENTER 
	CSLAF *laf = CSLAF::getCurrentLAF();
	if (mKind == TYPE_HORIZONTAL)
	{
		mMinWidth = mLess->getWidth() + mMore->getWidth();
		mMinHeight = mLess->getHeight();
	}
	if (mKind == TYPE_VERTICAL)
	{
		mMinHeight = mLess->getHeight() + mMore->getHeight();
		mMinWidth = mLess->getWidth();
	}
	
	LOG_EXIT
}
