#ifndef CSStringlist_h
#define CSStringlist_h

#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

#include <vector>
#include <string>
#include <map>

#include "CSTypes.h"

typedef std::map<int, std::string> StringMap;
typedef std::map<int, void *> ObjectMap;

// objects are NOT freed by CSStringList
class CSStringlist
{
	private: 
		StringMap mMap;
		ObjectMap mObjectMap;
	public:
		std::string getString(int i)
		{
			StringMap::iterator iter = mMap.find(i);
			if (iter != mMap.end())
			{
				return iter->second;
			}
			return "";
		}
		void clear()
		{
			mMap.clear();
			mObjectMap.clear();
		}
		void *getObject(int i)
		{
			ObjectMap::iterator iter = mObjectMap.find(i);
			if (iter != mObjectMap.end())
			{
				if ((iter->second) == ((void *) 1))
				{
					return 0;
				}
				return iter->second;
			}
			return 0;
		}

		void set(int i, const std::string & text, void *object = ((void *) 1))
		{
			mMap.erase(i);
			mMap.insert(StringMap::value_type(i, text));
			mObjectMap.erase(i);
			mObjectMap.insert(ObjectMap::value_type(i, object));
		}

		void add(const std::string & text, void *object = ((void *) 1))
		{
			mMap.insert(StringMap::value_type(mMap.size(), text));
			mObjectMap.insert(ObjectMap::value_type(mObjectMap.size(), object));
		}
		
		// inserts at index i
		void insert(int i, const std::string & text, void *object = ((void *) 1))
		{
			for (int t=mMap.size(); t>=i;t--)
			{
				set(t, getString(t-1), getObject(t-1));
			}
			set(i, text, object);
		}
		
		void remove(int i)
		{
			for (unsigned int t=i; t<mMap.size();t++)
			{
				set(t, getString(t+1), getObject(t+1));
			}
			mMap.erase(mMap.size()-1);
			mObjectMap.erase(mObjectMap.size()-1);
		}

		int getSize()
		{
			return mMap.size();
		}
};

#endif // CSStringlist_h
