#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

#include "CSTabbedPane.h"
#include "CSFont.h"
#include "CSButton.h"
#include "CSLAF.h"

const char *CSTabbedPane::CLASS = "CSTabbedPane";

CSTabbedPane::CSTabbedPane(int w, int h) : CSGrafikElement(w, h)
{
	static char *functionName="CSTabbedPane";
	LOG_ENTER 
	initTabbedPane();
	LOG_EXIT
}

void CSTabbedPane::initTabbedPane()
{
	mActivePanelNumber = -1; // no Panel active (since there is NO Panel)
	mActivePanelName = "";
	mButtonPanel = 0;
	setLayoutManager(new CSLayoutManagerStackVertical());
	CSLayoutData layoutData = getLayoutData();
	layoutData.setStretchedHorizontal(true);
	layoutData.setStretchedVertical(true);
layoutData.setCenteredHorizontal(true);
	setLayoutData(layoutData);
setBorder(CSBorder::getBorder(BORDER_TYPE_UP));

	rebuildElement();
	layoutSetup();
}

void CSTabbedPane::addElement(CSGrafikElement *element, std::string name)
{
	static char *functionName="addElement";
	LOG_ENTER 

	if (element->getType().compare(CSPanel::CLASS) == 0)
	{
		// only allow CSPanels to be added
		// TODO: must think about children of CSPanel
		NamedPanel *namedPanel = new NamedPanel((CSPanel *) element, name);
		mNamedPanels.push_back(namedPanel);
		CSButton *button = new CSButton(name);
		mButtonPanel->addElement(button, POSITION_WEST);
//		setActivePaneName(name);
	}
	LOG_EXIT
}

void CSTabbedPane::paint(SDL_Surface *destination, SDL_Rect *parentViewport)
{
	static char *functionName="paint";
	SDL_Rect rectSave;
	SDL_GetClipRect(destination, &rectSave);

	SDL_Rect rectClip = getViewportArea();
	rectClip.x += parentViewport->x + getX();
	rectClip.y += parentViewport->y + getY();

	SDLMain::adjustClipping(rectClip, rectSave);
	
	SDL_SetClipRect(destination, &rectClip);

//	SDL_Rect viewArea = getViewportArea();
//	SDL_Rect elementArea = getElementArea();

	// paint component
/*
	idee
	alle Namen als Button
	alle Buttons haben eine bestimmte border
	der Button mit dem ausgewhlten namen hat eine andere border!
	alle Panels als Panels, nur das sichtbare ist ge added

*/

	SDL_SetClipRect(destination, &rectSave);

	// paint panel?
	paintChildren(destination, parentViewport);
}

void CSTabbedPane::setActivePane(int n)
{
	int counter = -1;
	NamedPanels::iterator iter = mNamedPanels.begin();
	while (iter != mNamedPanels.end())
	{
		counter++;
		NamedPanel *namedPanel = *iter;
		if (counter == mActivePanelNumber)
		{
			removeElement(namedPanel->mPanel);
			mActivePanelNumber = -1;
			mActivePanelName = "";
			break;
		}
		iter++;
	}
	
	counter = -1;
	iter = mNamedPanels.begin();
	while (iter != mNamedPanels.end())
	{
		counter++;
		NamedPanel *namedPanel = *iter;
		if (counter == n)
		{
			addMainElement(namedPanel->mPanel, POSITION_SOUTH);
			mActivePanelName = namedPanel->mName;
			mActivePanelNumber = n;
			break;
		}
		iter++;
	}
}

int CSTabbedPane::getActivePane()
{
	return mActivePanelNumber;
}

void CSTabbedPane::setActivePaneName(std::string name)
{
	int counter = -1;
	NamedPanels::iterator iter = mNamedPanels.begin();
	while (iter != mNamedPanels.end())
	{
		counter++;
		NamedPanel *namedPanel = *iter;
		if (counter == mActivePanelNumber)
		{
			removeElement(namedPanel->mPanel);
			mActivePanelNumber = -1;
			mActivePanelName = "";
			break;
		}
		iter++;
	}
	
	iter = mNamedPanels.begin();
	counter = -1;
	while (iter != mNamedPanels.end())
	{
		counter++;
		NamedPanel *namedPanel = *iter;
		if (namedPanel->mName.compare(name) == 0)
		{
			addMainElement(namedPanel->mPanel, POSITION_SOUTH);
			mActivePanelName = namedPanel->mName;
			mActivePanelNumber = counter;
			break;
		}
		iter++;
	}
}

std::string CSTabbedPane::getActivePaneName()
{
	return mActivePanelName;
}

void CSTabbedPane::reactOnMessage(CSMessage *message)
{
	reactOnMessageTabbedPane(message);
}

void CSTabbedPane::reactOnMessageTabbedPane(CSMessage *message)
{
	static char *functionName="reactOnMessage";
	if (message->mIsHandled)
	{
		reactOnMessageGrafikElement(message);
		return;
	}
	switch (message->getType())
	{
		case GUI_MESSAGE:
		{
			GuiMessage *gm = (GuiMessage *) message;
			if (gm->receiver == this)
			{
			}
			if (gm->receiver != this)
			{
			}
		}
	}
	reactOnMessageGrafikElement(message);
}

void CSTabbedPane::rebuildElement()
{
	static char *functionName="rebuildElement";
	CSLAF *laf = CSLAF::getCurrentLAF();
	if (mButtonPanel)
	{
		mButtonPanel->removeMessageListener(this);
		removeElement(mButtonPanel);
		delete (mButtonPanel);
		mButtonPanel = 0;
	}
	mButtonPanel = new CSPanel(100, 30);
	mButtonPanel->setLayoutManager(new CSLayoutManagerStackHorizontal());

	CSLayoutData layoutData = mButtonPanel->getLayoutData();
	layoutData.setStretchedHorizontal(true);
	layoutData.setPackedVertical(true);
	mButtonPanel->setLayoutData(layoutData);

	addMainElement(mButtonPanel, POSITION_NORTH);
}

void CSTabbedPane::layoutSetupTabbedPane() 
{
//	mMinWidth = 10;
//	mMinHeight = 10;
}
