#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

#include "CSTextarea.h"
#include "CSBorder.h"
#include "CSFont.h"
#include "CSDesktop.h"
#include "CSLAF.h"

const char *CSTextarea::CLASS = "CSTextarea";

CSTextarea::CSTextarea(int height, int width) : CSGrafikElement(height, width)
{
	static char *functionName="CSTextarea";
	LOG_ENTER 
	init(""); 
	LOG_EXIT
}

void CSTextarea::init(const std::string &text)
{
	static char *functionName="init";
	LOG_ENTER 
	CSLAF *laf = CSLAF::getCurrentLAF();
	mCursorRate = laf->getCursorRate(); // 500 milliseconds
	
	mString = text;
	buildLines();
	mTickTimeNext = 0;

	mCursorX = 0;
	mCursorY = 0;
	mVerticalDisplayStart = 0;
	mVerticalDisplayEnd = 0;
	mVerticalDisplayLengthChar = 0;
	mHorizontalDisplayStart = 0;
	mHorizontalDisplayEnd = 0;
	mHorizontalDisplayLengthChar = 0;
	mExactCursorX = 0;
	mExactCursorY = 0;
	mIsEditable = true;
	mCursorShown = false;
	//setEnabled(false);

	layoutSetup();
	LOG_EXIT
}

void CSTextarea::setText(const std::string text)
{
	static char *functionName="setText";
	mString = text;
	buildLines();
}

void CSTextarea::buildLines()
{
	static char *functionName="buildLines";
	mLines.add(mString);
}

void CSTextarea::buildString()
{
	static char *functionName="buildString";
}

// position in viewport relation
// in pixel relative to textarea!
void CSTextarea::setGrafikCursor(int x, int y)
{
	static char *functionName="setCursor";
	if (mCursorY>=0)
	{
		mLines.set(mCursorY, mWorkingString);
	}

	// height of one line in textarea
	int yAdd = getFont()->getHeight() + getVerticalElementSpacing();
	
	// holds the information in which line of the
	// text the blinking cursor at one time
	// resides (might therefor be larger than the number of lines displayed)
	mCursorY = y/yAdd + mVerticalDisplayStart;
	mExactCursorY = (mCursorY-mVerticalDisplayStart)*yAdd;
	mWorkingString = mLines.getString(mCursorY);
	
	const char *s=mWorkingString.c_str();
	char test[2];
	test[1]=0;
	mExactCursorX = getHorizontalElementSpacing(); 
	mCursorX = mHorizontalDisplayStart;
	if (mCursorX > mWorkingString.size())
	{
		mCursorX = mWorkingString.size();
		if (mCursorX > 0)
		{
			mHorizontalDisplayStart = mCursorX-1;
			mExactCursorX += getFont()->getWidth(" ");
		}
	}
	int horizontalDisplayLengthPixel = getViewportWidth()-(getHorizontalElementSpacing());
	while (mCursorX < mWorkingString.length())
	{
		test[0] = s[mCursorX];
		int size = getFont()->getWidth(test);
		if ((mExactCursorX + size > x) || (mExactCursorX + size > horizontalDisplayLengthPixel))
		{
			break;
		}
		mExactCursorX += size;
		mCursorX++;
	}
	if (mCursorX <0)
	{
		mCursorX = 0;
	}
}

// in characters relative to complete text
void CSTextarea::setTextCursor(int x, int y)
{
	static char *functionName="setCursor";
	if (mCursorY>=0)
	{
		mLines.set(mCursorY, mWorkingString);
	}

	int yAdd = getFont()->getHeight() + getVerticalElementSpacing();
	int xAdd = getFont()->getWidth(" ");
	mCursorY = y;

	/////////////////////////
	mVerticalDisplayLengthChar = getViewportHeight() / yAdd;
	if (mCursorY < mVerticalDisplayStart)
	{
		mVerticalDisplayStart = mCursorY;
		mVerticalDisplayEnd = mVerticalDisplayStart + mVerticalDisplayLengthChar;
		if (mVerticalDisplayEnd > mLines.getSize())
		{
			mVerticalDisplayEnd = mLines.getSize();
		}
	}
	mVerticalDisplayEnd = mVerticalDisplayStart + mVerticalDisplayLengthChar;
	if (mCursorY > mVerticalDisplayEnd-1)
	{
		mVerticalDisplayEnd = mCursorY+1;
		mVerticalDisplayStart = mVerticalDisplayEnd - (mVerticalDisplayLengthChar);
		if (mVerticalDisplayStart < 0)
		{
			mVerticalDisplayStart = 0;
		}
	}
	/////////////////////////

	mExactCursorY = (mCursorY-mVerticalDisplayStart)*yAdd;
	mWorkingString = mLines.getString(mCursorY);
	const char *s=mWorkingString.c_str();
	char test[2];
	test[1]=0;
	mExactCursorX = getHorizontalElementSpacing();
	int pos = 0;
	while ((pos < x) &&  (pos < mWorkingString.length()))
	{
		test[0] = s[pos];
		int size = getFont()->getWidth(test);
		mExactCursorX += size;
		pos++;
	}
	mCursorX = pos--;

	/////////////////////////
	mHorizontalDisplayLengthChar = (getViewportWidth()-(2*getHorizontalElementSpacing())) / xAdd;
	if (mCursorX < mHorizontalDisplayStart)
	{
		mHorizontalDisplayStart = mCursorX;
	}
	mHorizontalDisplayEnd = mHorizontalDisplayStart + mHorizontalDisplayLengthChar;
	if (mCursorX > mHorizontalDisplayEnd-1)
	{
		mHorizontalDisplayEnd = mCursorX+1;
		mHorizontalDisplayStart = mHorizontalDisplayEnd - (mHorizontalDisplayLengthChar);
		if (mHorizontalDisplayStart < 0)
		{
			mHorizontalDisplayStart = 0;
		}
	}
	/////////////////////////
	mExactCursorX = (mCursorX-mHorizontalDisplayStart)*xAdd + getHorizontalElementSpacing();
}

void CSTextarea::putString(SDL_Surface *destination, SDL_Rect *viewport, int x, int y, const std::string &string)
{
	static char *functionName="putString";
	x += viewport->x;
	y += viewport->y;
	getFont()->putString(destination,x,y,getTextColor(), string);
}

void CSTextarea::paint(SDL_Surface *destination, SDL_Rect *parentViewport)
{
	// TODO: Textarea does not respect Insets!
	
	static char *functionName="paint";
	CSLAF *laf = CSLAF::getCurrentLAF();
	float currentTick = SDL_GetTicks() + mCursorRate;
	if (currentTick > mTickTimeNext)
	{
		mTickTimeNext = currentTick + mCursorRate;
		mCursorShown = !mCursorShown;
	}

	SDL_Rect thisViewportArea;
	thisViewportArea.x = parentViewport->x + getViewportX() + getX();
	thisViewportArea.y = parentViewport->y + getViewportY() + getY();
	thisViewportArea.h = getViewportHeight();
	thisViewportArea.w = getViewportWidth();

	int x = getHorizontalElementSpacing();
	int y = 0;

	int yAdd = getFont()->getHeight() + getVerticalElementSpacing();
	int xAdd = getFont()->getWidth(" ");
	
	mVerticalDisplayLengthChar = getViewportHeight() / yAdd;
	/////////////////////////
	if (mCursorY < mVerticalDisplayStart)
	{
		mVerticalDisplayStart = mCursorY;
		mVerticalDisplayEnd = mVerticalDisplayStart + mVerticalDisplayLengthChar;
		if (mVerticalDisplayEnd > mLines.getSize())
		{
			mVerticalDisplayEnd = mLines.getSize();
		}
	}
	mVerticalDisplayEnd = mVerticalDisplayStart + mVerticalDisplayLengthChar;
	if (mCursorY > mVerticalDisplayEnd-1)
	{
		mVerticalDisplayEnd = mCursorY+1;
		mVerticalDisplayStart = mVerticalDisplayEnd - (mVerticalDisplayLengthChar);
		if (mVerticalDisplayStart < 0)
		{
			mVerticalDisplayStart = 0;
		}
	}
	/////////////////////////
	/////////////////////////
	mHorizontalDisplayLengthChar = (getViewportWidth()-(2*(getHorizontalElementSpacing()))) / xAdd;
	if (mCursorX < mHorizontalDisplayStart)
	{
		mHorizontalDisplayStart = mCursorX;
		mHorizontalDisplayEnd = mHorizontalDisplayStart + mHorizontalDisplayLengthChar;
	}
	mHorizontalDisplayEnd = mHorizontalDisplayStart + mHorizontalDisplayLengthChar;
	if (mCursorX > mHorizontalDisplayEnd-1)
	{
		mHorizontalDisplayEnd = mCursorX+1;
		mHorizontalDisplayStart = mHorizontalDisplayEnd - (mHorizontalDisplayLengthChar);
		if (mHorizontalDisplayStart < 0)
		{
			mHorizontalDisplayStart = 0;
		}
	}
	/////////////////////////

	for (int i = mVerticalDisplayStart; i < mVerticalDisplayEnd; i++)
	{
		if (((i!= mCursorY) || (!getFocused())) || (!mIsEditable))
		{
			if (mLines.getString(i).size() >=mHorizontalDisplayStart)
			{
				std::string t = mLines.getString(i).substr(mHorizontalDisplayStart, mHorizontalDisplayLengthChar);
				putString(destination, &thisViewportArea, x, y, t);

			}
		}
		else
		{
			if (mWorkingString.size() >=mHorizontalDisplayStart)
			{
				std::string t = mWorkingString.substr(mHorizontalDisplayStart, mHorizontalDisplayLengthChar);
				putString(destination, &thisViewportArea, x, y, t);
			}
		}
		y += yAdd;
	}

	if ((getFocused() && (mCursorShown)) && (mIsEditable))
	{
		line(destination, &thisViewportArea, 
				mExactCursorX, mExactCursorY, 
				mExactCursorX, mExactCursorY+getFont()->getHeight(), 
				laf->getTextAreaCursorColor());
	}
}

void CSTextarea::deleteCharAtPos(int p)
{
	std::string t1 = mWorkingString.substr(0, p);
	std::string t2 = mWorkingString.substr(p+1, mWorkingString.length());
	mWorkingString = t1+t2;
}

void CSTextarea::insertCharAtPos(int p, char ch)
{
	std::string t1 = mWorkingString.substr(0, p);
	std::string t2 = mWorkingString.substr(p, mWorkingString.length());
	mWorkingString = t1+ch+t2;
}

void CSTextarea::reactOnMessage(CSMessage *message)
{
	reactOnMessageTextarea(message);
}

void CSTextarea::reactOnMessageTextarea(CSMessage *message)
{
	static char *functionName="reactOnMessage";
	if (message->mIsHandled)
	{
		reactOnMessageGrafikElement(message);
		return;
	}
	if (!getEnabled()) return;

	switch (message->getType())
	{
		case GUI_MESSAGE:
		{
			GuiMessage *gm = (GuiMessage *) message;
			if (gm->receiver != this)
			{
				break;
			}
			else // receiver == this
			{
				switch (message->getSubtype())
				{
					case MOUSE_BUTTON_RELEASED_MESSAGE:
					{
						int x,y;
						if (mIsEditable)
						{
							x = gm->receiverX;
							y = gm->receiverY;
							setGrafikCursor(x,y);
							gm->mIsHandled = true;
						}
						break;
					}
					
					case FOCUS_GAINED_MESSAGE:
					{
						if (mIsEditable)
						{
							setActive(true);
							if (mCursorY>=0)
							{
								mLines.set(mCursorY, mWorkingString);
							}
							gm->mIsHandled = true;
						}
						break;
					}
					
					case FOCUS_LOST_MESSAGE:
					{
						if (mIsEditable)
						{
							setActive(false);
						}

						mWorkingString = mLines.getString(mCursorY);
						gm->mIsHandled = true;
						break;
					}
					
					case KEY_RELEASED_MESSAGE:
					{
						gm->mIsHandled = false;
						break;
					}

					case KEY_PRESSED_MESSAGE:
					{
						int ch=gm->keyUnicodeChar;
						if (gm->keySymbolic == SDLK_TAB)
						{
							getDesktop()->focusedNextComponent();
							gm->mIsHandled = true;
							break;
						}
						if (gm->keySymbolic == SDLK_DELETE)
						{
							if (mCursorX<mWorkingString.length())
							{
								deleteCharAtPos(mCursorX);
								setTextCursor(mCursorX, mCursorY);
							}
							else
							{
								if (mCursorY<mLines.getSize())
								{
									mWorkingString += mLines.getString(mCursorY+1);
									mLines.remove(mCursorY+1);
									setTextCursor(mCursorX, mCursorY);
								}
							}
							gm->mIsHandled = true;
							break;
						}

						if (gm->keySymbolic == SDLK_BACKSPACE)
						{
							if (mCursorX>0)
							{
								deleteCharAtPos(mCursorX-1);
								setTextCursor(mCursorX-1, mCursorY);
							}
							else
							{
								if (mCursorY>0)
								{
									setTextCursor(mCursorX, mCursorY-1);
									mCursorX = mWorkingString.length();

									mWorkingString += mLines.getString(mCursorY+1);
									mLines.remove(mCursorY+1);
									setTextCursor(mCursorX, mCursorY);
								}
							}
							gm->mIsHandled = true;
							break;
						}
						if ((gm->keySymbolic == SDLK_RETURN) || (gm->keySymbolic == SDLK_KP_ENTER))
						{
							std::string t1 = mWorkingString.substr(0, mCursorX);
							std::string t2 = mWorkingString.substr(mCursorX, mWorkingString.length()-t1.length());
							mWorkingString = t1;
							mLines.insert(mCursorY+1,t2);
							mCursorX=0;
							setTextCursor(mCursorX, mCursorY+1);
							gm->mIsHandled = true;
							break;
						}
						
						if (gm->keySymbolic == SDLK_UP)
						{
							if (mCursorY>0)
							{
								setTextCursor(mCursorX, mCursorY-1);
							}
							gm->mIsHandled = true;
							break;
						}

						if (gm->keySymbolic == SDLK_DOWN)
						{
							if (mCursorY+1<mLines.getSize())
							{
								setTextCursor(mCursorX, mCursorY+1);
							}
							gm->mIsHandled = true;
							break;
						}

						if (gm->keySymbolic == SDLK_LEFT)
						{
							if (mCursorX>0)
							{
								setTextCursor(mCursorX-1, mCursorY);
							}
							gm->mIsHandled = true;
							break;
						}

						if (gm->keySymbolic == SDLK_RIGHT)
						{
							if (mCursorX<mWorkingString.length())
							{
								setTextCursor(mCursorX+1, mCursorY);
							}
							gm->mIsHandled = true;
							break;
						}

						if (gm->keySymbolic == SDLK_HOME)
						{
							setTextCursor(0, mCursorY);
							gm->mIsHandled = true;
							break;
						}

						if (gm->keySymbolic == SDLK_END)
						{
							setTextCursor(mWorkingString.length(), mCursorY);
							gm->mIsHandled = true;
							break;
						}
						if ((ch>31) && (ch<128)) 
						{
							int len = mWorkingString.length();
							insertCharAtPos(mCursorX, ch);
							setTextCursor(mCursorX+1, mCursorY);
							gm->mIsHandled = true;
							break;
						}

						break;
					}
					
				}
			}						
		}
	}
	reactOnMessageGrafikElement(message);
}

void CSTextarea::layoutSetupTextarea()
{
	static char *functionName="layoutSetupTextarea";
	LOG_ENTER 
	CSLAF *laf = CSLAF::getCurrentLAF();
	mMinHeight = mHeight - getBorder()->getTotalHeight() - getInset().getTotalHeight();
	mMinWidth = mWidth - getBorder()->getTotalWidth() - getInset().getTotalWidth();

	LOG_EXIT
}


