#ifndef CSTextarea_h
#define CSTextarea_h

// USES SDL_Surface

#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

#include <vector>
#include <string>
#include <map>
#include "CSTypes.h"

#include "CSGrafikElement.h"
#include "CSLog.h"
#include "CSTypes.h"
#include "CSStringlist.h"

class CSTextarea;
typedef std::vector<CSTextarea *> CSTextareas;

class CSTextarea : public CSGrafikElement
{
	private:
		std::string mString;
		CSStringlist mLines;
		std::string mWorkingString;
		void init(const std::string &text);
		float mTickTimeNext;
		float mCursorRate;
		bool mCursorShown;
		int mCursorX;	// in characters whole text
		int mCursorY;
		signed int mExactCursorX;	// in pixel - in relation to textarea only screen!
		signed int mExactCursorY;

		int mVerticalDisplayStart;			// offset where the displayed text starts (which) line!
		int mVerticalDisplayEnd;
		int mVerticalDisplayLengthChar;
		int mHorizontalDisplayStart;
		int mHorizontalDisplayEnd;
		int mHorizontalDisplayLengthChar;
		bool mIsEditable;

		void buildLines();
		void buildString();
		void setGrafikCursor(int x, int y); // in pixel relative to textarea! (position in viewport relation)
		void setTextCursor(int x, int y);	// in characters relative to complete text
		void deleteCharAtPos(int p);
		void insertCharAtPos(int p, char ch);

	protected:
		void layoutSetupTextarea();
	public:
		CSTextarea(int height, int width);
		virtual ~CSTextarea() {}
		static const char *CLASS;
		virtual std::string getType() {return (std::string) CLASS;}

		virtual void setText(const std::string text);
		virtual void setText(const CSStringlist &lines) {mLines = lines;}
		
		// lines are seperated by a "/n"
		virtual void setMultiLineText(const std::string text) 
		{
			mLines.clear();
			char *copy = strdup(text.c_str());
			char *tmp = copy;
			
			char *pointerToWord = strtok( copy, "\n");
			while (pointerToWord != 0)
			{
				mLines.add(std::string(pointerToWord));
				pointerToWord = strtok( 0, " \n\t");
			}
			free(tmp);
		}
		
		void reactOnMessageTextarea(CSMessage *message);
		virtual void reactOnMessage(CSMessage *message);

		void setEditable(bool b) {mIsEditable = b;}

		virtual void paint(SDL_Surface *destination, SDL_Rect *parentViewport);
		virtual void putString(SDL_Surface *destination, SDL_Rect *viewport, int x, int y, const std::string &string);
		virtual void layoutSetup() {layoutSetupTextarea();}
};

#endif // CSTextarea_h
