#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

#include "CSTextfield.h"
#include "CSFont.h"
#include "CSDesktop.h"
#include "CSBorder.h"
#include "CSLAF.h"

const char *CSTextfield::CLASS = "CSTextfield";

CSTextfield::CSTextfield() : CSGrafikElement(0, 0)
{
	static char *functionName="CSTextfield";
	LOG_ENTER 
	init("");
	LOG_EXIT
}

CSTextfield::CSTextfield(int width) : CSGrafikElement(0, 0)
{
	static char *functionName="CSTextfield";
	LOG_ENTER 
	setWidth(width);
	init("");
	LOG_EXIT
}

CSTextfield::CSTextfield(const std::string &text) : CSGrafikElement(0, 0)
{
	static char *functionName="CSTextfield";
	LOG_ENTER 
	init(text);
	LOG_EXIT
}

void CSTextfield::init(const std::string &text)
{
	static char *functionName="init";
	LOG_ENTER 
	CSLAF *laf = CSLAF::getCurrentLAF();
	mCursorRate = laf->getCursorRate(); // 500 milliseconds

	mString = text;
	mTickTimeNext = 0;
	mCursorShown = false;
	MESSAGE_TEXT_CHANGED.setSubtype(TEXT_CHANGED_MESSAGE);
	MESSAGE_TEXT_CHANGED.sender = this;
	//setActive(false);
	layoutSetup();
	LOG_EXIT
}

void CSTextfield::putString(SDL_Surface *destination, SDL_Rect *parentViewport, int x1, int y1, const std::string &string)
{
	static char *functionName="putString";
	x1 += parentViewport->x + getX();
	y1 += parentViewport->y + getY();
	getFont()->putString(destination,x1,y1,getTextColor(), string);
}

//! \todo maybe one can inherit from textAre? That way we would only have
//! one editing event loop...
void CSTextfield::paint(SDL_Surface *destination, SDL_Rect *parentViewport)
{
	static char *functionName="paint";
	int xOffset = getInset().getSizeWest() + getBorder()->getSizeWest();
	int yOffset = getInset().getSizeNorth() + getBorder()->getSizeNorth();

	if (getFocused())
	{
		float currentTick = SDL_GetTicks() + mCursorRate;
		if (currentTick > mTickTimeNext)
		{
			mTickTimeNext = currentTick + mCursorRate;
			mCursorShown = !mCursorShown;
		}
		
		putString(destination, parentViewport, xOffset, yOffset, mWorkingString);
		if (mCursorShown)
		{
			int witdh = getFont()->getWidth(mWorkingString);
			putString(destination, parentViewport, xOffset + witdh, yOffset, "_");
		}
	}
	else
	{
		putString(destination, parentViewport, xOffset, yOffset, mString);
	}
}

void CSTextfield::reactOnMessage(CSMessage *message)
{
	reactOnMessageTextField(message);
}

void CSTextfield::reactOnMessageTextField(CSMessage *message)
{
	static char *functionName="reactOnMessage";
	if (message->mIsHandled)
	{
		reactOnMessageGrafikElement(message);
		return;
	}
	if (!getEnabled()) return;
	switch (message->getType())
	{
		case GUI_MESSAGE:
		{
			GuiMessage *gm = (GuiMessage *) message;
			if (gm->receiver != this)
			{
				break;
			}
			else // receiver == this
			{
				switch (message->getSubtype())
				{
					case FOCUS_GAINED_MESSAGE:
					{
						setActive(true);
						mWorkingString = mString;
						gm->mIsHandled = true;
						break;
					}
					case FOCUS_LOST_MESSAGE:
					{
						setActive(false);
						mString = mWorkingString;
						gm->mIsHandled = true;
						break;
					}
					case KEY_RELEASED_MESSAGE:
					{
						gm->mIsHandled = false;
						break;
					}
					case KEY_PRESSED_MESSAGE:
					{
						int ch=gm->keyUnicodeChar;
						if ((gm->keySymbolic == SDLK_RETURN) || 
							(gm->keySymbolic == SDLK_KP_ENTER) || 
							(gm->keySymbolic == SDLK_TAB))
						{
							getDesktop()->focusedNextComponent();
							gm->mIsHandled = true;
							MESSAGE_TEXT_CHANGED.mIsHandled = false;
							MESSAGE_TEXT_CHANGED.actionId = mActionId;

							sendMessage(MESSAGE_TEXT_CHANGED);
							break;
						}
						if (((ch>31)||(ch=='\b')) && (ch<128)) 
						{
							int len = mWorkingString.length();
							if (ch=='\b')
							{
								if (len>0)
								{
									mWorkingString.resize(len-1);
								}
							}
							else
							{
								mWorkingString += (char) ch;
								
								len = mWorkingString.length();
								int witdh = getFont()->getWidth(mWorkingString) + getFont()->getWidth("_");
								if (witdh > mWidth - (getInset().getTotalWidth() + getBorder()->getTotalWidth()))
								{
									mWorkingString.resize(len-1);
								}
							}
							gm->mIsHandled = true;
						}
						break;
					}
				}
			}						
		}
	}
	reactOnMessageGrafikElement(message);
}

void CSTextfield::layoutSetupTextfield()
{
	static char *functionName="layoutSetupTextfield";
	LOG_ENTER 
	CSLAF *laf = CSLAF::getCurrentLAF();

	if (!mWidthSet)
	{
		mWidth =	getFont()->getWidth(mString) 
					+ getBorder()->getTotalWidth()
					+ getInset().getTotalWidth();
		mMinWidth = getFont()->getWidth("12345");
	}
	else
	{
		mMinWidth = mWidth - getBorder()->getTotalWidth() - getInset().getTotalWidth();
	}
	mHeight =	getFont()->getHeight() 
				+ getBorder()->getTotalHeight()
				+ getInset().getTotalHeight();
	mMinHeight = getFont()->getHeight();
	LOG_EXIT
}

