#ifdef WIN32
#pragma warning(disable : 4786 )
#endif


#include "CSWindow.h"
#include "CSScrollbar.h"
#include "CSIcon.h"
#include "CSLAF.h"
#include "CSPanel.h"
#include "CSDesktop.h"
#include "CSMessage.h"
#include "CSFont.h"

const char *CSWindow::CLASS = "CSWindow";
//! \todo correct picture window, so it works again!
CSWindow::CSWindow(CSPicture *picture) : CSGrafikElement(0,0)
{
	static char *functionName="CSWindow";
	LOG_ENTER 

	if (picture)
	{
		mPicture = picture;
		mHeight = mPicture->getMaxY();
		mWidth = mPicture->getMaxX();
		initWindow();
		setArea(0);
	}
	else
	{
		mPicture =0;
		initWindow();
	}
	mIsSizeable = true;
	LOG_EXIT
}

CSWindow::CSWindow(int height, int width, bool hasTitle) : CSGrafikElement(height,width), mCloseButton(0)
{
	static char *functionName="CSWindow";
	LOG_ENTER 
	mHasTitle = hasTitle;
	mIsSizeable = mHasTitle;
	mPicture = 0;
	initWindow();
	LOG_EXIT
}

CSWindow::~CSWindow()
{
	static char *functionName="~CSWindow";
	LOG_ENTER 
	// all elements are deleted by parent!
	LOG_EXIT
}

void CSWindow::initWindow()
{
	static char *functionName="initWindow";
	LOG_ENTER 
	mActive = true;
	mDragging = false;
	mSizing = false;
	mCursorChange = false;
	mCloseButton = 0;
	mTitleIcon = 0;
	mTitleBar = 0;
	mDragStartX = 0;
	mDragStartY = 0;
	mVerticalDisplayStart = -1;
	mVerticalDisplayEnd = -1;
	mVerticalDisplayLength = 0;
	mHorizontalDisplayStart = -1;
	mHorizontalDisplayEnd = -1;
	mHorizontalDisplayLength = 0;
	mVerticalScrollbar = 0;
	mHorizontalScrollbar = 0;
	mMinHeightInPixel = 0;
	mMinWidthInPixel = 0;
	mHorizontalDisplaySet = false;
	mVerticalDisplaySet = false;

	mTitle = "Window";
	MESSAGE_WINDOW_CLOSE.setSubtype(WINDOW_CLOSE_MESSAGE);
	mCloseable = true;
	setVisible(false);

	mNorth = 0;
	mWest = 0;
	mSouth = 0;
	mEast = 0;
	mCenter = 0;
	setMainLayoutManager(new CSLayoutManagerBorder());

	mNorth = new CSPanel();
	mNorth->setLayoutManager(new CSLayoutManagerStackVertical());
	mWest = new CSPanel();
	mWest->setLayoutManager(new CSLayoutManagerStackHorizontal());
	mSouth = new CSPanel();
	mSouth->setLayoutManager(new CSLayoutManagerStackVertical());
	mEast = new CSPanel();
	mEast->setLayoutManager(new CSLayoutManagerStackHorizontal());
	mCenter = new CSPanel();
	
	CSLayoutData layout = CSLayoutData(POSITION_NORTH);
	layout.setSpacing(0,0);

	layout.setStretchedHorizontal(true);
	layout.setPackedVertical(true);
	mNorth->setLayoutData(layout);
	addMainElement(mNorth);

	layout.setPosition(POSITION_SOUTH);
	mSouth->setLayoutData(layout);
	addMainElement(mSouth);
	
	layout.setPosition(POSITION_CENTER);
	layout.setStretchedVertical(true);
	mCenter->setLayoutData(layout);
	addMainElement(mCenter);
	setMainElement(mCenter);
	
	layout.setPackedHorizontal(true);
	layout.setPosition(POSITION_WEST);
	mWest->setLayoutData(layout);
	addMainElement(mWest);
	
	layout.setPosition(POSITION_EAST);
	mEast->setLayoutData(layout);
	addMainElement(mEast);

	rebuildElement();
	LOG_EXIT
}

void CSWindow::addElementNorth(CSGrafikElement *element)
{
	mNorth->addElement(element);
}

void CSWindow::addElementSouth(CSGrafikElement *element)
{
	mSouth->addElement(element);
}

void CSWindow::addElementEast(CSGrafikElement *element)
{
	mEast->addElement(element);
}

void CSWindow::addElementWest(CSGrafikElement *element)
{
	mWest->addElement(element);
}

void CSWindow::paint(SDL_Surface *destination, SDL_Rect *parentViewport)
{
	static char *functionName="paint";
	if (isPictureWindow())
	{
		mPicture->display(getX() + parentViewport->x, getY() + parentViewport->y);
	}
	paintChildren(destination, parentViewport);
}

bool CSWindow::isTransparent(int x, int y, SDL_Rect *parentViewport)
{
	static char *functionName="isTransparent";
	if (mPicture == 0)
	{
		return false;
	}

	bool b = mPicture->isTransparent(x+parentViewport->x, y+parentViewport->y);
	return b;
}

void CSWindow::addLeftIcon(CSIcon *icon)
{
	static char *functionName="addLeftIcon";
	LOG_ENTER 

	if (mTitleIcon)
	{
		mLeftIcons.pop_back();
	}
	mLeftIcons.push_back(icon);
	if (mTitleIcon)
	{
		mLeftIcons.push_back(mTitleIcon);
	}
	layoutChanged();
	LOG_EXIT
}

void CSWindow::reactOnMessage(CSMessage *message)
{
	reactOnMessageWindow(message);
}

void CSWindow::reactOnMessageWindow(CSMessage *message)
{
	static char *functionName="reactOnMessage";
	if (message->mIsHandled)
	{
		if (message->getType() == GUI_MESSAGE)
		{
			GuiMessage *gm = (GuiMessage *) message;
			switch (message->getSubtype())
			{
				case MOUSE_BUTTON_RELEASED_MESSAGE:
				{
					mDragging = false;
					if (mCursorChange)
					{
						cursorDefault();
						mSizing = false;
						mCursorChange = false;
					}
					break;
				}
			}
		}
		reactOnMessageGrafikElement(message);
		return;
	}
	switch (message->getType())
	{
		case GUI_MESSAGE:
		{
			GuiMessage *gm = (GuiMessage *) message;
			if (message->getSubtype() ==  MOUSE_MOTION_LOST_MESSAGE)
			{
				mDragging = false;
			}
			if (message->getSubtype() ==  MOUSE_MOTION_MESSAGE)
			{
				if (mDragging)
				{
					int deltaX = gm->screenX - mDragStartX;
					int deltaY = gm->screenY - mDragStartY;
					if (mSizing)
					{
						int height = getHeight() + deltaY;
						int width = getWidth() + deltaX;
				
						if (height < 100)  height = 100;
						if (width < 100)  width = 100; 
						
						setHeight(height);
						setWidth(width);
					}
					else
					{
						setPosition(getX()+deltaX, getY()+deltaY);
					}
					mDragStartX = gm->screenX;
					mDragStartY = gm->screenY;
				}
				else
				{
					int x = gm->receiverX;
					int y = gm->receiverY;
					if (mIsSizeable)
					{
						if ((x > getWidth()-mSizingOffset) && (y > getHeight()-mSizingOffset))
						{
							if (!mCursorChange)
							{
								cursorSizing();
								mSizing = true;
								mCursorChange = true;
							}
						}
						else
						{
							if (mCursorChange)
							{
								cursorDefault();
								mSizing = false;
								mCursorChange = false;
							}
						}
					}
				}
			}

			if (gm->receiver != this)
			{
				if (gm->getSubtype() == SCROLLBAR_POSITION_CHANGED_MESSAGE)
				{
					if (gm->sender == mVerticalScrollbar)
					{
						mVerticalDisplayStart = gm->start;
						mVerticalDisplayEnd = gm->end;
						mVerticalDisplaySet = true;
						mCenter->setViewportOffset(mHorizontalDisplayStart, mVerticalDisplayStart);
					}
					else if (gm->sender == mHorizontalScrollbar)
					{
						mHorizontalDisplayStart = gm->start;
						mHorizontalDisplayEnd = gm->end;
						mHorizontalDisplaySet = true;
						mCenter->setViewportOffset(mHorizontalDisplayStart, mVerticalDisplayStart);
					}
					gm->mIsHandled = true;
				}
				switch (message->getSubtype())
				{
					case MOUSE_BUTTON_RELEASED_MESSAGE:
					{
						mDragging = false;
						break;
					}
					case MOUSE_BUTTON_PRESSED_MESSAGE:
					{
						if (!mHasTitle)
						{
							break;
						}
						int tx = gm->receiverX;
						int ty = gm->receiverY;

						if (isChild(gm->receiver))
						{
							mDragging = true;
							mDragStartX = gm->screenX;
							mDragStartY = gm->screenY;
							gm->mIsHandled = true;
							mParent->toFront(this);
						}	
						break;
					}
					case BUTTON_PRESSED_MESSAGE:
					{
						if (gm->actionId == CLOSE_ACTION_ID)
						{
							CSGrafikElement *parent = getParent();
							if (parent) 
							{
								setVisible(false);
							}
							gm->mIsHandled = true;
						}
						break;
					}
				}
			}
			else // receiver == this
			{
				switch (message->getSubtype())
				{
					case MOUSE_BUTTON_PRESSED_MESSAGE:
					{
						if (!mHasTitle)
						{
							break;
						}
						if (mSizing)
						{
							mDragging = true;
							mDragStartX = gm->screenX;
							mDragStartY = gm->screenY;
							gm->mIsHandled = true;
							mParent->toFront(this);
						}	
						break;
					}
					case MOUSE_BUTTON_RELEASED_MESSAGE:
					{
						mDragging = false;
						break;
					}
				}
			}						
		}
	}
	reactOnMessageGrafikElement(message);
}

void CSWindow::rebuildElementWindow() 
{
	static char *functionName="rebuildElementWindow";
	LOG_ENTER 

	CSLAF *laf = CSLAF::getCurrentLAF();
	mSizingOffset = laf->getWindowSizingOffset();

	// Button & Icon must be deleted befor TitleBar, for titleBar will delete them anyway! 
	if (mCloseButton) 
	{
		mCloseButton->removeMessageListener(this);
		delete (mCloseButton);
		mCloseButton = 0;
	}
	if (mTitleIcon)
	{
		mLeftIcons.pop_back();
		mTitleIcon->removeMessageListener(this);
		delete (mTitleIcon);
		mTitleIcon = 0;
	}

	if (mHasTitle)
	{
		if (mTitleBar)
		{
			mNorth->removeElement(mTitleBar);
			mTitleBar->removeMessageListener(this);
			delete (mTitleBar);
			mTitleBar = 0;
		}
		mTitleBar = new CSBar(TYPE_HORIZONTAL);
		mTitleBar->setGUIType("Title");
		mTitleBar->addMessageListener(this, GUI_MESSAGE);
		mNorth->addElement(mTitleBar);
	}

	if (mTitleBar)
	{ 
		mTitleBar->removeAllElements(); 
	}

	if (mHasTitle)
	{	
		mCloseButton = new CSButton(laf->getIcon(LAF_ICON_TYPE_CLOSE));
		mCloseButton->setVisible(mCloseable);
		mCloseButton->setTooltipText("Button to hide a window...");
		mCloseButton->addMessageListener(this, GUI_MESSAGE);
		
		mTitleIcon = CSTextIcon::buildIcon(mTitle);
		mTitleIcon->setVisible(true);
		mTitleIcon->addMessageListener(this, GUI_MESSAGE);
		mTitleIcon->setBorderState(0);
		mLeftIcons.push_back(mTitleIcon);

		CSIcons::iterator iter = mLeftIcons.begin();
		CSLayoutData layoutData(POSITION_WEST);
		layoutData.setStretchedVertical(false);
		layoutData.setPackedVertical(true);
		layoutData.setPackedHorizontal(true);
		layoutData.setCenteredVertical(true);

		CSIcon* loopIcon = 0;
		while (iter != mLeftIcons.end())
		{
			CSIcon* loopIcon = *iter;
			loopIcon->setBorderState(0);
			loopIcon->addMessageListener(this, GUI_MESSAGE); // doesn't matter if readded, that is handled by message listerner internally
			loopIcon->setLayoutData(layoutData);
			mTitleBar->addElement(loopIcon);
			iter++;
		}
		if (mCloseable)
		{
			layoutData.setPosition(POSITION_EAST);
			mCloseButton->setLayoutData(layoutData);
			mTitleBar->addElement(mCloseButton);
			mCloseButton->setActionId(CLOSE_ACTION_ID);
		}
	}
	if (mHorizontalScrollbar != 0) 
	{
		mHorizontalScrollbar->removeMessageListener(this);
		removeElement(mHorizontalScrollbar);
		delete(mHorizontalScrollbar);
		mHorizontalScrollbar = 0;
	}
	mHorizontalScrollbar = new CSScrollbar(TYPE_HORIZONTAL);
	mHorizontalScrollbar->addMessageListener(this, GUI_MESSAGE);
	addElementSouth(mHorizontalScrollbar);
	
	if (mVerticalScrollbar != 0)
	{
		mVerticalScrollbar->removeMessageListener(this);
		removeElement(mVerticalScrollbar);
		delete (mVerticalScrollbar);
		mVerticalScrollbar = 0;
	}
	mVerticalScrollbar = new CSScrollbar(TYPE_VERTICAL);
	mVerticalScrollbar->addMessageListener(this, GUI_MESSAGE);
	addElementEast(mVerticalScrollbar);
	
	LOG_EXIT
}

// returns whether a scrollbar was added!
// /TODO auch fr andere Layouts
// only successfull on Center = XY Layout & Border
bool CSWindow::initHorizontalScrollbar()
{
	mMinHeightInPixel = 0;
	mMinWidthInPixel = 0;
	int minHeight = 0;
	int minWidth = 0;
	std::string s;
//	s.c;

	if ((mCenter->getLayoutManager()->getType().compare( CSLayoutManagerXY::CLASS) != 0) 
		&& (mCenter->getLayoutManager()->getType().compare(CSLayoutManagerBorder::CLASS) != 0))
	{
		mHorizontalScrollbar->setVisible(false);
		return false;
	}
		
	getCenterMinSize(minWidth, minHeight);

	// setting the complete size of the to be displayed are (in pixel)
	mHorizontalScrollbar->setStart(0);
	mHorizontalScrollbar->setEnd(minWidth);

	int xAdd = 1; // adding 1 pixel per scroll unit!

	mHorizontalDisplayLength = (mCenter->getViewportWidth()) / xAdd;	// maximal displayable "items" (scrollable entities) in viewport (without scrolling)
	mHorizontalDisplayEnd = mHorizontalDisplayStart + mHorizontalDisplayLength;
	
	// if the position is already set we don't want to change it
	// it upsets the user, if upon resizing
	// the position of the scrollbar resets
	if (!mHorizontalDisplaySet)
	{
		mHorizontalDisplayStart = 0;
	}
	else
	{
		if (mHorizontalDisplayStart + mHorizontalDisplayLength >minWidth)
		{
			mHorizontalDisplayStart = minWidth - mHorizontalDisplayLength;
			if (mHorizontalDisplayStart < 0)
			{
				mHorizontalDisplayStart = 0;
			}
		}
	}
	
	mHorizontalScrollbar->setDisplayStart(mHorizontalDisplayStart);
	mHorizontalScrollbar->setDisplayEnd(mHorizontalDisplayEnd);
	mHorizontalScrollbar->setDisplayLength(mHorizontalDisplayLength);

	// if the viewport is to small - add the scollbar
	// if not - the above was all not really neccessary
	if (mHorizontalDisplayLength < minWidth)
	{
		// reset an recount the size, since we also got now the size of the scrollbar!
		mMinHeightInPixel = 0;
		mMinWidthInPixel = 0;
		getCenterMinSize(minWidth, minHeight);
		mHorizontalScrollbar->setEnd(minWidth);
		mHorizontalScrollbar->setVisible(true);
		return true;
	}
	else
	{
		mHorizontalScrollbar->setVisible(false);
	}
	return false;
}

// returns whether a scrollbar was added!
// /TODO auch fr andere Layouts
// only successfull on Center = XY Layout && Border
// exactly the same as above - just switch horizontal an vertical arround!
bool CSWindow::initVerticalScrollbar()
{
	mMinHeightInPixel = 0;
	mMinWidthInPixel = 0;
	int minHeight = 0;
	int minWidth = 0;
	if ((mCenter->getLayoutManager()->getType().compare(CSLayoutManagerXY::CLASS) != 0) && (mCenter->getLayoutManager()->getType().compare(CSLayoutManagerBorder::CLASS) != 0))
	{
		mVerticalScrollbar->setVisible(false);
		return false;
	}

	getCenterMinSize(minWidth, minHeight);
	
	mVerticalScrollbar->setStart(0);
	mVerticalScrollbar->setEnd(minHeight);

	int yAdd = 1; // adding 1 pixel per scroll unit!
	mVerticalDisplayLength = (mCenter->getViewportHeight()) / yAdd;	// maximal displayable "items" (scrollable entities) in viewport (without scrolling)
	mVerticalDisplayEnd = mVerticalDisplayStart + mVerticalDisplayLength;

	if (!mVerticalDisplaySet)
	{
		mVerticalDisplayStart = 0;
	}
	else
	{
		if (mVerticalDisplayStart + mVerticalDisplayLength >minHeight)
		{
			mVerticalDisplayStart = minHeight - mVerticalDisplayLength;
			if (mVerticalDisplayStart < 0)
			{
				mVerticalDisplayStart = 0;
			}
		}
	}

	mVerticalScrollbar->setDisplayStart(mVerticalDisplayStart);
	mVerticalScrollbar->setDisplayEnd(mVerticalDisplayEnd);
	mVerticalScrollbar->setDisplayLength(mVerticalDisplayLength);

	if (mVerticalDisplayLength < minHeight)
	{
		// reset an recount the size, since we also got now the size of the scrollbar!
		mMinHeightInPixel = 0;
		mMinWidthInPixel = 0;
		getCenterMinSize(minWidth, minHeight);
		mVerticalScrollbar->setEnd(minHeight);
		mVerticalScrollbar->setVisible(true);
		return true;
	}
	else
	{
		mVerticalScrollbar->setVisible(false);
	}
	return false;
}

// /TODO auch fr andere Layouts
// only available for XY Layout! && Border layout
void CSWindow::getCenterMinSize(int &minWidth, int &minHeight)
{
	int height = 0;
	int width = 0;
	if (mCenter->getLayoutManager()->getType().compare(CSLayoutManagerXY::CLASS) == 0)	
	{
		CSGrafikElements::iterator iter = mCenter->getElements()->begin();
		while (iter != mCenter->getElements()->end())
		{
			CSGrafikElement *visitor = *iter;
			if (!visitor->getVisible())
			{
				iter++;
				continue;
			}
			CSLayoutData visitorLayoutData = visitor->getLayoutData();

			int h = visitorLayoutData.getY() + visitor->getHeight();
			int w = visitorLayoutData.getX() + visitor->getWidth();
			if ( h > height)
			{
				height = h;
			}
			if ( w > width)
			{
				width = w;
			}
			iter++;
		}
		minHeight = height;
		minWidth = width;
	}

	else if (mCenter->getLayoutManager()->getType().compare(CSLayoutManagerBorder::CLASS) == 0)	
	{
		CSGrafikElements::iterator iter = mCenter->getElements()->begin();
		while (iter != mCenter->getElements()->end())
		{
			CSGrafikElement *visitor = *iter;
			if (!visitor->getVisible())
			{
				iter++;
				continue;
			}
			CSLayoutData visitorLayoutData = visitor->getLayoutData();

			int h = visitorLayoutData.getY() + visitor->getHeight();
			int w = visitorLayoutData.getX() + visitor->getWidth();
			if ( h > height)
			{
				height = h;
			}
			if ( w > width)
			{
				width = w;
			}
			iter++;
		}
		minHeight = height;
		minWidth = width;
	}
}

void CSWindow::setVerticalDisplayStart(int y)
{
	mVerticalDisplayStart = y;
	mVerticalScrollbar->setDisplayStart(mVerticalDisplayStart);

	mVerticalDisplayEnd = mVerticalDisplayStart + mVerticalDisplayLength;
	mVerticalScrollbar->setDisplayEnd(mVerticalDisplayEnd);

	mCenter->setViewportOffset(mHorizontalDisplayStart, mVerticalDisplayStart);
}

void CSWindow::setHorizontalDisplayStart(int x)
{
	mHorizontalDisplayStart = x;
	mHorizontalScrollbar->setDisplayStart(mHorizontalDisplayStart);

	mHorizontalDisplayEnd = mHorizontalDisplayStart + mHorizontalDisplayLength;
	mHorizontalScrollbar->setDisplayEnd(mHorizontalDisplayEnd);
	mCenter->setViewportOffset(mHorizontalDisplayStart, mVerticalDisplayStart);
}


void CSWindow::layoutSetupWindow()
{
	static char *functionName="layoutSetupWindow";
	LOG_ENTER 
	CSLAF *laf = CSLAF::getCurrentLAF();

	// the size of the viewport changes if a scrollbar is added -> every size might change...
	// looks like ping pong - but is a must :-(
	bool tester = initVerticalScrollbar();
	if (initHorizontalScrollbar())
	{
		if (initVerticalScrollbar())
		{
			initHorizontalScrollbar();
		}
	}

	// correct scrollbars
	if (mVerticalDisplayEnd > mCenter->getLayoutManager()->getMinimumHeight(mCenter))
	{
		mVerticalDisplayEnd = mCenter->getLayoutManager()->getMinimumHeight(mCenter);
		mVerticalDisplayStart = mVerticalDisplayEnd - mVerticalDisplayLength;
		if (mVerticalDisplayStart < 0)
		{
			mVerticalDisplayStart = 0;
		}
		mVerticalScrollbar->setDisplayStart(mVerticalDisplayStart);
		mVerticalScrollbar->setDisplayEnd(mVerticalDisplayEnd);
		mCenter->setViewportOffset(mHorizontalDisplayStart, mVerticalDisplayStart);
	}

	if (mHorizontalDisplayEnd > mCenter->getLayoutManager()->getMinimumWidth(mCenter))
	{
		mHorizontalDisplayEnd = mCenter->getLayoutManager()->getMinimumWidth(mCenter);
		mHorizontalDisplayStart = mHorizontalDisplayEnd - mHorizontalDisplayLength;
		if (mHorizontalDisplayStart < 0)
		{
			mHorizontalDisplayStart = 0;
		}
		mHorizontalScrollbar->setDisplayStart(mHorizontalDisplayStart);
		mHorizontalScrollbar->setDisplayEnd(mHorizontalDisplayEnd);
		mCenter->setViewportOffset(mHorizontalDisplayStart, mVerticalDisplayStart);
	}
//printf("Height: %i, min: %i, dispend: %i\n", mCenter->getHeight(), mCenter->getLayoutManager()->getMinimumHeight(mCenter), mVerticalDisplayEnd);
	// correct viewport offsets
	LOG_EXIT
}




