#ifndef CSWINDOW_h
#define CSWINDOW_h

// USES SDL_Surface
// USES SDL_Rect 

#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

#include <vector>
#include <string>

#include "CSPicture.h"
#include "CSMessage.h"
#include "CSButton.h"
#include "CSGrafikElement.h"
#include "CSDesktop.h"
#include "CSLabel.h"
#include "CSLog.h"
#include "CSBar.h"

#include "CSIcon.h"

class CSPanel;
class CSWindow;
typedef std::vector<CSWindow *> CSWindows;

class CSWindow : public CSGrafikElement
{
	private:
		CSBar *mTitleBar;
		CSIcons mLeftIcons;
		CSTextIcon *mTitleIcon;
		std::string mTitle;
		CSPicture *mPicture;

		int mDragStartX;
		int mDragStartY;
		int mSizingOffset;
		bool mCloseable;
		bool mCursorChange;
		bool mSizing;
		bool mIsSizeable;
		bool mActive;
		bool mDragging;
		bool mHasTitle;
		CSPanel *mNorth;
		CSPanel *mWest;
		CSPanel *mSouth;
		CSPanel *mEast;
		CSPanel *mCenter;

		CSScrollbar *mVerticalScrollbar;
		CSScrollbar *mHorizontalScrollbar;
		int mVerticalDisplayStart;		
		int mVerticalDisplayEnd;		
		int mVerticalDisplayLength;		
		int mHorizontalDisplayStart;
		int mHorizontalDisplayEnd;
		int mHorizontalDisplayLength;
		void getCenterMinSize(int &minWidth, int &minHeight);
		int mMinHeightInPixel;
		int mMinWidthInPixel;
		bool mHorizontalDisplaySet;
		bool mVerticalDisplaySet;

		bool initVerticalScrollbar();
		bool initHorizontalScrollbar();

	protected:
		virtual void layoutSetupWindow();
		virtual void rebuildElementWindow();
		virtual void rebuildElement() {rebuildElementWindow();}

		GuiMessage MESSAGE_WINDOW_CLOSE;
		CSButton *mCloseButton;
	public:
		CSWindow(int height, int width, bool hasTitle = true);
		CSWindow(CSPicture *picture);
		virtual ~CSWindow();
		static const char *CLASS;
		virtual std::string getType() {return (std::string) CLASS;}

		void initWindow();
		virtual void paint(SDL_Surface *destination, SDL_Rect *parentViewport);
		bool isActive() {return mActive;}
		void reactOnMessageWindow(CSMessage *message);
		virtual void reactOnMessage(CSMessage *message);
		bool isPictureWindow(void) {return mPicture != 0;}
		virtual bool isTransparent(int x, int y, SDL_Rect *parentViewport);
		void setTitle(std::string s) 
		{
			if (s.compare(mTitle) == 0)
			{
				return;
			}
			mTitle = s;
			if (mTitleIcon)
			{
				mTitleIcon->setText(mTitle);
			}
			layoutChanged();
		}
		std::string getTitle(void) {return mTitle;}
		void addLeftIcon(CSIcon *icon);
		void addElementNorth(CSGrafikElement *element);
		void addElementSouth(CSGrafikElement *element);
		void addElementEast(CSGrafikElement *element);
		void addElementWest(CSGrafikElement *element);
		bool isClosable() {return mCloseable;}
		void setClosable(bool b) 
		{
			mCloseable = b;
			if (mCloseButton)
			{
				mCloseButton->setVisible(mCloseable);
			}
		}
		void setSizeable(bool s) {mIsSizeable = s;}
		bool isSizeable() {return mIsSizeable;}
		virtual void layoutSetup() {layoutSetupWindow();}
		void setVerticalDisplayStart(int y);
		void setHorizontalDisplayStart(int x);
		int getVerticalDisplayStart() {return mVerticalDisplayStart;}
		int getHorizontalDisplayStart() {return mHorizontalDisplayStart;}
};

#endif // CSWINDOW_h
