#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

#include "GhostNavigators.h"
#include "PacmanGame.h"

const char *GhostNavigator::CLASS = "GhostNavigator";

bool GhostNavigator::preNavigate(CSDisplayParams *displayParams)
{
	static char *functionName="preNavigate";
	mNextDirection = checkDirection(getCurrentDir());
	return true;
}

int GhostNavigator::checkDirection(int currentDir)
{
	static char *functionName="checkDirection";
	if (currentDir == NAV_STAY)
	{
		int i = initializeDirection();
		return i;
	}
	if (canContinueMove(currentDir))
	{
		if (isDecisionPoint())
		{
			int i = shouldDecideNewDeirection(currentDir);
			return i;
		}
		return currentDir;
	}
	int i = mustDecideNewDirection(currentDir);
	return i;
}

int GhostNavigator::initializeDirection()
{
	static char *functionName="initializeDirection";
	forceDecision();
	if (canMoveTo(NAV_NORTH))
	{
		return NAV_NORTH;
	}
	else if (canMoveTo(NAV_EAST))
	{
		return NAV_EAST;
	}
	else if (canMoveTo(NAV_SOUTH))
	{
		return NAV_SOUTH;
	}
	else if (canMoveTo(NAV_WEST))
	{
		return NAV_WEST;
	}
	return NAV_STAY;
}

bool GhostNavigator::canContinueMove(int currentDirection)
{
	static char *functionName="canContinueMove";
	if (isDecisionPoint())
	{
		bool b = canMoveTo(currentDirection);
		return b;
	}
	return true;
}

int TurnRightNavigator::mustDecideNewDirection(int currentDir)
{
	static char *functionName="mustDecideNewDirection";
	switch (currentDir)
	{
		case NAV_NORTH:
		{
			if (canMoveTo(NAV_EAST))
			{
				return NAV_EAST;
			}
			return mustDecideNewDirection(NAV_EAST);
		}
		case NAV_EAST:
		{
			if (canMoveTo(NAV_SOUTH))
			{
				return NAV_SOUTH;
			}
			return mustDecideNewDirection(NAV_SOUTH);
		}
		case NAV_SOUTH:
		{
			if (canMoveTo(NAV_WEST))
			{
				return NAV_WEST;
			}
			return mustDecideNewDirection(NAV_WEST);
		}
		case NAV_WEST:
		{
			if (canMoveTo(NAV_NORTH))
			{
				return NAV_NORTH;
			}
			return mustDecideNewDirection(NAV_NORTH);
		}
	}
	return NAV_STAY; 
}


int TurnLeftRightNavigator::mustDecideNewDirection(int currentDir)
{
	if (mLastChoice == 1)
	{
		switch (currentDir)
		{
			case NAV_NORTH:
			{
				if (canMoveTo(NAV_EAST))
				{
					mLastChoice = (mLastChoice +1)%2;
					return NAV_EAST;
				}
				return mustDecideNewDirection(NAV_EAST);
			}
			case NAV_EAST:
			{
				if (canMoveTo(NAV_SOUTH))
				{
					mLastChoice = (mLastChoice +1)%2;
					return NAV_SOUTH;
				}
				return mustDecideNewDirection(NAV_SOUTH);
			}
			case NAV_SOUTH:
			{
				if (canMoveTo(NAV_WEST))
				{
					mLastChoice = (mLastChoice +1)%2;
					return NAV_WEST;
				}
				return mustDecideNewDirection(NAV_WEST);
			}
			case NAV_WEST:
			{
				if (canMoveTo(NAV_NORTH))
				{
					mLastChoice = (mLastChoice +1)%2;
					return NAV_NORTH;
				}
				return mustDecideNewDirection(NAV_NORTH);
			}
		}
	}
	else
	{
		switch (currentDir)
		{
			case NAV_NORTH:
			{
				if (canMoveTo(NAV_WEST))
				{
					mLastChoice = (mLastChoice +1)%2;
					return NAV_WEST;
				}
				return mustDecideNewDirection(NAV_WEST);
			}
			case NAV_EAST:
			{
				if (canMoveTo(NAV_NORTH))
				{
					mLastChoice = (mLastChoice +1)%2;
					return NAV_NORTH;
				}
				return mustDecideNewDirection(NAV_NORTH);
			}
			case NAV_SOUTH:
			{
				if (canMoveTo(NAV_EAST))
				{
					mLastChoice = (mLastChoice +1)%2;
					return NAV_EAST;
				}
				return mustDecideNewDirection(NAV_EAST);
			}
			case NAV_WEST:
			{
				if (canMoveTo(NAV_SOUTH))
				{
					mLastChoice = (mLastChoice +1)%2;
					return NAV_SOUTH;
				}
				return mustDecideNewDirection(NAV_SOUTH);
			}
		}
	}
	mLastChoice = (mLastChoice +1)%2;
	return NAV_STAY; 
}

int FollowWallRightNavigator::mustDecideNewDirection(int currentDir)
{
	if (!mChecking)
	{
		mChecking = true;
		mCurrentStartDir = currentDir;
		switch (currentDir)
		{
			case NAV_NORTH:
			{
				currentDir = NAV_SOUTH;
				break;
			}
			case NAV_EAST:
			{
				currentDir = NAV_WEST;
				break;
			}
			case NAV_SOUTH:
			{
				currentDir = NAV_NORTH;
				break;
			}
			case NAV_WEST:
			{
				currentDir = NAV_EAST;
				break;
			}
		}
	}

	switch (currentDir)
	{
		case NAV_NORTH:
		{
			if (canMoveTo(NAV_EAST))
			{
				mChecking = false;
				return NAV_EAST;
			}
			return mustDecideNewDirection(NAV_EAST);
		}
		case NAV_EAST:
		{
			if (canMoveTo(NAV_SOUTH))
			{
				mChecking = false;
				return NAV_SOUTH;
			}
			return mustDecideNewDirection(NAV_SOUTH);
		}
		case NAV_SOUTH:
		{
			if (canMoveTo(NAV_WEST))
			{
				mChecking = false;
				return NAV_WEST;
			}
			return mustDecideNewDirection(NAV_WEST);
		}
		case NAV_WEST:
		{
			if (canMoveTo(NAV_NORTH))
			{
				mChecking = false;
				return NAV_NORTH;
			}
			return mustDecideNewDirection(NAV_NORTH);
		}
	}
	mChecking = false;
	return NAV_STAY; 
}

int ChaserNavigator::mustDecideNewDirection(int currentDir)
{
	int mx = mSprite->getXPos();
	int my = mSprite->getYPos();
	
	int ox = mGame->getMainSprite()->getXPos();
	int oy = mGame->getMainSprite()->getYPos();

	int diffX = mx-ox;
	int diffY = my-oy;
	int wanted1;
	int wanted2;
	int wanted3;
	int wanted4;

	int dirOrder1;
	int dirOrder2;
	int dirOrder3;
	int dirOrder4;

	if (ABS(diffX) > ABS(diffY))
	{
		if (ox>mx) // EAST
		{
			dirOrder1= NAV_EAST;
			dirOrder4 = NAV_WEST;
			if (oy>my) // EAST
			{
				dirOrder2= NAV_SOUTH;
				dirOrder3 = NAV_NORTH;
			}
			else
			{
				dirOrder3= NAV_SOUTH;
				dirOrder2 = NAV_NORTH;
			}
		}
		else
		{
			dirOrder4= NAV_EAST;
			dirOrder1 = NAV_WEST;
			if (oy>my) // EAST
			{
				dirOrder2= NAV_SOUTH;
				dirOrder3 = NAV_NORTH;
			}
			else
			{
				dirOrder3= NAV_SOUTH;
				dirOrder2 = NAV_NORTH;
			}
		}
	}
	else
	{
		if (oy>my) // EAST
		{
			dirOrder1= NAV_SOUTH;
			dirOrder4 = NAV_NORTH;
			if (oy>my) // EAST
			{
				dirOrder2= NAV_EAST;
				dirOrder3 = NAV_WEST;
			}
			else
			{
				dirOrder3= NAV_EAST;
				dirOrder2 = NAV_WEST;
			}
		}
		else
		{
			dirOrder4= NAV_SOUTH;
			dirOrder1 = NAV_NORTH; 
			if (oy>my) // EAST
			{
				dirOrder2= NAV_EAST;
				dirOrder3 = NAV_WEST;
			}
			else
			{
				dirOrder3= NAV_EAST;
				dirOrder2 = NAV_WEST;
			}
		}
	}
	
	if (dirOrder1 == getOpposite(currentDir))
	{
		wanted1 = dirOrder2;
		wanted2 = dirOrder3;
		wanted3 = dirOrder4;
		wanted4 = dirOrder1;
	}

	else if (dirOrder2 == getOpposite(currentDir))
	{
		wanted1 = dirOrder1;
		wanted2 = dirOrder3;
		wanted3 = dirOrder4;
		wanted4 = dirOrder2;
	}

	else if (dirOrder3 == getOpposite(currentDir))
	{
		wanted1 = dirOrder1;
		wanted2 = dirOrder2;
		wanted3 = dirOrder4;
		wanted4 = dirOrder3;
	}
	else
	{
		wanted1 = dirOrder1;
		wanted2 = dirOrder2;
		wanted3 = dirOrder3;
		wanted4 = dirOrder4;
	}

	if (canMoveTo(wanted1)) return wanted1;
	if (canMoveTo(wanted2)) return wanted2;
	if (canMoveTo(wanted3)) return wanted3;
	if (canMoveTo(wanted4)) return wanted4;
	return NAV_STAY; 
}

int PrudentNavigator::flight(int currentDir)
{
	int mx = mSprite->getXPos();
	int my = mSprite->getYPos();
	
	int ox = mGame->getMainSprite()->getXPos();
	int oy = mGame->getMainSprite()->getYPos();

	int diffX = mx-ox;
	int diffY = my-oy;
	int wanted1;
	int wanted2;
	int wanted3;
	int wanted4;

	int dirOrder1;
	int dirOrder2;
	int dirOrder3;
	int dirOrder4;

	if (ABS(diffX) < ABS(diffY))
	{
		if (ox<mx) // EAST
		{
			dirOrder1= NAV_EAST;
			dirOrder4 = NAV_WEST;
			if (oy<my) // EAST
			{
				dirOrder2= NAV_SOUTH;
				dirOrder3 = NAV_NORTH;
			}
			else
			{
				dirOrder3= NAV_SOUTH;
				dirOrder2 = NAV_NORTH;
			}
		}
		else
		{
			dirOrder4= NAV_EAST;
			dirOrder1 = NAV_WEST;
			if (oy<my) // EAST
			{
				dirOrder2= NAV_SOUTH;
				dirOrder3 = NAV_NORTH;
			}
			else
			{
				dirOrder3= NAV_SOUTH;
				dirOrder2 = NAV_NORTH;
			}
		}
	}
	else
	{
		if (oy<my) // EAST
		{
			dirOrder1= NAV_SOUTH;
			dirOrder4 = NAV_NORTH;
			if (oy<my) // EAST
			{
				dirOrder2= NAV_EAST;
				dirOrder3 = NAV_WEST;
			}
			else
			{
				dirOrder3= NAV_EAST;
				dirOrder2 = NAV_WEST;
			}
		}
		else
		{
			dirOrder4= NAV_SOUTH;
			dirOrder1 = NAV_NORTH; 
			if (oy<my) // EAST
			{
				dirOrder2= NAV_EAST;
				dirOrder3 = NAV_WEST;
			}
			else
			{
				dirOrder3= NAV_EAST;
				dirOrder2 = NAV_WEST;
			}
		}
	}
	
	if (dirOrder1 == getOpposite(currentDir))
	{
		wanted1 = dirOrder2;
		wanted2 = dirOrder3;
		wanted3 = dirOrder4;
		wanted4 = dirOrder1;
	}

	else if (dirOrder2 == getOpposite(currentDir))
	{
		wanted1 = dirOrder1;
		wanted2 = dirOrder3;
		wanted3 = dirOrder4;
		wanted4 = dirOrder2;
	}

	else if (dirOrder3 == getOpposite(currentDir))
	{
		wanted1 = dirOrder1;
		wanted2 = dirOrder2;
		wanted3 = dirOrder4;
		wanted4 = dirOrder3;
	}
	else
	{
		wanted1 = dirOrder1;
		wanted2 = dirOrder2;
		wanted3 = dirOrder3;
		wanted4 = dirOrder4;
	}

	if (canMoveTo(wanted1)) return wanted1;
	if (canMoveTo(wanted2)) return wanted2;
	if (canMoveTo(wanted3)) return wanted3;
	if (canMoveTo(wanted4)) return wanted4;
	return NAV_STAY; 
}

int PrudentNavigator::chase(int currentDir)
{
	int mx = mSprite->getXPos();
	int my = mSprite->getYPos();
	
	int ox = mGame->getMainSprite()->getXPos();
	int oy = mGame->getMainSprite()->getYPos();

	int diffX = mx-ox;
	int diffY = my-oy;
	int wanted1;
	int wanted2;
	int wanted3;
	int wanted4;

	int dirOrder1;
	int dirOrder2;
	int dirOrder3;
	int dirOrder4;

	if (ABS(diffX) > ABS(diffY))
	{
		if (ox>mx) // EAST
		{
			dirOrder1= NAV_EAST;
			dirOrder4 = NAV_WEST;
			if (oy>my) // EAST
			{
				dirOrder2= NAV_SOUTH;
				dirOrder3 = NAV_NORTH;
			}
			else
			{
				dirOrder3= NAV_SOUTH;
				dirOrder2 = NAV_NORTH;
			}
		}
		else
		{
			dirOrder4= NAV_EAST;
			dirOrder1 = NAV_WEST;
			if (oy>my) // EAST
			{
				dirOrder2= NAV_SOUTH;
				dirOrder3 = NAV_NORTH;
			}
			else
			{
				dirOrder3= NAV_SOUTH;
				dirOrder2 = NAV_NORTH;
			}
		}
	}
	else
	{
		if (oy>my) // EAST
		{
			dirOrder1= NAV_SOUTH;
			dirOrder4 = NAV_NORTH;
			if (oy>my) // EAST
			{
				dirOrder2= NAV_EAST;
				dirOrder3 = NAV_WEST;
			}
			else
			{
				dirOrder3= NAV_EAST;
				dirOrder2 = NAV_WEST;
			}
		}
		else
		{
			dirOrder4= NAV_SOUTH;
			dirOrder1 = NAV_NORTH; 
			if (oy>my) // EAST
			{
				dirOrder2= NAV_EAST;
				dirOrder3 = NAV_WEST;
			}
			else
			{
				dirOrder3= NAV_EAST;
				dirOrder2 = NAV_WEST;
			}
		}
	}
	
	if (dirOrder1 == getOpposite(currentDir))
	{
		wanted1 = dirOrder2;
		wanted2 = dirOrder3;
		wanted3 = dirOrder4;
		wanted4 = dirOrder1;
	}

	else if (dirOrder2 == getOpposite(currentDir))
	{
		wanted1 = dirOrder1;
		wanted2 = dirOrder3;
		wanted3 = dirOrder4;
		wanted4 = dirOrder2;
	}

	else if (dirOrder3 == getOpposite(currentDir))
	{
		wanted1 = dirOrder1;
		wanted2 = dirOrder2;
		wanted3 = dirOrder4;
		wanted4 = dirOrder3;
	}
	else
	{
		wanted1 = dirOrder1;
		wanted2 = dirOrder2;
		wanted3 = dirOrder3;
		wanted4 = dirOrder4;
	}

	if (canMoveTo(wanted1)) return wanted1;
	if (canMoveTo(wanted2)) return wanted2;
	if (canMoveTo(wanted3)) return wanted3;
	if (canMoveTo(wanted4)) return wanted4;
	return NAV_STAY; 
}

int PrudentNavigator::mustDecideNewDirection(int currentDir)
{
/*
const int GH0ST_DEFAULT_STATE = 0;
const int GH0ST_BLUE_STATE = 1;
const int GH0ST_EYE_STATE = 2;
const int GH0ST_SCORE_STATE = 6;
const int GH0ST_SCORE_STATE_200 = 6+8;
const int GH0ST_SCORE_STATE_400 = 6+16;
const int GH0ST_SCORE_STATE_800 = 6+32;
const int GH0ST_SCORE_STATE_1600 = 6+64;

*/
	int state = mSprite->getState().mCurrentBundleId;
	switch (state)
	{
		case GH0ST_DEFAULT_STATE:
		{
			return chase(currentDir);
		}
		case GH0ST_BLUE_STATE:
		{
			return flight(currentDir);
		}
		case GH0ST_EYE_STATE:
		case GH0ST_SCORE_STATE:
		case GH0ST_SCORE_STATE_200:
		case GH0ST_SCORE_STATE_400:
		case GH0ST_SCORE_STATE_800:
		case GH0ST_SCORE_STATE_1600:
		{
			return seekHome(currentDir);
		}
	}
	return NAV_STAY; 
}

int PrudentNavigator::seekHome(int currentDir)
{

	if (checkIfHome())
	{
		mustDecideNewDirection(currentDir);
	}

	int ox;
	int oy;
	getNearestHome(ox, oy);

	int mx = mSprite->getXPos();
	int my = mSprite->getYPos();

	int diffX = mx-ox;
	int diffY = my-oy;
	int wanted1;
	int wanted2;
	int wanted3;
	int wanted4;

	int dirOrder1;
	int dirOrder2;
	int dirOrder3;
	int dirOrder4;

	if (ABS(diffX) > ABS(diffY))
	{
		if (ox>mx) // EAST
		{
			dirOrder1= NAV_EAST;
			dirOrder4 = NAV_WEST;
			if (oy>my) // EAST
			{
				dirOrder2= NAV_SOUTH;
				dirOrder3 = NAV_NORTH;
			}
			else
			{
				dirOrder3= NAV_SOUTH;
				dirOrder2 = NAV_NORTH;
			}
		}
		else
		{
			dirOrder4= NAV_EAST;
			dirOrder1 = NAV_WEST;
			if (oy>my) // EAST
			{
				dirOrder2= NAV_SOUTH;
				dirOrder3 = NAV_NORTH;
			}
			else
			{
				dirOrder3= NAV_SOUTH;
				dirOrder2 = NAV_NORTH;
			}
		}
	}
	else
	{
		if (oy>my) // EAST
		{
			dirOrder1= NAV_SOUTH;
			dirOrder4 = NAV_NORTH;
			if (oy>my) // EAST
			{
				dirOrder2= NAV_EAST;
				dirOrder3 = NAV_WEST;
			}
			else
			{
				dirOrder3= NAV_EAST;
				dirOrder2 = NAV_WEST;
			}
		}
		else
		{
			dirOrder4= NAV_SOUTH;
			dirOrder1 = NAV_NORTH; 
			if (oy>my) // EAST
			{
				dirOrder2= NAV_EAST;
				dirOrder3 = NAV_WEST;
			}
			else
			{
				dirOrder3= NAV_EAST;
				dirOrder2 = NAV_WEST;
			}
		}
	}
	
	if (dirOrder1 == getOpposite(currentDir))
	{
		wanted1 = dirOrder2;
		wanted2 = dirOrder3;
		wanted3 = dirOrder4;
		wanted4 = dirOrder1;
	}

	else if (dirOrder2 == getOpposite(currentDir))
	{
		wanted1 = dirOrder1;
		wanted2 = dirOrder3;
		wanted3 = dirOrder4;
		wanted4 = dirOrder2;
	}

	else if (dirOrder3 == getOpposite(currentDir))
	{
		wanted1 = dirOrder1;
		wanted2 = dirOrder2;
		wanted3 = dirOrder4;
		wanted4 = dirOrder3;
	}
	else
	{
		wanted1 = dirOrder1;
		wanted2 = dirOrder2;
		wanted3 = dirOrder3;
		wanted4 = dirOrder4;
	}

	if (canMoveTo(wanted1)) return wanted1;
	if (canMoveTo(wanted2)) return wanted2;
	if (canMoveTo(wanted3)) return wanted3;
	if (canMoveTo(wanted4)) return wanted4;
	return NAV_STAY; 
}

#define _ABS_(a)    (((a) < (0)) ? (-(a)) : (a))

void PrudentNavigator::getNearestHome(int &ox, int &oy)
{
	int mx = mSprite->getXPos();
	int my = mSprite->getYPos();
	int layer = mLayer +1; // dots assumes one layer above labyrinth
	CSTileMap *map = mGame->getWorld()->getLayer(layer);
	int lw = map->getTileWidth();
	int lh = map->getTileHeight();
	ox = mx;
	oy = my;
	CSTile *tile;
	int tx;
	int ty;
	unsigned int nDistance = -1;
	for (int y=0; y < lh; y++)
	{
		for (int x=0; x < lw; x++)
		{
			tile = map->getTile(x,y);
			if (tile->getIDChar() == 'g')
			{
				// Ghost Home gefunden!!!
				tx = tile->getX();
				ty = tile->getY();
				if ((_ABS_(tx-mx) < nDistance) || (_ABS_(ty-my) < nDistance))
				{
					ox = tx;
					oy = ty;
				}
			}
		}
	}
}

bool PrudentNavigator::checkIfHome()
{
	int mx = mSprite->getXPos();
	int my = mSprite->getYPos();
	int layer = mLayer +1; // dots assumes one layer above labyrinth
	
	CSTiles tiles = mGame->getWorld()->getLayer(layer)->getTouchingTiles(mx+mSprite->getMaxWidth()/2,my+mSprite->getMaxHeight()/2, 1, 1);
	for (CSTiles::iterator iter = tiles.begin(); iter != tiles.end(); iter++)
	{
		if ((*iter)->getIDChar() == 'g')
		{
			MESSAGE_RESURRECT_GHOST.setSubtype(RESURRECT_GHOST_MESSAGE);
			MESSAGE_RESURRECT_GHOST.sprite = mSprite;
			mGame->sendMessage(MESSAGE_RESURRECT_GHOST);
			return true;
		}
	}
	
	return false;
}
