#ifndef GHOSTNAVIGATOR_H
#define GHOSTNAVIGATOR_H

#include "MazeNavigator.h"
#include "CSMessage.h"
#include "CSLog.h"

class GhostNavigator : public MazeNavigator
{
	private:
		int initializeDirection();
		int checkDirection(int currentDir);
	protected:
		virtual bool canContinueMove(int currentDir);
		virtual int shouldDecideNewDeirection(int currentDir) {return currentDir;}
		virtual int mustDecideNewDirection(int currentDir) = 0;
	public:
		static const char *CLASS;
		virtual std::string getType() {return (std::string) CLASS;}
		virtual bool preNavigate(CSDisplayParams *displayParams);
};

class TurnRightNavigator : public GhostNavigator
{
	protected:
		virtual int mustDecideNewDirection(int currentDir);
};

class TurnLeftRightNavigator : public GhostNavigator
{
	private:
		int mLastChoice;
	protected:
		virtual int mustDecideNewDirection(int currentDir);
	public :
		TurnLeftRightNavigator() {mLastChoice =1;};
};

class FollowWallRightNavigator : public GhostNavigator
{
	private:
		int mCurrentStartDir;
		bool mChecking;
	protected:
		virtual int mustDecideNewDirection(int currentDir);
		virtual int shouldDecideNewDeirection(int currentDir) {return mustDecideNewDirection(currentDir);}
	public :
		FollowWallRightNavigator() {mChecking = false;};
};

class PrudentNavigator;

class ChaserNavigator : public GhostNavigator
{
	friend PrudentNavigator;
	protected:
		virtual int mustDecideNewDirection(int currentDir);
		virtual int shouldDecideNewDeirection(int currentDir) {return mustDecideNewDirection(currentDir);}
	public :
};

class PrudentNavigator : public GhostNavigator
{
	private:
		void getNearestHome(int &ox, int &oy);
		bool checkIfHome();
		int seekHome(int currentDir);
		int flight(int currentDir);
		int chase(int currentDir);
		GameMessage MESSAGE_RESURRECT_GHOST;
	protected:
		virtual int mustDecideNewDirection(int currentDir);
		virtual int shouldDecideNewDeirection(int currentDir) {return mustDecideNewDirection(currentDir);}
	public:
};

#endif GHOSTNAVIGATOR_H
