#ifdef WIN32
#pragma warning(disable : 4786 )
#endif

#include "MazeNavigator.h"
#include "CSGame.h"

const char *MazeNavigator::CLASS = "MazeNavigator";

MazeNavigator::MazeNavigator()
{
	static char *functionName="MazeNavigator";
	LOG_ENTER 
	mNextDirection = -1;
	mLastDirection = -1;
	mSpriteWidth = -1;
	mSpriteHeight = -1;
	LOG_EXIT
}

MazeNavigator::~MazeNavigator()
{
	static char *functionName="~MazeNavigator";
	LOG_ENTER 
	LOG_EXIT
}

void MazeNavigator::navigate(CSDisplayParams *displayParams)
{
	static char *functionName="navigate";
	int y = mSprite->getYPos();
	int x = mSprite->getXPos();
	if ( (y+mSpriteHeight > mGame->getWorld()->getHeight()) || (y < 0) || (x+ mSpriteWidth > mGame->getWorld()->getWidth()) || (x < 0))
	{
		switch (mNextDirection)
		{
			case NAV_NORTH:
			{
				if (getCurrentDir() == NAV_SOUTH)
				{
					mSprite->setPosition(ALIGNX(x), y, mSprite->getWorldPos());
					mSprite->dispatchMessage(&mGame->MESSAGE_UP);
					mLastDirection = mNextDirection;
				}
				break;
			}
			case NAV_SOUTH:
			{
				if (getCurrentDir() == NAV_NORTH)
				{
					mSprite->setPosition(ALIGNX(x), y, mSprite->getWorldPos());
					mSprite->dispatchMessage(&mGame->MESSAGE_DOWN);
					mLastDirection = mNextDirection;
				}
				break;
			}
			case NAV_EAST:
			{
				if (getCurrentDir() == NAV_WEST)
				{
					mSprite->setPosition(x, ALIGNY(y), mSprite->getWorldPos());
					mSprite->dispatchMessage(&mGame->MESSAGE_RIGHT);
					mLastDirection = mNextDirection;
				}
				break;
			}
			case NAV_WEST:
			{
				if (getCurrentDir() == NAV_EAST)
				{
					mSprite->setPosition(x, ALIGNY(y), mSprite->getWorldPos());
					mSprite->dispatchMessage(&mGame->MESSAGE_LEFT);
					mLastDirection = mNextDirection;
				}
				break;
			}
		}
		return;
	}
	bool didChange = false;
	mX = (x-X_OFFSET) / X_MOVE_OK;
	mY = (y-Y_OFFSET) / Y_MOVE_OK;
	if (x-X_OFFSET < 0) mX = -1;
	if (y-Y_OFFSET < 0) mY = -1;
	if (!preNavigate(displayParams))
	{
		return;
	}

	if (mNextDirection != mLastDirection)
	{
		if (canMoveTo(mNextDirection))
		{
			didChange = true;
			switch (mNextDirection)
			{
				case NAV_NORTH:
				{
					mSprite->setPosition(ALIGNX(x), y-1, mSprite->getWorldPos());
					mSprite->dispatchMessage(&mGame->MESSAGE_UP);
					break;
				}
				case NAV_SOUTH:
				{
					mSprite->setPosition(ALIGNX(x), y+1, mSprite->getWorldPos());
					mSprite->dispatchMessage(&mGame->MESSAGE_DOWN);
					break;
				}
				case NAV_EAST:
				{
					mSprite->setPosition(x+1, ALIGNY(y), mSprite->getWorldPos());
					mSprite->dispatchMessage(&mGame->MESSAGE_RIGHT);
					break;
				}
				case NAV_WEST:
				{
					mSprite->setPosition(x-1, ALIGNY(y), mSprite->getWorldPos());
					mSprite->dispatchMessage(&mGame->MESSAGE_LEFT);
					break;
				}
			}
			mLastDirection = mNextDirection;
		}
	}
	if (!didChange)
	{
		int currentDirection = getCurrentDir();
		if ((currentDirection == NAV_NORTH) || (currentDirection == NAV_SOUTH))
		{
			if (isDecisionPointY())
			{
				if (!canMoveTo(currentDirection))
				{
					mSprite->setPosition(x, ALIGNY(y), mSprite->getWorldPos());
					mSprite->dispatchMessage(&mGame->MESSAGE_DEFAULT);
					mLastDirection = NAV_STAY;
					mNextDirection = mLastDirection;
				}
			}
		}
		if ((currentDirection == NAV_EAST) || (currentDirection == NAV_WEST))
		{
			if (isDecisionPointX())
			{
				if (!canMoveTo(currentDirection))
				{
					mSprite->setPosition(ALIGNX(x), y, mSprite->getWorldPos());
					mSprite->dispatchMessage(&mGame->MESSAGE_DEFAULT);
					mLastDirection = NAV_STAY;
					mNextDirection = mLastDirection;
				}
			}
		}
	}

	mLastX = (mSprite->getXPos()-X_OFFSET) / X_MOVE_OK;
	mLastY = (mSprite->getYPos()-Y_OFFSET) / Y_MOVE_OK;
}

void MazeNavigator::reset()
{
	static char *functionName="reset";
	LOG_ENTER 
	mLastDirection = -1;
	mLastX = -1;
	mLastY = -1;
	mX = -1;
	mY = -1;
	LOG_EXIT
}

int MazeNavigator::getCurrentDir()
{
	static char *functionName="getCurrentDir";
	if (mSprite->getCurrentActionType() == NORTH_ACTION)
	{
		return NAV_NORTH;
	}
	if (mSprite->getCurrentActionType() == SOUTH_ACTION)
	{
		return NAV_SOUTH;
	}
	if (mSprite->getCurrentActionType() == EAST_ACTION)
	{
		return NAV_EAST;
	}
	if (mSprite->getCurrentActionType() == WEST_ACTION)
	{
		return NAV_WEST;
	}
	return NAV_STAY;
}

bool MazeNavigator::canMoveTo(int dir)
{
	static char *functionName="canMoveTo";
	if (mSpriteWidth == -1)
	{
		mSpriteWidth = mSprite->getMaxWidth();
		mSpriteHeight = mSprite->getMaxHeight();
	}
	int currentDir = getCurrentDir();
	switch (dir)
	{
		case NAV_NORTH:
		{
			if (currentDir == NAV_SOUTH)
			{
				return true;
			}
			unsigned int y = mSprite->getYPos(); 
			unsigned int x = mSprite->getXPos(); 
			if ((!isDecisionPointX()) && (!BORDERX(x)))
			{
				break;
			}
			CSTile *tile = mGame->getWorld()->getTileAtPixel(x,y,mLayer);
			return !((tile->getTileType() & TEST_NORTH) == TEST_NORTH);
		}
		case NAV_SOUTH:
		{
			if (currentDir == NAV_NORTH)
			{
				return true;
			}
			unsigned int y = mSprite->getYPos() + mSpriteHeight; // height
			unsigned int x = mSprite->getXPos(); 
			if ((!isDecisionPointX()) && (!BORDERX(x)))
			{
				break;
			}
			CSTile *tile = mGame->getWorld()->getTileAtPixel(x,y,mLayer);
			return !((tile->getTileType() & TEST_SOUTH) == TEST_SOUTH);
		}
		case NAV_EAST:
		{
			if (currentDir == NAV_WEST)
			{
				return true;
			}
			unsigned int y = mSprite->getYPos(); 
			unsigned int x = mSprite->getXPos() + mSpriteWidth; // width
			if ((!isDecisionPointY()) && (!BORDERY(y)))
			{
				break;
			}
			CSTile *tile = mGame->getWorld()->getTileAtPixel(x,y,mLayer);
			return !((tile->getTileType() & TEST_EAST) == TEST_EAST);
		}
		case NAV_WEST:
		{
			if (currentDir == NAV_EAST)
			{
				return true;
			}
			unsigned int y = mSprite->getYPos(); 
			unsigned int x = mSprite->getXPos(); 
			if ((!isDecisionPointY()) && (!BORDERY(y)))
			{
				break;
			}
			CSTile *tile = mGame->getWorld()->getTileAtPixel(x,y,mLayer);
			return !((tile->getTileType() & TEST_WEST) == TEST_WEST);
		}
	}
	return false;
}
