#ifndef NAZENAVIGATOR_H
#define NAZENAVIGATOR_H

#include "CSNavigator.h"
#include "CSLog.h"

const int X_MOVE_OK = 50;	// TileSize X
const int Y_MOVE_OK = 40;	// TileSize X
const int X_OFFSET = 30;	// Pacman middle X
const int Y_OFFSET = 24;	// Pacman middle y

const int NAV_NORTH = 0;
const int NAV_SOUTH = 1;
const int NAV_WEST  = 2;
const int NAV_EAST  = 3;
const int NAV_STAY  = 4;
const int NAV_START  = -1;

const int TEST_NORTH = 1;
const int TEST_SOUTH = 2;
const int TEST_EAST  = 4;
const int TEST_WEST  = 8;

#define ABS(x) (((x)>=0) ? (x) : -(x))
#define ALIGNX(x) (((((x+(X_OFFSET-1))-X_OFFSET) / X_MOVE_OK) * X_MOVE_OK)+X_OFFSET)
#define ALIGNY(y) (((((y+(Y_OFFSET-1))-Y_OFFSET) / Y_MOVE_OK) * Y_MOVE_OK)+Y_OFFSET)
#define BORDERX(x) (((x-X_OFFSET) % X_MOVE_OK) == 0)
#define BORDERY(y) (((y-Y_OFFSET) % Y_MOVE_OK) == 0)

class MazeNavigator : public CSNavigator
{
	private:
		int mSpriteWidth;
		int mSpriteHeight;
		int mLastDirection;
		int mLastX;
		int mLastY;
		int mX;
		int mY;
	protected:
		int mLayer;
		int mNextDirection;
		int getCurrentDir();
		void forceDecision(){mLastX = mX+1;mLastY = mY+1;}
		bool canMoveTo(int dir);
		
		bool isDecisionPointX() {return ABS(mX - mLastX) == 1;}
		bool isDecisionPointY() {return ABS(mY - mLastY) == 1;}
		bool isDecisionPoint() {return (isDecisionPointX() || isDecisionPointY());}

	public:
		MazeNavigator();
		~MazeNavigator();
		static const char *CLASS;
		virtual std::string getType() {return (std::string) CLASS;}

		virtual void navigate(CSDisplayParams *displayParams);
		virtual bool preNavigate(CSDisplayParams *displayParams) {return true;}
		virtual void setLayer(int layer){mLayer = layer;}
		virtual void reset();

		static int getOpposite(int dir)
		{
			if (dir == NAV_NORTH)
			{
				return NAV_SOUTH;
			}
			else if (dir == NAV_SOUTH)
			{
				return NAV_NORTH;
			}
			else if (dir == NAV_EAST)
			{
				return NAV_WEST;
			}
			else if (dir == NAV_WEST)
			{
				return NAV_EAST;
			}
			return NAV_STAY;
		}
		static int getClock(int dir)
		{
			if (dir == NAV_NORTH)
			{
				return NAV_EAST;
			}
			else if (dir == NAV_SOUTH)
			{
				return NAV_WEST;
			}
			else if (dir == NAV_EAST)
			{
				return NAV_SOUTH;
			}
			else if (dir == NAV_WEST)
			{
				return NAV_NORTH;
			}
			return NAV_STAY;
		}
		static int getAntiClock(int dir)
		{
			if (dir == NAV_NORTH)
			{
				return NAV_WEST;
			}
			else if (dir == NAV_SOUTH)
			{
				return NAV_EAST;
			}
			else if (dir == NAV_EAST)
			{
				return NAV_NORTH;
			}
			else if (dir == NAV_WEST)
			{
				return NAV_SOUTH;
			}
			return NAV_STAY;
		}

};

#endif // NAZENAVIGATOR_H
