#ifndef PACMANGAME_H
#define PACMANGAME_H

// USES SDL_Rect 

#include "CSGame.h"
#include "CSWindow.h"
#include "CSLog.h"
#include "PacmanLAF.h"

class CSSprite;
class CSCombobox;
class CSTextfield;
class RedGhostNavigator;
class PacmanNavigator;
class PacmanConfigWindow;

// (sprite) states as in XML Definition
const int GH0ST_DEFAULT_STATE = 0;
const int GH0ST_BLUE_STATE = 1;
const int GH0ST_EYE_STATE = 2;
const int GH0ST_SCORE_STATE = 6;
const int GH0ST_SCORE_STATE_200 = 6+8;
const int GH0ST_SCORE_STATE_400 = 6+16;
const int GH0ST_SCORE_STATE_800 = 6+32;
const int GH0ST_SCORE_STATE_1600 = 6+64;

// event ids
const int GHOST_TURN_NORMAL = 0;
const int GHOST_TURN_FLICKER = 1;
const int GHOST_TURN_EYES = 2;
const int RESET_LEVEL = 3;
const int DISPLAY_BONUS_EVENT = 4;
const int REMOVE_BONUS_EVENT = 5;

// SubType GameMessage
const int RESURRECT_GHOST_MESSAGE = LAST_GAME_MESSAGE+1;

const int BONUS_WAIT_TIME = 20000; // bonus will appear every 20 seconds
const int BONUS_DISPLAY_TIME = 10000; // bonus will be visible for 10 seconds
class PacmanGame : public CSGame
{
	friend PacmanConfigWindow;
	private:
		CSSprite *mPacman;
		PacmanNavigator *mNavigator;
		void adjustPositionToTile(CSSprite *sprite);

		int mLevelCurrentGhostScore;
		int mLevelCurrentDotCount;
		int mLevelCurrentPillCount;
		int mLevelCurrentPillCountInARow;

		int mLevelDotCount;
		int mLevelPillCount;
		int mLevelGhostAfraidTime;
		int mLevelGhostBlueSpeed;
		int mLevelGhostEyeSpeed;
		int mLevelBonusValue;
		int mLevelBonusX;
		int mLevelBonusY;
		double mScaleFactor;
		std::string mLevelBonusFilename;
		std::string mScaleCenterSpriteId;
		CSTile *mBonusSaveTile;
		CSTile *mBonusTile;
		
		void removeBonus();
		void reInitLevel();
		void makeGhostsBlue();
		void makeGhostsNormal();
		CSTile * PacmanGame::buildBonusTile(std::string bonusName);

		CSPicture *mSkinUp;
		CSPicture *mSkinDown;
		CSPicture *mSkinLeft;
		CSPicture *mSkinRight;
		CSPicture *mSkinHorizontalLeft;
		CSPicture *mSkinHorizontalRight;
		CSPicture *mSkinVerticalMid;
		CSMenu *mPacmanMenu;
		CSMenuItem *mMenuItemConfiguration;
		PacmanConfigWindow *mPacmanConfigWindow;
		PacmanLAF mPacmanLAF;
		CSLAF mWindowLAF;

	protected:
		virtual SDL_Rect getDisplayArea();
		virtual SDL_Rect getScaledDisplayArea();
		virtual double getScaleFactor();

		virtual void initLevelSpecific(CSLevelData *data);
		virtual void freeLevelSpecific();
		virtual void resetSpritesSpecific();
		virtual void displayGameData();
		virtual void startRun();
	
	public:
		PacmanGame(std::string gameFileName);
		virtual ~PacmanGame();
		virtual void intro();
		static const char *CLASS;
		virtual std::string getType() {return (std::string) CLASS;}
		void reactOnMessagePacmanGame(CSMessage *message);
		virtual void reactOnMessage(CSMessage *message);
		virtual void reactOnTimedEvent(TimedEvent *event);
		virtual void loadLevelDataSpecific(CSXMLHelper &xmlSupport);
		int getDotLayer();
		int getMazeLayer();
		int getGhostState();
};		

class PacmanConfigWindow : public CSWindow
{
	private:
		PacmanGame *mGame;
		CSCombobox *mCombobox;
		CSTextfield * mTextfield;
	public:
		PacmanConfigWindow(PacmanGame *game);
		void fillData();
		virtual void setVisible(bool v);
		void reactOnMessagePacmanConfigWindow(CSMessage *message);
		virtual void reactOnMessage(CSMessage *message);

};

#endif // PACMANGAME_H
